(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*) (******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: Bu.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:47:53 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 

IMPLEMENTATION MODULE Bu   ADT Buchung
                           --  Enthalten in Buchungen

Sigrid Preuss

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE Bu;

(* ADT Buchung *)

(* IMPORTE *)

FROM Storage	IMPORT ALLOCATE,DEALLOCATE;
FROM SYSTEM	IMPORT TSIZE, ADDRESS;
FROM GENERIC	IMPORT TObjektreferenz;
FROM WholeStr   IMPORT ConvResults, StrToCard, CardToStr, StrToInt, IntToStr;
FROM Datum      IMPORT TDatum, CreateDatum, DisposeDatum,
                       DuplicateDatum, EqualizeDatum, IsEqualDatum,
                       DatumAlsString, DatumVonString;
FROM BuenIMPORTE IMPORT
     TBuID,     DisposeBuID, DuplicateBuID, IsEqualBuID, EqualizeBuID,
                SameBuID, SmallerBuID, PruefeBuID, BuIDAlsString, StringInBuID,
     TKstID,    DisposeKstID, DuplicateKstID, EqualizeKstID, IsEqualKstID,
     TKst,      PruefeKstID,KstIDAlsString, StringInKstID,
     TVtrID,    DisposeVtrID, DuplicateVtrID, EqualizeVtrID, IsEqualVtrID,
     TVtr,      PruefeVtrID, VtrIDAlsString, StringInVtrID;


IMPORT String, Stringliste;

CONST LaengeBuIDNR   = 5;
      KonstGehalt    = "Gehalt";
      KonstWG        = "WG";
      KonstAbschluss = "Abschluss";
      KonstSonst     = "Sonst";


TYPE TString = String.TString;
     TStringliste = Stringliste.TStringliste;



     TBu_DS = RECORD
                Identifikator  : RECORD
                                   ID : TBuID;
                                   existiert : BOOLEAN;
                                 END;
                KstIdent       : RECORD
                                   KstID : TKstID;
                                   existiert : BOOLEAN;
                                 END;
                VtrIdent       : RECORD
                                   VtrID : TVtrID;
                                   existiert : BOOLEAN;
                                 END;
                Betrag      : INTEGER;
                GebuchtAm   : TDatum;
                BuchungsArt : TBuArt;
                Storno      : RECORD
                                StorniertAm : TDatum;
                                StornoGesetzt : BOOLEAN;
                              END;
              END;

TYPE  TBu = POINTER TO TBu_DS;


(*
SEMANTIK
Die Basis-Operationen Dispose, Duplicate, IsEqual, Equalize, SmallerBu,
SameBu besitzen die uebliche Semantik.

Die Attribut-Operationen
  Lies<Attribut>
  Setze<Attribut>
haben folgende Semantik:

Lies<Attribut>:
Vb:	keine
Liefert ein ADO des entsprechenden Attributs
nb:	Rueckgabe-ADO existiert

Setze<Attribut>:
vb:	keine
Gleicht den Zustand des entsprechenden Attribut-ADOs in inoutBu dem Zustand des
Parameter-Attributs in<Attribut> an.
Nb:	IsEqual(Lies<Attribut>(), in<Attribut>)
*)



(* Typ-Wandlung *)

PROCEDURE BU (inRef : TObjektreferenz) : TBu;
(*
Vb:	inRef referenziert ein Buchung-Objekt
Typtransfer von TObjektreferenz nach TBu
Nb:    inRef = OBJEKTREFERENZ(Rueckgabe-Bu)
*)

  VAR Bu : TBu;

BEGIN
    Bu := inRef;
    RETURN Bu;
END BU;

PROCEDURE OBJEKTREFERENZ (inBu : TBu) : TObjektreferenz;
(*
Vb:	keine
Typtransfer von TBu nach TObjektreferenz
Nb:    inBu = Bu(Rueckgabe-Objektreferenz)
*)

  VAR BuRef : TObjektreferenz;

BEGIN
    BuRef := inBu;
    RETURN BuRef;
END OBJEKTREFERENZ;



(* Basis-Operationen *)


PROCEDURE	Create ():TBu;
(*
Vb:	keine
Generiert ein ADO Bu.
Die Attribute dieses ADOs werden folgendermassen initialisert:
INTEGER-Attribut: 0
Datum-Attribut:   Eternity (31.12.9999)
BuchArt:          Sonst

Nb:	Rueckgabe-ADO Bu existiert
*)

  VAR Bu : TBu;

BEGIN
  ALLOCATE (Bu, TSIZE(TBu_DS));

  WITH Bu^ DO
    Identifikator.existiert := FALSE;
    KstIdent.existiert      := FALSE;
    VtrIdent.existiert      := FALSE;
    Betrag       := 0;
    GebuchtAm    := CreateDatum();
    BuchungsArt  := Sonst;
    Storno.StornoGesetzt := FALSE;

  END; (* WITH Bu^ *)

  RETURN Bu;

END Create;


PROCEDURE	Dispose (VAR inoutBu:TBu);

BEGIN
  WITH inoutBu^ DO
    IF Identifikator.existiert THEN
      DisposeBuID(Identifikator.ID);
    END; (* IF *)

    IF KstIdent.existiert THEN
      DisposeKstID(KstIdent.KstID);
    END; (* IF *)

    IF VtrIdent.existiert THEN
      DisposeVtrID(VtrIdent.VtrID);
    END; (* IF *)

    DisposeDatum(GebuchtAm);
    IF Storno.StornoGesetzt THEN
       DisposeDatum(Storno.StorniertAm);
    END (* IF *)


  END; (* WITH inoutBu^ *)

  DEALLOCATE (inoutBu, TSIZE(TBu_DS));
  inoutBu := NIL;

END Dispose;


PROCEDURE	Duplicate (inBu:TBu):TBu;

  VAR Duplikat : TBu;

BEGIN
  Duplikat := Create();
  Equalize(Duplikat,inBu);
  RETURN Duplikat;
END Duplicate;

PROCEDURE	IsEqual (inBu1,inBu2:TBu):BOOLEAN;

  VAR gleicheAttribute : BOOLEAN;
      gleicheBuID      : BOOLEAN;
      gleicheKstID     : BOOLEAN;
      gleicheVtrID     : BOOLEAN;
      gleichesStorno   : BOOLEAN;

BEGIN

(* BuID *)
    IF inBu1^.Identifikator.existiert AND inBu2^.Identifikator.existiert THEN
      gleicheBuID := SameBu (inBu1, inBu2)
    ELSIF inBu1^.Identifikator.existiert OR inBu2^.Identifikator.existiert THEN
      gleicheBuID := FALSE
    ELSE
      gleicheBuID := TRUE
    END; (* IF *)

(* KstID *)
    IF inBu1^.KstIdent.existiert AND inBu2^.KstIdent.existiert THEN
      gleicheKstID := IsEqualKstID (inBu1^.KstIdent.KstID,
                                    inBu2^.KstIdent.KstID)
    ELSIF inBu1^.KstIdent.existiert OR inBu2^.KstIdent.existiert THEN
      gleicheKstID := FALSE
    ELSE
      gleicheKstID := TRUE
    END; (* IF *)

(* VtrID *)
    IF inBu1^.VtrIdent.existiert AND inBu2^.VtrIdent.existiert THEN
      gleicheVtrID := IsEqualVtrID (inBu1^.VtrIdent.VtrID,
                                    inBu2^.VtrIdent.VtrID)
    ELSIF inBu1^.VtrIdent.existiert OR inBu2^.VtrIdent.existiert THEN
      gleicheVtrID := FALSE
    ELSE
      gleicheVtrID := TRUE
    END; (* IF *)

(* StorniertAm *)
    IF inBu1^.Storno.StornoGesetzt AND inBu2^.Storno.StornoGesetzt THEN
      gleichesStorno := IsEqualDatum (inBu1^.Storno.StorniertAm,
                                      inBu2^.Storno.StorniertAm)
    ELSIF inBu1^.Storno.StornoGesetzt OR inBu2^.Storno.StornoGesetzt THEN
      gleichesStorno := FALSE
    ELSE
      gleichesStorno := TRUE
    END; (* IF *)


(* Uebrige Attribute *)
    IF gleicheBuID  AND  gleicheKstID   AND  gleicheVtrID  AND  gleichesStorno
    THEN
      gleicheAttribute := 
        (inBu1^.Betrag =  inBu2^.Betrag)           AND
        IsEqualDatum(inBu1^.GebuchtAm,
                     inBu2^.GebuchtAm)             AND
        (inBu1^.BuchungsArt = inBu2^.BuchungsArt);

    ELSE
      gleicheAttribute := FALSE
    END; (* IF gleicheID *)

    RETURN gleicheAttribute;

END IsEqual;



PROCEDURE	Equalize (inoutZielBu:TBu; inQuelleBu:TBu);
BEGIN
  WITH inoutZielBu^ DO
    WITH Identifikator DO
      IF existiert AND inQuelleBu^.Identifikator.existiert THEN
        EqualizeBuID (ID, inQuelleBu^.Identifikator.ID)
      ELSIF existiert THEN
        (* -- inQuelleBu^.Identifikator.existiert = FALSE *)
        existiert := FALSE;
        DisposeBuID (ID)
      ELSIF inQuelleBu^.Identifikator.existiert THEN
        (* -- inoutZielBu^.Identifikator.existiert = FALSE *)
        ID := DuplicateBuID (inQuelleBu^.Identifikator.ID);
        existiert := TRUE
      ELSE 
        (* -- keine der beiden IDs existiert, es ist nichts zu tun *)
      END (* IF *)
    END; (* WITH Identifikator*)

    WITH KstIdent DO
      IF existiert AND inQuelleBu^.KstIdent.existiert THEN
        EqualizeKstID (KstID, inQuelleBu^.KstIdent.KstID)
      ELSIF existiert THEN
        (* -- inQuelleBu^.KstIdent.existiert = FALSE *)
        existiert := FALSE;
        DisposeKstID (KstID)
      ELSIF inQuelleBu^.KstIdent.existiert THEN
        (* -- inoutZielBu^.KstIdent.existiert = FALSE *)
        KstID := DuplicateKstID (inQuelleBu^.KstIdent.KstID);
        existiert := TRUE
      ELSE 
        (* -- keine der beiden IDs existiert, es ist nichts zu tun *)
      END (* IF *)
    END; (* WITH Identifikator*)

    WITH VtrIdent DO
      IF existiert AND inQuelleBu^.VtrIdent.existiert THEN
        EqualizeVtrID (VtrID, inQuelleBu^.VtrIdent.VtrID)
      ELSIF existiert THEN
        (* -- inQuelleBu^.VtrIdent.existiert = FALSE *)
        existiert := FALSE;
        DisposeVtrID (VtrID)
      ELSIF inQuelleBu^.VtrIdent.existiert THEN
        (* -- inoutZielBu^.VtrIdent.existiert = FALSE *)
        VtrID := DuplicateVtrID (inQuelleBu^.VtrIdent.VtrID);
        existiert := TRUE
      ELSE 
        (* -- keine der beiden IDs existiert, es ist nichts zu tun *)
      END (* IF *)
    END; (* WITH Identifikator*)


    WITH Storno DO
      IF StornoGesetzt AND inQuelleBu^.Storno.StornoGesetzt THEN
        EqualizeDatum (StorniertAm, inQuelleBu^.Storno.StorniertAm)
      ELSIF StornoGesetzt THEN
        (* -- inQuelleBu^.Storno.StornoGesetzt = FALSE *)
        StornoGesetzt := FALSE;
        DisposeDatum (StorniertAm)
      ELSIF inQuelleBu^.Storno.StornoGesetzt THEN
        (* -- inoutZielBu^.Storno.StornoGesetzt = FALSE *)
        StorniertAm := DuplicateDatum (inQuelleBu^.Storno.StorniertAm);
        StornoGesetzt := TRUE
      ELSE 
        (* -- keine der beiden IDs existiert, es ist nichts zu tun *)
      END (* IF *)
    END; (* WITH Storno*)

    Betrag := inQuelleBu^.Betrag;
    EqualizeDatum(GebuchtAm,inQuelleBu^.GebuchtAm);
    BuchungsArt := inQuelleBu^.BuchungsArt;

  END; (* WITH inoutZielBu^ *)

END Equalize;



PROCEDURE	SmallerBu (inBu1,inBu2:TBu):BOOLEAN;
BEGIN
  RETURN SmallerBuID(inBu1^.Identifikator.ID,
                             inBu2^.Identifikator.ID);
END SmallerBu;


PROCEDURE	SameBu (inBu1,inBu2:TBu):BOOLEAN;
BEGIN
  RETURN SameBuID(inBu1^.Identifikator.ID,
                          inBu2^.Identifikator.ID);
END SameBu;




(* BuID-Operationen *)

PROCEDURE	LiesBuID (inBu:TBu):TBuID;
(*
Vb:	 ExistiertBuID(inBu)
Liefert den Identifikator BuID der Buchung inBu
Nb:	RueckgabeADO BuID existiert
*)

BEGIN
  RETURN DuplicateBuID(inBu^.Identifikator.ID);
END LiesBuID;


PROCEDURE	SetzeBuID (inBu:TBu; inBuID : TBuID);
(*
Vb:	NOT ExistiertBuID(inBu)
Versetzt die Buchung inBu mit dem Identifikator BuID
Nb:	ExistiertBuID()
*)

BEGIN

  WITH inBu^.Identifikator DO
    ID := DuplicateBuID(inBuID);
    existiert := TRUE;
  END (* WITH *)

END SetzeBuID;

PROCEDURE	ExistiertBuID (inBu:TBu):BOOLEAN;
(*
Vb:	keine
Prueft, ob die Buchung bereits mit einer BuID versehen wurde
Nb:	Rueckgabewert = TRUE <=> BuID existiert
*)

BEGIN
  RETURN inBu^.Identifikator.existiert;
END ExistiertBuID;




(******** Operationen auf den Attributen einer Buchung **********)


(* KstID-Operationen *)

PROCEDURE	LiesKstID (inBu:TBu):TKstID;
(*
Vb:	 ExistiertKstID(inBu)
Liefert die KstID der Buchung inBu
Nb:	RueckgabeADO KstID existiert
*)

BEGIN
  RETURN DuplicateKstID(inBu^.KstIdent.KstID);
END LiesKstID;


PROCEDURE	SetzeKstID (inBu:TBu; inKstID : TKstID);
(*
Vb:	NOT ExistiertKstID(inBu)
Versetzt die Buchung inBu mit der KstID
Nb:	ExistiertKstID()
*)

BEGIN

  WITH inBu^.KstIdent DO
    KstID := DuplicateKstID(inKstID);
    existiert := TRUE;
  END (* WITH *)

END SetzeKstID;

PROCEDURE	ExistiertKstID (inBu:TBu):BOOLEAN;
(*
Vb:	keine
Prueft, ob die Buchung bereits mit einer KstID versehen wurde
Nb:	Rueckgabewert = TRUE <=> KstID existiert
*)

BEGIN
  RETURN inBu^.KstIdent.existiert;
END ExistiertKstID;



(* VtrID-Operationen *)


PROCEDURE	LiesVtrID (inBu:TBu):TVtrID;
(*
Vb:	ExistiertVtrID()
Liefert den Identifikator VtrID der Buchung inBu
Nb:	RueckgabeADO VtrID existiert
*)

BEGIN
  RETURN DuplicateVtrID(inBu^.VtrIdent.VtrID);
END LiesVtrID;


PROCEDURE	SetzeVtrID (inBu:TBu; inVtrID : TVtrID);
(*
Vb:	NOT ExistiertVtrID()
Versetzt die Buchung inBu mit dem Identifikator VtrID
Nb:	ExistiertVtrID()
*)

BEGIN

  WITH inBu^.VtrIdent DO
    VtrID := DuplicateVtrID(inVtrID);
    existiert := TRUE;
  END (* WITH *)

END SetzeVtrID;

PROCEDURE	ExistiertVtrID (inBu:TBu):BOOLEAN;
(*
Vb:	keine
Prueft, ob die Buchung bereits mit einer VtrID versehen wurde
Nb:	Rueckgabewert = TRUE <=> VtrID existiert
*)

BEGIN
  RETURN inBu^.VtrIdent.existiert;
END ExistiertVtrID;





PROCEDURE	LiesBetrag (inBu:TBu):INTEGER;
BEGIN
  RETURN inBu^.Betrag;
END LiesBetrag;


PROCEDURE	SetzeBetrag (inoutBu:TBu; inBetrag:INTEGER);
BEGIN
  inoutBu^.Betrag := inBetrag;
END SetzeBetrag;


PROCEDURE	LiesGebuchtAm (inBu:TBu):TDatum;
BEGIN
  RETURN DuplicateDatum(inBu^.GebuchtAm);
END LiesGebuchtAm;


PROCEDURE	SetzeGebuchtAm (inoutBu:TBu; inGebuchtAm:TDatum);
BEGIN
  EqualizeDatum(inoutBu^.GebuchtAm, inGebuchtAm);
END SetzeGebuchtAm;


PROCEDURE	LiesBuchungsArt (inBu:TBu):TBuArt;
BEGIN
  RETURN inBu^.BuchungsArt;
END LiesBuchungsArt;


PROCEDURE	SetzeBuchungsArt (inoutBu:TBu; inBuArt:TBuArt);
BEGIN
  inoutBu^.BuchungsArt := inBuArt;
END SetzeBuchungsArt;


PROCEDURE	LiesStorniertAm (inBu:TBu):TDatum;
(*
Vb:     IstStorniert(inBu)
Liefert das Storno-Datum der Buchung inBu
Nb:	RueckgabeADO StorniertAm existiert
*)

BEGIN
  RETURN DuplicateDatum(inBu^.Storno.StorniertAm);
END LiesStorniertAm;


PROCEDURE	SetzeStorniertAm (inoutBu:TBu; inStorniertAm:TDatum);
(*
Vb:     NOT IstStorniert(inBu)
Setzt das Storno-Datum der Buchung inBu auf inStorniertAm
Nb:	IstStorniert(inBu)
*)

BEGIN
  inoutBu^.Storno.StorniertAm := DuplicateDatum(inStorniertAm);
  inoutBu^.Storno.StornoGesetzt := TRUE;
END SetzeStorniertAm;


PROCEDURE	IstStorniert (inBu:TBu):BOOLEAN;
BEGIN
  RETURN inBu^.Storno.StornoGesetzt;
END IstStorniert;



(* Operationen zur persistenten Speicherung *)


PROCEDURE       CreateStringlisteFromObject (inBu : TBu) : TStringliste;
(*
Vb:    keine
Saemtliche Attribute des ADOs inBu werden in Strings konvertiert und
in eine Stringliste eingefuegt.
Nb:    Rueckgabe-ADO Stringliste existiert  AND
       CreateObjectFromStringliste(Rueckgabe-ADO Stringliste) = inBu
*)

  VAR BuStringliste : TStringliste;
      lfdString     : TString;
      zwString      : TString; (* String zum Zwischenspeichern *)
      strArray      : ARRAY [0..255] OF CHAR;
      OneCharString : ARRAY [0..0] OF CHAR;
BEGIN
  BuStringliste := Stringliste.Create();
  zwString := String.Create();

  (* BuID *)
  lfdString := BuIDAlsString(inBu^.Identifikator.ID, LaengeBuIDNR);
  Stringliste.Append(BuStringliste, lfdString);
  String.Dispose(lfdString);

  (* KstID *)
  IF inBu^.KstIdent.existiert THEN
     lfdString := KstIDAlsString(inBu^.KstIdent.KstID);
  ELSE
     lfdString := String.Create();
  END; (* IF *)
  Stringliste.Append(BuStringliste, lfdString);
  String.Dispose(lfdString);

  (* VtrID *)
  IF inBu^.VtrIdent.existiert THEN
     lfdString := VtrIDAlsString(inBu^.VtrIdent.VtrID);
  ELSE
     lfdString := String.Create();
  END; (* IF *)
  Stringliste.Append(BuStringliste, lfdString);
  String.Dispose(lfdString);

  (* Betrag *)
  IntToStr(inBu^.Betrag,strArray);
  String.Schreibe(zwString,strArray);
  Stringliste.Append(BuStringliste, zwString);

  (* GebuchtAm *)
  lfdString := DatumAlsString(inBu^.GebuchtAm);
  Stringliste.Append(BuStringliste, lfdString);
  String.Dispose(lfdString);

  (* Buchungsart *)
  CASE inBu^.BuchungsArt OF

       Gehalt : String.Schreibe(zwString, KonstGehalt);
      |WG     : String.Schreibe(zwString, KonstWG);
      |Abschluss : String.Schreibe(zwString, KonstAbschluss);
      |Sonst : String.Schreibe(zwString, KonstSonst);

  END; (* CASE *)
  Stringliste.Append(BuStringliste, zwString);

  (* StorniertAm *)
  IF inBu^.Storno.StornoGesetzt THEN
     lfdString := DatumAlsString(inBu^.Storno.StorniertAm);
  ELSE
     lfdString := String.Create();
  END; (* IF *)
  Stringliste.Append(BuStringliste, lfdString);
  String.Dispose(lfdString);


  String.Dispose(zwString);
  RETURN BuStringliste;

END CreateStringlisteFromObject;




PROCEDURE       CreateObjectFromStringliste (inoutStringliste : TStringliste;
                                             VAR outBu :TBu;
                                             VAR outMessage : TString
                                            ) : BOOLEAN;
(*
Vb:    inoutStringliste ist mit CreateStringlisteFromObject erstellt.
Erzeugt aus inoutStringliste ein ADO Bu
Nb:     (Rueckgabewert = TRUE
         => CreateStringlisteFromObject(outBu) = inoutStringliste
            AND
            outMessage undefiniert)
        AND
        (Rueckgabewert = FALSE
         => outMessage enthaelt eine Fehlermeldung AND
            outBu undefiniert)
*)

  VAR rc         : BOOLEAN;
      index      : CARDINAL;
      lfdString  : TString;
      zwString   : TString; (* String zum Zwischenspeichern *)
      StrBuArt   : TString;
      StrBuID    : TString;
      lfdDatum   : TDatum;
      strArray   : ARRAY [0..255] OF CHAR;
      KonvResult : ConvResults;

BEGIN
  rc := TRUE;
  outBu := Create();
  zwString := String.Create();
  StrBuID := String.Create();


  IF NOT Stringliste.IsEmpty(inoutStringliste) THEN
     Stringliste.GoStart(inoutStringliste);
     index := 0;

     WHILE (rc AND NOT Stringliste.OffEnd(inoutStringliste)) DO
        lfdString := Stringliste.Current(inoutStringliste);
        CASE index OF
            0 :  (* BuID *)
                 String.Equalize(StrBuID, lfdString);
                 IF PruefeBuID(lfdString, outMessage) THEN
                    WITH outBu^.Identifikator DO
                      ID := StringInBuID(lfdString);
                      existiert := TRUE;
                    END; (* WITH *)
                 ELSE
                    String.Schreibe(zwString,
                           " - CreateObject-Fehler bei BuID, String=");
                    String.Anhaengen(zwString, outMessage);
                    String.Anhaengen(lfdString, outMessage);
                    rc := FALSE;
                 END (* IF BuID OK *)
          | 1 :  (* KstID *)
                 IF String.Laenge(lfdString) > 0 THEN
                    IF PruefeKstID(lfdString, outMessage) THEN
                       WITH outBu^.KstIdent DO
                         KstID := StringInKstID(lfdString);
                         existiert := TRUE;
                       END; (* WITH *)
                    ELSE
                      String.Schreibe(zwString,
                              " - CreateObject-Fehler bei KstID, String=");
                       String.Anhaengen(zwString, outMessage);
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END (* IF KstID OK *)
                 END; (* IF StringLaenge > 0 *)
          | 2 :  (* VtrID *)
                 IF String.Laenge(lfdString) > 0 THEN
                    IF PruefeVtrID(lfdString, outMessage) THEN
                       WITH outBu^.VtrIdent DO
                         VtrID := StringInVtrID(lfdString);
                         existiert := TRUE;
                       END; (* WITH *)
                    ELSE
                      String.Schreibe(zwString,
                              " - CreateObject-Fehler bei VtrID, String=");
                       String.Anhaengen(zwString, outMessage);
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END (* IF VtrID OK *)
                 END; (* IF StringLaenge > 0 *)
          | 3 :  (* Betrag *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Lies(lfdString, strArray);
                    StrToInt(strArray, outBu^.Betrag, KonvResult);
                    IF KonvResult # strAllRight  THEN
                       outMessage := String.Create();
                       String.Schreibe(outMessage,
                              "CreateObject-Fehler bei Betrag, String=");
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END; (* IF StrToInt OK *)
                 END; (* IF StringLaenge > 0 *)

          | 4 :  (* GebuchtAm *)
                 IF String.Laenge(lfdString) > 0 THEN
                    IF DatumVonString(lfdString, lfdDatum, outMessage) THEN
                       EqualizeDatum(outBu^.GebuchtAm, lfdDatum);
                       DisposeDatum(lfdDatum);
                    ELSE
                       String.Schreibe(zwString,
                           " - CreateObject-Fehler bei GebuchtAm, String=");
                       String.Anhaengen(zwString, outMessage);
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END (* IF GebuchtAm OK *)
                 END; (* IF StringLaenge > 0 *)

          | 5 :  (* BuchungsArt *)
                 IF String.Laenge(lfdString) > 0 THEN
                    StrBuArt := String.Create();
                    String.Schreibe(StrBuArt, KonstGehalt);
                    IF String.IsEqual(lfdString, StrBuArt) THEN
                       outBu^.BuchungsArt := Gehalt;
                    ELSE
                       String.Schreibe(StrBuArt, KonstWG);
                       IF String.IsEqual(lfdString, StrBuArt) THEN
                          outBu^.BuchungsArt := WG;
                       ELSE
                          String.Schreibe(StrBuArt, KonstAbschluss);
                          IF String.IsEqual(lfdString, StrBuArt) THEN
                             outBu^.BuchungsArt := Abschluss;
                          ELSE
                             String.Schreibe(StrBuArt, KonstSonst);
                             IF String.IsEqual(lfdString, StrBuArt) THEN
                                outBu^.BuchungsArt := Sonst;
                             ELSE
                               outMessage := String.Create();
                               String.Schreibe(outMessage,
                             " - CreateObject-Fehler bei BuchungsArt, String=");
                               String.Anhaengen(lfdString, outMessage);
                               rc := FALSE;
                             END (* IF BuchungsArt = Sonst*)
                          END (* IF BuchungsArt = Abschluss*)
                       END (* IF BuchungsArt = WG*)
                    END; (* IF BuchungsArt = Gehalt*)
                    String.Dispose(StrBuArt);
                 END; (* IF StringLaenge > 0 *)

          | 6 :  (* StorniertAm *)
                 IF String.Laenge(lfdString) > 0 THEN
                    IF DatumVonString(lfdString, lfdDatum, outMessage) THEN
                       outBu^.Storno.StorniertAm := DuplicateDatum(lfdDatum);
                       outBu^.Storno.StornoGesetzt := TRUE;
                       DisposeDatum(lfdDatum);
                    ELSE
                       String.Schreibe(zwString,
                           " - CreateObject-Fehler bei StorniertAm, String=");
                       String.Anhaengen(zwString, outMessage);
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END (* IF StorniertAm OK *)
                 END; (* IF StringLaenge > 0 *)
          ELSE
                 outMessage := String.Create();
                 String.Schreibe(outMessage,
                        "CreateObject-Fehler, mehr als 7 Attribute, String=");
                 String.Anhaengen(lfdString, outMessage);
                 rc := FALSE;

        END; (* CASE ueber den Index der Stringliste *)

        index := index + 1;
        String.Dispose(lfdString);
        Stringliste.StepNext(inoutStringliste);

     END; (* WHILE Schleife ueber die Elemente der Stringliste *)

  END; (* IF inoutStringliste nicht leer *)

  IF NOT rc THEN
     (* Im Fehlerfall BuID in die outMessage *)
     String.Schreibe(zwString, "; BuID=");
     String.Anhaengen(zwString, outMessage);
     String.Anhaengen(StrBuID, outMessage);
     Dispose(outBu);
  END; (* IF *)
  String.Dispose(zwString);
  String.Dispose(StrBuID);

  RETURN rc;

END CreateObjectFromStringliste;



END Bu.
