(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*) (******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: BuListe.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:47:53 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 

IMPLEMENTATION MODULE BuListe   ADT SortierteBuchungenListe
                                Instantiierungs-Modul
                                        --  Enthalten in Hilfskraefte

Sigrid Preuss

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE BuListe;

(* ADT SortierteBuchungenListe *)

(* IMPORTE *)


  IMPORT Bu;
  IMPORT SortierteObjektliste;

  TYPE TBuListe =  SortierteObjektliste.TSortierteObjektliste;
       TObjektreferenz = SortierteObjektliste.TObjektreferenz;


(* SEMANTIK:

Die Semantik ist dem Modul "SortierteObjektliste.def" entnommen

Die Semantik wird auf Basis zweier lokaler Integervariablen len und pos,
die den Zustand der sortierten Objektliste beschreiben, und einer
lokalen Kapazitaetskonstanten maxlen spezifiziert: 
Sei len die Laenge der sortierten Objektliste, maxlen > 0 die Maximal-
kapazitaet der Liste, d.h. 0 <= len <= maxlen, und pos die Position 
des aktuellen Listenelements mit 0 <= pos <= len + 1.
Die Indizes alt und neu bezeichnen den Wert einer Zustandsvariablen vor
bzw. nach Ausfuehren einer Operation. Werden die neuen Zustaende der
Variablen nicht explizit angegeben, so sind sie invariant.
*)

PROCEDURE  Create () : TBuListe;
(*
Vb:     keine
Generiert und initialisert ein ADO TBuListe.
Nb:     Rueckgabe-ADO TBuListe existiert AND IsEmpty(TBuListe)
*)

  VAR outBuListe : TBuListe;

BEGIN
  outBuListe := SortierteObjektliste.Create (DisposeBu,
                                             DuplicateBu,
                                             EqualizeBu,
                                             IsEqualBu,
                                             SmallerBu,
                                             SameBu);
  RETURN outBuListe;

END Create;


PROCEDURE Dispose (VAR inoutBuListe : TBuListe);
(*
Vb:     IsEmpty (inoutBuListe : TBuListe)
Entfernt das ADO inoutBuListe aus dem System.
Nb:     ADO inoutBuListe existiert nicht mehr
*)
BEGIN
  SortierteObjektliste.Dispose (inoutBuListe);
END Dispose;


PROCEDURE Current(inBuListe : TBuListe) : TBu;
(*
Vb:     NOT (OffStart (inBuListe) OR OffEnd (inBuListe))
Liefert das Objekt an der aktuellen Position.
Nb:     NOT (OffStart(inBuListe) OR OffEnd(inBuListe))
*)
BEGIN
  RETURN Bu.BU (SortierteObjektliste.Current (inBuListe));
END Current;


PROCEDURE Update (inoutBuListe : TBuListe;
                  inObjekt     : TBu);
(*
Vb:     NOT (OffStart (inoutBuListe) OR OffEnd (inoutBuListe))
Ueberschreibt unter Beachtung der Ordnung die Attributwerte des 
Objekts an der aktuellen Position mit denen von inObjekt. 
Anschliessend sind das Objekt an der ggf. neuen aktuellen 
Position und inObjekt zustandsgleich
Nb:   NOT (OffStart (inoutBuListe) OR OffEnd (inoutBuListe)
*)
BEGIN
  SortierteObjektliste.Update (inoutBuListe, Bu.OBJEKTREFERENZ (inObjekt));
END Update;


PROCEDURE Insert (inoutBuListe : TBuListe;
                  inObjekt     : TBu);
(*
Vb:     keine
Einfuegen eines Objekts bzgl. der Sortierreihenfolge
Anschliessend befindet sich inObjekt an der aktuellen Position 
Nb:     (1 <= inoutBuListe.pos_neu <= inoutBuListe.len) AND
              (inoutBuListe.len_neu = inoutBuListe.len_alt + 1) AND NOT 
              (IsEmpty (inoutBuListe) OR OffStart (inoutBuListe)
               OR OffEnd (inoutBuListe) )
*)

BEGIN
  SortierteObjektliste.Insert (inoutBuListe, Bu.OBJEKTREFERENZ (inObjekt));
END Insert;


PROCEDURE Search (inoutBuListe : TBuListe;
                  inObjekt     : TBu) : BOOLEAN;
(*
Vb:     keine
Suche nach inObjekt; wird ein zustandsgleiches Objekt gefunden, 
ist dort die neue aktuelle Position; sonst OffEnd
Sei found der boole'sche Rueckgabeparameter, dann gilt:
Nb:     (found => NOT (OffStart (inoutBuListe) OR OffEnd (inoutBuListe))) AND
        (NOT found => OffEnd (inoutBuListe))
*)

BEGIN
  RETURN SortierteObjektliste.Search (inoutBuListe,
                                      Bu.OBJEKTREFERENZ (inObjekt));
END Search;


PROCEDURE GoSmallest (inoutBuListe : TBuListe);
(*
Vb:     keine
Die Position des kleinsten Objekts der Liste wird zur aktuellen Position
Nb:     inoutBuListe.pos_neu = 1
*)
BEGIN
  SortierteObjektliste.GoSmallest (inoutBuListe);
END GoSmallest;


PROCEDURE StopAt (inoutBuListe : TBuListe;
                  inObjekt     : TBu) : BOOLEAN;
(*
Vb:     keine
Suche nach einem bzgl. SameValue Objekt gleichen Objekt. Wird ein
solches Objekt gefunden, ist dort die neue aktuelle Position; 
Seien found der boole'sche Rueckgabeparameter und aktObjekt bzw.   
MaxObjekt die Objekte an pos_neu bzw. pos = len (falls diese 
existieren), dann gilt:
Nb:    (found => NOT (OffStart (inoutBuListe) OR OffEnd (inoutBuListe)) AND
                 Objekt.SameValue (inObjekt, aktObjekt)) AND
       (NOT found => (IsEmpty (inoutBuListe) AND OffEnd (inoutBuListe)) OR
                     (NOT IsEmpty (inoutBuListe) AND OffEnd (inoutBuListe) AND
                      Objekt.SmallerValue (MaxObjekt, inObjekt)) OR
                      (NOT (OffStart (inoutBuListe) OR
                            OffEnd (inoutBuListe)) AND
                       Objekt.SmallerValue (aktObjekt, inObjekt))
*)

BEGIN
  RETURN SortierteObjektliste.StopAt (inoutBuListe, Bu.OBJEKTREFERENZ (inObjekt));
END StopAt;



PROCEDURE GoLargest (inoutBuListe : TBuListe);
(*
Vb:     keine
Die Position des groessten Objekts der Liste wird zur aktuellen Position
Nb:     inoutBuListe.pos_neu = inoutBuListe.len
*)

BEGIN
  SortierteObjektliste.GoLargest (inoutBuListe);
END GoLargest;


PROCEDURE StepNext (inoutBuListe : TBuListe);
(*
Vb:     NOT OffEnd (inoutBuListe)
Die Position hinter der aktuellen Position wird zur aktuellen Position
Nb:     (inoutBuListe.pos_neu = inoutBuListe.pos_alt + 1
         OR (IsEmpty (inoutBuListe) => OffEnd (inoutBuListe)
*)

BEGIN
  SortierteObjektliste.StepNext (inoutBuListe);
END StepNext;



PROCEDURE StepBack (inoutBuListe : TBuListe);
(*
Vb:     NOT OffStart (inoutBuListe)
Die Position vor der aktuellen Position wird zur aktuellen Position
Nb:     inoutBuListe.pos_neu = inoutBuListe.pos_alt - 1) OR
        (IsEmpty (inoutBuListe) => OffStart (inoutBuListe)
*)

BEGIN
  SortierteObjektliste.StepBack (inoutBuListe);
END StepBack;


PROCEDURE IsEmpty (inBuListe : TBuListe) : BOOLEAN;
(*
Vb:     keine
Test, ob die Liste leer ist:
IsEmpty (inBuListe) <=> (inBuListe.len = 0)
Nb:     IsEmpty (inBuListe) => OffStart (inBuListe) OR OffEnd (inBuListe)
*)

BEGIN
  RETURN SortierteObjektliste.IsEmpty (inBuListe);
END IsEmpty;


PROCEDURE OffStart (inBuListe : TBuListe) : BOOLEAN;
(*
Vb:     keine
Test, ob die Position vor dem ersten Objekt aktuell ist:
OffStart (inBuListe) <=> inBuListe.pos = 0
Nb:     keine
*)

BEGIN
  RETURN SortierteObjektliste.OffStart (inBuListe);
END OffStart;


PROCEDURE OffEnd (inBuListe : TBuListe) : BOOLEAN;
(*
Vb:     keine
Test, ob die Position hinter dem letzten Objekt aktuell ist:
OffEnd (inBuListe) <=> inBuListe.pos = inBuListe.len + 1
Nb:     Rueckgabe-ADO TBuListe existiert AND IsEmpty(TBuListe)
*)

BEGIN
  RETURN SortierteObjektliste.OffEnd (inBuListe);
END OffEnd;
 

(* ------------------------------------------------------------------
   -- Typkonvertierungen von TBu und TObjektreferenz
   ------------------------------------------------------------------
*)

PROCEDURE DisposeBu (VAR inoutReferenz : TObjektreferenz);
 VAR aktBu : TBu;
BEGIN
  aktBu := Bu.BU (inoutReferenz);
  Bu.Dispose (aktBu);
  inoutReferenz := Bu.OBJEKTREFERENZ (aktBu);
END DisposeBu;

PROCEDURE DuplicateBu (inReferenz : TObjektreferenz) : TObjektreferenz;
  VAR Source      : TBu;
      outDuplikat : TBu;
BEGIN
  Source :=Bu.BU (inReferenz);
  outDuplikat := Bu.Duplicate (Source);
  RETURN Bu.OBJEKTREFERENZ (outDuplikat)
END DuplicateBu;

PROCEDURE EqualizeBu (inoutZiel, inQuelle : TObjektreferenz);
  VAR ZielBu  : TBu;
      QuellBu : TBu;
BEGIN
 ZielBu :=Bu.BU (inoutZiel);
 QuellBu :=Bu.BU (inQuelle);
 Bu.Equalize (ZielBu, QuellBu)
END EqualizeBu;

PROCEDURE IsEqualBu (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Bu1 : TBu;
      Bu2 : TBu;
BEGIN
  Bu1 :=Bu.BU (inReferenz1);
  Bu2 :=Bu.BU (inReferenz2);
  RETURN Bu.IsEqual (Bu1, Bu2)
END IsEqualBu;

PROCEDURE SmallerBu (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Bu1 : TBu;
      Bu2 : TBu;
BEGIN
  Bu1 :=Bu.BU (inReferenz1);
  Bu2 :=Bu.BU (inReferenz2);
  RETURN Bu.SmallerBu (Bu1, Bu2)
END SmallerBu;

PROCEDURE SameBu (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Bu1 : TBu;
      Bu2 : TBu;
BEGIN
  Bu1 :=Bu.BU (inReferenz1);
  Bu2 :=Bu.BU (inReferenz2);
  RETURN Bu.SameBu (Bu1, Bu2)
END SameBu;

END BuListe.
