(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*)
(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: BuOrdnerSchablone.mod,v $
# $Author: Sopra $
# $Date: 2001/03/13 18:32:04 $
# $Revision: 1.7 $
# Bemerkungen: 

IMPLEMENTATION MODULE BuOrdnerSchablone  ADT BuchungenOrdnerSchablone

                                         --  Enthalten in Buchungen

Sigrid Preuss

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE BuOrdnerSchablone;

(* ADT BuchungenOrdnerSchablone *)

(* IMPORTE *)

FROM Storage     IMPORT ALLOCATE, DEALLOCATE;
FROM SYSTEM      IMPORT TSIZE;
FROM WholeStr    IMPORT IntToStr, CardToStr;
FROM Strings     IMPORT Append;
FROM Datum       IMPORT TDatum, CreateDatum, DisposeDatum,
                        DatumAlsString, DatumVonString;
FROM BuenIMPORTE IMPORT TBuID, TVtrID, TVtr, TKstID,
                        DisposeBuID, IsEqualBuID,
                        HoleNeueBuID, UeberlasseAlteBuID,
                        BuIDAlsString,
                        DisposeKstID, DuplicateKstID, IsEqualKstID,
                        DisposeVtrID, DuplicateVtrID, IsEqualVtrID,
                        LiesSystemDatumInSysDat;

IMPORT Bu, BuListe, String, Stringliste;

CONST LaengeBuIDNR = 5;

TYPE TBu = Bu.TBu;
     TBuArt = Bu.TBuArt;   (* = (Gehalt, WG, Abschluss, Sonst) *)
     TString = String.TString;
     TStringliste = Stringliste.TStringliste;
     TBuOrdner = BuListe.TBuListe;



(*
SEMANTIK
Die Semantik wird mit Hilfe dreier lokaler Integervariablen kap, 
pos und maxkap spezifiziert. Die Variablen kap und pos beschreiben  
den aktuellen Zustand (aktuelle Position und Anzahl der enthaltenen
Elemente) und  maxkap > 0 die Maximalkapazitaet des Ordners, 
d.h. es gilt stets:
            0 <= kap <= maxkap   und    0 <= pos <= kap+1

*)


PROCEDURE	Create () : TBuOrdner;
(*
Vb:	keine
Erzeugt und initialisiert ein ADO BuOrdner.
Nb:	Rueckgabe-ADO BuOrdner existiert und ist leer
*)

  VAR BuOrdner : TBuOrdner;

BEGIN
  BuOrdner := BuListe.Create();

  RETURN BuOrdner;

END Create;


PROCEDURE	Dispose (VAR inoutBuOrdner : TBuOrdner);
(*
Vb:	keine
Entfernt alle Realisierungs-ADOs, die in inoutBuOrdner enthalten sind.
Entfernt das ADO inoutBuOrdner aus dem System
Nb:	ADO BuOrdner existiert nicht mehr
*)

BEGIN
  BuListe.Dispose(inoutBuOrdner);
  inoutBuOrdner := NIL;

END Dispose;



PROCEDURE IsEmpty (inBuOrdner : TBuOrdner) : BOOLEAN;
(*
VB:	keine
Test, ob der Ordner leer ist
Nb:	 IsEmpty (inBuOrdner) <=> kap = 0
*)

BEGIN

  RETURN BuListe.IsEmpty (inBuOrdner);

END IsEmpty;




PROCEDURE	Insert(inoutBuOrdner : TBuOrdner; inoutBu: TBu;
                       VAR outMessage : TString) : BOOLEAN;
(*
Vb:	NOT SearchBu(LiesBuIDBu(inoutBu), x)
Fuegt die Buchung in den Ordner ein und setzt den Identifikator der Buchung
Nb:    (Rueckgabewert = TRUE
        => (kap_neu = kap_alt+1)  AND
           (SearchBu(LiesBuIDBu(inoutBu), x) = TRUE)  AND
           outMessage undefiniert)
        AND
        (Rueckgabewert = FALSE
         => outMessage enthaelt eine Fehlermeldung)
*)

  VAR aktBuID : TBuID;
      rc        : BOOLEAN;

BEGIN
  IF NOT Bu.ExistiertBuID(inoutBu) THEN
     aktBuID := HoleNeueBuID();
     Bu.SetzeBuID(inoutBu, aktBuID);
     BuListe.Insert(inoutBuOrdner,inoutBu);
     DisposeBuID(aktBuID);
     rc := TRUE;
  ELSE
      outMessage := String.Create();
      String.Schreibe(outMessage, "BuID ist bereits gesetzt.");
      rc := FALSE;
  END; (* IF *)

  RETURN rc;

END Insert;



PROCEDURE	Search(inoutBuOrdner : TBuOrdner;
                       inBuID : TBuID; VAR outBu : TBu) : BOOLEAN;
(*
Vb:	keine
Sucht die Buchung mit der uebergebenen BuID; wird die Buchung gefunden,
so wird sie in outBu bereitgestellt;
Nb:    (Rueckgabewert = TRUE
        <=> LiesBuIDBu(inoutBuOrdner,outBu) = inBuID 
*)

  VAR aktBuID : TBuID;
      rc        : BOOLEAN;
      lfdBu     : TBu;
      lfdBuID : TBuID;

BEGIN
  rc := FALSE;
  IF NOT IsEmpty (inoutBuOrdner) THEN  
     BuListe.GoSmallest (inoutBuOrdner);
     WHILE NOT (rc OR BuListe.OffEnd (inoutBuOrdner)) DO
       lfdBu := BuListe.Current (inoutBuOrdner);
       lfdBuID := Bu.LiesBuID (lfdBu);
       IF IsEqualBuID (lfdBuID, inBuID) THEN
          rc := TRUE;
          outBu := Bu.Duplicate (lfdBu)  
       ELSE 
          BuListe.StepNext (inoutBuOrdner)
       END; (* IF *)
       DisposeBuID (lfdBuID);
       Bu.Dispose (lfdBu)
     END; (* WHILE nicht gefunden*)
  END; (* IF Ordner nicht leer*)
  RETURN rc
   
END Search;


PROCEDURE	Update(inoutBuOrdner : TBuOrdner; inBu : TBu;
                       VAR outMessage : TString):BOOLEAN;
(*
Vb:	SearchBu(LiesBuIDBu(inBu), x)
Aktualisiert die Daten der Buchung Bu im Ordner
Nb:    (Rueckgabewert = TRUE
        => 	SearchBu(LiesBuIDBu(inBu), x)  AND
           outMessage undefiniert)
        AND
        (Rueckgabewert = FALSE
         => outMessage enthaelt eine Fehlermeldung)
*)

  VAR  rc : BOOLEAN;

BEGIN
  IF BuListe.StopAt(inoutBuOrdner, inBu) THEN
     BuListe.Update(inoutBuOrdner, inBu);
     rc := TRUE;
  ELSE
     outMessage := String.Create();
     String.Schreibe(outMessage,
                     "Buchung ist nicht im Buchungen-Ordner.");
     rc := FALSE;

  END; (* IF *)

  RETURN rc;

END Update;


PROCEDURE BerGebGelderDerKst (inoutBuOrdner : TBuOrdner; inKstID : TKstID
                             ) : INTEGER; 
(* Vb:  keine 
Berechne gebuchte Gelder der Kostenstelle inKstID
Prozess D3.6.1.1 S.44 
FOR alle B in Buchungen mit B.StorniertAm = UNDEF AND  B.KstID = inKSTID DO
    Summe += B.Betrag (=Rueckgabe)
Nb:     Rueckgabe der gebuchten Gelder zu der Kostenstelle inKstID 
*)

  VAR Summe : INTEGER;
      lfdBu : TBu;
      lfdKstID : TKstID;

BEGIN
  Summe := 0;
  IF First(inoutBuOrdner, lfdBu) THEN
     IF Bu.ExistiertKstID(lfdBu) THEN
        lfdKstID := Bu.LiesKstID(lfdBu);
        IF IsEqualKstID(inKstID, lfdKstID) THEN
           Summe := Bu.LiesBetrag(lfdBu);
        END; (* IF *)
        DisposeKstID(lfdKstID);
     END; (* IF ExistiertKstID *)

     WHILE Next(inoutBuOrdner, lfdBu) DO
        IF Bu.ExistiertKstID(lfdBu) THEN
           lfdKstID := Bu.LiesKstID(lfdBu);
           IF IsEqualKstID(inKstID, lfdKstID) THEN
             Summe := Summe + Bu.LiesBetrag(lfdBu);
           END; (* IF *)
           DisposeKstID(lfdKstID);
        END; (* IF ExistiertKstID *)
     END; (* WHILE Schleife ueber alle nichtstornierten Buchungen *)
     Bu.Dispose(lfdBu);
  END; (* IF inoutBuOrdner nichtleer*)

  RETURN Summe;

END BerGebGelderDerKst;


PROCEDURE StorniereBuZuVtr (inoutBuOrdner  : TBuOrdner; inVtrID : TVtrID ;
                            VAR outMessage : TString) : BOOLEAN;
(*
Vb: --
Storniert alle Buchungen der Buchungsart "Gehalt" zu dem Vertrag inVtr
Stornierungsdatum ist das aktuelle Systemdatum
Prozess D3.6.2.3 S.49
FOR alle B in Buchungen mit B.VtrID = inVtrID AND BuArt = Gehalt DO
    B.StorniertAm := Heute;

Sei OK der boolesche Rueckgabeparameter, dann gilt:
Nb: OK <=> Alle Buchungen  der Buchungsart "Gehalt" zu dem Vertrag inVtr
           sind storniert
*)

  VAR rc : BOOLEAN;
      lfdBu : TBu;
      heute : TDatum;


BEGIN
  rc := TRUE;
  IF FirstBuMitBuArtZuVtr (inoutBuOrdner, inVtrID, Gehalt, lfdBu) THEN

     heute := LiesSystemDatumInSysDat();

     Bu.SetzeStorniertAm(lfdBu, heute);
     rc := Update(inoutBuOrdner, lfdBu, outMessage);
       
     WHILE rc AND
           NextBuMitBuArtZuVtr(inoutBuOrdner, inVtrID, Gehalt, lfdBu) DO
         Bu.SetzeStorniertAm(lfdBu, heute);
         rc := Update(inoutBuOrdner, lfdBu, outMessage);
     END; (* WHILE *)

    DisposeDatum(heute);
    Bu.Dispose(lfdBu);

  END; (* IF First *)

  RETURN rc;

END StorniereBuZuVtr;





(* -------------------------------------------------------------------------- *)
(* Iteratoren ueber nicht stornierte Buchungen                                *)
(* -------------------------------------------------------------------------- *)


PROCEDURE	First (inoutBuOrdner : TBuOrdner; VAR outBu : TBu) : BOOLEAN;
(*
Vb:     keine
Die erste nicht stornierte Buchung wird in outBu bereitgestellt und auf diese 
positioniert
Nb:     (Rueckgabewert = TRUE
         <=> (pos_neu = 1) AND (outBu enthaelt die 1. nicht stornierte Buchung)
*)

  VAR  rc    : BOOLEAN;
       lfdBu : TBu;

BEGIN
  rc := FALSE;
  IF NOT IsEmpty(inoutBuOrdner) THEN
     BuListe.GoSmallest (inoutBuOrdner);
     WHILE NOT (rc OR BuListe.OffEnd (inoutBuOrdner)) DO
       lfdBu := BuListe.Current (inoutBuOrdner);
       IF NOT Bu.IstStorniert(lfdBu) THEN
         outBu := Bu.Duplicate(lfdBu);
         rc := TRUE;
       ELSE
         BuListe.StepNext (inoutBuOrdner)
       END; (* IF NOT IstStorniert *)
       Bu.Dispose (lfdBu)
     END; (* WHILE Schleife ueber die Elemente im BuchungenOrdner *)

  END; (* IF NOT IsEmpty*)

  RETURN rc;

END First;




PROCEDURE	Next(inoutBuOrdner : TBuOrdner; inoutBu : TBu) : BOOLEAN;
(*
Vb:	SearchBu(LiesBuIDBu(inoutBu), x)
Die naechste nicht stornierte Buchung wird zurueckgegeben und auf diese
positioniert
Nb:     (Rueckgabewert = TRUE
         <=> (pos_neu = pos_alt + 1) AND (inoutBu enthaelt die naechste
                                          nicht stornierte Buchung des Ordners)
*)

  VAR rc    : BOOLEAN;
      lfdBu : TBu;

BEGIN
   rc := FALSE;

   IF BuListe.StopAt (inoutBuOrdner, inoutBu) THEN
      BuListe.StepNext (inoutBuOrdner);
      WHILE NOT (rc OR BuListe.OffEnd (inoutBuOrdner)) DO
        lfdBu := BuListe.Current (inoutBuOrdner);
        IF NOT Bu.IstStorniert(lfdBu) THEN
           Bu.Equalize (inoutBu, lfdBu);
           rc := TRUE;
        ELSE
          BuListe.StepNext (inoutBuOrdner)
        END; (* IF NOT IstStorniert *)
        Bu.Dispose (lfdBu);
      END; (* WHILE Schleife ueber die Elemente im BuchungenOrdner*)
   END; (* IF BuListe.StopAt *)

   RETURN rc;

END Next;




PROCEDURE FirstBuZuKst (inoutBuOrdner : TBuOrdner; inKstID : TKstID;
                        VAR outBu : TBu) : BOOLEAN;
(*
Vb:     keine
Die erste nicht stornierte Buchung B mit B.KstID = inKstID wird
in outBu bereitgestellt und auf diese positioniert
Sei OK der boolsche Rueckgabewert, dann gilt:
Nb:     Ordner nicht leer <=> outBu enthaelt die erste nicht stornierte
        Buchung B mit B.KstID = inKstID
*)

  VAR  rc    : BOOLEAN;  (* Returnwert *)
       lfdrc : BOOLEAN;  (* Interner Returncode beim Ordner-Durchlauf *)
       lfdKstID : TKstID;
       lfdBu    : TBu;

BEGIN
  rc := FALSE;
  lfdrc := First(inoutBuOrdner, lfdBu);

  IF lfdrc THEN
     (* IF-Abfrage ist erforderlich, da nur im TRUE-Fall
        ein Dispose fuer lfdBu durchgefuehrt werden muss
     *)
     WHILE lfdrc DO
       IF Bu.ExistiertKstID(lfdBu) THEN
          lfdKstID := Bu.LiesKstID(lfdBu);
          IF (NOT Bu.IstStorniert(lfdBu)) AND
                  IsEqualKstID(inKstID, lfdKstID) THEN
             outBu := Bu.Duplicate(lfdBu);
             rc := TRUE;
             lfdrc := FALSE; (* Durchlauf beenden *)
          END; (* IF IsEqualKstID *)
          DisposeKstID(lfdKstID);
       END; (* IF ExistiertKstID *)

       IF lfdrc THEN
          lfdrc := Next(inoutBuOrdner, lfdBu);
       END; (* IF *)
     END; (* WHILE-Schleife ueber die Elemente von inoutBuOrdner *)
     Bu.Dispose(lfdBu);
  END; (* IF lfdrc *)

  RETURN rc;

END FirstBuZuKst;


 
PROCEDURE NextBuZuKst  (inoutBuOrdner : TBuOrdner; inKstID : TKstID;
                        inoutBu : TBu) : BOOLEAN;
(*
Vb:	SearchBu(LiesBuIDBu(inoutBu), x)
Die naechste nicht stornierte Buchung B mit  B.KstID = inKstID wird
zurueckgegeben und auf diese positioniert
Sei OK der boolsche Rueckgabewert, dann gilt:
Nb:	OK <=> inoutBu enthaelt die naechste nicht stornierte Buchung B
                       mit B.KstID = inKstID
               AND pos_neu = pos_alt + 1
*)

  VAR  rc    : BOOLEAN;  (* Returnwert *)
       lfdrc : BOOLEAN;  (* Interner Returncode beim Ordner-Durchlauf *)
       lfdKstID : TKstID;
       lfdBu    : TBu;

BEGIN
  rc := FALSE;
  lfdBu := Bu.Duplicate(inoutBu);
  lfdrc := Next(inoutBuOrdner, lfdBu);

  IF lfdrc THEN
     WHILE lfdrc DO
       IF Bu.ExistiertKstID(lfdBu) THEN
          lfdKstID := Bu.LiesKstID(lfdBu);
          IF (NOT Bu.IstStorniert(lfdBu))    AND
                  IsEqualKstID(inKstID, lfdKstID)  THEN
             Bu.Equalize(inoutBu, lfdBu);
             rc := TRUE;
             lfdrc := FALSE; (* Durchlauf beenden *)
          END; (* IF IsEqualKstID *)

          DisposeKstID(lfdKstID);
       END; (* IF ExistiertKstID *)

       IF NOT rc THEN
          lfdrc := Next(inoutBuOrdner, lfdBu);
       END; (* IF *)
     END; (* WHILE-Schleife ueber die Elemente von inoutBuOrdner *)
     Bu.Dispose(lfdBu);
  END; (* IF lfdrc *)

  RETURN rc;

END NextBuZuKst;




PROCEDURE FirstBuZuVtr (inoutBuOrdner : TBuOrdner; inVtrID : TVtrID;
                        VAR outBu : TBu) : BOOLEAN;
(*
Vb:     keine
Die erste nicht stornierte Buchung B mit B.VtrID = inVtrID wird
in outBu bereitgestellt und auf diese positioniert
Sei OK der boolsche Rueckgabewert, dann gilt:
Nb:     Ordner nicht leer <=> outBu enthaelt die erste nicht stornierte
        Buchung B mit B.VtrID = inVtrID
*)

  VAR  rc    : BOOLEAN;  (* Returnwert *)
       lfdrc : BOOLEAN;  (* Interner Returncode beim Ordner-Durchlauf *)
       lfdVtrID : TVtrID;
       lfdBu    : TBu;

BEGIN
  rc := FALSE;
  lfdrc := First(inoutBuOrdner, lfdBu);

  IF lfdrc THEN
     (* IF-Abfrage ist erforderlich, da nur im TRUE-Fall
        ein Dispose fuer lfdBu durchgefuehrt werden muss
     *)
     WHILE lfdrc DO
       IF Bu.ExistiertVtrID(lfdBu) THEN
          lfdVtrID := Bu.LiesVtrID(lfdBu);
          IF (NOT Bu.IstStorniert(lfdBu))     AND
                  IsEqualVtrID(inVtrID, lfdVtrID)  THEN
             outBu := Bu.Duplicate(lfdBu);
             rc := TRUE;
             lfdrc := FALSE; (* Durchlauf beenden *)
          END; (* IF IsEqualVtrID *)
          DisposeVtrID(lfdVtrID);
       END; (* IF ExistiertVtrID *)

       IF lfdrc THEN
          lfdrc := Next(inoutBuOrdner, lfdBu);
       END; (* IF *)
     END; (* WHILE-Schleife ueber die Elemente von inoutBuOrdner *)
     Bu.Dispose(lfdBu);
  END; (* IF lfdrc *)

  RETURN rc;

END FirstBuZuVtr;


PROCEDURE  NextBuZuVtr  (inoutBuOrdner : TBuOrdner; inVtrID : TVtrID;
                         inoutBu : TBu) : BOOLEAN;
(*
Vb:	SearchBu(LiesBuIDBu(inoutBu), x)
Die naechste nicht stornierte Buchung B mit B.VtrID = inVtrID wird
zurueckgegeben und auf diese positioniert
Sei OK der boolsche Rueckgabewert, dann gilt:
Nb:	OK <=> inoutBu enthaelt die naechste nicht stornierte Buchung
               mit B.VtrID = inVtrID AND pos_neu = pos_alt + 1
*)

  VAR  rc    : BOOLEAN;  (* Returnwert *)
       lfdrc : BOOLEAN;  (* Interner Returncode beim Ordner-Durchlauf *)
       lfdVtrID : TVtrID;
       lfdBu    : TBu;

BEGIN
  rc := FALSE;
  lfdBu := Bu.Duplicate(inoutBu);
  lfdrc := Next(inoutBuOrdner, lfdBu);

  IF lfdrc THEN
     WHILE lfdrc DO
       IF Bu.ExistiertVtrID(lfdBu) THEN
          lfdVtrID := Bu.LiesVtrID(lfdBu);
          IF (NOT Bu.IstStorniert(lfdBu))     AND
                  IsEqualVtrID(inVtrID, lfdVtrID)     THEN
             Bu.Equalize(inoutBu, lfdBu);
             rc := TRUE;
             lfdrc := FALSE; (* Durchlauf beenden *)
          END; (* IF IsEqualVtrID *)
          DisposeVtrID(lfdVtrID);
       END; (* IF ExistiertVtrID *)

       IF NOT rc THEN
          lfdrc := Next(inoutBuOrdner, lfdBu);
       END; (* IF *)
     END; (* WHILE-Schleife ueber die Elemente von inoutBuOrdner *)
     Bu.Dispose(lfdBu);
  END; (* IF lfdrc *)

  RETURN rc;

END NextBuZuVtr;



PROCEDURE FirstBuMitBuArtZuVtr (inoutBuOrdner : TBuOrdner; inVtrID : TVtrID;
                                inBuArt : TBuArt; VAR outBu : TBu) : BOOLEAN;
(*
Vb:     keine
Die erste nicht stornierte Buchung B mit BuchungsArt inBuArt und
B.VtrID = inVtrID wird zurueckgegeben und auf diese positioniert
Sei OK der boolsche Rueckgabewert, dann gilt:
Nb:     Ordner nicht leer <=> outBu enthaelt die erste nicht stornierte
        Buchung B mit BuchungsArt = inBuArt und mit B.VtrID = inVtrID
*)

  VAR  rc    : BOOLEAN;  (* Returnwert *)
       lfdrc : BOOLEAN;  (* Interner Returncode beim Ordner-Durchlauf *)
       lfdVtrID : TVtrID;
       lfdBu    : TBu;

BEGIN
  rc := FALSE;
  lfdrc := First(inoutBuOrdner, lfdBu);

  IF lfdrc THEN
     (* IF-Abfrage ist erforderlich, da nur im TRUE-Fall
        ein Dispose fuer lfdBu durchgefuehrt werden muss
     *)
     WHILE lfdrc DO
       IF Bu.ExistiertVtrID(lfdBu) THEN
          lfdVtrID := Bu.LiesVtrID(lfdBu);
          IF (NOT Bu.IstStorniert(lfdBu))          AND
                  (Bu.LiesBuchungsArt(lfdBu) = inBuArt)   AND
                  IsEqualVtrID(inVtrID, lfdVtrID)    THEN
             outBu := Bu.Duplicate(lfdBu);
             rc := TRUE;
             lfdrc := FALSE; (* Durchlauf beenden *)
          END; (* IF IsEqualVtrID *)
          DisposeVtrID(lfdVtrID);
       END; (* IF ExistiertVtrID *)

       IF lfdrc THEN
          lfdrc := Next(inoutBuOrdner, lfdBu);
       END; (* IF *)
     END; (* WHILE-Schleife ueber die Elemente von inoutBuOrdner *)
     Bu.Dispose(lfdBu);
  END; (* IF lfdrc *)

  RETURN rc;

END FirstBuMitBuArtZuVtr;


PROCEDURE  NextBuMitBuArtZuVtr  (inoutBuOrdner : TBuOrdner; inVtrID : TVtrID;
                                 inBuArt : TBuArt; inoutBu : TBu) : BOOLEAN;
(*
Vb:	SearchBu(LiesBuIDBu(inoutBu), x)
Die naechste nicht stornierte Buchung B mit BuchungsArt = inBuArt
und B.VtrID = inVtrID wird zurueckgegeben und auf diese positioniert
Sei OK der boolsche Rueckgabewert, dann gilt:
Nb:	OK <=> inoutBu enthaelt die naechste nicht stornierte Buchung B mit
               BuchungsArt = inBuArt und mit B.VtrID = inVtrID
               AND pos_neu = pos_alt + 1
*)

  VAR  rc    : BOOLEAN;  (* Returnwert *)
       lfdrc : BOOLEAN;  (* Interner Returncode beim Ordner-Durchlauf *)
       lfdVtrID : TVtrID;
       lfdBu    : TBu;

BEGIN
  rc := FALSE;
  lfdBu := Bu.Duplicate(inoutBu);
  lfdrc := Next(inoutBuOrdner, lfdBu);

  IF lfdrc THEN
     WHILE lfdrc DO
       IF Bu.ExistiertVtrID(lfdBu) THEN
          lfdVtrID := Bu.LiesVtrID(lfdBu);
          IF (NOT Bu.IstStorniert(lfdBu))          AND
                  (Bu.LiesBuchungsArt(lfdBu) = inBuArt)   AND
                  IsEqualVtrID(inVtrID, lfdVtrID)    THEN
             Bu.Equalize(inoutBu, lfdBu);
             rc := TRUE;
             lfdrc := FALSE; (* Durchlauf beenden *)
          END; (* IF IsEqualVtrID *)
          DisposeVtrID(lfdVtrID);
       END; (* IF ExistiertVtrID *)

       IF NOT rc THEN
          lfdrc := Next(inoutBuOrdner, lfdBu);
       END; (* IF *)
     END; (* WHILE-Schleife ueber die Elemente von inoutBuOrdner *)
     Bu.Dispose(lfdBu);
  END; (* IF lfdrc *)

  RETURN rc;

END NextBuMitBuArtZuVtr;






(* -------------------------------------------------------------------------- *)
(* Iterator ueber alle Buchungen (einschl. der stornierten)                   *)
(* -------------------------------------------------------------------------- *)



PROCEDURE	FirstBuUndStorno (inoutBuOrdner : TBuOrdner; VAR outBu : TBu
                                 ) : BOOLEAN;
(*
Vb:     keine
Die erste Buchung wird in outBu bereitgestellt und auf diese positioniert
Nb:     (Rueckgabewert = TRUE
         <=> (pos_neu = 1) AND (outBu enthaelt die 1. Buchung des Ordners)
*)


  VAR  rc : BOOLEAN;

BEGIN

  IF NOT IsEmpty(inoutBuOrdner) THEN
    BuListe.GoSmallest(inoutBuOrdner);
    outBu := BuListe.Current(inoutBuOrdner);
    rc := TRUE;
  ELSE
    rc := FALSE;
  END; (* IF *)

  RETURN rc;

END FirstBuUndStorno;



PROCEDURE	NextBuUndStorno (inoutBuOrdner : TBuOrdner; inoutBu : TBu
                                ) : BOOLEAN;
(*
Vb:	SearchBu(LiesBuIDBu(inoutBu), x)
Rueckgabe der Buchung an der naechsten Position; (posalt+1, falls existent)
Nb:     (Rueckgabewert = TRUE
         <=> (pos_neu = pos_alt + 1) AND (inoutBu enthaelt die naechste
                                          Buchung des Ordners)
*)

  VAR aktBuID : TBuID;
      rc        : BOOLEAN;
      lfdBu     : TBu;

BEGIN
    rc := FALSE;

    IF BuListe.StopAt (inoutBuOrdner, inoutBu) THEN
       BuListe.StepNext (inoutBuOrdner);
       IF NOT BuListe.OffEnd (inoutBuOrdner) THEN
          lfdBu := BuListe.Current (inoutBuOrdner);
          Bu.Equalize (inoutBu, lfdBu);
          Bu.Dispose (lfdBu);
          rc := TRUE
      END (* IF NOT BuListe.OffEnd *)
    END; (* IF BuListe.StopAt *)

    RETURN rc;

END NextBuUndStorno;




(*************************** StringOperationen ***************************)
 
                                         
PROCEDURE GibBuenAlsStringliste(inoutBuOrdner : TBuOrdner): TStringliste;
(* Gib die Buchungen mit folgender Formatierung als Stringliste aus;
   Formatierung: BuenID (5Zeichen), 1 Leerzeichen, BuchungsArt (6Zeichen),
                 1 Leerzeichen, Buchungsbetrag (10Zeichen), 1 Leerzeichen,
                 Gebucht Am (TT.MM.JJJJ - 10Zeichen)
   insgesamt also 34 Zeichen
   Ist die BuchungsID laenger als 5 Stellen, werden alle Stellen der Buchungs-
   ID uebertragen und die uebrigen Positionen verschieben sich entsprechend
   nach rechts.
*)

  VAR BuenStringliste : TStringliste;
      BuString        : TString;
      lfdBu           : TBu;

      (* -------------------- lokale Prozedur ------------------------------ *)
      PROCEDURE GenerateString(inBu : TBu; inoutBuString : TString);
        VAR lfdBuID   : TBuID;
            lfdDatum  : TDatum;
            lfdString : TString;
            Blank     : TString;
            Komma     : TString;
            Null      : TString;
            LaengeString : CARDINAL;
            strArray   : ARRAY [0..255] OF CHAR;
            i : CARDINAL;
      BEGIN
        Blank := String.Create();
        String.Schreibe(Blank," ");
        Komma := String.Create();
        String.Schreibe(Komma,",");
        Null := String.Create();
        String.Schreibe(Null,"0");

        (* 5 Stellen BuID *)
        lfdBuID := Bu.LiesBuID(inBu);
        lfdString := BuIDAlsString(lfdBuID, LaengeBuIDNR);
        DisposeBuID(lfdBuID);
        LaengeString := String.Laenge(lfdString);
        IF LaengeString < 5 THEN
           FOR i := 1 TO 5 - LaengeString DO
               String.Anhaengen(Blank, inoutBuString);
           END; (* FOR *)
        END; (* IF LaengeBuID < 5 *)
        String.Anhaengen(lfdString, inoutBuString);
        String.Anhaengen(Blank, inoutBuString);

        (* 6 Stellen Buchungsart *)
        CASE Bu.LiesBuchungsArt(inBu) OF

             Gehalt : String.Schreibe(lfdString,"Gehalt ");
            |WG     : String.Schreibe(lfdString,"WG     ");
            |Abschluss : String.Schreibe(lfdString,"Abschl ");
            |Sonst : String.Schreibe(lfdString,"Sonst  ");

        END; (* CASE *)

        String.Anhaengen(lfdString, inoutBuString);
       
        (* 10 Stellen Buchungsbetrag *)
        IntToStr(Bu.LiesBetrag(inBu), strArray);
        String.Schreibe(lfdString, strArray);
        LaengeString := String.Laenge(lfdString);
        IF LaengeString = 1 THEN
           String.Einfuegen(Null, 0, lfdString);
           LaengeString := LaengeString + 1;
        END;
        IF LaengeString < 3 THEN
           String.Einfuegen(Null, 0, lfdString);
           LaengeString := LaengeString + 1;
        END;
        (* String hat jetzt mindestens die Laenge 3 *)
        String.Einfuegen(Komma, LaengeString - 2, lfdString);
                  
        LaengeString := String.Laenge(lfdString);
        IF LaengeString < 10 THEN
           FOR i := 1 TO 10 - LaengeString DO
               String.Anhaengen(Blank, inoutBuString);
           END; (* FOR *)
        END; (* IF *)

        String.Anhaengen(lfdString, inoutBuString);
        String.Anhaengen(Blank, inoutBuString);
        String.Dispose(lfdString);


        (* 10 Stellen GebuchtAm *)
        lfdDatum := Bu.LiesGebuchtAm(inBu);
        lfdString := DatumAlsString(lfdDatum);
        DisposeDatum(lfdDatum);
        IF String.Laenge(lfdString) < 9 THEN
           String.Anhaengen(Blank, inoutBuString);
        END; (* IF *)
        IF String.Laenge(lfdString) < 10 THEN
           String.Anhaengen(Blank, inoutBuString);
        END; (* IF *)
           
        String.Anhaengen(lfdString, inoutBuString);

        String.Dispose(lfdString);
        String.Dispose(Blank);
        String.Dispose(Komma);
        String.Dispose(Null);

      END GenerateString;

      (* -------------------- Ende lokale Prozedur ------------------------- *)


BEGIN                       (* ProzedurKoerper GibBuenAlsStringliste *)
  BuenStringliste := Stringliste.Create();
  BuString    := String.Create();

  IF First (inoutBuOrdner, lfdBu) THEN
     GenerateString(lfdBu, BuString);
     Stringliste.Append(BuenStringliste, BuString);
     String.Loeschen(BuString, 0, String.Laenge(BuString));
       
     WHILE Next(inoutBuOrdner, lfdBu) DO
        GenerateString(lfdBu, BuString);
        Stringliste.Append(BuenStringliste, BuString);
        String.Loeschen(BuString, 0, String.Laenge(BuString));
     END; (* WHILE *)

     Bu.Dispose(lfdBu);
     String.Dispose(BuString);

  END; (* IF First *)

  RETURN BuenStringliste;

END GibBuenAlsStringliste;
 


PROCEDURE       GibAllesAlsStringliste(inoutBuOrdner : TBuOrdner
                                      ) : TStringliste;
(* Gib saemtliche Buchungen (einschl. der stornierten) mit saemtlichen
   Attributen als Stringliste aus.
   Aufbau eines Strings:

   Anzahl
   Zeichen  Inhalt
    5       BuID
    1       Zwischenraum
    5       KstID
    1       Zwischenraum
    5       VtrID
    1       Zwischenraum
   10       Betrag
    1       Zwischenraum
   10       Betrag
    1       Zwischenraum
   10       GebuchtAm
    1       Zwischenraum
    6       BuchungsArt
    1       Zwischenraum
   10       StornoDatum
  ---
   57 Zeichen insgesamt
*)
  CONST INDEX_KstID  = 1; (* Index fuer das Attribut KstID *)
  CONST INDEX_VtrID  = 2; (* Index fuer das Attribut VtrID *)
  CONST INDEX_Betrag = 3; (* Index fuer das Attribut Betrag *)
  CONST INDEX_BuArt  = 5; (* Index fuer das Attribut BuchungsArt *)

  VAR rc         : BOOLEAN;
      index      : CARDINAL; (* Index ueber die Attribute einer Buchung *)
      i          : CARDINAL; (* Laufvariable *)
      rcOrdner   : BOOLEAN;
      lfdBu      : TBu;
      lfdString  : TString;
      Blank      : TString;
      ObjektAlsStringliste : TStringliste;
      BuenStringliste      : TStringliste;
      BuString             : TString;

BEGIN
   Blank := String.Create();
   String.Schreibe(Blank," ");
   BuenStringliste := Stringliste.Create();
   BuString    := String.Create();

   rc := TRUE;
   rcOrdner := FirstBuUndStorno (inoutBuOrdner, lfdBu);

   IF rcOrdner THEN
      (* Abfrage erforderlich, da im FALSE-Fall DisposeBu(lfdBu) entfaellt *)

      WHILE (rc AND rcOrdner)  DO

         ObjektAlsStringliste := Bu.CreateStringlisteFromObject(lfdBu);
         IF NOT Stringliste.IsEmpty(ObjektAlsStringliste) THEN
            Stringliste.GoStart(ObjektAlsStringliste);
            index := 0;

            WHILE (rc AND NOT Stringliste.OffEnd(ObjektAlsStringliste)) DO
               lfdString := Stringliste.Current(ObjektAlsStringliste);
               (* Buchungs-Zeile aufbauen *)
               IF ((index = INDEX_KstID) OR (index = INDEX_VtrID)) THEN
                  IF String.Laenge(lfdString) > 0  THEN
                    (* ID 5-stellig mit fuehrenden Nullen *)
                    IDAlsString5(lfdString);
                 ELSE
                   (* ID nicht vorhanden, Blanks eintragen *)
                   FOR i:= 1 TO 5 DO
                      String.Anhaengen(Blank, lfdString);
                   END; (* FOR *)
                 END; (* IF ID vorhanden *)
               END; (* IF KstID OR VtrID *)
               IF index = INDEX_Betrag THEN
                  (* Buchungsbetrag mit Komma und 10-stellig *)
                  BetragAlsString10(lfdString);
               END; (* IF Attribut Betrag *)
               IF index = INDEX_BuArt THEN
                  (* BuchungsArt linksbuendig und maximal 6-stellig *)
                  BuArtAlsString6(lfdString);
               END; (* IF Attribut Betrag *)
               String.Anhaengen(lfdString, BuString);
               String.Anhaengen(Blank, BuString);
               String.Anhaengen(Blank, BuString);
               String.Dispose(lfdString);
               Stringliste.StepNext(ObjektAlsStringliste);
               index := index + 1;
            END; (* WHILE Schleife ueber die Elemente der Stringliste *)

            (* String mit den Attributen eines Buchungs-Objekts in die
               Stringliste anhaengen
            *)

            Stringliste.Append(BuenStringliste, BuString);
            String.Loeschen(BuString, 0, String.Laenge(BuString));

         END; (* IF ObjektAlsStringliste nicht leer *)

         Stringliste.Dispose(ObjektAlsStringliste);
       
         rcOrdner := NextBuUndStorno(inoutBuOrdner, lfdBu);

      END; (* WHILE Schleife ueber die Elemente des Buchungen-Ordners *)

      Bu.Dispose(lfdBu);
      String.Dispose(Blank);

   END; (* IF First *)

   RETURN BuenStringliste;

END GibAllesAlsStringliste;


PROCEDURE       BuArtAlsString6(inoutBuArtString : TString);

(* BuchungsArt linksbuendig und maximal 6-stellig *)

  VAR Blank  : TString;
      LaengeString : CARDINAL;
      i : CARDINAL;

BEGIN
  Blank := String.Create();
  String.Schreibe(Blank," ");

  LaengeString := String.Laenge(inoutBuArtString);
  IF LaengeString < 6 THEN
     FOR i := 1 TO  6 - LaengeString DO
         String.Anhaengen(Blank,inoutBuArtString);
     END; (* FOR *)
  END; (* IF *)
  IF LaengeString > 6 THEN
     FOR i := 1 TO  LaengeString - 6 DO
         String.Loeschen(inoutBuArtString, 6, LaengeString - 6);
     END; (* FOR *)
  END; (* IF *)

  String.Dispose(Blank);

END BuArtAlsString6;


PROCEDURE       IDAlsString5(inoutIDString : TString);

(* ID mit fuehrenden Nullen als 5-stelliger String *)

  VAR Null  : TString;
      LaengeString : CARDINAL;
      i : CARDINAL;

BEGIN
  Null := String.Create();
  String.Schreibe(Null,"0");

  LaengeString := String.Laenge(inoutIDString);
  IF LaengeString < 5 THEN
     FOR i := 1 TO  5 - LaengeString DO
         String.Einfuegen(Null, 0, inoutIDString);
     END; (* FOR *)
  END; (* IF *)

  String.Dispose(Null);

END IDAlsString5;





PROCEDURE       BetragAlsString10(inoutBetragString : TString);

(* Buchungsbetrag rechtsbuendig als 10-stelliger String *)

  VAR Null  : TString;
      Blank : TString;
      Komma : TString;
      i : CARDINAL;
      LaengeString : CARDINAL;

BEGIN
  Blank := String.Create();
  String.Schreibe(Blank," ");
  Komma := String.Create();
  String.Schreibe(Komma,",");
  Null := String.Create();
  String.Schreibe(Null,"0");

  LaengeString := String.Laenge(inoutBetragString);
  IF LaengeString = 1 THEN
     String.Einfuegen(Null, 0, inoutBetragString);
     LaengeString := LaengeString + 1;
  END;
  IF LaengeString < 3 THEN
     String.Einfuegen(Null, 0, inoutBetragString);
     LaengeString := LaengeString + 1;
  END;
  (* String hat jetzt mindestens die Laenge 3 *)
  String.Einfuegen(Komma, LaengeString - 2, inoutBetragString);
                  
  LaengeString := String.Laenge(inoutBetragString);
  IF LaengeString < 10 THEN
     FOR i := 1 TO 10 - LaengeString DO
         String.Einfuegen(Blank, 0, inoutBetragString);
     END; (* FOR *)
  END; (* IF *)

  String.Dispose(Null);
  String.Dispose(Blank);
  String.Dispose(Komma);

END BetragAlsString10;


(********************** Backup/Restore-Operationen **********************)


PROCEDURE       BackupInsert(inoutBuOrdner : TBuOrdner; inBu: TBu);
(*
Vb:     Bu.ExistiertBuID(inBu) = TRUE
Fuegt die Buchung in den Buchungen-Ordner ein.
Es wird kein Identifikator gesetzt, da das Objekt bereits mit einem
Identifikator versehen ist
Nb:    inBu existiert im Buchungen-Ordner
*)

BEGIN
  BuListe.Insert(inoutBuOrdner, inBu);
END BackupInsert;

END BuOrdnerSchablone.
