(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*)

(* Test-Operationen:
      OpInsertBuen   Buchungen aus der Datei budateninsert einfuegen
      OpUpdateBuen   Buchungen aus der Datei budatenupdate aendern
      OpGebGelderKst Summe gebuchter Gelder zu einer Kostenstelle
                     auf Logdatei ausgeben
      OpStorniereBuZuVtr
                     Alle Buchungen zu einem Vertrag mit BuchungsArt
                     "Gehalt" stornieren
      OpShowListBuen Inhalt des BuchungenOrdners auf Logdatei ausgeben
      OpShowBuenKst  Alle Buchungen zu einer Kostenstelle  auf Logdatei ausgeben
      OpShowBuenVtr  Alle Buchungen zu einem Vertrag  auf Logdatei ausgeben
      OpShowBuenBuArtVtr
                     Alle Buchungen einer BuArt zu einem Vertrag
                     auf Logdatei ausgeben
      OpShowBuenAlsStringliste
                     Alle Buchungen als Stringliste auf Logdatei ausgeben
      OpShowAllesAlsStringliste
                     Alle Buchungen (einschl. der stornierten) als
                     Stringliste auf Logdatei ausgeben
      OpSaveBuen     Alle Buchungen auf Datei auslagern
      OpLoadBuen     Alle Buchungen von Datei einspielen


*)

IMPLEMENTATION MODULE TestOpBuen;

   FROM Storage IMPORT ALLOCATE, DEALLOCATE;
   FROM SYSTEM  IMPORT TSIZE, ADDRESS;
   FROM SeqFile IMPORT ChanId, FlagSet, read,
                       OpenRead, Close, OpenResults;
   FROM  IOResult IMPORT ReadResults, ReadResult;
   FROM  TextIO   IMPORT ReadString, ReadToken, ReadRestLine, ReadChar,
                         SkipLine;
   FROM  InOut    IMPORT WriteString, WriteLn, Write;
   FROM  WholeStr IMPORT ConvResults, StrToCard, CardToStr, StrToInt, IntToStr;
   FROM  Ascii    IMPORT lf;
   FROM  LogDatei IMPORT WriteLogString, WriteLogChar;
   FROM  VDat     IMPORT THandleInVDat, OpenInVDat, CloseInVDat, SetWriteInVDat,
                         SetReadInVDat;
   FROM  HIDVerwaltung   IMPORT SaveInHID, LoadInHID;
   FROM  Ksten    IMPORT TKstID, DisposeKstID,
                         KstIDAlsString, StringInKstID, PruefeKstID;
   FROM  Vtre     IMPORT TVtrID, DisposeVtrID,
                         VtrIDAlsString, StringInVtrID, PruefeVtrID;
   FROM  BelBez   IMPORT BerGebGelderDerKst,FirstBuZuKst, NextBuZuKst;
   FROM  HatBuBez IMPORT StorniereBuZuVtr,
                         FirstBuZuVtr, NextBuZuVtr,
                         FirstBuMitBuArtZuVtr, NextBuMitBuArtZuVtr;
   FROM  Buen     IMPORT TBu, TBuID, TBuArt,
            CreateBu, DisposeBu, DuplicateBu, EqualizeBu,IsEqualBu,
            LiesBuIDBu, BuIDAlsString, DisposeBuID, PruefeBuID, StringInBuID,
            LiesKstIDinBu, SetzeKstIDinBu, ExistiertKstIDinBu,
            LiesVtrIDinBu,SetzeVtrIDinBu, ExistiertVtrIDinBu,
            LiesBetraginBu, SetzeBetraginBu,
            LiesGebuchtAminBu, SetzeGebuchtAminBu,
            LiesBuchungsArtinBu, SetzeBuchungsArtinBu,
            LiesStorniertAminBu, SetzeStorniertAminBu, IstStorniertinBu,
            InsertBu, SearchBu, UpdateBu,
            FirstBuUndStorno, NextBuUndStorno,
            GibBuenAlsStringlisteinBuen, GibAllesAlsStringlisteinBuen,
            SaveinBuen, LoadinBuen;

   IMPORT String, Stringliste;
   IMPORT Datum;

   CONST DateinameBuen = "BuchungenOrdner";
         DateinameHID = "HIDOrdner";
         DateiNameBuDatenInsert = "TS_Buen/Testumgebung/buendateninsert";
                                  (* Eingabedaten fuer die Operation Insert *)
         DateiNameBuDatenUpdate = "TS_Buen/Testumgebung/buendatenupdate";
                                  (* Eingabedaten fuer die Operation Update *)
         (* LaengePersIDNR = 5; *)


   TYPE TString = String.TString;
        TStringliste = Stringliste.TStringliste;
        TDatum  = Datum.TDatum;



PROCEDURE OpInsertBuen();


  VAR rc : BOOLEAN;
      cidinput :ChanId;
      openres : OpenResults;
      KonvResult : ConvResults;
      EingabeTyp     : CHAR;
      EingabeZeichen : CHAR;
      EingabeName    : ARRAY [0..255] OF CHAR;
      EingabeDaten   : ARRAY [0..255] OF CHAR;
      StrArray       : ARRAY [0..255] OF CHAR;
        
      lfdBu       : TBu;
      lfdDatum    : TDatum;
      lfdCardinal : CARDINAL;
      lfdInteger  : INTEGER;
      lfdString   : TString;
      outMessage  : TString;
      lfdKstID    : TKstID;
      lfdVtrID    : TVtrID;



BEGIN

   lfdString := String.Create();

   OpenRead (cidinput, DateiNameBuDatenInsert, read, openres);
   IF  (openres # opened) THEN
      WriteLogString("OpenRead-Fehler bei ");
      WriteLogString(DateiNameBuDatenInsert); WriteLogChar(lf);
      WriteLogString("FehlerCode = ORD(OpenResults): ");
      WriteLogChar(CHR(ORD(openres)+48)); WriteLogChar(lf);
      (* Fehlermeldung zusaetzlich auf STDOUT ausgeben *)
      WriteString("OpenRead-Fehler bei ");
      WriteString(DateiNameBuDatenInsert); WriteLn();
      WriteString("FehlerCode = ORD(OpenResults): ");
      Write(CHR(ORD(openres)+48)); WriteLn();
      HALT();
   END; (* IF OpenRead NOT OK *)

   (* Aufbau der Zeilen der Eingabedaten siehe in der Eingabedatei *)

   ReadChar(cidinput, EingabeTyp);

   WHILE NOT (ReadResult(cidinput) = endOfInput) DO

     (* Kommentar ueberlesen *)
     IF EingabeTyp = '#' THEN
        WHILE EingabeTyp = '#' DO
          SkipLine(cidinput);
          ReadChar(cidinput, EingabeTyp);
        END (* WHILE *)
     END; (* IF *)
     

     ReadToken(cidinput, EingabeName);

     (* Zeichen bis zum Beginn der Daten (|) ueberlesen *)
     EingabeZeichen := ' ';
     WHILE EingabeZeichen <> '|' DO
         ReadChar(cidinput, EingabeZeichen);
     END; (* WHILE *)
     ReadRestLine(cidinput, EingabeDaten);
     SkipLine(cidinput);


     CASE EingabeTyp OF

       '#' : (* Kommentar, nichts zu tun *) ;

      |'-' : (* Beginn der Attribute eines Objekts,
                Initialisierung
             *)
             lfdBu := CreateBu();

      |'=' : (* Ende der Attribute eines Objekts,
                Objekt in den Ordner einfuegen
             *)
             rc := InsertBu(lfdBu, outMessage);
             IF NOT rc THEN
                WriteLogString("Fehler bei InsertBu: ");
                String.Lies(outMessage,StrArray);
                WriteLogString(StrArray); WriteLogChar(lf);
                String.Dispose(outMessage);
             END; (* IF *)
             DisposeBu(lfdBu);

      |'K' : (* KstID-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                (* KstID-Nr *)
                (* Aus KstID-Nr ein KstID-Objekt erzeugen *)
                String.Schreibe(lfdString,EingabeDaten);
                IF PruefeKstID(lfdString, outMessage)  THEN
                   lfdKstID :=StringInKstID(lfdString);
                   SetzeKstIDinBu(lfdBu, lfdKstID);
                   DisposeKstID(lfdKstID);
                ELSE
                   WriteLogString("Fehler bei PruefeKstID: ");
                   WriteLogString(EingabeDaten);
                   WriteLogString("; ");
                   String.Lies(outMessage,StrArray);
                   WriteLogString(StrArray); WriteLogChar(lf);
                   String.Dispose(outMessage);
                END; (* IF  KstID korrekt *)
             END; (* IF LENGTH(EingabeDaten) > 0 *)

      |'V' : (* VtrID-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                (* VtrID-Nr *)
                (* Aus VtrID-Nr ein VtrID-Objekt erzeugen *)
                String.Schreibe(lfdString,EingabeDaten);
                IF PruefeVtrID(lfdString, outMessage)  THEN
                   lfdVtrID := StringInVtrID(lfdString);
                   SetzeVtrIDinBu(lfdBu, lfdVtrID);
                   DisposeVtrID(lfdVtrID);
                 ELSE
                   WriteLogString("Fehler bei PruefeVtrID: ");
                   WriteLogString(EingabeDaten);
                   WriteLogString("; ");
                   String.Lies(outMessage,StrArray);
                   WriteLogString(StrArray); WriteLogChar(lf);
                   String.Dispose(outMessage);
                END; (* IF  VtrID korrekt *)
             END; (* IF LENGTH(EingabeDaten) > 0 *)


      |'B' : (* Betrag-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                StrToInt(EingabeDaten, lfdInteger, KonvResult);
                IF KonvResult = strAllRight  THEN
                   SetzeBetraginBu(lfdBu, lfdInteger);
                ELSE
                  WriteLogString("Fehler bei StrToInt: ");
                  WriteLogString(EingabeDaten);
                  WriteLogChar(lf);
                END; (* IF *)
             END; (* IF LENGTH(EingabeDaten) > 0 *)

      |'G' : (* GebuchtAm-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                String.Schreibe(lfdString,EingabeDaten);
                rc := Datum.DatumVonString(lfdString, lfdDatum, outMessage);
                IF rc THEN
                   SetzeGebuchtAminBu(lfdBu, lfdDatum);
                   Datum.DisposeDatum(lfdDatum);
                ELSE
                   WriteLogString("Fehler bei DatumVonString: ");
                   WriteLogString(EingabeDaten);
                   WriteLogString("; ");
                   String.Lies(outMessage,StrArray);
                   WriteLogString(StrArray); WriteLogChar(lf);
                   String.Dispose(outMessage);
                END; (* IF *)
             END; (* IF LENGTH(EingabeDaten) > 0 *)

      |'A' : (* BuchungsArt-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN

                CASE EingabeDaten[0] OF

                   'G' : SetzeBuchungsArtinBu(lfdBu, Gehalt);
                  |'W' : SetzeBuchungsArtinBu(lfdBu, WG);
                  |'A' : SetzeBuchungsArtinBu(lfdBu, Abschluss);
                  |'S' : SetzeBuchungsArtinBu(lfdBu, Sonst);
                ELSE
                  WriteLogString("BuchungsArt falsch: ");
                  WriteLogString(EingabeName);
                  WriteLogString("; ");
                  WriteLogString(EingabeDaten);
                  WriteLogChar(lf);
                END; (* CASE EingabeDaten[0] *)
             END; (* IF LENGTH(EingabeDaten) > 0 *)

      ELSE
             WriteLogString("Eingabedaten falsch: ");
             WriteLogString(EingabeName);
             WriteLogString("; ");
             WriteLogString(EingabeDaten);
             WriteLogChar(lf);


     END; (* CASE EingabeTyp *)


     ReadChar(cidinput, EingabeTyp);


   END; (* WHILE Schleife ueber die Buchung-Eingabedaten-Datei *)

   Close(cidinput);
   String.Dispose(lfdString);

END OpInsertBuen;



(* --------------------------------------------------------------------------*)

PROCEDURE OpUpdateBuen();


  VAR rc : BOOLEAN;
      cidinput :ChanId;
      openres : OpenResults;
      KonvResult : ConvResults;
      EingabeTyp     : CHAR;
      EingabeZeichen : CHAR;
      EingabeName    : ARRAY [0..255] OF CHAR;
      EingabeDaten   : ARRAY [0..255] OF CHAR;
      StrArray       : ARRAY [0..255] OF CHAR;
        
      lfdBu       : TBu;
      vorhandBu   : TBu;
      lfdBuID     : TBuID;
      lfdKstID    : TKstID;
      lfdVtrID    : TVtrID;
      lfdDatum    : TDatum;
      lfdCardinal : CARDINAL;
      lfdInteger  : INTEGER;
      lfdString   : TString;
      outMessage  : TString;
      buchung_vorhanden : BOOLEAN;



BEGIN

   lfdString := String.Create();

   OpenRead (cidinput, DateiNameBuDatenUpdate, read, openres);
   IF  (openres # opened) THEN
      WriteLogString("OpenRead-Fehler bei ");
      WriteLogString(DateiNameBuDatenUpdate); WriteLogChar(lf);
      WriteLogString("FehlerCode = ORD(OpenResults): ");
      WriteLogChar(CHR(ORD(openres)+48)); WriteLogChar(lf);
      (* Fehlermeldung zusaetzlich auf STDOUT ausgeben *)
      WriteString("OpenRead-Fehler bei ");
      WriteString(DateiNameBuDatenUpdate); WriteLn();
      WriteString("FehlerCode = ORD(OpenResults): ");
      Write(CHR(ORD(openres)+48)); WriteLn();
      HALT();
   END; (* IF OpenRead NOT OK *)

   (* Aufbau der Zeilen der Eingabedaten siehe in der Eingabedatei *)

   ReadChar(cidinput, EingabeTyp);

   WHILE NOT (ReadResult(cidinput) = endOfInput) DO

     (* Kommentar ueberlesen *)
     IF EingabeTyp = '#' THEN
        WHILE EingabeTyp = '#' DO
          SkipLine(cidinput);
          ReadChar(cidinput, EingabeTyp);
        END (* WHILE *)
     END; (* IF *)
     

     ReadToken(cidinput, EingabeName);

     (* Zeichen bis zum Beginn der Daten (|) ueberlesen *)
     EingabeZeichen := ' ';
     WHILE EingabeZeichen <> '|' DO
         ReadChar(cidinput, EingabeZeichen);
     END; (* WHILE *)
     ReadRestLine(cidinput, EingabeDaten);
     SkipLine(cidinput);


     CASE EingabeTyp OF

       '#' : (* Kommentar, nichts zu tun *) ;

      |'-' : (* Beginn der Attribute eines Objekts,
                Initialisierung
             *)
             lfdBu := CreateBu();
             buchung_vorhanden := FALSE;

      |'=' : (* Ende der Attribute eines Objekts,
                Objekt in dem Ordner aendern
             *)
             IF buchung_vorhanden THEN
             rc := UpdateBu(lfdBu, outMessage);
                IF NOT rc THEN
                   WriteLogString("Fehler bei UpdateBu: ");
                   String.Lies(outMessage,StrArray);
                   WriteLogString(StrArray); WriteLogChar(lf);
                   String.Dispose(outMessage);
                END; (* IF UpdateBu OK*)
             END; (* IF buchung_vorhanden*)
             DisposeBu(lfdBu);

      |'I' : (* BuID der Buchung *)
             (* BuID-Nr *)
             (* Aus BuID-Nr ein BuID-Objekt erzeugen *)
             String.Schreibe(lfdString,EingabeDaten);
             rc := PruefeBuID(lfdString, outMessage);
             IF rc  THEN
                lfdBuID :=StringInBuID(lfdString);
                buchung_vorhanden := SearchBu(lfdBuID,vorhandBu);
                IF buchung_vorhanden THEN
                   EqualizeBu(lfdBu,vorhandBu);
                   DisposeBu(vorhandBu);
                ELSE
                  WriteLogString
                           ("Fehler bei SearchBu, Element nicht vorhanden: ");
                  WriteLogString(EingabeDaten);
                  WriteLogChar(lf);
                END; (* IF buchung_vorhanden *)
                DisposeBuID(lfdBuID);
             ELSE
                WriteLogString("Fehler bei PruefeBuID: ");
                WriteLogString(EingabeDaten);
                WriteLogString("; ");
                String.Lies(outMessage,StrArray);
                WriteLogString(StrArray); WriteLogChar(lf);
                String.Dispose(outMessage);
             END; (* IF  String fuer BuID korrekt *)


     |'K' : (* KstID-Attribut *)
            IF LENGTH(EingabeDaten) > 0 THEN
                (* KstID-Nr *)
                (* Aus KstID-Nr ein KstID-Objekt erzeugen *)
                String.Schreibe(lfdString,EingabeDaten);
                IF PruefeKstID(lfdString, outMessage)  THEN
                   IF NOT ExistiertKstIDinBu(lfdBu) THEN
                      lfdKstID :=StringInKstID(lfdString);
                      SetzeKstIDinBu(lfdBu, lfdKstID);
                      DisposeKstID(lfdKstID);
                   ELSE
                      WriteLogString("Fehler bei KstID-Attribut, ");
                      WriteLogString("KstID ist bereits gesetzt, KstID=");
                      WriteLogString(EingabeDaten); WriteLogChar(lf);
                   END; (* IF NOT ExistiertKstID *)
                 ELSE
                   WriteLogString("Fehler bei PruefeKstID: ");
                   WriteLogString(EingabeDaten);
                   WriteLogString("; ");
                   String.Lies(outMessage,StrArray);
                   WriteLogString(StrArray); WriteLogChar(lf);
                   String.Dispose(outMessage);
                END; (* IF PruefeKstID *)
            END; (* IF LENGTH(EingabeDaten) > 0 *)
            (* Bei leerer Eingabe keine Aktion bei KstID *)


      |'V' : (* VtrID-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                (* VtrID-Nr *)
                (* Aus VtrID-Nr ein VtrID-Objekt erzeugen *)
                String.Schreibe(lfdString,EingabeDaten);
                IF PruefeVtrID(lfdString, outMessage)  THEN
                   IF NOT ExistiertVtrIDinBu(lfdBu) THEN
                      lfdVtrID := StringInVtrID(lfdString);
                      SetzeVtrIDinBu(lfdBu, lfdVtrID);
                      DisposeVtrID(lfdVtrID);
                   ELSE
                      WriteLogString("Fehler bei VtrID-Attribut, ");
                      WriteLogString("VtrID ist bereits gesetzt, VtrID=");
                      WriteLogString(EingabeDaten); WriteLogChar(lf);
                   END; (* IF NOT ExistiertVtrID *)
                 ELSE
                   WriteLogString("Fehler bei PruefeVtrID: ");
                   WriteLogString(EingabeDaten);
                   WriteLogString("; ");
                   String.Lies(outMessage,StrArray);
                   WriteLogString(StrArray); WriteLogChar(lf);
                   String.Dispose(outMessage);
                END; (* IF  VtrID korrekt *)
             END; (* IF LENGTH(EingabeDaten) > 0 *)
            (* Bei leerer Eingabe keine Aktion bei VtrID *)


      |'B' : (* Betrag-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                StrToInt(EingabeDaten, lfdInteger, KonvResult);
                IF KonvResult = strAllRight  THEN
                   SetzeBetraginBu(lfdBu, lfdInteger);
                ELSE
                  WriteLogString("Fehler bei StrToInt: ");
                  WriteLogString(EingabeDaten); WriteLogChar(lf);
                END; (* IF *)
             ELSE
               (* leere Eingabe, Betrag initialisieren *)
                SetzeBetraginBu(lfdBu, 0);
             END; (* IF LENGTH(EingabeDaten) > 0 *)

      |'G' : (* GebuchtAm-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                String.Schreibe(lfdString,EingabeDaten);
                rc := Datum.DatumVonString(lfdString, lfdDatum, outMessage);
                IF rc THEN
                   SetzeGebuchtAminBu(lfdBu, lfdDatum);
                   Datum.DisposeDatum(lfdDatum);
                ELSE
                   WriteLogString("Fehler bei DatumVonString: ");
                   WriteLogString(EingabeDaten);
                   WriteLogString("; ");
                   String.Lies(outMessage,StrArray);
                   WriteLogString(StrArray); WriteLogChar(lf);
                   String.Dispose(outMessage);
                END; (* IF DatumVonString OK *)
             ELSE
               (* leere Eingabe, Datum initialisieren *)
               lfdDatum := Datum.CreateDatum();
               SetzeGebuchtAminBu(lfdBu, lfdDatum);
               Datum.DisposeDatum(lfdDatum);
            END; (* LENGTH(EingabeDaten) > 0 *)

      |'A' : (* BuchungsArt-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN

                CASE EingabeDaten[0] OF

                   'G' : SetzeBuchungsArtinBu(lfdBu, Gehalt);
                  |'W' : SetzeBuchungsArtinBu(lfdBu, WG);
                  |'A' : SetzeBuchungsArtinBu(lfdBu, Abschluss);
                  |'S' : SetzeBuchungsArtinBu(lfdBu, Sonst);
                ELSE
                  WriteLogString("BuchungsArt falsch: ");
                  WriteLogString(EingabeName);
                  WriteLogString("; ");
                  WriteLogString(EingabeDaten); WriteLogChar(lf);
                END; (* CASE EingabeDaten[0] *)
             ELSE
                (* leere Eingabe, BuchungsArt initialisieren *)
                SetzeBuchungsArtinBu(lfdBu, Sonst);
            END; (* LENGTH(EingabeDaten) > 0 *)

      |'S' :  (* StorniertAm-Attribut *)
              IF LENGTH(EingabeDaten) > 0 THEN
                String.Schreibe(lfdString,EingabeDaten);
                rc := Datum.DatumVonString(lfdString, lfdDatum, outMessage);
                IF rc THEN
                   IF NOT IstStorniertinBu(lfdBu) THEN
                     SetzeStorniertAminBu(lfdBu, lfdDatum);
                     Datum.DisposeDatum(lfdDatum);
                   ELSE
                     WriteLogString("Fehler bei IstStorniertinBu, ");
                     WriteLogString("Buchung ist bereits storniert, ");
                     WriteLogString(EingabeDaten); WriteLogChar(lf);
                   END; (* IF NOT IstStorniert *)
                ELSE
                   WriteLogString("Fehler bei DatumVonString: ");
                   WriteLogString(EingabeDaten);
                   WriteLogString("; ");
                   String.Lies(outMessage,StrArray);
                   WriteLogString(StrArray); WriteLogChar(lf);
                   String.Dispose(outMessage);
                END; (* IF DatumVonString OK *)
            END; (* LENGTH(EingabeDaten) > 0 *)
            (* Bei leerer Eingabe keine Aktion beim StornoDatum *)


      ELSE
             WriteLogString("Eingabedaten falsch: ");
             WriteLogString(EingabeName);
             WriteLogString("; ");
             WriteLogString(EingabeDaten);
             WriteLogChar(lf);


     END; (* CASE EingabeTyp *)


     ReadChar(cidinput, EingabeTyp);


   END; (* WHILE Schleife ueber die Buchung-Eingabedaten-Datei *)

   Close(cidinput);
   String.Dispose(lfdString);



END OpUpdateBuen;


(* --------------------------------------------------------------------------*)


PROCEDURE OpGebGelderKst(inKstIDNr : CARDINAL);

(* Summe gebuchter Gelder zu einer Kostenstelle auf Logdatei ausgeben *)

  VAR StrArray   : ARRAY [0..255] OF CHAR;
      lfdString  : TString;
      lfdKstID   : TKstID;
      outMessage : TString;
      Summe      : INTEGER;

BEGIN

  lfdString := String.Create();
  CardToStr(inKstIDNr,StrArray);
  String.Schreibe(lfdString,StrArray);

  IF PruefeKstID(lfdString, outMessage) THEN
     lfdKstID := StringInKstID(lfdString);

     Summe := BerGebGelderDerKst(lfdKstID);

     WriteLogString(StrArray);
     WriteLogString("; ");
     DisposeKstID(lfdKstID);
     String.Dispose(lfdString);

     IntToStr(Summe, StrArray);
     WriteLogString(StrArray);
     WriteLogChar(lf);
  ELSE
     WriteLogString("Fehler bei PruefeKstID, KstID=");
     WriteLogString(StrArray);
     WriteLogString(";  ");
     String.Lies(outMessage,StrArray);
     WriteLogString(StrArray); WriteLogChar(lf);
     String.Dispose(outMessage);
  END; (* KstID korrekt *)

END OpGebGelderKst;


(* --------------------------------------------------------------------------*)


PROCEDURE OpStorniereBuZuVtr(inVtrIDNr : CARDINAL);

(* Alle Buchungen zu einem Vertrag mit BuchungsArt "Gehalt" stornieren *)

  VAR rc : BOOLEAN;
      lfdVtrID   : TVtrID;
      lfdString  : TString;
      outMessage : TString;
      StrArray   : ARRAY [0..255] OF CHAR;

BEGIN

  lfdString := String.Create();
  CardToStr(inVtrIDNr,StrArray);
  String.Schreibe(lfdString,StrArray);
  IF PruefeVtrID(lfdString, outMessage) THEN
     lfdVtrID := StringInVtrID(lfdString);

     rc := StorniereBuZuVtr(lfdVtrID, outMessage);
     DisposeVtrID(lfdVtrID);
     IF NOT rc THEN
        WriteLogString("Fehler bei StorniereBuZuVtr, VtrID=");
        WriteLogString(StrArray);
        WriteLogString(";  ");
        String.Lies(outMessage,StrArray);
        WriteLogString(StrArray); WriteLogChar(lf);
        String.Dispose(outMessage);
     END; (* IF StorniereBuZuVtr OK *)
  ELSE
     WriteLogString("Fehler bei PruefeVtrID, VtrID=");
     WriteLogString(StrArray);
     WriteLogString(";  ");
     String.Lies(outMessage,StrArray);
     WriteLogString(StrArray); WriteLogChar(lf);
     String.Dispose(outMessage);
  END; (* VtrID korrekt *)

  String.Dispose(lfdString);

END OpStorniereBuZuVtr;



(* --------------------------------------------------------------------------*)


PROCEDURE OpShowListBuen();

(* Inhalt des BuchungenOrdners auf Logdatei ausgeben *)

  VAR lfdBu       : TBu;


BEGIN
   IF FirstBuUndStorno (lfdBu) THEN

      AusgabeBu(lfdBu);
       
      WHILE NextBuUndStorno(lfdBu) DO
          AusgabeBu(lfdBu); 

      END; (* WHILE *)

    DisposeBu(lfdBu);

   END; (* IF First *)


END OpShowListBuen;

(* --------------------------------------------------------------------------*)


PROCEDURE OpShowBuenKst(inKstIDNr : CARDINAL);

(* Alle Buchungen zu einer Kostenstelle auf Logdatei ausgeben *)

  VAR lfdBu      : TBu;
      lfdKstID   : TKstID;
      lfdString  : TString;
      outMessage : TString;
      StrArray   : ARRAY [0..255] OF CHAR;

BEGIN

  lfdString := String.Create();
  CardToStr(inKstIDNr,StrArray);
  String.Schreibe(lfdString,StrArray);

  IF PruefeKstID(lfdString, outMessage) THEN
     lfdKstID := StringInKstID(lfdString);

      IF FirstBuZuKst (lfdKstID, lfdBu) THEN

         AusgabeBu(lfdBu);
       
         WHILE NextBuZuKst(lfdKstID, lfdBu) DO
             AusgabeBu(lfdBu); 

         END; (* WHILE *)

         DisposeBu(lfdBu);
         DisposeKstID(lfdKstID);
     END; (* IF First *)
  ELSE
     WriteLogString("Fehler bei PruefeKstID, KstID=");
     WriteLogString(StrArray);
     WriteLogString(";  ");
     String.Lies(outMessage,StrArray);
     WriteLogString(StrArray); WriteLogChar(lf);
     String.Dispose(outMessage);
  END; (* KstID korrekt *)

  String.Dispose(lfdString);


END OpShowBuenKst;



(* --------------------------------------------------------------------------*)


PROCEDURE OpShowBuenVtr(inVtrIDNr : CARDINAL);

(* Alle Buchungen zu einem Vertrag auf Logdatei ausgeben *)

  VAR lfdBu      : TBu;
      lfdVtrID   : TVtrID;
      StrArray   : ARRAY [0..255] OF CHAR;
      lfdString  : TString;
      outMessage : TString;


BEGIN

  lfdString := String.Create();
  CardToStr(inVtrIDNr,StrArray);
  String.Schreibe(lfdString,StrArray);

  IF PruefeVtrID(lfdString, outMessage) THEN
     lfdVtrID := StringInVtrID(lfdString);

     IF FirstBuZuVtr (lfdVtrID, lfdBu) THEN

        AusgabeBu(lfdBu);
       
        WHILE NextBuZuVtr(lfdVtrID, lfdBu) DO
            AusgabeBu(lfdBu); 

        END; (* WHILE *)

       DisposeBu(lfdBu);
       DisposeVtrID(lfdVtrID);
     END; (* IF First *)
  ELSE
     WriteLogString("Fehler bei PruefeVtrID, VtrID=");
     WriteLogString(StrArray);
     WriteLogString(";  ");
     String.Lies(outMessage,StrArray);
     WriteLogString(StrArray); WriteLogChar(lf);
     String.Dispose(outMessage);
  END; (* VtrID korrekt *)

  String.Dispose(lfdString);

END OpShowBuenVtr;

(* --------------------------------------------------------------------------*)


PROCEDURE OpShowBuenBuArtVtr(inVtrIDNr : CARDINAL);

(* Alle Buchungen einer BuArt zu einem Vertrag auf Logdatei ausgeben *)

  VAR lfdVtrID   : TVtrID;
      StrArray   : ARRAY [0..255] OF CHAR;
      lfdString  : TString;
      outMessage : TString;

  PROCEDURE IteratorBuenBuArtZuVtr(inVtrID : TVtrID; inBuArt : TBuArt);
    VAR lfdBu    : TBu;

  BEGIN
    IF FirstBuMitBuArtZuVtr (inVtrID, inBuArt, lfdBu) THEN

       AusgabeBu(lfdBu);
       
       WHILE NextBuMitBuArtZuVtr(inVtrID, inBuArt, lfdBu) DO
           AusgabeBu(lfdBu); 

       END; (* WHILE *)

     DisposeBu(lfdBu);

    END; (* IF First *)

  END IteratorBuenBuArtZuVtr;  (* Ende lokale Prozedur *)

BEGIN (* Proz.Koerper OpShowBuenBuArtVtre *)

  lfdString := String.Create();
  CardToStr(inVtrIDNr,StrArray);
  String.Schreibe(lfdString,StrArray);

  IF PruefeVtrID(lfdString, outMessage) THEN
     lfdVtrID := StringInVtrID(lfdString);
     WriteLogChar(lf);
     WriteLogString("==== BuchungsArt: Gehalt");
     IteratorBuenBuArtZuVtr(lfdVtrID, Gehalt);
     WriteLogChar(lf);
     WriteLogString("==== BuchungsArt: WG");
     IteratorBuenBuArtZuVtr(lfdVtrID, WG);
     WriteLogChar(lf);
     WriteLogString("==== BuchungsArt: Abschluss");
     IteratorBuenBuArtZuVtr(lfdVtrID, Abschluss);
     WriteLogChar(lf);
     WriteLogString("==== BuchungsArt: Sonst");
     IteratorBuenBuArtZuVtr(lfdVtrID, Sonst);
     WriteLogChar(lf);
     DisposeVtrID(lfdVtrID);
  ELSE
     WriteLogString("Fehler bei PruefeVtrID, VtrID=");
     WriteLogString(StrArray);
     WriteLogString(";  ");
     String.Lies(outMessage,StrArray);
     WriteLogString(StrArray); WriteLogChar(lf);
     String.Dispose(outMessage);
  END; (* VtrID korrekt *)

  String.Dispose(lfdString);


END OpShowBuenBuArtVtr;


(* --------------------------------------------------------------------------*)


PROCEDURE OpShowBuenAlsStringliste();

(* Alle Buchungen als Stringliste auf Logdatei ausgeben *)

  VAR lfdString       : TString;
      strArray        : ARRAY [0..255] OF CHAR;
      Buenstringliste : TStringliste;


BEGIN

  Buenstringliste :=
  GibBuenAlsStringlisteinBuen();

  IF NOT Stringliste.IsEmpty(Buenstringliste) THEN
     Stringliste.GoStart(Buenstringliste);

     WHILE NOT Stringliste.OffEnd(Buenstringliste) DO
        lfdString := Stringliste.Current(Buenstringliste);
        String.Lies(lfdString,strArray);
        WriteLogString(strArray);
        WriteLogChar(lf);
        String.Dispose(lfdString);
        Stringliste.StepNext(Buenstringliste);
     END; (* WHILE *)

   END; (* IF Buenstringliste nicht leer *)

   Stringliste.Dispose(Buenstringliste);

END OpShowBuenAlsStringliste;


(* --------------------------------------------------------------------------*)


PROCEDURE OpShowAllesAlsStringliste();

(* Alle Buchungen einschl. der stornierten) als Stringliste auf
   Logdatei ausgeben
*)

  VAR lfdString       : TString;
      strArray        : ARRAY [0..255] OF CHAR;
      Buenstringliste : TStringliste;


BEGIN

  Buenstringliste :=
  GibAllesAlsStringlisteinBuen();

  IF NOT Stringliste.IsEmpty(Buenstringliste) THEN
     Stringliste.GoStart(Buenstringliste);

     WHILE NOT Stringliste.OffEnd(Buenstringliste) DO
        lfdString := Stringliste.Current(Buenstringliste);
        String.Lies(lfdString,strArray);
        WriteLogString(strArray);
        WriteLogChar(lf);
        String.Dispose(lfdString);
        Stringliste.StepNext(Buenstringliste);
     END; (* WHILE *)

   END; (* IF Buenstringliste nicht leer *)

   Stringliste.Dispose(Buenstringliste);

END OpShowAllesAlsStringliste;


(* --------------------------------------------------------------------------*)

PROCEDURE OpSaveBuen();

(* Alle Buchungen auf Datei auslagern *)
(* HID-Liste auf Datei auslagern *)

  VAR Dateiname  : TString;
      strArray   : ARRAY [0..255] OF CHAR;
      outMessage : TString;
      FileHandle : THandleInVDat;


BEGIN

   Dateiname := String.Create();
   String.Schreibe(Dateiname, DateinameBuen);

   (* Buchungen-Ordner auf Datei auslagern *)
   IF OpenInVDat(FileHandle, Dateiname, SetWriteInVDat()) THEN
      IF NOT SaveinBuen(FileHandle, outMessage) THEN
         WriteLogString("Fehler bei SaveinBuen, ");
         String.Lies(outMessage, strArray);
         WriteLogString(strArray); WriteLogChar(lf);
         String.Dispose(outMessage);
      END; (* IF Save NOT OK *)
   ELSE
      WriteLogString("Fehler bei OpenInVDat fuer Save Buen"); WriteLogChar(lf);
   END; (* IF OpenInVDat NOT OK *)

   IF NOT CloseInVDat(FileHandle) THEN
      WriteLogString("Fehler bei CloseInVDat fuer Save Buen"); WriteLogChar(lf);
   END; (* IF *)

   (* HIDOrdner auf Datei auslagern *)
   String.Schreibe(Dateiname, DateinameHID);
   IF OpenInVDat(FileHandle, Dateiname, SetWriteInVDat()) THEN
      IF NOT SaveInHID(FileHandle, outMessage) THEN
         WriteLogString("Fehler bei SaveInHID, ");
         String.Lies(outMessage, strArray);
         WriteLogString(strArray); WriteLogChar(lf);
         String.Dispose(outMessage);
      END; (* IF Save NOT OK *)
   ELSE
      WriteLogString("Fehler bei OpenInVDat fuer Save HID"); WriteLogChar(lf);
   END; (* IF OpenInVDat NOT OK *)

   IF NOT CloseInVDat(FileHandle) THEN
      WriteLogString("Fehler bei CloseInVDat fuer Save HID"); WriteLogChar(lf);
   END; (* IF *)

   String.Dispose(Dateiname);

END OpSaveBuen;



(* --------------------------------------------------------------------------*)

PROCEDURE OpLoadBuen();

(* Alle Buchungen von Datei einspielen *)

  VAR Dateiname  : TString;
      strArray   : ARRAY [0..255] OF CHAR;
      outMessage : TString;
      FileHandle : THandleInVDat;


BEGIN

   Dateiname := String.Create();
   String.Schreibe(Dateiname, DateinameBuen);

   (* Buchungen-Ordner von Datei einspielen *)
   IF OpenInVDat(FileHandle, Dateiname, SetReadInVDat()) THEN
      IF NOT LoadinBuen(FileHandle, outMessage) THEN
         WriteLogString("Fehler bei LoadinBuen, ");
         String.Lies(outMessage, strArray);
         WriteLogString(strArray); WriteLogChar(lf);
         String.Dispose(outMessage);
      END; (* IF Load OK *)
   ELSE
      WriteLogString("Fehler bei OpenInVDat fuer Load"); WriteLogChar(lf);
   END; (* IF OpenInVDat NOT OK *)

   IF NOT CloseInVDat(FileHandle) THEN
      WriteLogString("Fehler bei CloseInVDat fuer Load"); WriteLogChar(lf);
   END; (* IF Close NOT OK *)

   (* HIDOrdner von Datei einspielen *)

   String.Schreibe(Dateiname, DateinameHID);
   IF OpenInVDat(FileHandle, Dateiname, SetReadInVDat()) THEN
      IF NOT LoadInHID(FileHandle, outMessage) THEN
         WriteLogString("Fehler bei LoadInHID, ");
         String.Lies(outMessage, strArray);
         WriteLogString(strArray); WriteLogChar(lf);
         String.Dispose(outMessage);
      END; (* IF Load OK *)
   ELSE
      WriteLogString("Fehler bei OpenInVDat fuer Load HID"); WriteLogChar(lf);
   END; (* IF OpenInVDat NOT OK *)

   IF NOT CloseInVDat(FileHandle) THEN
      WriteLogString("Fehler bei CloseInVDat fuer Load HID"); WriteLogChar(lf);
   END; (* IF Close NOT OK *)

   String.Dispose(Dateiname);

END OpLoadBuen;


(* -------------------------------------------------------------------------*)

PROCEDURE AusgabeBu(inBu : TBu); 
  VAR rc : BOOLEAN;
      lfdBuID     : TBuID;
      lfdKstID    : TKstID;
      lfdVtrID    : TVtrID;
      lfdDatum    : TDatum;
      lfdCardinal : CARDINAL;
      lfdString   : TString;
      lfdBuArt    : TBuArt;
      outMessage  : TString;
      StrArray    : ARRAY [0..255] OF CHAR;



BEGIN

  WriteLogChar(lf);
  WriteLogString("Buchung: ");
  lfdBuID := LiesBuIDBu(inBu);
  lfdString := BuIDAlsString(lfdBuID);
  String.Lies(lfdString,StrArray);
  WriteLogString(StrArray);
  WriteLogString("; ");
  DisposeBuID(lfdBuID);
  String.Dispose(lfdString);

  IF ExistiertKstIDinBu(inBu) THEN
     lfdKstID := LiesKstIDinBu(inBu);
     lfdString := KstIDAlsString(lfdKstID);
     String.Lies(lfdString,StrArray);
     WriteLogString(StrArray);
     DisposeKstID(lfdKstID);
  END; (* IF ExistiertKstID *)
  WriteLogString("; ");

  IF ExistiertVtrIDinBu(inBu) THEN
     lfdVtrID := LiesVtrIDinBu(inBu);
     lfdString := VtrIDAlsString(lfdVtrID);
     String.Lies(lfdString,StrArray);
     WriteLogString(StrArray);
     DisposeVtrID(lfdVtrID);
  END; (* IF ExistiertVtrID *)
  WriteLogString("; ");
  String.Dispose(lfdString);


  IntToStr(LiesBetraginBu(inBu), StrArray);
  WriteLogString(StrArray);
  WriteLogString("; ");

  lfdDatum := LiesGebuchtAminBu(inBu);
  lfdString := Datum.DatumAlsString(lfdDatum);
  String.Lies(lfdString,StrArray);
  WriteLogString(StrArray);
  WriteLogString("; ");
  Datum.DisposeDatum(lfdDatum);
  String.Dispose(lfdString);

  lfdBuArt := LiesBuchungsArtinBu(inBu);
  CASE lfdBuArt OF
     Gehalt    : StrArray := "Gehalt";
    |WG        : StrArray := "WG";
    |Abschluss : StrArray := "Abschluss";
    |Sonst     : StrArray := "Sonst";
  END; (* CASE *)

  WriteLogString(StrArray);

  IF IstStorniertinBu(inBu) THEN
     WriteLogString(", S T O R N O :  ");
     lfdDatum := LiesStorniertAminBu(inBu);
     lfdString := Datum.DatumAlsString(lfdDatum);
     String.Lies(lfdString,StrArray);
     WriteLogString(StrArray);
     WriteLogString("; ");
     Datum.DisposeDatum(lfdDatum);
     String.Dispose(lfdString);
  END; (* IF *)

  WriteLogChar(lf);

END AusgabeBu; 



END TestOpBuen.
