(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: Date.mod,v $
# $Author: Sopra $
# $Date: 2001/03/15 07:41:47 $
# $Revision: 1.5 $
# Bemerkungen: Hier sind die Funktionen und Typen fuer's ADO Datum drin!
        
*)
(******************************************************************************)


IMPLEMENTATION MODULE Date;

(* TS Datum *)
(* Die Woche geht von Montag(1) bis Sonntag(7)  *)
(* DatenTeilSystem zum Erzeugen, Lschen von Datum ADOs. *)
(* Operatoren auf Datums ADOs *)

IMPORT String;
IMPORT WholeStr;
IMPORT DatumIMPORTE;
FROM Storage IMPORT ALLOCATE, DEALLOCATE;
FROM SYSTEM  IMPORT TSIZE,CAST,ENTIER,ROUND;
FROM Jahr 	IMPORT 	TJahr, CreateJahr, DisposeJahr,SetzeJahr, IsEqualJahr,
			JahrAlsCard, EqualizeJahr;
FROM Monat 	IMPORT 	TMonat, CreateMonat, DisposeMonat, IsEqualMonat, 
			SetzeMonat,MonatAlsCard,EqualizeMonat;
FROM Tag 	IMPORT 	TTag, CreateTag, DisposeTag, SetzeTag, 
			TagAlsCard, EqualizeTag, IsEqualTag;

(**************************** Datentypvereinbarungen ***********************)

TYPE TString	= 	String.TString;
TYPE TDatum     = 	POINTER TO TDatumDS;
TYPE TDatumDS   =       RECORD
                                T:TTag;
                                M:TMonat;
                                J:TJahr;
                                isUndef:BOOLEAN;
                        END;
TYPE Monate = [1..12];
TYPE MonatsMenge = SET OF Monate; 

(************************** Datentyp TDatum ******************************)

PROCEDURE CreateDatum ():TDatum;
(*
Vb:             keine
Erzeugt ein neues ADO D vom Typ TDatum. Als Defaultwerte wird der 31.12.9999
und isUndef:=TRUE gesetzt.
Nb:     ADO D existiert
*)
VAR aktDatum : TDatum;
BEGIN
    (* -- Vorbed.: keine *)
     ALLOCATE (aktDatum, TSIZE (TDatumDS)) ;
     WITH aktDatum^ DO
        T := CreateTag();
        SetzeTag(T, 31);
        M := CreateMonat();
        SetzeMonat(M, 12);
        J := CreateJahr();
        SetzeJahr(J, 9999);
        isUndef:=TRUE;
     END; (* WITH *)
     RETURN aktDatum;
    (* -- Nachbed.: RueckgabeADO aktID existiert *)
END CreateDatum;


PROCEDURE DisposeDatum (VAR inoutD:TDatum);
(*
Vb: keine
ADO D wird aus dem System entfernt
Nb:     ADO D existiert nicht mehr
*)
BEGIN
        DisposeTag(inoutD^.T);
        DisposeMonat(inoutD^.M);
        DisposeJahr(inoutD^.J);
        DEALLOCATE (inoutD, TSIZE (TDatumDS));
        inoutD:=NIL;
END DisposeDatum;

PROCEDURE DuplicateDatum (VAR inD:TDatum):TDatum;
(*
Vb:     keine
Erzeugt das Rueckgabe ADO und gleicht die Werte der Attribute an;
Nb:     IsEqual(D, RueckgabeD) and not (D=RueckgabeD)
*)
VAR Duplikat: TDatum;
BEGIN
        Duplikat:=CreateDatum();
        WITH Duplikat^ DO
                EqualizeTag(T,inD^.T);
                EqualizeMonat(M, inD^.M);
                EqualizeJahr(J, inD^.J);
                isUndef:=inD^.isUndef;
        END (* WITH *);
        RETURN Duplikat;
END DuplicateDatum;  

PROCEDURE IsEqualDatum (inD1,inD2: TDatum ):BOOLEAN;
(*
Vb:             keine
berprueft die beiden Konditionen auf Wertegleichheit der aller Attribute
Nb:     keine
*)
VAR gleich: BOOLEAN;
BEGIN
        gleich :=       IsEqualTag(inD1^.T,inD2^.T) AND
                        IsEqualMonat(inD1^.M, inD2^.M)  AND
                        IsEqualJahr(inD1^.J,inD2^.J) AND
                        (inD1^.isUndef = inD2^.isUndef);
        RETURN gleich;

END IsEqualDatum; 

PROCEDURE EqualizeDatum (VAR inoutZielD:TDatum ; inQuellD:TDatum);
(*
Vb:     keine                                                                   gleicht die Werte der Attribute von ZielD an die von QuellD an, wobei die Identi
taet der ADO's nicht veraendert wird
Nb:     IsEqual(QuellD,ZielD) and not (QuellD=ZielD)
*)
BEGIN
        WITH inoutZielD^ DO
                EqualizeTag(T,inQuellD^.T);
                EqualizeMonat(M, inQuellD^.M);
                EqualizeJahr(J, inQuellD^.J);
                isUndef:=inQuellD^.isUndef; 
        END (* WITH *);
END EqualizeDatum;

PROCEDURE PruefeDatum (inDatum : TDatum) : BOOLEAN;
(*
Vb: --
Prueft die Konsistenz (Gueltigkeit des uebergebenen Datums.
So ist der 31.02. fuer jedes Jahr verboten. Schaltjahre usw. sollten beruecksichtigt sein.
nb: (false <=> inDatum ungueltig) sonst true
*)

VAR 	gueltig		: BOOLEAN;
	VAR J,M,T	: CARDINAL;
BEGIN
        gueltig:=TRUE;
	T:=TagAlsCard(inDatum^.T);
	M:=MonatAlsCard(inDatum^.M);
	J:=JahrAlsCard(inDatum^.J);
        IF (M IN MonatsMenge{2,4,6,9,11}) AND (T = 31) THEN
               gueltig:=FALSE;
        ELSIF (M = 2)  THEN
        	IF isSchaltJahr(inDatum) THEN
                        IF (T > 29) THEN
                                 gueltig:=FALSE;
                        END (* IF *);
                ELSE
                        IF (T > 28) THEN
                                 gueltig:=FALSE;
                        END (* IF *);
                END (* IF *);
        ELSIF NOT (M IN MonatsMenge{1,2,3,4,5,6,7,8,9,10,11,12}) THEN
                        gueltig:=FALSE;
        ELSIF (J < 0) OR (J > 10000) THEN
                        gueltig:=FALSE;
        ELSIF (T < 1) OR (T > 31) THEN
                        gueltig:=FALSE;
        END (* IF *);
        RETURN gueltig;
END PruefeDatum; 

PROCEDURE PruefeObMonatsErster (inDatum : TDatum) : BOOLEAN;
(*
Vb: --
Prueft, ob es sich bei dem Datum inDatum um den Monatsersten (01.xx.xx) handelt.
Nb: (True <=> inDatum.Tag = 01) oder (False sonst)
*)
BEGIN
        RETURN (TagAlsCard(inDatum^.T) = 1);
END PruefeObMonatsErster;

PROCEDURE PruefeObMonatJanuar (inDatum : TDatum) : BOOLEAN;
(*
Vb: --
Prueft, ob es sich bei dem Datum inDatum um den Monat Januar
(xx.01.xx) handelt.
Nb: (True <=> inDatum.Monat = 01) oder (False sonst)
*)
BEGIN
        RETURN (MonatAlsCard(inDatum^.M) = 1);
END PruefeObMonatJanuar;

PROCEDURE LiesTagInDatum (inD:TDatum):TTag;
(*
Vb:             keine
Liefert den Tag des ADO D
Nb:     keine
*)
VAR t:TTag;
BEGIN
        t:=CreateTag();
        SetzeTag(t,TagAlsCard(inD^.T));
        RETURN t;
END LiesTagInDatum;
                      
PROCEDURE SetzeTagInDatum (VAR inoutD:TDatum; inTag:TTag);
(*
Vb: Tag in [1..31]
Setzt den Tag im ADO D
Nb:     GetTag(Dneu)=Tag
*)
BEGIN
        SetzeTag(inoutD^.T,TagAlsCard(inTag));
END SetzeTagInDatum;

PROCEDURE LiesMonatInDatum (inD:TDatum):TMonat;
(*
Vb:             keine
Liefert den Monat des ADO D
Nb:     keine
*)
VAR m:TMonat;
BEGIN
        m:=CreateMonat();
        SetzeMonat(m, MonatAlsCard(inD^.M));
        RETURN m;
END LiesMonatInDatum;

PROCEDURE SetzeMonatInDatum (VAR inoutD:TDatum; inMonat:TMonat);
(*
Vb:             keine
Setze Monat [1..12] des ADO D
Nb:     keine
*)
BEGIN
        SetzeMonat(inoutD^.M,MonatAlsCard(inMonat));
END SetzeMonatInDatum;
   
PROCEDURE LiesJahrInDatum (inD:TDatum):TJahr;
(*
Vb:             keine
Liefert das Jahr des ADO D
Nb:     keine
*)
VAR j:TJahr;
BEGIN
        j:=CreateJahr();
        SetzeJahr(j, JahrAlsCard(inD^.J));
        RETURN j;
END LiesJahrInDatum;

PROCEDURE SetzeJahrInDatum (VAR inoutD:TDatum; inJahr:TJahr);
(*
Vb:             keine
Setze das Jahr [1..9999] des ADO D
Nb:     keine
*)
BEGIN
        SetzeJahr(inoutD^.J,JahrAlsCard(inJahr));
END SetzeJahrInDatum;

PROCEDURE SetzeJahresanfang(VAR inoutDatum : TDatum);
BEGIN
        WITH inoutDatum^ DO
                SetzeTag(T,1);
                SetzeMonat(M,1);
        END (* WITH *);
END SetzeJahresanfang;

PROCEDURE SetzeJahresende(VAR inoutDatum : TDatum);
BEGIN
        WITH inoutDatum^ DO
                SetzeTag(T,31);
                SetzeMonat(M,12);
        END (* WITH *);
END SetzeJahresende;
  
PROCEDURE SetzeDatumEternity (VAR inoutD:TDatum);
(*
Vb:             keine
Setze Datum auf UNDEF
Nb:     keine
*)
BEGIN
        SetzeTag(inoutD^.T,31);
        SetzeMonat(inoutD^.M,12);
        SetzeJahr(inoutD^.J,9999);
        inoutD^.isUndef:=TRUE;
END SetzeDatumEternity;

PROCEDURE  SetzeValidInDatum (VAR inoutD:TDatum);
(*
Vb:             keine
Setze Datum auf NOT UNDEF
Nb:     keine
*)
BEGIN
        inoutD^.isUndef:=FALSE;
END SetzeValidInDatum;

PROCEDURE IstDatumEternity (inD:TDatum):BOOLEAN;
(*
Vb:             keine
TRUE, falls Datum = UNDEF
Nb:     keine
*)
BEGIN
        RETURN inD^.isUndef;
END IstDatumEternity;

PROCEDURE SmallerDatum(inSmallerD,inGreaterD:TDatum):BOOLEAN ;
(*
Vb:     keine
Prueft, ob SmallerD ein frueheres Datum als GreaterD enthaelt
Nb:     Reuckgabewert <=>       SmallerD "<" GreaterD
*)
VAR     SmallerMJD      : REAL;
        GreaterMJD      : REAL;
BEGIN
        IF IstDatumEternity(inGreaterD) THEN
                RETURN TRUE;
        ELSE
                SmallerMJD:=Datum2MJD(inSmallerD);
                GreaterMJD:=Datum2MJD(inGreaterD);
                RETURN (SmallerMJD < GreaterMJD);
        END (* IF *);

END SmallerDatum;
 
PROCEDURE MaxDatum (inDatum1, inDatum2:TDatum) : TDatum;
(*
Vb: (PrufeDatum (inDatum1) = true) AND ( PruefeDatum (inDatum2) = true)
Gibt das groessere Datum zurueck
Nb: --
*)
VAR 	outDatum : TDatum;
BEGIN
        IF SmallerDatum(inDatum1,inDatum2) THEN
		outDatum:=DuplicateDatum(inDatum2);	
                RETURN outDatum;
        ELSE
		outDatum:=DuplicateDatum(inDatum1);
                RETURN outDatum;
        END (* IF *)
END MaxDatum;

PROCEDURE MinDatum (inDatum1, inDatum2:TDatum) : TDatum;
(*
Vb: (PrufeDatum (inDatum1) = true) AND ( PruefeDatum (inDatum2) = true)
Gibt das kleinere Datum zurueck
Nb: --
*)
VAR 	outDatum : TDatum;
BEGIN
        IF SmallerDatum(inDatum1,inDatum2) THEN
		outDatum:=DuplicateDatum(inDatum1);
                RETURN outDatum;
        ELSE
		outDatum:=DuplicateDatum(inDatum2); 
                RETURN outDatum;
        END (* IF *)
END MinDatum;
 
PROCEDURE IncDatum(VAR inoutD:TDatum);
(*
Vb: Datum < 31.12.9999
D wird auf das Datum des nachfolgenden Tages gesetzt
Nb:     IsEqual(Dec(Dneu),Dalt) and not IsEqual(Dalt,Dneu)
*)
VAR MJD		: REAL;
    MaxDatum 	: TDatum;
    MaxMJD	: REAL;
BEGIN
	MaxDatum := CreateDatum();
        MaxMJD := Datum2MJD(MaxDatum);	
        IF NOT IstDatumEternity(inoutD) THEN
                MJD := Datum2MJD(inoutD);
		IF MJD < MaxMJD THEN
                	MJD := MJD + 1.0;
		END (* IF *);
                DisposeDatum(inoutD);
                inoutD:=MJD2Datum(MJD);
        END (* IF *);
	DisposeDatum(MaxDatum);
END IncDatum;

PROCEDURE DecDatum(VAR inoutD:TDatum);
(*
Vb: Datum > 1.1.1
D wird auf das Datum des vorhergehenden Tages gesetzt
Nb:     IsEqual(Inc(Dneu),Dalt) and not IsEqual(Dalt,Dneu)
*)
VAR MJD		: REAL;
    MinDatum 	: TDatum;	
    MinMJD	: REAL;
    DStr	: TString;
    mesg	: TString;
BEGIN
	DStr:=String.Create();
	String.Schreibe(DStr,"01.01.01");
	IF DatumVonString(DStr,MinDatum,mesg) THEN
		MinMJD := Datum2MJD(MinDatum);
        	IF NOT IstDatumEternity(inoutD) THEN
                	MJD := Datum2MJD(inoutD);
			IF MJD > MinMJD THEN
                		MJD := MJD - 1.0;
			END (* IF *);
                	DisposeDatum(inoutD);
                	inoutD:=MJD2Datum(MJD);
		END (* IF *);
		DisposeDatum(MinDatum);
	ELSE
		String.Dispose(mesg);
        END (* IF *);
	String.Dispose(DStr);
END DecDatum;
 
PROCEDURE DatumInZeitraum( inD,inVonD,inBisD:TDatum ): BOOLEAN;
(*
Vb: keine
prueft ob D zwischen vonD und bisD liegt
Nb: true falls inD in [vonD,bisD], false sonst
*)
BEGIN
        RETURN (SmallerDatum(inVonD,inD) AND
                SmallerDatum(inD,inBisD)) OR
                IsEqualDatum(inD,inVonD) OR IsEqualDatum(inD,inBisD);

END DatumInZeitraum;

PROCEDURE AnzahlMonate( inVonD,inBisD:TDatum ): CARDINAL;
(*
Vb: vonD <= bisd
gib die Anzahl der Monate zwischen vonD und bisD zurueck. Falls die Datums
angaben undef sind wird 0 returniert
Semantik: angeschnittene Monate zaehlen mit.
Nb: --
*)
VAR     J : CARDINAL;
        M : CARDINAL;
BEGIN
        IF NOT (IstDatumEternity(inVonD) OR IstDatumEternity(inBisD)) THEN
          IF (JahrAlsCard(inBisD^.J) - JahrAlsCard(inVonD^.J)) > 1  THEN
                J:=	JahrAlsCard(inBisD^.J) - 
			JahrAlsCard(inVonD^.J) - 1 (* Anzahl der vollen Jahre *)
          ELSE
                J:=0;
          END (* IF *);
          IF (JahrAlsCard(inBisD^.J) > JahrAlsCard(inVonD^.J)) THEN
                M:= 	MonatAlsCard(inBisD^.M) + 12 - MonatAlsCard(inVonD^.M);
          ELSE
                M:= 	MonatAlsCard(inBisD^.M) - 
		    	MonatAlsCard(inVonD^.M) + 1 ; (* mindestens 1 Monat *)
          END (* IF *);
          RETURN (J * 12 + M);
        ELSE
          RETURN 0;
        END (* IF *);
END AnzahlMonate;
 
PROCEDURE AnzahlWochen( inVonD,inBisD:TDatum ): CARDINAL;
(*
Vb: vonD <= bisD
gib die Anzal der Wochen zwischen vonD und bisD zurueck
Semantik: angeschnittene Wochen zaehlen mit
Nb: --
*)
VAR DStart, DEnde,D 	: TDatum;
    MJDStart, MJDEnde	: REAL;

BEGIN
        IF NOT (IstDatumEternity(inVonD) OR IstDatumEternity(inBisD)) THEN
		AnfEndDatumKWZuDatum(inVonD,DStart,D);
		DisposeDatum(D);
		AnfEndDatumKWZuDatum(inBisD,D,DEnde);
		DisposeDatum(D);
		MJDStart:=Datum2MJD(DStart);
		MJDEnde:=Datum2MJD(DEnde);
		DisposeDatum(DStart);
		DisposeDatum(DEnde);
                RETURN ROUND((MJDEnde-MJDStart)/7.0 + 0.5);
        ELSE
                RETURN 0;
        END (* IF *);
END AnzahlWochen;

PROCEDURE AnzahlTage( inVonD,inBisD:TDatum ): CARDINAL;
(*
Vb: vonD <= bisD
gib die Anzal der Tage zwischen vonD und bisD zurueck
Nb: --
*)
BEGIN
        IF NOT (IstDatumEternity(inVonD) OR IstDatumEternity(inBisD)) THEN
                RETURN TRUNC(Datum2MJD(inBisD)-Datum2MJD(inVonD));
        ELSE
                RETURN 0;
        END (* IF *);
END AnzahlTage;
  
PROCEDURE AnfEndDatumKWZuDatum (inDatum : TDatum;
                                VAR outAnfang, outEnde : TDatum);
(*
Vb: (1 <= inDatum.Tag <= 31) AND (PruefeDatum(inDatum)=True)
Ermittelt zu dem uebergebenen Datum die Kalenderwoche, in der das Datum
inDatum liegt. Der Mittwoch hat im System des MJD offenbar die Tagesnummer 1
daher der Ausdruck MJD-4.0.
NB: --
*)
VAR MJD	: REAL;
    TNr	: INTEGER;
    D	: TString;
    str	: ARRAY [1..255] OF CHAR;
BEGIN
        IF NOT IstDatumEternity(inDatum) THEN
          MJD:=Datum2MJD(inDatum);
          TNr:=Wochentag(inDatum);
	  outAnfang:=MJD2Datum(MJD-FLOAT(TNr-1));
          outEnde:=MJD2Datum(MJD+FLOAT(7-TNr));
        ELSE
          outAnfang:=CreateDatum();
          outEnde:=CreateDatum();
        END (* IF *);
END AnfEndDatumKWZuDatum;

PROCEDURE Wochentag ( inDatum:TDatum):CARDINAL;
(*
vb: --
liefere die Nr des Wochentages zurck: 1=Montag, 7=Sonntag. Der Mittwoch hat im System des MJD offenbar die Tagesnummer 1 daher der Ausdruck MJD-4.0. 
nb: --
*)
VAR TNr : INTEGER;
    MJD : REAL;
BEGIN
        IF NOT IstDatumEternity(inDatum) THEN
          MJD:=Datum2MJD(inDatum);
          TNr:=ENTIER(MJD-4.0) MOD 7;
          IF TNr < 0 THEN
                TNr:=TNr * (-1);
          END (* IF *);
          IF TNr=0 THEN
                TNr:=7;
          END (* IF *);
	  RETURN CAST(CARDINAL,TNr);
	ELSE
	  RETURN 0;
        END (* IF *); 
END Wochentag; 
 
PROCEDURE isSchaltJahr (inDatum : TDatum):BOOLEAN;
VAR 	J : CARDINAL;
BEGIN
	J:=JahrAlsCard(inDatum^.J);
        IF((J MOD 4) # 0) THEN
                RETURN FALSE;
        ELSIF ((J MOD 400) = 0) THEN
                RETURN TRUE;
        ELSIF ((J MOD 100) = 0) THEN
                RETURN FALSE;
        ELSE
        	RETURN TRUE;
        END (* IF *);
END isSchaltJahr;
  
PROCEDURE AnfEndDatumKWZuMonat (inDatum : TDatum;
                                 VAR outAnfang, outEnde : TDatum);
(*
Vb: 1 <= inDatum.Monat <= 12
Ermittelt zu dem uebergebenen Monat die Kalenderwochen, in denen der erste und
letzte Tag des Monats liegt. outAnfang ist das Kalenderdatum des ersten Tages
der Woche, in der der erste des Monats liegt. outEnde ist das Kalenderdatum des
letzten Tages der Woche, in der der letzte Tag des Monats liegt.
NB: --
*)
VAR     D, inDat : TDatum;
BEGIN
        IF NOT IstDatumEternity(inDatum) THEN
          D:=CreateDatum();
          inDat:=DuplicateDatum(inDatum);
          SetzeTag(inDat^.T,1);
          AnfEndDatumKWZuDatum(inDat,outAnfang,D);
          DisposeDatum(D);
          WITH inDat^ DO
                IF MonatAlsCard(M) IN MonatsMenge{4,6,9,11} THEN
                        SetzeTag(T,30);
                ELSIF MonatAlsCard(M) IN MonatsMenge{1,3,5,7,8,10,12} THEN
                        SetzeTag(T,31);
                ELSIF MonatAlsCard(M) = 2 THEN
                        IF isSchaltJahr(inDatum)  THEN
                                SetzeTag(T,29);
                        ELSE
                                SetzeTag(T,28);
                        END (* IF *);
                END (* IF *);
          END (* WITH *);
          AnfEndDatumKWZuDatum(inDat,D,outEnde);
          DisposeDatum(D);
          DisposeDatum(inDat);
        ELSE
          outAnfang:=CreateDatum();
          outEnde:=CreateDatum();
        END (* IF *);
END AnfEndDatumKWZuMonat;
 
PROCEDURE ErsteWocheInMonat (inDatum:TDatum; VAR outWAnfang, outWEnde:TDatum);
(*
Vb: PrufeDatum (inDatum) = true
Ermittelt das Anfangsdatum und Endedatum der Kalenderwoche, in der der Monatserstevon inDatum liegt. Angeschnitte Wochen zaehlen mit.
Nb: ADOs outWAnfang, outWEnde existieren
*)
VAR D:TDatum;
BEGIN
        D:=DuplicateDatum(inDatum);
        SetzeTag(D^.T,1);
        AnfEndDatumKWZuDatum(D, outWAnfang, outWEnde);
        DisposeDatum(D);
END ErsteWocheInMonat;
  
PROCEDURE NaechsteWocheInMonat (inDatum:TDatum; VAR inoutWAnfang, inoutWEnde:TDatum):BOOLEAN;
(*
Vb: PrufeDatum (inDatum) = true
inDatum liefert den relevanten Monat. Zu diesem Monat sollen alle folgenden Wochen ausgegeben werden.
inoutWAnfang, inoutWEnde enthalten als Eingabe die aktuelle Woche. Zu dieser Woche wird eine Woche hinzugezaehlt und wieder ausgegeben.
Ist das neue inoutWAnfang nicht mehr im betrachteten Monat (inDatum), dann gibt es keine NaechsteWocheImMonat
NB: (False <=> wenn keine naechste Woche existiert)
*)
VAR     MJDA    : REAL;
        MJDE    : REAL;
        D1,D2   : TDatum;
        imMonat : BOOLEAN;
BEGIN
        IF NOT IstDatumEternity(inDatum) THEN
          MJDE:=Datum2MJD(inoutWEnde);
          MJDA:= MJDE + 1.0;              (* neuer Wochenanfang ist Ende + 1 *)
          D1:=MJD2Datum(MJDA);
          IF MonatAlsCard(D1^.M) = MonatAlsCard(inDatum^.M) THEN 
	        (* noch im gleichen Monat? *)
                EqualizeDatum(inoutWAnfang,D1);
                D2:=MJD2Datum(MJDE + 7.0);
                EqualizeDatum(inoutWEnde,D2);
                DisposeDatum(D2);
                imMonat:=TRUE;
          ELSE
                imMonat:=FALSE;
          END (* IF *);
          DisposeDatum(D1);
        ELSE
          imMonat:=FALSE;               (* falls ein undef Datum uebergeben *)
                                        (* wurde, wird auch undef returniert*)
          SetzeDatumEternity(inoutWAnfang);
          SetzeDatumEternity(inoutWEnde);
        END (* IF *);
        RETURN imMonat;

END NaechsteWocheInMonat;

 
(************************* Konvertierungsoperatoren ***************************)

PROCEDURE DatumAlsString (inDatum : TDatum) : TString;
(*
Vb: --
Konvertiert ein uebergebenes Datum inDatum in einen les-/druckbaren String "31.1
2.2001"
Nb: --
*)
VAR     str1:TString;
        str2:TString;
        arr:ARRAY [1..5] OF CHAR;
        str0:TString;
BEGIN
        str1:= String.Create();
        IF inDatum^.isUndef THEN
          String.Schreibe(str1,"undef     ");
        ELSE
          str0:= String.Create();
          String.Schreibe(str0,"0");
          str2:= String.Create();
          WholeStr.CardToStr(TagAlsCard(inDatum^.T),arr);
          String.Schreibe(str2,arr);
          IF TagAlsCard(inDatum^.T) < 10 THEN
                String.Anhaengen(str0,str1);
          END (* IF *);
          String.Anhaengen(str2,str1);
          String.Schreibe(str2,".");
          String.Anhaengen(str2,str1);
          WholeStr.CardToStr(MonatAlsCard(inDatum^.M),arr);
          String.Schreibe(str2,arr);
          IF MonatAlsCard(inDatum^.M) < 10 THEN
                String.Anhaengen(str0,str1);
          END (* IF *);
          String.Anhaengen(str2,str1);
          String.Schreibe(str2,".");
          String.Anhaengen(str2,str1);
          WholeStr.CardToStr(JahrAlsCard(inDatum^.J),arr);
          String.Schreibe(str2,arr);
          IF JahrAlsCard(inDatum^.J) < 1000 THEN
                String.Anhaengen(str0,str1);
                IF JahrAlsCard(inDatum^.J) < 100 THEN
                        String.Anhaengen(str0,str1);
                        IF JahrAlsCard(inDatum^.J) < 10 THEN
                                String.Anhaengen(str0,str1);
                        END (* IF *);
                END (* IF *);
          END (* IF *);
          String.Anhaengen(str2,str1);
          String.Dispose(str2);
          String.Dispose(str0);
        END (* IF *);
        RETURN str1;
END DatumAlsString;



PROCEDURE DatumVonString (inDatum : TString; VAR outDatum:TDatum;
                          VAR outMessage:TString) : BOOLEAN;
(*
Vb: --
Konvertiert einen uebergebenen String "31.12.2001" in ein internes Datum-ADO
Falls 'undef' uebergeben wird dann wird ein Undefiniertes (31.12.9999) ADO
zurueckgegeben. Der Wertebereich ist 0-9999;
Nb: --
*)
TYPE    TDatumKomp = (Tag,Monat,Jahr);
CONST   LEN=20;
VAR     Muster   : TString;
        TeilStr  : TString;
        undefStr : TString;
        inString : TString;
        CharArray: ARRAY[1..LEN] OF CHAR;
        gefunden : BOOLEAN;
        gueltig  : BOOLEAN;
        posStart : CARDINAL;
        posEnde  : CARDINAL;
        val      : CARDINAL;
        i        : TDatumKomp;
        Result   : WholeStr.ConvResults;

        (* Entfernen von fuehrenden und folgenden Leerzeichen *)
        PROCEDURE trim (VAR inoutS:TString);
        VAR TrimArray : ARRAY[1..LEN] OF CHAR;
            j         : CARDINAL;
        BEGIN
		(* Arrays initialisieren *)
                FOR j:=1 TO LEN DO
                        TrimArray[j]:='';
                        CharArray[j]:='';
                END (* FOR *);
                String.Lies(inoutS,CharArray);  
                j:=1;
                WHILE (j <= LEN) AND (CharArray[j]<>' ') DO
                        TrimArray[j]:=CharArray[j];
                        INC(j);
                END (* WHILE *);
                String.Schreibe(inoutS,TrimArray);
        END trim;

BEGIN
        Muster:=String.Create();  (* hier kommt der . rein *)
        TeilStr:=String.Create(); (* fuer Tag,Monat, Jahr *)
        undefStr:=String.Create();(* zum Vergleich auf undef *)
        outDatum:=CreateDatum();
        gueltig:=TRUE;
        inString:=String.Duplicate(inDatum);
        trim(inString);
        String.Schreibe(undefStr,"UNDEF");
        String.GrossBuchstaben(inString);
        IF NOT String.IsEqual(inString,undefStr) THEN
          posStart:=0;
          String.Schreibe(Muster,".");
          FOR i:=Tag TO Jahr DO
                String.FindeNaechsten(
                        Muster,inString,posStart,gefunden,posEnde);
                IF i=Jahr THEN
                  posEnde:=String.Laenge(inString);
                  gefunden:=TRUE;
                END (* IF *);
                IF gefunden THEN
                  String.Auslesen(inString,posStart,posEnde-posStart,TeilStr);
                  String.Lies(TeilStr,CharArray);
                  WholeStr.StrToCard(CharArray,val,Result);
                  IF  Result=WholeStr.strAllRight THEN
                    CASE i OF
                       Tag:     IF (val >= 1) AND (val <= 31) THEN
                                  SetzeTag(outDatum^.T, val);
                                ELSE
                                  gueltig:=FALSE;
                                END (* IF *);
                      |Monat:   IF (val >= 1) AND (val <= 12) THEN
                                  SetzeMonat(outDatum^.M, val);
                                ELSE
                                  gueltig:=FALSE;
                                END (* IF *); 
                      |Jahr:    IF (val >= 0) AND (val <= 9999) THEN
                                  SetzeJahr(outDatum^.J, val);
                                ELSE
                                  gueltig:=FALSE;
                                END (* IF *);
                    END;
                  ELSE
                    gueltig:=FALSE;
                  END (* IF *);
                  posStart:=posEnde + 1;
                ELSE
                  gueltig:=FALSE;
                END (* IF *);
          END (* FOR *);
          IF PruefeDatum(outDatum) AND gueltig THEN
                outDatum^.isUndef:=FALSE;
          ELSE
                outMessage:=String.Create();
                String.Schreibe(
                  outMessage,"Das Datum konnte nicht umgewandelt werden!");
                DisposeDatum(outDatum);
                gueltig:=FALSE;
          END (* IF *);
        END (* IF *);
        String.Dispose(Muster);
        String.Dispose(TeilStr);
        String.Dispose(undefStr);
        String.Dispose(inString);
        RETURN gueltig;
END DatumVonString; 


(********************Funktionen zum Rechnen mit julianischem Datum************)
(* Algorithmen aus 'Mathematisches Hilfsbuch fuer Studierende und Freunde der*)
(* Astronomie', W.Wepner, Verlag Vehrenberg                                  *)
(* Funktion zur Umwandlung eines gregorianischen Datums in ein modifiziertes *)
(* julianisches Datum                                                        *)
PROCEDURE Datum2MJD( inD:TDatum):REAL;
VAR 	y,m,A,K     : INTEGER;
 	B           : REAL;
 	JD          : REAL;
 	J,M,T	    : INTEGER; (* Eingabe Tag Monat Jahr *)

BEGIN
	J := JahrAlsCard(LiesJahrInDatum(inD));
	M := MonatAlsCard(LiesMonatInDatum(inD));
	T := TagAlsCard(LiesTagInDatum(inD));

        K := 10000 * J + 100 * M + T;
        B := -63.5;
        y := J + 4712 ;
        m := M + 1;
        IF M <= 2 THEN
                y := y - 1;
                m := m + 12;
        END (* IF *);
        IF K >= 15821015 THEN
                A := ENTIER(FLOAT(y + 88)/100.0);
                B := B + FLOAT(38 - A + ENTIER(FLOAT(A)/4.0));
        END (* IF *);
        JD := FLOAT(ENTIER(365.25 * FLOAT(y)) +
              ENTIER(30.6001 * FLOAT(m)) + T) + B;
        JD := JD - 2400000.5;               (* so wird aus JD das MJD *)
        RETURN JD;
END Datum2MJD; 


(* Prozedur zur Umwandlung eines MJD in ein gregorianisches Datum            *)
(* Langversion nach Jan Meeus, gilt fuer alle julianische Daten              *)
(* von -4712 - 9999                                                          *)
PROCEDURE MJD2Datum (inJD : REAL) :TDatum;
VAR A,B,C,E,K,M,Z,a,m	: INTEGER;
    JD,N,F			: REAL;
    outD		: TDatum;
    Ta			: TTag;
    Mo			: TMonat;
    Ja			: TJahr;

BEGIN
        outD:=CreateDatum();
	Ta:=CreateTag();
	Mo:=CreateMonat();
	Ja:=CreateJahr();
	
        JD := inJD + 2400000.5; (* aus MJD wird JD *)

        N := JD + 0.5;
        Z := ENTIER(N);
        F := N - FLOAT(Z);
        IF Z < 2299161 THEN
                A := Z;
        ELSE
                a := ENTIER((FLOAT(Z)-1867216.25)/36524.25);
                A := Z + 1 + a - ENTIER(FLOAT(a)/4.0);
        END (* IF *);
        B := A + 1524;
        C := ENTIER((FLOAT(B) - 122.1)/365.25);
        K := ENTIER(365.25 * FLOAT(C));
        E := ENTIER(FLOAT(B - K)/30.6001);
        SetzeTag(Ta, ROUND(FLOAT(B - K - ENTIER( 30.6001*FLOAT(E))) + F));
        m := E - 1;
        SetzeJahr(Ja, C - 4716);

        IF E > 13 THEN
                SetzeMonat(Mo, m - 12);
                SetzeJahr(Ja, JahrAlsCard(Ja) + 1);
        ELSE
                SetzeMonat(Mo, m) ;
        END (* IF *);
	SetzeTagInDatum(outD,Ta);
	SetzeMonatInDatum(outD,Mo);
	SetzeJahrInDatum(outD,Ja);
        SetzeValidInDatum(outD);
	DisposeTag(Ta);
	DisposeMonat(Mo);
	DisposeJahr(Ja);
        RETURN outD;
END MJD2Datum; 

END Date.
