(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: STag.mod,v $
# $Author: Sopra $
# $Date: 2001/03/13 18:05:20 $
# $Revision: 1.2 $
# Bemerkungen: Hier sind die Funktionen und Typen fuer's ADO STag drin!
        
*)
(******************************************************************************)


IMPLEMENTATION MODULE STag;

(* TS Datum *)
(* Die Woche geht von Montag(1) bis Sonntag(7)  *)
(* DatenTeilSystem zum Erzeugen, Lschen von Datum ADOs. *)
(* Operatoren auf STag ADOs *)


IMPORT String;
IMPORT WholeStr;
IMPORT DatumIMPORTE;
FROM Storage IMPORT ALLOCATE, DEALLOCATE;
FROM SYSTEM  IMPORT TSIZE;  
FROM 	Monat 	IMPORT TMonat, CreateMonat, DisposeMonat, SetzeMonat, 
		MonatAlsCard, IsEqualMonat;
FROM    Tag  	IMPORT TTag, CreateTag, DisposeTag, SetzeTag, TagAlsCard, 
		IsEqualTag;

TYPE TString	= 	String.TString;
TYPE TStag      = 	POINTER TO TStagDS;
TYPE TStagDS    =       RECORD
                                T : TTag;
                                M : TMonat;
                                isUndef:BOOLEAN;
                        END; 
TYPE Monate 	= [1..12];
TYPE MonatsMenge = SET OF Monate;  

(**************************** Datentyp TStag = Stichtag ***********************)


PROCEDURE CreateStag ():TStag;
(*
Vb:             keine
Erzeugt ein neues ADO D vom Typ TStag
Default-Wert (nicht gesetztes Datum) ist D.isUndef=TRUE;
Nb:     ADO D existiert
*)
VAR aktStag : TStag;
BEGIN (* -- Create *)
    (* -- Vorbed.: keine *)
     ALLOCATE (aktStag, TSIZE (TStagDS)) ;
     WITH aktStag^ DO
        T := CreateTag();
        M := CreateMonat();
        isUndef:=TRUE;
     END; (* WITH *)
     RETURN aktStag;
    (* -- Nachbed.: RueckgabeADO aktID existiert *)
END CreateStag;

PROCEDURE DisposeStag (VAR inoutD:TStag);
(*
Vb: keine
ADO D wird aus dem System entfernt
Nb:     ADO D existiert nicht mehr
*)
BEGIN
        DisposeTag(inoutD^.T);
        DisposeMonat(inoutD^.M);
        DEALLOCATE (inoutD, TSIZE (TStagDS));
        inoutD:=NIL;
END DisposeStag;   

PROCEDURE DuplicateStag ( inD:TStag):TStag;
(*
Vb:     keine
Erzeugt das Rueckgabe ADO und gleicht die Werte der Attribute an;
Nb:     IsEqual(D, RueckgabeD) and not (D=RueckgabeD)
*)
VAR Duplikat: TStag;
BEGIN
        Duplikat:=CreateStag();
        WITH Duplikat^ DO
                SetzeTag(T,TagAlsCard(inD^.T));
                SetzeMonat(M,MonatAlsCard(inD^.M));
                isUndef:=inD^.isUndef;
        END (* WITH *);
        RETURN Duplikat;

END DuplicateStag;

PROCEDURE IsEqualStag (inD1,inD2: TStag ):BOOLEAN;
(*
Vb:             keine
berprueft die beiden Datuemer auf Wertegleichheit aller Attribute
Nb:     keine
*)

VAR gleich: BOOLEAN;
BEGIN
        gleich :=       (IsEqualTag(inD1^.T, inD2^.T) AND
                         IsEqualMonat(inD1^.M, inD2^.M)) OR
                        (inD1^.isUndef = inD2^.isUndef);
        RETURN gleich;

END IsEqualStag;  

PROCEDURE EqualizeStag (VAR inoutZielD:TStag ; inQuellD:TStag);
(*
Vb:     keine
gleicht die Werte der Attribute von ZielD an die von QuellD an, wobei die Identi
taet der ADO's nicht veraendert wird
Nb:     IsEqual(QuellD,ZielD) and not (QuellD=ZielD)
*)
BEGIN
        WITH inoutZielD^ DO
                SetzeTag(T,TagAlsCard(inQuellD^.T));
                SetzeMonat(M,MonatAlsCard(inQuellD^.M));
                isUndef:=inQuellD^.isUndef;
        END (* WITH *);
END EqualizeStag;


PROCEDURE LiesTagInStag ( inStag :TStag):TTag;
(*
Vb:             keine
Liefert den Tag des ADO  Stag
Nb:     keine
*)
VAR t:TTag;
BEGIN
        t:=CreateTag();
        SetzeTag(t,TagAlsCard(inStag^.T));
        RETURN t;
END LiesTagInStag;

PROCEDURE SetzeTagInStag (VAR inoutStag:TStag; inTag:TTag);
(*
Vb: Tag in [1..31]
Setzt den Tag im ADO Stag
Nb:     GetTag(Dneu)=Tag
*)
BEGIN
        SetzeTag(inoutStag^.T,TagAlsCard(inTag));
END SetzeTagInStag; 

PROCEDURE LiesMonatInStag (inStag:TStag):TMonat;
(*
Vb:             keine
Liefert den Monat des ADO  Stag
Nb:     keine
*)
VAR m:TMonat;
BEGIN
        m:=CreateMonat();
        SetzeMonat(m, MonatAlsCard(inStag^.M));
        RETURN m;
END LiesMonatInStag;

PROCEDURE SetzeMonatInStag (VAR inoutStag:TStag; inMonat:TMonat);
(*
Vb:             keine
Setze Monat [1..12] des ADO Stag
Nb:     keine
*)
BEGIN
        SetzeMonat(inoutStag^.M, MonatAlsCard(inMonat));
END SetzeMonatInStag;
 
PROCEDURE PruefeStag (inStag: TStag) : BOOLEAN;
(*
Vb: --
Prueft, ob es sich bei dem Stichtag inStag um ein gueltiges Datum handelt.
z.B. wird bei inStag = 31.02. ein false geliefert.
Da ein Stichtag 29.02. nur alle vier Jahre auftreten wuerde, ist ein Stichtag 29
.02. ebenfalls verboten
nb: (false <=> inStag ungueltig) sonst true
*)
VAR 	gueltig	: BOOLEAN;
	T,M	: CARDINAL;
BEGIN
	T:=TagAlsCard(inStag^.T);
	M:=MonatAlsCard(inStag^.M);

        IF (M IN MonatsMenge{2,4,6,9,11}) AND (T = 31) THEN
                        gueltig:=FALSE;
        ELSIF   (M = 2) AND ((T =29) OR (T = 30)) THEN
                        gueltig:=FALSE;
        ELSIF NOT (M IN MonatsMenge{1,2,3,4,5,6,7,8,9,10,11,12}) THEN
                        gueltig:=FALSE;
        ELSIF (T < 1) OR (T > 31) THEN
                        gueltig:=FALSE;
        ELSE
                        gueltig:=TRUE;
        END (* IF *);

        RETURN gueltig;
END PruefeStag;

PROCEDURE SmallerStag ( inKleinerStag, inGroesserStag : TStag):BOOLEAN;
VAR M1,M2	: CARDINAL;
    T1,T2	: CARDINAL;

BEGIN
	M1:= MonatAlsCard(inKleinerStag^.M);
	M2:= MonatAlsCard(inGroesserStag^.M);
	T1:= TagAlsCard(inKleinerStag^.T);
	T2:= TagAlsCard(inGroesserStag^.T);
 
	IF M1 < M2 THEN
	  RETURN TRUE;
	ELSIF M1 = M2 THEN
	      IF T1 < T2 THEN
	  	RETURN TRUE;
	      ELSE 
		RETURN FALSE;
              END (* IF *);
	ELSE
	  RETURN FALSE;
	END (* IF *);
END SmallerStag;
	
	

PROCEDURE SetzeStagEternity (VAR inoutStag:TStag);
(*
Vb:             keine
Setze inoutStag auf UNDEF
Nb:     keine
*)
BEGIN
        inoutStag^.isUndef:=TRUE;
END SetzeStagEternity;
 
PROCEDURE IstStagEternity (inStag:TStag):BOOLEAN;
(*
Vb:             keine
TRUE, falls inoutStag = UNDEF
Nb:     keine
*)
BEGIN
        RETURN inStag^.isUndef;
END IstStagEternity;
 
(************************** Konvertierungsoperatoren **************************)
 
PROCEDURE StagAlsString (inStag : TStag) : TString;
(*
Vb: --
Konvertiert ein uebergebenes Stag inStag in einen les-/druckbaren String "31.12.2001"
Nb: --
*)
VAR     str0:TString;
        str1:TString;                                                                   str2:TString;
        arr:ARRAY [1..5] OF CHAR;
BEGIN
        str1:= String.Create();
        IF inStag^.isUndef THEN
          String.Schreibe(str1,"undef");
        ELSE
          str2:= String.Create();
          str0:= String.Create();
          String.Schreibe(str0,"0");
          WholeStr.CardToStr(TagAlsCard(inStag^.T),arr);
          String.Schreibe(str2,arr);
          IF TagAlsCard(inStag^.T) < 10 THEN
                String.Anhaengen(str0,str1);
          END (* IF *);
          String.Anhaengen(str2,str1);
          String.Schreibe(str2,".");
          String.Anhaengen(str2,str1);
          WholeStr.CardToStr(MonatAlsCard(inStag^.M),arr);
          String.Schreibe(str2,arr);
          IF MonatAlsCard(inStag^.M) < 10 THEN
                String.Anhaengen(str0,str1);
          END (* IF *);
          String.Anhaengen(str2,str1);
          String.Schreibe(str2,".");
          String.Anhaengen(str2,str1);
          String.Dispose(str0);
          String.Dispose(str2);
        END (* IF *);
        RETURN str1;
END StagAlsString;      

PROCEDURE StagVonString (inStagString : TString;
 VAR outStag:TStag; VAR outMessage:TString) : BOOLEAN;
(*
Vb: --
Konvertiert einen uebergebenen String "31.12." in ein internes Stag-ADO
Nb: --
*)
TYPE    TDatumKomp = (Tag,Monat);
VAR     Muster   : TString;
        TeilStr  : TString;
        inString : TString;
        CharArray: ARRAY[1..255] OF CHAR;
        gefunden : BOOLEAN;
        gueltig  : BOOLEAN;
        posStart : CARDINAL;
        posEnde  : CARDINAL;
        val      : CARDINAL;
        i        : TDatumKomp;
        Result   : WholeStr.ConvResults;

BEGIN
        inString:=String.Duplicate(inStagString);
        Muster:=String.Create();
        TeilStr:=String.Create();
        outStag:=CreateStag();
        gueltig:=TRUE;
        posStart:=0;
        String.Schreibe(Muster,".");
        (* Sonderbehandlung fuer Stichtage ohne 2. '.' *)
        IF String.Laenge(inString) > 1 THEN
                String.Auslesen(inString,String.Laenge(inString),1,TeilStr);
                IF NOT String.IsEqual(TeilStr,Muster) THEN
                        String.Anhaengen(Muster,inString);
                END (* IF *);
        END (* IF *);
        FOR i:=Tag TO Monat DO
                String.FindeNaechsten(
                        Muster,inString,posStart,gefunden,posEnde);
                IF gefunden THEN
                  String.Auslesen(inString,posStart,posEnde-posStart,TeilStr);
                  String.Lies(TeilStr,CharArray);
                  WholeStr.StrToCard(CharArray,val,Result);
                  IF gefunden AND (Result=WholeStr.strAllRight) THEN
                    CASE i OF 
                       Tag:     IF (val > 0) AND (val < 32) THEN
                                  SetzeTag(outStag^.T, val);
                                ELSE
                                  gueltig:=FALSE;
                                END (* IF *);

                      |Monat:   IF (val > 0) AND (val < 13) THEN
                                  SetzeMonat(outStag^.M, val);
                                ELSE
                                  gueltig:=FALSE;
                                END (* IF *);
                    END;
                  ELSE
                    gueltig:=FALSE;
                  END (* IF *);
                  posStart:=posEnde + 1;
                ELSE
                  gueltig:=FALSE;
                END (* IF *);
        END (* FOR *);
        IF PruefeStag(outStag) AND gueltig THEN
                outStag^.isUndef:=FALSE;
        ELSE
                outMessage:=String.Create();
                String.Schreibe(outMessage,
                        "Das Stichtags-Datum konnte nicht umgewandelt werden!");
                gueltig:=FALSE;
                DisposeStag(outStag);
        END (* IF *);
        String.Dispose(Muster);
        String.Dispose(TeilStr);
        String.Dispose(inString);
        RETURN gueltig;
END StagVonString; 

END STag.
