(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*)
   (******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: Adresse.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:47:57 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 

IMPLEMENTATION MODULE Adresse   ADT Adresse
                                --  Enthalten in Hilfskraefte

Sigrid Preuss

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE Adresse;

(* ADT Adresse *)

FROM Storage	IMPORT ALLOCATE,DEALLOCATE;
FROM SYSTEM	IMPORT TSIZE;


IMPORT String;

TYPE TString = String.TString;


TYPE TAdresse_DS  = RECORD
                      Strasse      : TString;
                      Postleitzahl : TString;
                      Stadt        : TString;
                      Land         : TString;
                      Telefon      : TString;
                      Fax          : TString;
                      Mail         : TString;
                    END;

TYPE  TAdresse = POINTER TO TAdresse_DS;


(*
SEMANTIK
Die Basis-Operationen Dispose, Duplicate, IsEqual, Equalize
besitzen die uebliche Semantik.

Die Attribut-Operationen
  Lies<Attribut>
  Setze<Attribut>
haben folgende Semantik:

Lies<Attribut>:
Vb:	keine
Liefert ein ADO des entsprechenden Attributs
nb:	Rueckgabe-ADO existiert

Setze<Attribut>:
Vb:	keine
Gleicht den Zustand des entsprechenden Attribut-ADOs in inoutAdresse dem Zustand des
Parameter-Attributs in<Attribut> an.
Nb:	IsEqual(Lies<Attribut>(), in<Attribut>)
*)






(* Basis-Operationen *)

PROCEDURE	Create ():TAdresse;
(*
Vb:	keine
Generiert ein ADO Adresse.
Die Adresse enthaelt auscchliesslich String-Attribute. Sie werden
mit einem Leerstring initialisiert.

Nb:	Rueckgabe-ADO Adresse existiert
*)


  VAR Adresse : TAdresse;

BEGIN
  ALLOCATE (Adresse, TSIZE(TAdresse_DS));

  WITH Adresse^ DO
    Strasse      := String.Create();
    Postleitzahl := String.Create();
    Stadt        := String.Create();
    Land         := String.Create();
    Telefon      := String.Create();
    Fax          := String.Create();
    Mail         := String.Create();
  END; (* WITH Adresse *)

  RETURN Adresse;

END Create;

PROCEDURE	Dispose (VAR inoutAdresse:TAdresse);

BEGIN
  WITH inoutAdresse^ DO
    String.Dispose(Strasse);
    String.Dispose(Postleitzahl);
    String.Dispose(Stadt);
    String.Dispose(Land);
    String.Dispose(Telefon);
    String.Dispose(Fax);
    String.Dispose(Mail);
  END; (* WITH inoutAdresse^ *)

  DEALLOCATE (inoutAdresse, TSIZE(TAdresse_DS));
  inoutAdresse := NIL;

END Dispose;


PROCEDURE	Duplicate (inAdresse:TAdresse):TAdresse;

  VAR Duplikat : TAdresse;

BEGIN
  Duplikat := Create();
  Equalize(Duplikat,inAdresse);
  RETURN Duplikat;
END Duplicate;


PROCEDURE	IsEqual (inAdresse1,inAdresse2:TAdresse):BOOLEAN;

  VAR gleicheAttribute : BOOLEAN;

BEGIN
  gleicheAttribute := 
       (String.IsEqual(inAdresse1^.Strasse,
                       inAdresse2^.Strasse)       AND
        String.IsEqual(inAdresse1^.Postleitzahl,
                       inAdresse2^.Postleitzahl)  AND
        String.IsEqual(inAdresse1^.Stadt,
                       inAdresse2^.Stadt)         AND
        String.IsEqual(inAdresse1^.Land,
                       inAdresse2^.Land)          AND
        String.IsEqual(inAdresse1^.Telefon,
                       inAdresse2^.Telefon)       AND
        String.IsEqual(inAdresse1^.Fax,
                       inAdresse2^.Fax)           AND
        String.IsEqual(inAdresse1^.Mail,
                       inAdresse2^.Mail));

  RETURN gleicheAttribute;

END IsEqual;


PROCEDURE	Equalize (inoutZielAdresse:TAdresse;
                          inQuelleAdresse:TAdresse);

BEGIN

  WITH inoutZielAdresse^ DO
       String.Equalize(Strasse, inQuelleAdresse^.Strasse);
       String.Equalize(Postleitzahl,inQuelleAdresse^.Postleitzahl);
       String.Equalize(Stadt, inQuelleAdresse^.Stadt);
       String.Equalize(Land, inQuelleAdresse^.Land);
       String.Equalize(Telefon, inQuelleAdresse^.Telefon);
       String.Equalize(Fax, inQuelleAdresse^.Fax);
       String.Equalize(Mail, inQuelleAdresse^.Mail);

 END; (* WITH inoutZielAdresse^ *)

END Equalize;



(* Operationen auf den Attributen von Adresse *)

PROCEDURE	LiesStrasse (inAdresse:TAdresse):TString;
BEGIN
  RETURN String.Duplicate(inAdresse^.Strasse);
END LiesStrasse;

PROCEDURE	SetzeStrasse (inoutAdresse:TAdresse; inStrasse:TString);
BEGIN
  String.Equalize(inoutAdresse^.Strasse, inStrasse);
END SetzeStrasse;

PROCEDURE	LiesPostleitzahl (inAdresse:TAdresse):TString;
BEGIN
  RETURN String.Duplicate(inAdresse^.Postleitzahl);
END LiesPostleitzahl;

PROCEDURE	SetzePostleitzahl (inoutAdresse:TAdresse; inPLZ:TString);
BEGIN
  String.Equalize(inoutAdresse^.Postleitzahl, inPLZ);
END SetzePostleitzahl;

PROCEDURE	LiesStadt (inAdresse:TAdresse):TString;
BEGIN
  RETURN String.Duplicate(inAdresse^.Stadt);
END LiesStadt;

PROCEDURE	SetzeStadt (inoutAdresse:TAdresse; inStadt:TString);
BEGIN
  String.Equalize(inoutAdresse^.Stadt, inStadt);
END SetzeStadt;

PROCEDURE	LiesLand (inAdresse:TAdresse):TString;
BEGIN
  RETURN String.Duplicate(inAdresse^.Land);
END LiesLand;

PROCEDURE	SetzeLand (inoutAdresse:TAdresse; inLand:TString);
BEGIN
  String.Equalize(inoutAdresse^.Land, inLand);
END SetzeLand;

PROCEDURE	LiesTelefon (inAdresse:TAdresse):TString;
BEGIN
  RETURN String.Duplicate(inAdresse^.Telefon);
END LiesTelefon;

PROCEDURE	SetzeTelefon (inoutAdresse:TAdresse; inTelefon:TString);
BEGIN
  String.Equalize(inoutAdresse^.Telefon, inTelefon);
END SetzeTelefon;

PROCEDURE	LiesFax (inAdresse:TAdresse):TString;
BEGIN
  RETURN String.Duplicate(inAdresse^.Fax);
END LiesFax;

PROCEDURE	SetzeFax (inoutAdresse:TAdresse; inFax:TString);
BEGIN
  String.Equalize(inoutAdresse^.Fax, inFax);
END SetzeFax;

PROCEDURE	LieseMail (inAdresse:TAdresse):TString;
BEGIN
  RETURN String.Duplicate(inAdresse^.Mail);
END LieseMail;

PROCEDURE	SetzeeMail (inoutAdresse:TAdresse; ineMail:TString);
BEGIN
  String.Equalize(inoutAdresse^.Mail, ineMail);
END SetzeeMail;






END Adresse.

