(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*)
   (******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: DAdresse.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:47:58 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 

IMPLEMENTATION MODULE DAdresse   ADT DienstAdresse
                                 --  Enthalten in Hilfskraefte

Sigrid Preuss

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE DAdresse;

(* ADT DienstAdresse *)

(* IMPORTE *)

FROM Storage	IMPORT ALLOCATE,DEALLOCATE;
FROM SYSTEM	IMPORT TSIZE;

IMPORT String, Adresse;

TYPE TString = String.TString;
     TAdresse  = Adresse.TAdresse;


      TFirma   = RECORD
                   FirmenName    : TString;
                   Gebaeude      : TString;
                   Zimmer        : TString;
                 END;


      TDAdresse_DS = RECORD
                       Firma       : TFirma;
                       Adr         : TAdresse;
                     END;

TYPE  TDAdresse = POINTER TO TDAdresse_DS;




(*
SEMANTIK
Die Basis-Operationen Dispose, Duplicate, IsEqual, Equalize
besitzen die uebliche Semantik.

Die Attribut-Operationen
  Lies<Attribut>
  Setze<Attribut>
haben folgende Semantik:

Lies<Attribut>:
Vb:	keine
Liefert ein ADO des entsprechenden Attributs
nb:	Rueckgabe-ADO existiert

Setze<Attribut>:
Vb:	keine
Gleicht den Zustand des entsprechenden Attribut-ADOs in inoutAdresse dem Zustand des
Parameter-Attributs in<Attribut> an.
Nb:	IsEqual(Lies<Attribut>(), in<Attribut>)
*)





(* Basis-Operationen *)

PROCEDURE	Create ():TDAdresse;
(*
Vb:	keine
Generiert ein ADO Adresse.
Die Adresse enthaelt ausschliesslich String-Attribute. Sie werden
mit einem Leerstring initialisiert.

Nb:	Rueckgabe-ADO Adresse existiert
*)
  VAR DienstAdresse : TDAdresse;

BEGIN
  ALLOCATE (DienstAdresse, TSIZE(TDAdresse_DS));

    WITH DienstAdresse^ DO
      WITH Firma DO
        FirmenName := String.Create();
        Gebaeude   := String.Create();
        Zimmer     := String.Create();
      END; (* WITH Firma *)

      Adr := Adresse.Create();

    END; (* WITH DienstAdresse *)

    RETURN DienstAdresse;

END Create;

PROCEDURE	Dispose (VAR inoutDienstAdresse:TDAdresse);

BEGIN
  WITH inoutDienstAdresse^ DO
      WITH Firma DO
        String.Dispose(FirmenName);
        String.Dispose(Gebaeude);
        String.Dispose(Zimmer);
      END; (* WITH Firma *)

      Adresse.Dispose(Adr);

  END; (* WITH inoutDienstAdresse^ *)

  DEALLOCATE (inoutDienstAdresse, TSIZE(TDAdresse_DS));
  inoutDienstAdresse := NIL;

END Dispose;



PROCEDURE	Duplicate (inDienstAdresse:TDAdresse):TDAdresse;

  VAR Duplikat : TDAdresse;

BEGIN
  Duplikat := Create();
  Equalize(Duplikat,inDienstAdresse);
  RETURN Duplikat;
END Duplicate;


PROCEDURE	IsEqual (inDienstAdresse1,inDienstAdresse2:TDAdresse):BOOLEAN;

  VAR gleicheAttribute : BOOLEAN;

BEGIN
  gleicheAttribute := 
        String.IsEqual(inDienstAdresse1^.Firma.FirmenName,
                       inDienstAdresse2^.Firma.FirmenName)      AND
        String.IsEqual(inDienstAdresse1^.Firma.Gebaeude,
                       inDienstAdresse2^.Firma.Gebaeude)        AND
        String.IsEqual(inDienstAdresse1^.Firma.Zimmer,
                       inDienstAdresse2^.Firma.Zimmer)          AND
        Adresse.IsEqual(inDienstAdresse1^.Adr,
                       inDienstAdresse2^.Adr);

  RETURN gleicheAttribute;

END IsEqual;



PROCEDURE	Equalize (inoutZielDienstAdresse:TDAdresse;
                          inQuelleDienstAdresse:TDAdresse);
BEGIN
  WITH inoutZielDienstAdresse^ DO
    WITH Firma DO
      String.Equalize(FirmenName,
                      inQuelleDienstAdresse^.Firma.FirmenName);
      String.Equalize(Gebaeude, inQuelleDienstAdresse^.Firma.Gebaeude);
      String.Equalize(Zimmer, inQuelleDienstAdresse^.Firma.Zimmer);
    END; (* WITH Firma *)

    Adresse.Equalize (Adr, inQuelleDienstAdresse^.Adr);

  END; (* WITH inoutZielDienstAdresse^ *)

END Equalize;


(* Operationen auf den Attributen von DienstAdresse *)

PROCEDURE	LiesDienstName (inDienstAdresse:TDAdresse):TString;
BEGIN
  RETURN String.Duplicate(inDienstAdresse^.Firma.FirmenName);
END LiesDienstName;

PROCEDURE	SetzeDienstName (inoutDienstAdresse:TDAdresse;
                                 inDienstName:TString);
BEGIN
  String.Equalize(inoutDienstAdresse^.Firma.FirmenName, inDienstName);
END SetzeDienstName;

PROCEDURE	LiesDienstGebaeude (inDienstAdresse:TDAdresse):TString;
BEGIN
  RETURN String.Duplicate(inDienstAdresse^.Firma.Gebaeude);
END LiesDienstGebaeude;

PROCEDURE	SetzeDienstGebaeude (inoutDienstAdresse:TDAdresse;
                                     inGebaeude:TString);
BEGIN
  String.Equalize(inoutDienstAdresse^.Firma.Gebaeude, inGebaeude);
END SetzeDienstGebaeude;

PROCEDURE	LiesDienstZimmer (inDienstAdresse:TDAdresse):TString;
BEGIN
  RETURN String.Duplicate(inDienstAdresse^.Firma.Zimmer);
END LiesDienstZimmer;

PROCEDURE	SetzeDienstZimmer(inoutDienstAdresse:TDAdresse;
                                  inZimmer:TString);
BEGIN
  String.Equalize(inoutDienstAdresse^.Firma.Zimmer, inZimmer);
END SetzeDienstZimmer;

PROCEDURE	LiesDienstStr(inDienstAdresse:TDAdresse):TString;
BEGIN
  RETURN Adresse.LiesStrasse(inDienstAdresse^.Adr);
END LiesDienstStr;

PROCEDURE	SetzeDienstStr (inoutDienstAdresse:TDAdresse;
                                inStrasse:TString);
BEGIN
  Adresse.SetzeStrasse(inoutDienstAdresse^.Adr, inStrasse);
END SetzeDienstStr;

PROCEDURE	LiesDienstPlz (inDienstAdresse:TDAdresse):TString;
BEGIN
  RETURN Adresse.LiesPostleitzahl(inDienstAdresse^.Adr);
END LiesDienstPlz;

PROCEDURE	SetzeDienstPlz (inoutDienstAdresse:TDAdresse; inPLZ:TString);
BEGIN
  Adresse.SetzePostleitzahl(inoutDienstAdresse^.Adr, inPLZ);
END SetzeDienstPlz;

PROCEDURE	LiesDienstStadt (inDienstAdresse:TDAdresse):TString;
BEGIN
  RETURN Adresse.LiesStadt(inDienstAdresse^.Adr);
END LiesDienstStadt;

PROCEDURE	SetzeDienstStadt (inoutDienstAdresse:TDAdresse;
                                  inStadt:TString);
BEGIN
  Adresse.SetzeStadt(inoutDienstAdresse^.Adr, inStadt);
END SetzeDienstStadt;

PROCEDURE	LiesDienstLand (inDienstAdresse:TDAdresse):TString;
BEGIN
  RETURN Adresse.LiesLand(inDienstAdresse^.Adr);
END LiesDienstLand;

PROCEDURE	SetzeDienstLand (inoutDienstAdresse:TDAdresse; inLand:TString);
BEGIN
  Adresse.SetzeLand(inoutDienstAdresse^.Adr, inLand);
END SetzeDienstLand;

PROCEDURE	LiesDienstTel (inDienstAdresse:TDAdresse):TString;
BEGIN
  RETURN Adresse.LiesTelefon(inDienstAdresse^.Adr);
END LiesDienstTel;

PROCEDURE	SetzeDienstTel (inoutDienstAdresse:TDAdresse;
                                inTelefon:TString);
BEGIN
  Adresse.SetzeTelefon(inoutDienstAdresse^.Adr, inTelefon);
END SetzeDienstTel;

PROCEDURE	LiesDienstFax (inDienstAdresse:TDAdresse):TString;
BEGIN
  RETURN Adresse.LiesFax(inDienstAdresse^.Adr);
END LiesDienstFax;

PROCEDURE	SetzeDienstFax (inoutDienstAdresse:TDAdresse; inFax:TString);
BEGIN
  Adresse.SetzeFax(inoutDienstAdresse^.Adr, inFax);
END SetzeDienstFax;

PROCEDURE	LiesDiensteMail (inDienstAdresse:TDAdresse):TString;
BEGIN
  RETURN Adresse.LieseMail(inDienstAdresse^.Adr);
END LiesDiensteMail;

PROCEDURE	SetzeDiensteMail (inoutDienstAdresse:TDAdresse;
                                  ineMail:TString);
BEGIN
  Adresse.SetzeeMail(inoutDienstAdresse^.Adr, ineMail);
END SetzeDiensteMail;



END DAdresse.

