(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*)
   (******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: HK.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:47:58 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 

IMPLEMENTATION MODULE HK   ADT Hilfskraft
                                        --  Enthalten in Hilfskraefte

Sigrid Preuss

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE HK;

(* ADT Hilfskraft *)

(* IMPORTE *)

FROM Storage	IMPORT ALLOCATE,DEALLOCATE;
FROM SYSTEM	IMPORT TSIZE;
FROM GENERIC	IMPORT TObjektreferenz;
FROM WholeStr   IMPORT ConvResults, StrToCard, CardToStr;
FROM Datum      IMPORT TDatum, CreateDatum, DisposeDatum,
                       DuplicateDatum, EqualizeDatum, IsEqualDatum,
                       DatumAlsString, DatumVonString;
FROM HKeIMPORTE IMPORT TPersID, DisposePersID, DuplicatePersID, IsEqualPersID,
                       EqualizePersID, SamePersID, SmallerPersID,
                       PruefePersID, PersIDAlsString, StringInPersID;

IMPORT String, Stringliste, DAdresse, Adresse, Wochentage;

CONST LaengePersIDNR = 5;
      INITVALUECARDINAL = 99999999;

TYPE TString      = String.TString;
     TStringliste = Stringliste.TStringliste;
     TAdresse     = Adresse.TAdresse;
     TDAdresse    = DAdresse.TDAdresse;  (* DienstAdresse *)
     TPraesTage   = Wochentage.TWochentage;


(* Zusammgehoerende Attribute werden in RECORDS gestellt *)
TYPE  TExistenz = RECORD
                    Geburtsdatum : TDatum;
                    Geburtsort   : TString;
                    Staatsang    : TString;
                    Geschlecht   : CHAR;
                    Famstand     : CHAR;
                    Kinder       : CARDINAL;
                  END;

      TBank    = RECORD
                   BankinstOrt   : TString;
                   Blz           : CARDINAL;
                   Kontonr       : TString;
                 END;



      TVertragsAngaben    = RECORD
                              VorStdzahl  : CARDINAL;
                              VerfuegAb   : TDatum;
                              PraesTage   : TPraesTage;
                              Abschluss   : TString;
                              Interessen  : TString;
                              Bemerkungen : TString;

                            END;

      THK_DS = RECORD
                 Identifikator     : RECORD
                                       ID : TPersID;
                                       existiert : BOOLEAN;
                                     END;
                 Name            : TString;
                 Vorname         : TString;
                 Personalnr      : CARDINAL;
                 Existenz        : TExistenz;
                 PrivatAdresse   : TAdresse;
                 DienstAdresse   : TDAdresse;
                 Bankverbindung  : TBank;
                 VertragsAngaben : TVertragsAngaben;
               END;

TYPE  THK = POINTER TO THK_DS;


(*
SEMANTIK
Die Basis-Operationen Dispose, Duplicate, IsEqual, Equalize, SmallerHK,
SameHK besitzen die uebliche Semantik.

Die Attribut-Operationen
  Lies<Attribut>
  Setze<Attribut>
haben folgende Semantik:

Lies<Attribut>:
Vb:     keine
Liefert ein ADO des entsprechenden Attributs
nb:     Rueckgabe-ADO existiert

Setze<Attribut>:
vb:     keine
Gleicht den Zustand des entsprechenden Attribut-ADOs in inoutHK dem Zustand des
Parameter-Attributs in<Attribut> an.
Nb:     IsEqual(Lies<Attribut>(), in<Attribut>)
*)



(* Typ-Wandlung *)

PROCEDURE       HK (inRef : TObjektreferenz) : THK;
(*
Vb:     inRef referenziert ein Hilfskraft-Objekt
Typtransfer von TObjektreferenz nach THK
Nb:     inRef = OBJEKTREFERENZ(Rueckgabe-HK)
*)

  VAR HK : THK;

BEGIN
    HK := inRef;
    RETURN HK;
END HK;

PROCEDURE       OBJEKTREFERENZ (inHK : THK) : TObjektreferenz;
(*
Vb:     keine
Typtransfer von THK nach TObjektreferenz
Nb:     inHK = HK(Rueckgabe-Objektreferenz)
*)

  VAR HKRef : TObjektreferenz;

BEGIN
    HKRef := inHK;
    RETURN HKRef;
END OBJEKTREFERENZ;



(* Basis-Operationen *)


PROCEDURE       Create () : THK;
(*
Vb:     keine
Generiert ein ADO HK.
Die Attribute dieses ADOs werden folgendermassen initialisert:
String-Attribut:   Leerstring
CHAR-Attribut:     Blank
CARDINAL-Attribut: 999999
Datum-Attribut:    Eternity (31.12.9999)
Praesenztage:      Leere Menge

Nb:     Rueckgabe-ADO HK existiert
*)

  VAR HK : THK;

BEGIN
  ALLOCATE (HK, TSIZE(THK_DS));

  WITH HK^ DO
    Identifikator.existiert := FALSE;
    Name       := String.Create();
    Vorname    := String.Create();
    Personalnr := INITVALUECARDINAL;

    WITH Existenz DO
      Geburtsdatum := CreateDatum();
      Geburtsort   := String.Create();
      Staatsang    := String.Create();
      Geschlecht   := ' ';
      Famstand     := ' ';
      Kinder       := INITVALUECARDINAL;
    END; (* WITH Existenz *)


    PrivatAdresse := Adresse.Create(); 
    DienstAdresse := DAdresse.Create(); 


    WITH Bankverbindung DO
       BankinstOrt  := String.Create();
       Blz          := INITVALUECARDINAL;
       Kontonr      := String.Create();
    END; (* WITH Bankverbindung *)

    WITH VertragsAngaben DO
      VorStdzahl   := INITVALUECARDINAL;
      VerfuegAb    := CreateDatum();
      PraesTage    := Wochentage.Create();
      Abschluss    := String.Create();
      Interessen   := String.Create();
      Bemerkungen  := String.Create();
    END; (* WITH VertragsAngaben *)

  END; (* WITH HK^ *)

  RETURN HK;

END Create;

PROCEDURE       Dispose (VAR inoutHK : THK);

BEGIN
  WITH inoutHK^ DO
    IF Identifikator.existiert THEN
      DisposePersID(Identifikator.ID);
    END; (* IF *)
    String.Dispose(Name);
    String.Dispose(Vorname);

    WITH Existenz DO
      DisposeDatum(Geburtsdatum);
      String.Dispose(Geburtsort);
      String.Dispose(Staatsang);
    END; (* WITH Existenz *)

    Adresse.Dispose(PrivatAdresse);
    DAdresse.Dispose(DienstAdresse);

    WITH Bankverbindung DO
       String.Dispose(BankinstOrt);
       String.Dispose(Kontonr);
    END; (* WITH Bankverbindung *)

    WITH VertragsAngaben DO
      DisposeDatum(VerfuegAb);
      Wochentage.Dispose(PraesTage);
      String.Dispose(Abschluss);
      String.Dispose(Interessen);
      String.Dispose(Bemerkungen);

    END; (* WITH Bankverbindung *)

  END; (* WITH inoutHK^ *)

  DEALLOCATE (inoutHK, TSIZE(THK_DS));
  inoutHK := NIL;

END Dispose;

PROCEDURE       Duplicate (inHK : THK) : THK;

  VAR Duplikat : THK;

BEGIN
  Duplikat := Create();
  Equalize(Duplikat,inHK);
  RETURN Duplikat;
END Duplicate;

PROCEDURE       IsEqual (inHK1, inHK2 : THK) : BOOLEAN;

  VAR gleicheAttribute : BOOLEAN;
      gleicheID        : BOOLEAN;

BEGIN

(* PersID *)
    IF inHK1^.Identifikator.existiert AND inHK2^.Identifikator.existiert THEN
      gleicheID := SameHK (inHK1, inHK2)
    ELSIF inHK1^.Identifikator.existiert OR inHK2^.Identifikator.existiert THEN
      gleicheID := FALSE
    ELSE
      gleicheID := TRUE
    END; (* IF *)
    IF gleicheID THEN
      gleicheAttribute := 
(* Name, Personalnr *)
        String.IsEqual(inHK1^.Name, inHK2^.Name)        AND
        String.IsEqual(inHK1^.Vorname, inHK2^.Vorname)  AND
        (inHK1^.Personalnr = inHK2^.Personalnr)         AND
(* Existenz *)
        IsEqualDatum(inHK1^.Existenz.Geburtsdatum,
                     inHK2^.Existenz.Geburtsdatum)    AND
        String.IsEqual(inHK1^.Existenz.Geburtsort,
                       inHK2^.Existenz.Geburtsort)    AND
        String.IsEqual(inHK1^.Existenz.Staatsang,
                       inHK2^.Existenz.Staatsang)     AND
        (inHK1^.Existenz.Geschlecht = inHK2^.Existenz.Geschlecht) AND
        (inHK1^.Existenz.Famstand = inHK2^.Existenz.Famstand)     AND
        (inHK1^.Existenz.Kinder = inHK2^.Existenz.Kinder)         AND
(* Privatadresse *)
        Adresse.IsEqual(inHK1^.PrivatAdresse,
                        inHK2^.PrivatAdresse)         AND
(* Dienstadresse *)
        DAdresse.IsEqual(inHK1^.DienstAdresse,
                         inHK2^.DienstAdresse)         AND
(* Bankverbindung *)
        String.IsEqual(inHK1^.Bankverbindung.BankinstOrt,
                       inHK2^.Bankverbindung.BankinstOrt)               AND
        (inHK1^.Bankverbindung.Blz = inHK2^.Bankverbindung.Blz)         AND
        String.IsEqual(inHK1^.Bankverbindung.Kontonr,
                       inHK2^.Bankverbindung.Kontonr)                   AND
(* Vertragsangaben *)
        (inHK1^.VertragsAngaben.VorStdzahl =
                                inHK2^.VertragsAngaben.VorStdzahl) AND
        IsEqualDatum(inHK1^.VertragsAngaben.VerfuegAb,
                       inHK2^.VertragsAngaben.VerfuegAb)           AND
        Wochentage.IsEqual(inHK1^.VertragsAngaben.PraesTage,
                       inHK2^.VertragsAngaben.PraesTage)           AND
        String.IsEqual(inHK1^.VertragsAngaben.Abschluss,
                       inHK2^.VertragsAngaben.Abschluss)           AND
        String.IsEqual(inHK1^.VertragsAngaben.Interessen,
                       inHK2^.VertragsAngaben.Interessen)          AND
        String.IsEqual(inHK1^.VertragsAngaben.Bemerkungen,
                       inHK2^.VertragsAngaben.Bemerkungen);


    ELSE
      gleicheAttribute := FALSE
    END; (* IF gleicheID *)

    RETURN gleicheAttribute;

END IsEqual;


PROCEDURE       Equalize (inoutZielHK : THK; inQuelleHK : THK);
BEGIN
  WITH inoutZielHK^ DO
    WITH Identifikator DO
      IF existiert AND inQuelleHK^.Identifikator.existiert THEN
        EqualizePersID (ID, inQuelleHK^.Identifikator.ID)
      ELSIF existiert THEN
        (* -- inQuelleHK^.Identifikator.existiert = FALSE *)
        existiert := FALSE;
        DisposePersID (ID)
      ELSIF inQuelleHK^.Identifikator.existiert THEN
        (* -- inoutZielHK^.Identifikator.existiert = FALSE *)
        ID := DuplicatePersID (inQuelleHK^.Identifikator.ID);
        existiert := TRUE
      ELSE 
        (* -- keine der beiden IDs existiert, es ist nichts zu tun *)
      END (* IF *)
    END; (* WITH *)

    String.Equalize (Name, inQuelleHK^.Name);
    String.Equalize (Vorname, inQuelleHK^.Vorname);
    Personalnr := inQuelleHK^.Personalnr;

    WITH Existenz DO
      EqualizeDatum(Geburtsdatum,inQuelleHK^.Existenz.Geburtsdatum);
      String.Equalize(Geburtsort, inQuelleHK^.Existenz.Geburtsort);
      String.Equalize(Staatsang, inQuelleHK^.Existenz.Staatsang);
      Geschlecht := inQuelleHK^.Existenz.Geschlecht;
      Famstand   := inQuelleHK^.Existenz.Famstand;
      Kinder     := inQuelleHK^.Existenz.Kinder;
    END; (* WITH Existenz *)

    Adresse.Equalize (PrivatAdresse, inQuelleHK^.PrivatAdresse);
    DAdresse.Equalize (DienstAdresse, inQuelleHK^.DienstAdresse);

    WITH Bankverbindung DO
      String.Equalize (BankinstOrt, inQuelleHK^.Bankverbindung.BankinstOrt);
      Blz := inQuelleHK^.Bankverbindung.Blz;
      String.Equalize (Kontonr, inQuelleHK^.Bankverbindung.Kontonr);
    END; (* WITH Bankverbindung *)

 
    WITH VertragsAngaben DO
      VorStdzahl := inQuelleHK^.VertragsAngaben.VorStdzahl;
      EqualizeDatum (VerfuegAb, inQuelleHK^.VertragsAngaben.VerfuegAb);
      Wochentage.Equalize (PraesTage, inQuelleHK^.VertragsAngaben.PraesTage);
      String.Equalize (Abschluss, inQuelleHK^.VertragsAngaben.Abschluss);
      String.Equalize (Interessen, inQuelleHK^.VertragsAngaben.Interessen);
      String.Equalize (Bemerkungen, inQuelleHK^.VertragsAngaben.Bemerkungen);

    END; (* WITH VertragsAngaben *)


  END; (* WITH inoutZielHK^ *)

END Equalize;



PROCEDURE       SmallerHK (inHK1,inHK2 : THK) : BOOLEAN;
BEGIN
  RETURN SmallerPersID(inHK1^.Identifikator.ID, inHK2^.Identifikator.ID);
END SmallerHK;


PROCEDURE       SameHK (inHK1,inHK2 : THK) : BOOLEAN;
BEGIN
  RETURN SamePersID(inHK1^.Identifikator.ID,
                    inHK2^.Identifikator.ID);
END SameHK;



(* PersID-Operationen *)


PROCEDURE       LiesPersID (inHK : THK) : TPersID;
(*
Vb:     ExistiertPersID()
Liefert den Identifikator PersID der Hilfskraft inHK
Nb:     RueckgabeADO PersID existiert
*)

BEGIN
  RETURN DuplicatePersID(inHK^.Identifikator.ID);
END LiesPersID;


PROCEDURE       SetzePersID (inHK : THK; inPersID : TPersID);
(*
Vb:     NOT ExistiertPersID()
Versetzt die Hilfskraft inHK mit dem Identifikator PersID
Nb:     ExistiertPersID()
*)

BEGIN

  WITH inHK^.Identifikator DO
    ID := DuplicatePersID(inPersID);
    existiert := TRUE;
  END (* WITH *)

END SetzePersID;

PROCEDURE       ExistiertPersID (inHK : THK) : BOOLEAN;
(*
Vb:     keine
Prueft, ob die Hilfskraft bereits mit einer PersID versehen wurde
Nb:     Rueckgabewert = TRUE <=> PersID existiert
*)

BEGIN
  RETURN inHK^.Identifikator.existiert;
END ExistiertPersID;



(* PraesTage-Operationen *)

PROCEDURE       DisposePraesTage (VAR inoutPraesTage : TPraesTage);

BEGIN
  Wochentage.Dispose(inoutPraesTage);
END DisposePraesTage;


PROCEDURE       DuplicatePraesTage (inPraesTage : TPraesTage) : TPraesTage;
BEGIN
  RETURN Wochentage.Duplicate(inPraesTage);
END DuplicatePraesTage;


PROCEDURE       IsEqualPraesTage (inPraesTage1 : TPraesTage;
                                  inPraesTage2 : TPraesTage) : BOOLEAN;
BEGIN
  RETURN Wochentage.IsEqual(inPraesTage1,inPraesTage2)
END IsEqualPraesTage;


PROCEDURE       EqualizePraesTage (inoutZielPraesTage : TPraesTage;
                                   inQuellPraesTage   : TPraesTage);
BEGIN
  Wochentage.Equalize(inoutZielPraesTage, inQuellPraesTage);
END EqualizePraesTage;


PROCEDURE       PruefePraesTage (inStringTage : TString;
                                 VAR outMessage : TString) : BOOLEAN;
BEGIN
  RETURN Wochentage.PruefeWochentage(inStringTage, outMessage);
END PruefePraesTage;

PROCEDURE       PraesTageAlsString (inPraesTage : TPraesTage) : TString;
BEGIN
  RETURN Wochentage.WochentageAlsString(inPraesTage);
END PraesTageAlsString;

PROCEDURE       StringInPraesTage (inStringTage : TString) : TPraesTage;
BEGIN
  RETURN Wochentage.StringInWochentage(inStringTage);
END StringInPraesTage;





(******** Operationen auf den Attributen einer Hilfskraft **********)


(* Name, Personalnr *) 

PROCEDURE       LiesName (inHK : THK) : TString;
BEGIN
  RETURN String.Duplicate(inHK^.Name);
END LiesName;

PROCEDURE       SetzeName (inoutHK : THK; inName : TString);
BEGIN
  String.Equalize(inoutHK^.Name, inName);
END SetzeName;

PROCEDURE       LiesVorname (inHK : THK) : TString;
BEGIN
  RETURN String.Duplicate(inHK^.Vorname);
END LiesVorname;

PROCEDURE       SetzeVorname (inoutHK : THK; inVorname : TString);
BEGIN
  String.Equalize(inoutHK^.Vorname, inVorname);
END SetzeVorname;

PROCEDURE       LiesPersonalnr (inHK : THK) : CARDINAL;
BEGIN
  RETURN inHK^.Personalnr;
END LiesPersonalnr;

PROCEDURE       SetzePersonalnr (inoutHK : THK;
                                 inPersonalnummer : CARDINAL);
BEGIN
  inoutHK^.Personalnr := inPersonalnummer;
END SetzePersonalnr;


(* Existenz-Angaben *)

PROCEDURE       LiesGeburtsdatum (inHK : THK) : TDatum;
BEGIN
  RETURN DuplicateDatum(inHK^.Existenz.Geburtsdatum);
END LiesGeburtsdatum;

PROCEDURE       SetzeGeburtsdatum (inoutHK : THK; inGeburtsdatum : TDatum);
BEGIN
  EqualizeDatum(inoutHK^.Existenz.Geburtsdatum, inGeburtsdatum);
END SetzeGeburtsdatum;

PROCEDURE       LiesGeburtsort (inHK : THK) : TString;
BEGIN
  RETURN String.Duplicate(inHK^.Existenz.Geburtsort);
END LiesGeburtsort;

PROCEDURE       SetzeGeburtsort (inoutHK : THK; inGeburtsort : TString);
BEGIN
  String.Equalize(inoutHK^.Existenz.Geburtsort, inGeburtsort);
END SetzeGeburtsort;

PROCEDURE       LiesStaatsang (inHK : THK) : TString;
BEGIN
  RETURN String.Duplicate(inHK^.Existenz.Staatsang);
END LiesStaatsang;

PROCEDURE       SetzeStaatsang (inoutHK : THK; inStaatsangkeit : TString);
BEGIN
  String.Equalize(inoutHK^.Existenz.Staatsang, inStaatsangkeit);
END SetzeStaatsang;

PROCEDURE       LiesGeschlecht (inHK : THK) : CHAR;
BEGIN
  RETURN inHK^.Existenz.Geschlecht;
END LiesGeschlecht;

PROCEDURE       SetzeGeschlecht (inoutHK : THK; inGeschlecht:CHAR);
BEGIN
  inoutHK^.Existenz.Geschlecht := inGeschlecht;
END SetzeGeschlecht;

PROCEDURE       LiesFamstand (inHK : THK) : CHAR;
BEGIN
  RETURN inHK^.Existenz.Famstand;
END LiesFamstand;

PROCEDURE       SetzeFamstand (inoutHK : THK; inFamilienstand:CHAR);
BEGIN
  inoutHK^.Existenz.Famstand := inFamilienstand;
END SetzeFamstand;

PROCEDURE       LiesKinder (inHK : THK) : CARDINAL;
BEGIN
  RETURN inHK^.Existenz.Kinder;
END LiesKinder;

PROCEDURE       SetzeKinder (inoutHK : THK; inKinder : CARDINAL);
BEGIN
  inoutHK^.Existenz.Kinder := inKinder;
END SetzeKinder;



(* Privat-Adresse *)

PROCEDURE       LiesStrasse (inHK : THK) : TString;
BEGIN
  RETURN Adresse.LiesStrasse(inHK^.PrivatAdresse);
END LiesStrasse;

PROCEDURE       SetzeStrasse (inoutHK : THK; inStrasse : TString);
BEGIN
  Adresse.SetzeStrasse(inoutHK^.PrivatAdresse, inStrasse);
END SetzeStrasse;

PROCEDURE       LiesPostleitzahl (inHK : THK) : TString;
BEGIN
  RETURN Adresse.LiesPostleitzahl(inHK^.PrivatAdresse);
END LiesPostleitzahl;

PROCEDURE       SetzePostleitzahl (inoutHK : THK; inPLZ : TString);
BEGIN
  Adresse.SetzePostleitzahl(inoutHK^.PrivatAdresse, inPLZ);
END SetzePostleitzahl;

PROCEDURE       LiesStadt (inHK : THK) : TString;
BEGIN
  RETURN Adresse.LiesStadt(inHK^.PrivatAdresse);
END LiesStadt;

PROCEDURE       SetzeStadt (inoutHK : THK; inStadt : TString);
BEGIN
  Adresse.SetzeStadt(inoutHK^.PrivatAdresse, inStadt);
END SetzeStadt;

PROCEDURE       LiesLand (inHK : THK) : TString;
BEGIN
  RETURN Adresse.LiesLand(inHK^.PrivatAdresse);
END LiesLand;

PROCEDURE       SetzeLand (inoutHK : THK; inLand : TString);
BEGIN
  Adresse.SetzeLand(inoutHK^.PrivatAdresse, inLand);
END SetzeLand;

PROCEDURE       LiesTelefon (inHK : THK) : TString;
BEGIN
  RETURN Adresse.LiesTelefon(inHK^.PrivatAdresse);
END LiesTelefon;

PROCEDURE       SetzeTelefon (inoutHK : THK; inTelefon : TString);
BEGIN
  Adresse.SetzeTelefon(inoutHK^.PrivatAdresse, inTelefon);
END SetzeTelefon;

PROCEDURE       LiesFax (inHK : THK) : TString;
BEGIN
  RETURN Adresse.LiesFax(inHK^.PrivatAdresse);
END LiesFax;

PROCEDURE       SetzeFax (inoutHK : THK; inFax : TString);
BEGIN
  Adresse.SetzeFax(inoutHK^.PrivatAdresse, inFax);
END SetzeFax;

PROCEDURE       LieseMail (inHK : THK) : TString;
BEGIN
  RETURN Adresse.LieseMail(inHK^.PrivatAdresse);
END LieseMail;

PROCEDURE       SetzeeMail (inoutHK : THK; ineMail : TString);
BEGIN
  Adresse.SetzeeMail(inoutHK^.PrivatAdresse, ineMail);
END SetzeeMail;



(* Dienst-Adresse *)

PROCEDURE       LiesDienstName (inHK : THK) : TString;
BEGIN
  RETURN DAdresse.LiesDienstName(inHK^.DienstAdresse);
END LiesDienstName;

PROCEDURE       SetzeDienstName (inoutHK : THK; inDienstName : TString);
BEGIN
  DAdresse.SetzeDienstName(inoutHK^.DienstAdresse, inDienstName);
END SetzeDienstName;

PROCEDURE       LiesDienstGebaeude (inHK : THK) : TString;
BEGIN
  RETURN DAdresse.LiesDienstGebaeude(inHK^.DienstAdresse);
END LiesDienstGebaeude;

PROCEDURE       SetzeDienstGebaeude (inoutHK : THK; inGebaeude : TString);
BEGIN
  DAdresse.SetzeDienstGebaeude(inoutHK^.DienstAdresse, inGebaeude);
END SetzeDienstGebaeude;

PROCEDURE       LiesDienstZimmer (inHK : THK) : TString;
BEGIN
  RETURN DAdresse.LiesDienstZimmer(inHK^.DienstAdresse);
END LiesDienstZimmer;

PROCEDURE       SetzeDienstZimmer(inoutHK : THK; inZimmer : TString);
BEGIN
  DAdresse.SetzeDienstZimmer(inoutHK^.DienstAdresse, inZimmer);
END SetzeDienstZimmer;

PROCEDURE       LiesDienstStr (inHK : THK) : TString;
BEGIN
  RETURN DAdresse.LiesDienstStr(inHK^.DienstAdresse);
END LiesDienstStr;

PROCEDURE       SetzeDienstStr (inoutHK : THK; inStrasse : TString);
BEGIN
  DAdresse.SetzeDienstStr(inoutHK^.DienstAdresse, inStrasse);
END SetzeDienstStr;

PROCEDURE       LiesDienstPlz (inHK : THK) : TString;
BEGIN
  RETURN DAdresse.LiesDienstPlz(inHK^.DienstAdresse);
END LiesDienstPlz;

PROCEDURE       SetzeDienstPlz(inoutHK : THK; inPLZ : TString);
BEGIN
  DAdresse.SetzeDienstPlz(inoutHK^.DienstAdresse, inPLZ);
END SetzeDienstPlz;

PROCEDURE       LiesDienstStadt (inHK : THK) : TString;
BEGIN
  RETURN DAdresse.LiesDienstStadt(inHK^.DienstAdresse);
END LiesDienstStadt;

PROCEDURE       SetzeDienstStadt (inoutHK : THK; inStadt : TString);
BEGIN
  DAdresse.SetzeDienstStadt(inoutHK^.DienstAdresse, inStadt);
END SetzeDienstStadt;

PROCEDURE       LiesDienstLand (inHK : THK) : TString;
BEGIN
  RETURN DAdresse.LiesDienstLand(inHK^.DienstAdresse);
END LiesDienstLand;

PROCEDURE       SetzeDienstLand (inoutHK : THK; inLand : TString);
BEGIN
  DAdresse.SetzeDienstLand(inoutHK^.DienstAdresse, inLand);
END SetzeDienstLand;

PROCEDURE       LiesDienstTel (inHK : THK) : TString;
BEGIN
  RETURN DAdresse.LiesDienstTel(inHK^.DienstAdresse);
END LiesDienstTel;

PROCEDURE       SetzeDienstTel (inoutHK : THK; inTelefon : TString);
BEGIN
  DAdresse.SetzeDienstTel(inoutHK^.DienstAdresse, inTelefon);
END SetzeDienstTel;

PROCEDURE       LiesDienstFax (inHK : THK) : TString;
BEGIN
  RETURN DAdresse.LiesDienstFax(inHK^.DienstAdresse);
END LiesDienstFax;

PROCEDURE       SetzeDienstFax (inoutHK : THK; inFax : TString);
BEGIN
  DAdresse.SetzeDienstFax(inoutHK^.DienstAdresse, inFax);
END SetzeDienstFax;

PROCEDURE       LiesDiensteMail (inHK : THK) : TString;
BEGIN
  RETURN DAdresse.LiesDiensteMail(inHK^.DienstAdresse);
END LiesDiensteMail;

PROCEDURE       SetzeDiensteMail (inoutHK : THK; ineMail : TString);
BEGIN
  DAdresse.SetzeDiensteMail(inoutHK^.DienstAdresse, ineMail);
END SetzeDiensteMail;



(* Bank-Verbindung *)

PROCEDURE       LiesBankinstOrt (inHK : THK) : TString; 
BEGIN
  RETURN String.Duplicate(inHK^.Bankverbindung.BankinstOrt);
END LiesBankinstOrt;

PROCEDURE       SetzeBankinstOrt (inoutHK : THK; inBankinstitut : TString);
BEGIN
  String.Equalize(inoutHK^.Bankverbindung.BankinstOrt, inBankinstitut);
END SetzeBankinstOrt;

PROCEDURE       LiesBlz (inHK : THK) : CARDINAL;
BEGIN
  RETURN inHK^.Bankverbindung.Blz;
END LiesBlz;

PROCEDURE       SetzeBlz (inoutHK : THK; inBankleitzahl : CARDINAL);
BEGIN
  inoutHK^.Bankverbindung.Blz := inBankleitzahl;
END SetzeBlz;

PROCEDURE       LiesKontonr (inHK : THK) : TString;
BEGIN
  RETURN String.Duplicate(inHK^.Bankverbindung.Kontonr);
END LiesKontonr;

PROCEDURE       SetzeKontonr (inoutHK : THK; inKontonummer : TString);
BEGIN
  String.Equalize(inoutHK^.Bankverbindung.Kontonr, inKontonummer);
END SetzeKontonr;


(* Vertrags-Angaben *)

PROCEDURE       LiesVorStdzahl (inHK : THK) : CARDINAL;
BEGIN
  RETURN inHK^.VertragsAngaben.VorStdzahl;
END LiesVorStdzahl;

PROCEDURE       SetzeVorStdzahl (inoutHK : THK; inVorStdzahl : CARDINAL);
BEGIN
  inoutHK^.VertragsAngaben.VorStdzahl := inVorStdzahl;
END SetzeVorStdzahl;

PROCEDURE       LiesVerfuegAb (inHK : THK) : TDatum;
BEGIN
  RETURN DuplicateDatum(inHK^.VertragsAngaben.VerfuegAb);
END LiesVerfuegAb;

PROCEDURE       	SetzeVerfuegAb (inoutHK : THK; inVerfuegbarAb : TDatum);
BEGIN
  EqualizeDatum(inoutHK^.VertragsAngaben.VerfuegAb, inVerfuegbarAb);
END SetzeVerfuegAb;


PROCEDURE       LiesPraestage (inHK : THK) : TPraesTage;
BEGIN
  RETURN Wochentage.Duplicate(inHK^.VertragsAngaben.PraesTage);
END LiesPraestage;

PROCEDURE       SetzePraestage (inoutHK : THK; inPraesenztage : TPraesTage);
BEGIN
  Wochentage.Equalize(inoutHK^.VertragsAngaben.PraesTage, inPraesenztage);
END SetzePraestage;

PROCEDURE       LiesAbschluss (inHK : THK) : TString;
BEGIN
  RETURN String.Duplicate(inHK^.VertragsAngaben.Abschluss);
END LiesAbschluss;

PROCEDURE       SetzeAbschluss (inoutHK : THK; inAbschluss : TString);
BEGIN
  String.Equalize(inoutHK^.VertragsAngaben.Abschluss, inAbschluss);
END SetzeAbschluss;

PROCEDURE       LiesInteressen (inHK : THK) : TString;
BEGIN
  RETURN String.Duplicate(inHK^.VertragsAngaben.Interessen);
END LiesInteressen;

PROCEDURE       SetzeInteressen (inoutHK : THK; inInteresse : TString);
BEGIN
  String.Equalize(inoutHK^.VertragsAngaben.Interessen, inInteresse);
END SetzeInteressen;

PROCEDURE       LiesBemerkungen (inHK : THK) : TString;
BEGIN
  RETURN String.Duplicate(inHK^.VertragsAngaben.Bemerkungen);
END LiesBemerkungen;

PROCEDURE       SetzeBemerkungen (inoutHK : THK; inBemerkungen : TString);
BEGIN
  String.Equalize(inoutHK^.VertragsAngaben.Bemerkungen, inBemerkungen);
END SetzeBemerkungen;



(* Operationen zur persistenten Speicherung *)


PROCEDURE       CreateStringlisteFromObject (inHK : THK) : TStringliste;
(*
Vb:    keine
Saemtliche Attribute des ADOs inHK werden in Strings konvertiert und
in eine Stringliste eingefuegt.
Nb:    Rueckgabe-ADO Stringliste existiert  AND
       CreateObjectFromStringliste(Rueckgabe-ADO Stringliste) = inHK
*)

  VAR HKStringliste : TStringliste;
      lfdString     : TString;
      zwString      : TString; (* String zum Zwischenspeichern *)
      strArray      : ARRAY [0..255] OF CHAR;
      OneCharString : ARRAY [0..0] OF CHAR;
BEGIN
  HKStringliste := Stringliste.Create();
  zwString := String.Create();

  (* PersID *)
  lfdString := PersIDAlsString(inHK^.Identifikator.ID, LaengePersIDNR);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  (* Name, Personalnr*)
  Stringliste.Append(HKStringliste, inHK^.Name);
  Stringliste.Append(HKStringliste, inHK^.Vorname);
  CardToStr(inHK^.Personalnr,strArray);
  String.Schreibe(zwString,strArray);
  Stringliste.Append(HKStringliste, zwString);

  (* Existenz-Angaben *)
  WITH inHK^.Existenz DO
    lfdString := DatumAlsString(Geburtsdatum);
    Stringliste.Append(HKStringliste, lfdString);
    String.Dispose(lfdString);

    Stringliste.Append(HKStringliste, Geburtsort);
    Stringliste.Append(HKStringliste, Staatsang);

    OneCharString[0] := Geschlecht;
    String.Schreibe(zwString,OneCharString);
    Stringliste.Append(HKStringliste, zwString);

    OneCharString[0] := Famstand;
    String.Schreibe(zwString,OneCharString);
    Stringliste.Append(HKStringliste, zwString);

    CardToStr(Kinder,strArray);
    String.Schreibe(zwString,strArray);
    Stringliste.Append(HKStringliste, zwString);
  END; (* WITH inHK^.Existenz *)

  (* Privat-Adresse *)
  lfdString := Adresse.LiesStrasse(inHK^.PrivatAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := Adresse.LiesPostleitzahl(inHK^.PrivatAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := Adresse.LiesStadt(inHK^.PrivatAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := Adresse.LiesLand(inHK^.PrivatAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := Adresse.LiesTelefon(inHK^.PrivatAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := Adresse.LiesFax(inHK^.PrivatAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := Adresse.LieseMail(inHK^.PrivatAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);


  (* Dienst-Adresse *)
  lfdString := DAdresse.LiesDienstName(inHK^.DienstAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := DAdresse.LiesDienstGebaeude(inHK^.DienstAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);


  lfdString := DAdresse.LiesDienstZimmer(inHK^.DienstAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := DAdresse.LiesDienstStr(inHK^.DienstAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := DAdresse.LiesDienstPlz(inHK^.DienstAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := DAdresse.LiesDienstStadt(inHK^.DienstAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := DAdresse.LiesDienstLand(inHK^.DienstAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := DAdresse.LiesDienstTel(inHK^.DienstAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := DAdresse.LiesDienstFax(inHK^.DienstAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);

  lfdString := DAdresse.LiesDiensteMail(inHK^.DienstAdresse);
  Stringliste.Append(HKStringliste, lfdString);
  String.Dispose(lfdString);


  (* Bank-Verbindung *)
  WITH inHK^.Bankverbindung DO
    Stringliste.Append(HKStringliste, BankinstOrt);

    CardToStr(Blz,strArray);
    String.Schreibe(zwString,strArray);
    Stringliste.Append(HKStringliste, zwString);

    Stringliste.Append(HKStringliste, Kontonr);
  END; (* WITH inHK^.Bankverbindung *)


  (* Vertrags-Angaben *)
  WITH inHK^.VertragsAngaben DO
    CardToStr(VorStdzahl,strArray);
    String.Schreibe(zwString,strArray);
    Stringliste.Append(HKStringliste, zwString);

    lfdString := DatumAlsString(VerfuegAb);
    Stringliste.Append(HKStringliste, lfdString);
    String.Dispose(lfdString);

    lfdString := PraesTageAlsString(PraesTage);
    Stringliste.Append(HKStringliste, lfdString);
    String.Dispose(lfdString);

    Stringliste.Append(HKStringliste, Abschluss);
    Stringliste.Append(HKStringliste, Interessen);
    Stringliste.Append(HKStringliste, Bemerkungen);

  END; (* WITH inHK^.VertragsAngaben *)

  String.Dispose(zwString);
  RETURN HKStringliste;

END CreateStringlisteFromObject;




PROCEDURE       CreateObjectFromStringliste (inoutStringliste : TStringliste;
                                             VAR outHK :THK;
                                             VAR outMessage : TString
                                            ) : BOOLEAN;
(*
Vb:    inoutStringliste ist mit CreateStringlisteFromObject erstellt.
Erzeugt aus inoutStringliste ein ADO HK
Nb:     (Rueckgabewert = TRUE
         => CreateStringlisteFromObject(outHK) = inoutStringliste
            AND
            outMessage undefiniert)
        AND
        (Rueckgabewert = FALSE
         => outMessage enthaelt eine Fehlermeldung AND
            outHK undefiniert)
*)

  VAR rc           : BOOLEAN;
      index        : CARDINAL;
      lfdString    : TString;
      zwString     : TString; (* String zum Zwischenspeichern *)
      strPersID    : TString;
      lfdDatum     : TDatum;
      lfdPraesTage : TPraesTage;
      strArray     : ARRAY [0..255] OF CHAR;
      KonvResult   : ConvResults;

BEGIN
  rc := TRUE;
  outHK := Create();
  zwString := String.Create();
  strPersID := String.Create();

  IF NOT Stringliste.IsEmpty(inoutStringliste) THEN
     Stringliste.GoStart(inoutStringliste);
     index := 0;

     WHILE (rc AND NOT Stringliste.OffEnd(inoutStringliste)) DO
        lfdString := Stringliste.Current(inoutStringliste);
        CASE index OF
            0 :  (* PersID *)
                 String.Equalize(strPersID, lfdString);
                 IF PruefePersID(lfdString, outMessage) THEN
                    WITH outHK^.Identifikator DO
                      ID := StringInPersID(lfdString);
                      existiert := TRUE;
                    END; (* WITH *)
                 ELSE
                    String.Schreibe(zwString,
                           " - CreateObject-Fehler bei PersID, String=");
                    String.Anhaengen(zwString, outMessage);
                    String.Anhaengen(lfdString, outMessage);
                    rc := FALSE;
                 END (* IF PersID OK *)
          | 1 :  (* Name *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Equalize(outHK^.Name, lfdString);
                 END; (* IF StringLaenge > 0 *)
          | 2 :  (* Vorname *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Equalize(outHK^.Vorname, lfdString);
                 END; (* IF StringLaenge > 0 *)
          | 3 :  (* Personalnr *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Lies(lfdString, strArray);
                    StrToCard(strArray, outHK^.Personalnr, KonvResult);
                    IF KonvResult # strAllRight  THEN
                       outMessage := String.Create();
                       String.Schreibe(outMessage,
                              "CreateObject-Fehler bei Personalnr, String=");
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END; (* IF StrToCard OK *)
                 END; (* IF StringLaenge > 0 *)

                 (* Existenz-Angaben *)
          | 4 :  (* Geburtsdatum *)
                 IF String.Laenge(lfdString) > 0 THEN
                    IF DatumVonString(lfdString, lfdDatum, outMessage) THEN
                       EqualizeDatum(outHK^.Existenz.Geburtsdatum, lfdDatum);
                       DisposeDatum(lfdDatum);
                    ELSE
                       String.Schreibe(zwString,
                           " - CreateObject-Fehler bei Geburtsdatum, String=");
                       String.Anhaengen(zwString, outMessage);
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END (* IF Geburtsdatum OK *)
                 END; (* IF StringLaenge > 0 *)

          | 5 :  (* Geburtsort *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Equalize(outHK^.Existenz.Geburtsort, lfdString);
                 END; (* IF StringLaenge > 0 *)
          | 6 :  (* Staatsangehoerigkeit *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Equalize(outHK^.Existenz.Staatsang, lfdString);
                 END; (* IF StringLaenge > 0 *)
          | 7 :  (* Geschlecht *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Lies(lfdString, strArray);
                    outHK^.Existenz.Geschlecht := strArray[0];
                 END; (* IF StringLaenge > 0 *)
          | 8 :  (* Familienstand *)
                 IF String.Laenge(lfdString) > 0 THEN
                   String.Lies(lfdString, strArray);
                   outHK^.Existenz.Famstand := strArray[0];
                 END; (* IF StringLaenge > 0 *)
          | 9 :  (* Kinder *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Lies(lfdString, strArray);
                    StrToCard(strArray, outHK^.Existenz.Kinder, KonvResult);
                    IF KonvResult # strAllRight  THEN
                       outMessage := String.Create();
                       String.Schreibe(outMessage,
                              "CreateObject-Fehler bei Kinder, String=");
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END; (* IF StrToCard OK *)
                 END; (* IF StringLaenge > 0 *)

                 (* Privat-Adresse *)
          |10 :  (* Strasse *)
                 IF String.Laenge(lfdString) > 0 THEN
                    Adresse.SetzeStrasse(outHK^.PrivatAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |11 :  (* Postleitzahl *)
                 IF String.Laenge(lfdString) > 0 THEN
                    Adresse.SetzePostleitzahl(outHK^.PrivatAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |12 :  (* Stadt *)
                 IF String.Laenge(lfdString) > 0 THEN
                    Adresse.SetzeStadt(outHK^.PrivatAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |13 :  (* Land *)
                 IF String.Laenge(lfdString) > 0 THEN
                    Adresse.SetzeLand(outHK^.PrivatAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |14 :  (* Telefon *)
                 IF String.Laenge(lfdString) > 0 THEN
                    Adresse.SetzeTelefon(outHK^.PrivatAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |15 :  (* Fax *)
                 IF String.Laenge(lfdString) > 0 THEN
                    Adresse.SetzeFax(outHK^.PrivatAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |16 :  (* eMail *)
                 IF String.Laenge(lfdString) > 0 THEN
                    Adresse.SetzeeMail(outHK^.PrivatAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)

                 (* Dienst-Adresse *)
          |17 :  (* DienstName *)
                 IF String.Laenge(lfdString) > 0 THEN
                    DAdresse.SetzeDienstName(outHK^.DienstAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |18 :  (* DienstGebaeude *)
                 IF String.Laenge(lfdString) > 0 THEN
                    DAdresse.SetzeDienstGebaeude(outHK^.DienstAdresse,
                                                 lfdString);
                 END; (* IF StringLaenge > 0 *)
          |19 :  (* DienstZimmer *)
                 IF String.Laenge(lfdString) > 0 THEN
                    DAdresse.SetzeDienstZimmer(outHK^.DienstAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |20 :  (* DienstStrasse *)
                 IF String.Laenge(lfdString) > 0 THEN
                    DAdresse.SetzeDienstStr(outHK^.DienstAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |21 :  (* DienstPostleitzahl *)
                 IF String.Laenge(lfdString) > 0 THEN
                    DAdresse.SetzeDienstPlz(outHK^.DienstAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |22 :  (* DienstStadt *)
                 IF String.Laenge(lfdString) > 0 THEN
                    DAdresse.SetzeDienstStadt(outHK^.DienstAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |23 :  (* DienstLand *)
                 IF String.Laenge(lfdString) > 0 THEN
                    DAdresse.SetzeDienstLand(outHK^.DienstAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |24 :  (* DienstTelefon *)
                 IF String.Laenge(lfdString) > 0 THEN
                    DAdresse.SetzeDienstTel(outHK^.DienstAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |25 :  (* DienstFax *)
                 IF String.Laenge(lfdString) > 0 THEN
                    DAdresse.SetzeDienstFax(outHK^.DienstAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)
          |26 :  (* DiensteMail *)
                 IF String.Laenge(lfdString) > 0 THEN
                    DAdresse.SetzeDiensteMail(outHK^.DienstAdresse, lfdString);
                 END; (* IF StringLaenge > 0 *)

                 (* Bank-Verbindung *)
          |27 :  (* BankinstOrt *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Equalize(outHK^.Bankverbindung.BankinstOrt,
                                    lfdString);
                 END; (* IF StringLaenge > 0 *)
          |28 :  (* Bankleitzahl *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Lies(lfdString, strArray);
                    StrToCard(strArray, outHK^.Bankverbindung.Blz, KonvResult);
                    IF KonvResult # strAllRight  THEN
                       outMessage := String.Create();
                       String.Schreibe(outMessage,
                              "CreateObject-Fehler bei DienstBlz, String=");
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END; (* IF StrToCard OK *)
                 END; (* IF StringLaenge > 0 *)

          |29 :  (* Kontonr *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Equalize(outHK^.Bankverbindung.Kontonr, lfdString);
                 END; (* IF StringLaenge > 0 *)

                 (* Vertrags-Angaben *)
          |30 :  (* VoraussichtlicheStdZahl *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Lies(lfdString, strArray);
                    StrToCard(strArray, outHK^.VertragsAngaben.VorStdzahl,
                              KonvResult);
                    IF KonvResult # strAllRight  THEN
                       outMessage := String.Create();
                       String.Schreibe(outMessage,
                              "CreateObject-Fehler bei VorStdzahl, String=");
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END; (* IF StrToCard OK *)
                 END; (* IF StringLaenge > 0 *)

          |31 :  (* VerfuegbarAb *)
                 IF String.Laenge(lfdString) > 0 THEN
                    IF DatumVonString(lfdString, lfdDatum, outMessage) THEN
                       EqualizeDatum(outHK^.VertragsAngaben.VerfuegAb,
                                     lfdDatum);
                       DisposeDatum(lfdDatum);
                    ELSE
                       String.Schreibe(zwString,
                           " - CreateObject-Fehler bei VerfuegAb, String=");
                       String.Anhaengen(zwString, outMessage);
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END (* IF VerfuegAb OK *)
                 END; (* IF StringLaenge > 0 *)

          |32 :  (* Praesenztage *)
                 IF String.Laenge(lfdString) > 0 THEN
                    IF Wochentage.PruefeWochentage(lfdString, outMessage) THEN
                       lfdPraesTage := StringInPraesTage(lfdString);
                       Wochentage.Equalize(outHK^.VertragsAngaben.PraesTage,
                                           lfdPraesTage);
                       Wochentage.Dispose(lfdPraesTage);
                    ELSE
                       String.Schreibe(zwString,
                              " - CreateObject-Fehler bei PraesTage, String=");
                       String.Anhaengen(zwString, outMessage);
                       String.Anhaengen(lfdString, outMessage);
                       rc := FALSE;
                    END (* IF PraesTage OK *)
                 END; (* IF StringLaenge > 0 *)

          |33 :  (* Abschluss *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Equalize(outHK^.VertragsAngaben.Abschluss,
                                    lfdString);
                 END; (* IF StringLaenge > 0 *)
          |34 :  (* Interessen *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Equalize(outHK^.VertragsAngaben.Interessen,
                                    lfdString);
                 END; (* IF StringLaenge > 0 *)
          |35 :  (* Bemerkungen *)
                 IF String.Laenge(lfdString) > 0 THEN
                    String.Equalize(outHK^.VertragsAngaben.Bemerkungen,
                                    lfdString);
                 END; (* IF StringLaenge > 0 *)
          ELSE
                 outMessage := String.Create();
                 String.Schreibe(outMessage,
                        "CreateObject-Fehler, mehr als 36 Attribute, String=");
                 String.Anhaengen(lfdString, outMessage);
                 rc := FALSE;

        END; (* CASE ueber den Index der Stringliste *)

        index := index + 1;
        String.Dispose(lfdString);
        Stringliste.StepNext(inoutStringliste);

     END; (* WHILE Schleife ueber die Elemente der Stringliste *)

  END; (* IF inoutStringliste nicht leer *)

  IF NOT rc THEN
     (* Im Fehlerfall PersID in die outMessage *)
     String.Schreibe(zwString, "; PersID=");
     String.Anhaengen(zwString, outMessage);
     String.Anhaengen(strPersID, outMessage);
     Dispose(outHK);
  END; (* IF *)
  String.Dispose(zwString);
  String.Dispose(strPersID);

  RETURN rc;

END CreateObjectFromStringliste;

END HK.
