(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*)

(* Test-Operationen fuer das Teilsystem Hilfskraefte:
      OpInsertHKe   Hilfskraefte aus der Datei hkdateninsert einfuegen
      OpUpdateHKe   Hilfskraefte aus der Datei hkdatenupdate aendern
      OpDeleteHKe   Hilfskraft loeschen
      OpShowListHKe Inhalt des HilfskraefteOrdners auf Logdatei ausgeben
      OpShowHKeAlsStringliste
                     Alle Hilfskraefte als Stringliste auf Logdatei ausgeben
      OpShowAlleAlsStringliste
                     Alle Hilfskraefte mit fast allen Attributen als
                     Stringliste auf Logdatei ausgeben
      OpSaveHK       Alle Hilfskraefte auf Datei auslagern
      OpLoadHKe      Alle Hilfskraefte von Datei einspielen


Implementation der Setze-Operationen:
Die Setze-Operationen werden nach Typen aufgeteilt - String, Datum, CARDINAL,
CHAR - und entsprechend ihren Typen als Prozedur-Pointer in verschiedenen
Arrays gespeichert. Da die Eingabedaten in vorgeschriebener Reihenfolge
vorliegen, werden die Setze-Operationen ueber ihren Array-Index aufgerufen.
*)

IMPLEMENTATION MODULE TestOpHKe;

   FROM  SeqFile  IMPORT ChanId, FlagSet, read,
                         OpenRead, Close, OpenResults;
   FROM  IOResult IMPORT ReadResults, ReadResult;
   FROM  TextIO   IMPORT ReadString, ReadToken, ReadRestLine, ReadChar,
                         SkipLine;
   FROM  InOut    IMPORT WriteString, WriteLn, Write;
   FROM  WholeStr IMPORT ConvResults, StrToCard, CardToStr;
   FROM  Ascii    IMPORT lf;
   FROM  LogDatei IMPORT WriteLogString, WriteLogChar;
   FROM  VDat     IMPORT THandleInVDat, OpenInVDat, CloseInVDat, SetWriteInVDat,
                         SetReadInVDat;
   FROM  HIDVerwaltung   IMPORT SaveInHID, LoadInHID;
   FROM  HKe      IMPORT THK, TPersID, TPraesTage,
            CreateHK, DisposeHK, DuplicateHK, EqualizeHK,IsEqualHK,
            DisposePersID, LiesPersIDHK, PersIDAlsString, StringInPersID,
            PruefePersID,
            DisposePraesTage, PruefePraesTage, IsEqualPraesTage,
            PraesTageAlsString, StringInPraesTage,
            LiesNameinHK, SetzeNameinHK, LiesVornameinHK, SetzeVornameinHK,
            LiesPersonalnrinHK, SetzePersonalnrinHK,
            LiesGeburtsdatuminHK, SetzeGeburtsdatuminHK,
            LiesGeburtsortinHK, SetzeGeburtsortinHK,
            LiesStaatsanginHK, SetzeStaatsanginHK,
            LiesGeschlechtinHK, SetzeGeschlechtinHK,
            LiesFamstandinHK, SetzeFamstandinHK,
            LiesKinderinHK,  SetzeKinderinHK,
            LiesStrasseinHK, SetzeStrasseinHK,
            LiesPostleitzahlinHK, SetzePostleitzahlinHK,
            LiesStadtinHK, SetzeStadtinHK,
            LiesLandinHK, SetzeLandinHK,
            LiesTelefoninHK, SetzeTelefoninHK,
            LiesFaxinHK, SetzeFaxinHK,
            LieseMailinHK, SetzeeMailinHK,
            LiesDienstNameinHK, SetzeDienstNameinHK,
            LiesDienstGebaeudeinHK, SetzeDienstGebaeudeinHK,
            LiesDienstZimmerinHK, SetzeDienstZimmerinHK,
            LiesDienstStrinHK, SetzeDienstStrinHK,
            LiesDienstPlzinHK, SetzeDienstPlzinHK,
            LiesDienstStadtinHK, SetzeDienstStadtinHK,
            LiesDienstLandinHK, SetzeDienstLandinHK,
            LiesDienstTelinHK, SetzeDienstTelinHK,
            LiesDienstFaxinHK, SetzeDienstFaxinHK,
            LiesDiensteMailinHK, SetzeDiensteMailinHK,
            LiesBankinstOrtinHK, SetzeBankinstOrtinHK,
            LiesBlzinHK, SetzeBlzinHK, LiesKontonrinHK, SetzeKontonrinHK,
            LiesVorStdzahlinHK, SetzeVorStdzahlinHK,
            LiesVerfuegAbinHK, SetzeVerfuegAbinHK,
            LiesPraestageinHK, SetzePraestageinHK,
            LiesAbschlussinHK, SetzeAbschlussinHK,
            LiesInteresseninHK, SetzeInteresseninHK,
            LiesBemerkungeninHK, SetzeBemerkungeninHK,
            InsertHK, DeleteHK, SearchHK, UpdateHK, FirstHK, NextHK,
            GibHKeAlsStringlisteinHKe, GibAllesAlsStringlisteinHKe,
            SaveinHKe, LoadinHKe;

   IMPORT String, Stringliste;
   IMPORT Datum;

   CONST DateinameHKe = "HilfskraefteOrdner";
         DateinameHID = "HIDOrdner";
         DateiNameHKDatenInsert = "TS_HKe/Testumgebung/hkdateninsert";
                                  (* Eingabedaten fuer die Operation Insert *)
         DateiNameHKDatenUpdate = "TS_HKe/Testumgebung/hkdatenupdate";
                                  (* Eingabedaten fuer die Operation Update *)
         (* LaengePersIDNR = 5; *)
         INITVALUECARDINAL = 99999999;

   TYPE TString = String.TString;
        TStringliste = Stringliste.TStringliste;
        TDatum  = Datum.TDatum;

       (* Prozedur-Typen fuer die Setze-Prozeduren *)
       TSetzeStringinHK   = PROCEDURE (THK,TString);
       TSetzeDatuminHK    = PROCEDURE (THK,TDatum);
       TSetzeCharinHK     = PROCEDURE (THK,CHAR);
       TSetzeCardinalinHK = PROCEDURE (THK,CARDINAL);


   VAR   (* Arrays fuer die Prozedur-Pointer *)
         SetzePROCsString   : ARRAY [0..25] OF TSetzeStringinHK;
         SetzePROCsDatum    : ARRAY [0..1]  OF TSetzeDatuminHK;
         SetzePROCsChar     : ARRAY [0..1]  OF TSetzeCharinHK;
         SetzePROCsCardinal : ARRAY [0..3]  OF TSetzeCardinalinHK;



PROCEDURE OpInsertHKe();


  VAR rc : BOOLEAN;
      cidinput :ChanId;
      openres : OpenResults;
      KonvResult : ConvResults;
      (* Indizes fuer die Prozedur-Pointer-Arrays *)
      istring   : CARDINAL;
      idatum    : CARDINAL;
      ichar     : CARDINAL;
      icardinal : CARDINAL;
      EingabeTyp     : CHAR;
      EingabeZeichen : CHAR;
      EingabeName    : ARRAY [0..255] OF CHAR;
      EingabeDaten   : ARRAY [0..255] OF CHAR;
      StrArray       : ARRAY [0..255] OF CHAR;
        
      lfdHK        : THK;
      lfdDatum     : TDatum;
      lfdPraesTage : TPraesTage;
      lfdZahl      : CARDINAL;
      lfdString    : TString;
      outMessage   : TString;



BEGIN

   lfdString := String.Create();

   OpenRead (cidinput, DateiNameHKDatenInsert, read, openres);
   IF  (openres # opened) THEN
      WriteLogString("OpenRead-Fehler bei ");
      WriteLogString(DateiNameHKDatenInsert); WriteLogChar(lf);
      WriteLogString("FehlerCode = ORD(OpenResults): ");
      WriteLogChar(CHR(ORD(openres)+48)); WriteLogChar(lf);
      (* Fehlermeldung zusaetzlich auf STDOUT ausgeben *)
      WriteString("OpenRead-Fehler bei ");
      WriteString(DateiNameHKDatenInsert); WriteLn();
      WriteString("FehlerCode = ORD(OpenResults): ");
      Write(CHR(ORD(openres)+48)); WriteLn();
      HALT();
   END; (* IF OpenRead NOT OK *)

   (* Aufbau der Zeilen der Eingabedaten siehe in der Eingabedatei *)

   ReadChar(cidinput, EingabeTyp);

   WHILE NOT (ReadResult(cidinput) = endOfInput)  DO

     (* Kommentar ueberlesen *)
     IF EingabeTyp = '#' THEN
        WHILE EingabeTyp = '#'  DO
          SkipLine(cidinput);
          ReadChar(cidinput, EingabeTyp);
        END (* WHILE *)
     END; (* IF *)
     

     ReadToken(cidinput, EingabeName);

     (* Zeichen bis zum Beginn der Daten (|) ueberlesen *)
     EingabeZeichen := ' ';
     WHILE EingabeZeichen <> '|' DO
         ReadChar(cidinput, EingabeZeichen);
     END; (* WHILE *)
     ReadRestLine(cidinput, EingabeDaten);
     SkipLine(cidinput);


     CASE EingabeTyp OF
       '#' : (* Kommentar, nichts zu tun *) ;

      |'-' : (* Beginn der Attribute eines Objekts,
                Initialisierung
             *)
             lfdHK := CreateHK();
             istring   := 0;
             idatum    := 0;
             ichar     := 0;
             icardinal := 0;

      |'=' : (* Ende der Attribute eines Objekts,
                Objekt in den Ordner einfuegen
             *)
             rc := InsertHK(lfdHK, outMessage);
             IF NOT rc THEN
                WriteLogString("Fehler bei InsertHK: ");
                String.Lies(outMessage,StrArray);
                WriteLogString(StrArray); WriteLogChar(lf);
                String.Dispose(outMessage);
             END; (* IF *)
             DisposeHK(lfdHK);

      |'S' : (* String-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                String.Schreibe(lfdString,EingabeDaten);
                SetzePROCsString[istring](lfdHK, lfdString);
             END; (* IF *)
             istring := istring + 1;

      |'D' : (* Datum-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                String.Schreibe(lfdString,EingabeDaten);
                rc := Datum.DatumVonString(lfdString, lfdDatum, outMessage);
                IF rc THEN
                   SetzePROCsDatum[idatum](lfdHK, lfdDatum);
                   Datum.DisposeDatum(lfdDatum);
                ELSE
                   WriteLogString("Fehler bei DatumVonString: ");
                   WriteLogString(EingabeDaten);
                   WriteLogString("; ");
                   String.Lies(outMessage,StrArray);
                   WriteLogString(StrArray); WriteLogChar(lf);
                   String.Dispose(outMessage);
                END; (* IF *)
             END; (* IF *)
             idatum := idatum + 1;

      |'C' : (* CHAR-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                SetzePROCsChar[ichar](lfdHK, EingabeDaten[0]);
             END; (* IF *)
             ichar := ichar + 1;

      |'Z' :  (* CARDINAL-Attribut *)
              IF LENGTH(EingabeDaten) > 0 THEN
                StrToCard(EingabeDaten, lfdZahl, KonvResult);
                IF KonvResult = strAllRight  THEN
                   SetzePROCsCardinal[icardinal](lfdHK, lfdZahl);
                ELSE
                  WriteLogString("Fehler bei StrToCard: ");
                  WriteLogString(EingabeDaten);
                  WriteLogChar(lf);
                END; (* IF *)
             END; (* IF *)
             icardinal := icardinal + 1;

      |'P' :  (* Attribut Praesenztage *)
              IF LENGTH(EingabeDaten) > 0 THEN
                String.Schreibe(lfdString,EingabeDaten);
                rc := PruefePraesTage(lfdString, outMessage);
                IF rc  THEN
                   lfdPraesTage :=StringInPraesTage(lfdString);
                   SetzePraestageinHK(lfdHK, lfdPraesTage);
                   DisposePraesTage(lfdPraesTage);
                ELSE
                  WriteLogString("Fehler bei PruefePraesTage: ");
                  WriteLogString(EingabeDaten);
                  WriteLogChar(lf);
                END; (* IF *)
             END; (* IF *)

      ELSE
             WriteLogString("Eingabedaten falsch: ");
             WriteLogString(EingabeName);
             WriteLogString("; ");
             WriteLogString(EingabeDaten);
             WriteLogChar(lf);


     END; (* CASE EingabeTyp *)


     ReadChar(cidinput, EingabeTyp);


   END; (* WHILE Schleife ueber die Hilfskraft-Eingabedaten-Datei *)

   Close(cidinput);
   String.Dispose(lfdString);

END OpInsertHKe;



(* --------------------------------------------------------------------------*)

PROCEDURE OpUpdateHKe();


  VAR rc : BOOLEAN;
      cidinput :ChanId;
      openres : OpenResults;
      KonvResult : ConvResults;
      (* Indizes fuer die Prozedur-Pointer-Arrays *)
      istring   : CARDINAL;
      idatum    : CARDINAL;
      ichar     : CARDINAL;
      icardinal : CARDINAL;
      EingabeTyp     : CHAR;
      EingabeZeichen : CHAR;
      EingabeName    : ARRAY [0..255] OF CHAR;
      EingabeDaten   : ARRAY [0..255] OF CHAR;
      StrArray       : ARRAY [0..255] OF CHAR;
        
      lfdHK        : THK;
      vorhandHK    : THK;
      lfdPersID    : TPersID;
      lfdDatum     : TDatum;
      lfdPraesTage : TPraesTage;
      lfdZahl      : CARDINAL;
      lfdString    : TString;
      outMessage   : TString;
      hilfskraft_vorhanden : BOOLEAN;



BEGIN

   lfdString := String.Create();

   OpenRead (cidinput, DateiNameHKDatenUpdate, read, openres);
   IF  (openres # opened) THEN
      WriteLogString("OpenRead-Fehler bei ");
      WriteLogString(DateiNameHKDatenUpdate); WriteLogChar(lf);
      WriteLogString("FehlerCode = ORD(OpenResults): ");
      WriteLogChar(CHR(ORD(openres)+48)); WriteLogChar(lf);
      (* Fehlermeldung zusaetzlich auf STDOUT ausgeben *)
      WriteString("OpenRead-Fehler bei ");
      WriteString(DateiNameHKDatenUpdate); WriteLn();
      WriteString("FehlerCode = ORD(OpenResults): ");
      Write(CHR(ORD(openres)+48)); WriteLn();
      HALT();
   END; (* IF OpenRead NOT OK *)

   (* Aufbau der Zeilen der Eingabedaten siehe in der Eingabedatei *)

   ReadChar(cidinput, EingabeTyp);

   WHILE NOT (ReadResult(cidinput) = endOfInput) DO

     (* Kommentar ueberlesen *)
     IF EingabeTyp = '#' THEN
        WHILE EingabeTyp = '#' DO
          SkipLine(cidinput);
          ReadChar(cidinput, EingabeTyp);
        END (* WHILE *)
     END; (* IF *)
     

     ReadToken(cidinput, EingabeName);

     (* Zeichen bis zum Beginn der Daten (|) ueberlesen *)
     EingabeZeichen := ' ';
     WHILE EingabeZeichen <> '|' DO
         ReadChar(cidinput, EingabeZeichen);
     END; (* WHILE *)
     ReadRestLine(cidinput, EingabeDaten);
     SkipLine(cidinput);



     CASE EingabeTyp OF

       '#' : (* Kommentar, nichts zu tun *) ;

      |'-' : (* Beginn der Attribute eines Objekts,
                Initialisierung
             *)
             lfdHK := CreateHK();
             istring   := 0;
             idatum    := 0;
             ichar     := 0;
             icardinal := 0;
             hilfskraft_vorhanden := FALSE;

      |'=' : (* Ende der Attribute eines Objekts,
                Objekt in dem Ordner aendern
             *)
             IF hilfskraft_vorhanden THEN
                rc := UpdateHK(lfdHK, outMessage);
                IF NOT rc THEN
                   WriteLogString("Fehler bei UpdateHK: ");
                   String.Lies(outMessage,StrArray);
                   WriteLogString(StrArray); WriteLogChar(lf);
                   String.Dispose(outMessage);
                   WriteLogChar(lf);
                END; (* IF UpdateHK OK *)
             END; (* IF hilfskraft_vorhanden*)
             DisposeHK(lfdHK);

      |'S' : (* String-Attribut *)
             String.Schreibe(lfdString,EingabeDaten);
             SetzePROCsString[istring](lfdHK, lfdString);
             istring := istring + 1;


      |'I' : (* PersID der Hilfskraft *)
             (* PersID-Nr *)
             (* Aus PersID-Nr ein PersID-Objekt erzeugen *)
             String.Schreibe(lfdString,EingabeDaten);
             rc := PruefePersID(lfdString, outMessage);
             IF rc  THEN
                lfdPersID :=StringInPersID(lfdString);
                hilfskraft_vorhanden := SearchHK(lfdPersID,vorhandHK);
                IF hilfskraft_vorhanden THEN
                   EqualizeHK(lfdHK, vorhandHK);
                   DisposeHK(vorhandHK);
                ELSE
                  WriteLogString
                           ("Fehler bei SearchHK, Element nicht vorhanden: ");
                  WriteLogString(EingabeDaten);
                  WriteLogChar(lf);
                END; (* IF hilfskraft_vorhanden *)
                DisposePersID(lfdPersID);
             ELSE
               WriteLogString("Fehler bei PruefePersID: ");
               WriteLogString(EingabeDaten);
               WriteLogString("; ");
               String.Lies(outMessage,StrArray);
               WriteLogString(StrArray); WriteLogChar(lf);
               String.Dispose(outMessage);
            END; (* IF PersID korrekt *)

      |'D' : (* Datum-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                String.Schreibe(lfdString,EingabeDaten);
                rc := Datum.DatumVonString(lfdString, lfdDatum, outMessage);
                IF rc THEN
                   SetzePROCsDatum[idatum](lfdHK, lfdDatum);
                   Datum.DisposeDatum(lfdDatum);
                ELSE
                   WriteLogString("Fehler bei DatumVonString: ");
                   WriteLogString(EingabeDaten);
                   WriteLogString("; ");
                   String.Lies(outMessage,StrArray);
                   WriteLogString(StrArray); WriteLogChar(lf);
                   String.Dispose(outMessage);
               END; (* IF DatumVonString OK *)
            ELSE
               (* leere Eingabe, Datum initialisieren *)
               lfdDatum := Datum.CreateDatum();
               SetzePROCsDatum[idatum](lfdHK, lfdDatum);
               Datum.DisposeDatum(lfdDatum);
            END; (* LENGTH(EingabeDaten) > 0 *)
            idatum := idatum + 1;

      |'C' : (* CHAR-Attribut *)
             IF LENGTH(EingabeDaten) > 0 THEN
                SetzePROCsChar[ichar](lfdHK, EingabeDaten[0]);
             ELSE
                SetzePROCsChar[ichar](lfdHK, ' ');
             END; (* IF *)
             ichar := ichar + 1;

      |'Z' :  (* CARDINAL-Attribut *)
              IF LENGTH(EingabeDaten) > 0 THEN
                StrToCard(EingabeDaten, lfdZahl, KonvResult);
                IF KonvResult = strAllRight  THEN
                   SetzePROCsCardinal[icardinal](lfdHK, lfdZahl);
                ELSE
                  WriteLogString("Fehler bei StrToCard: ");
                  WriteLogString(EingabeDaten);
                  WriteLogChar(lf);
                END; (* IF *)
             ELSE
               SetzePROCsCardinal[icardinal](lfdHK, INITVALUECARDINAL);
             END; (* IF LENGTH(EingabeDaten) > 0 *)
             icardinal := icardinal + 1;

      |'P' :  (* Attribut Praesenztage *)
              String.Schreibe(lfdString,EingabeDaten);
              rc := PruefePraesTage(lfdString, outMessage);
              IF rc  THEN
                 lfdPraesTage :=StringInPraesTage(lfdString);
                 SetzePraestageinHK(lfdHK, lfdPraesTage);
                 DisposePraesTage(lfdPraesTage);
              ELSE
                WriteLogString("Fehler bei PruefePraesTage: ");
                WriteLogString(EingabeDaten);
                WriteLogChar(lf);
              END; (* IF PruefePraesTage OK *)

     END; (* CASE EingabeTyp *)


     ReadChar(cidinput, EingabeTyp);


   END; (* WHILE Schleife ueber die Hilfskraft-Eingabedaten-Datei *)

   Close(cidinput);
   String.Dispose(lfdString);

END OpUpdateHKe;



(* --------------------------------------------------------------------------*)

PROCEDURE OpDeleteHKe(inPersIDNr : CARDINAL);


  VAR rc : BOOLEAN;
        
      vorhandHK  : THK;
      lfdPersID  : TPersID;
      lfdString  : TString;
      outMessage : TString;
      StrArray   : ARRAY [0..255] OF CHAR;

BEGIN

  lfdString := String.Create();
  CardToStr(inPersIDNr,StrArray);
  String.Schreibe(lfdString,StrArray);
  lfdPersID :=StringInPersID(lfdString);
  rc := SearchHK(lfdPersID,vorhandHK);
  IF rc THEN
     rc := DeleteHK(vorhandHK, outMessage);
     IF NOT rc THEN
        WriteLogString("Fehler bei DeleteHK, PersIDNR: ");
        WriteLogString(StrArray);
        WriteLogChar(lf);
        String.Lies(outMessage,StrArray);
        WriteLogString(StrArray); WriteLogChar(lf);
        String.Dispose(outMessage);
        WriteLogChar(lf);
     END; (* IF *)
     DisposeHK(vorhandHK);
  ELSE
    WriteLogString ("Fehler bei SearchHK, Element nicht vorhanden: ");
    WriteLogString(StrArray);
    WriteLogChar(lf);
  END; (* IF gefunden *)

  String.Dispose(lfdString);
  DisposePersID(lfdPersID);

END OpDeleteHKe;



(* --------------------------------------------------------------------------*)


PROCEDURE OpShowListHKe();

(* Inhalt des HilfskraefteOrdners auf Logdatei ausgeben *)

  VAR lfdHK : THK;

       (* -------------------------------------------------------------------*)
       PROCEDURE AusgabeHK(inHK : THK);  (* lokale Prozedur *)

         VAR lfdPersID    : TPersID;
             lfdDatum     : TDatum;
             lfdPraesTage : TPraesTage;
             lfdZahl    : CARDINAL;
             lfdString  : TString;
             outMessage : TString;
             StrArray   : ARRAY [0..255] OF CHAR;

       BEGIN
          WriteLogChar(lf);
          WriteLogString("Name: ");
          lfdPersID := LiesPersIDHK(inHK);
          (*lfdString := PersIDAlsString(lfdPersID, LaengePersIDNR);*)
          lfdString := PersIDAlsString(lfdPersID);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          DisposePersID(lfdPersID);
          String.Dispose(lfdString);

          lfdString := LiesNameinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);

          lfdString := LiesVornameinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          CardToStr(LiesPersonalnrinHK(inHK), StrArray);
          WriteLogString(StrArray);
          WriteLogString(";");
          WriteLogChar(lf);

          WriteLogString("Existenz: ");
          lfdDatum := LiesGeburtsdatuminHK(inHK);
          lfdString := Datum.DatumAlsString(lfdDatum);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          Datum.DisposeDatum(lfdDatum);
          String.Dispose(lfdString);
          lfdString := LiesGeburtsortinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesStaatsanginHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          WriteLogChar(LiesGeschlechtinHK(inHK));
          WriteLogString("; ");
          WriteLogChar(LiesFamstandinHK(inHK));
          WriteLogString("; ");
          CardToStr(LiesKinderinHK(inHK), StrArray);
          WriteLogString(StrArray);
          WriteLogString(";");
          WriteLogChar(lf);
          WriteLogString("Adresse: ");
          lfdString := LiesStrasseinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesPostleitzahlinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesStadtinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesLandinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesTelefoninHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesFaxinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LieseMailinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString(";");
          WriteLogChar(lf);
          String.Dispose(lfdString);

          WriteLogString("DienstAdresse: ");
          lfdString := LiesDienstNameinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesDienstGebaeudeinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesDienstZimmerinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesDienstStrinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesDienstPlzinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesDienstStadtinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesDienstLandinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesDienstTelinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesDienstFaxinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesDiensteMailinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString(";");
          WriteLogChar(lf);
          String.Dispose(lfdString);

          WriteLogString("Bankverbindung: ");
          lfdString := LiesBankinstOrtinHK(lfdHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          CardToStr(LiesBlzinHK(inHK), StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          lfdString := LiesKontonrinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString(";");
          WriteLogChar(lf);
          String.Dispose(lfdString);

          WriteLogString("Vertragsangaben: ");
          CardToStr(LiesVorStdzahlinHK(inHK), StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          lfdDatum := LiesVerfuegAbinHK(inHK);
          lfdString := Datum.DatumAlsString(lfdDatum);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          Datum.DisposeDatum(lfdDatum);
          String.Dispose(lfdString);
          lfdPraesTage := LiesPraestageinHK(inHK);
          lfdString := PraesTageAlsString(lfdPraesTage);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          DisposePraesTage(lfdPraesTage);
          String.Dispose(lfdString);
          lfdString := LiesAbschlussinHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesInteresseninHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString("; ");
          String.Dispose(lfdString);
          lfdString := LiesBemerkungeninHK(inHK);
          String.Lies(lfdString,StrArray);
          WriteLogString(StrArray);
          WriteLogString(";");
          WriteLogChar(lf);
          String.Dispose(lfdString);

        END AusgabeHK;    (* Ende lokale Prozedur *)

       (* -------------------------------------------------------------------*)


BEGIN
   IF FirstHK (lfdHK) THEN

      AusgabeHK(lfdHK);
       
      WHILE NextHK(lfdHK) DO
          AusgabeHK(lfdHK); 

      END; (* WHILE *)

    DisposeHK(lfdHK);

   END; (* IF First *)

END OpShowListHKe;


(* --------------------------------------------------------------------------*)

PROCEDURE OpShowHKeAlsStringliste();

(* Alle Hilfskraefte als Stringliste auf Logdatei ausgeben *)

  VAR lfdString       : TString;
      strArray        : ARRAY [0..255] OF CHAR;
      HKestringliste  : TStringliste;


BEGIN

  HKestringliste := GibHKeAlsStringlisteinHKe();

  IF NOT Stringliste.IsEmpty(HKestringliste) THEN
     Stringliste.GoStart(HKestringliste);

     WHILE NOT Stringliste.OffEnd(HKestringliste) DO
        lfdString := Stringliste.Current(HKestringliste);
        String.Lies(lfdString,strArray);
        WriteLogString(strArray);
        WriteLogChar(lf);
        String.Dispose(lfdString);
        Stringliste.StepNext(HKestringliste);
     END; (* WHILE *)

  END; (* IF HKestringliste nicht leer *)

  Stringliste.Dispose(HKestringliste);

END OpShowHKeAlsStringliste;


(* --------------------------------------------------------------------------*)

PROCEDURE OpShowAlleAlsStringliste();

(* Alle Hilfskraefte mit fast allen Attributen als Stringliste
   auf Logdatei ausgeben *)

  VAR lfdString       : TString;
      strArray        : ARRAY [0..255] OF CHAR;
      HKestringliste  : TStringliste;


BEGIN

  HKestringliste := GibAllesAlsStringlisteinHKe();

  IF NOT Stringliste.IsEmpty(HKestringliste) THEN
     Stringliste.GoStart(HKestringliste);

     WHILE NOT Stringliste.OffEnd(HKestringliste) DO
        lfdString := Stringliste.Current(HKestringliste);
        String.Lies(lfdString,strArray);
        WriteLogString(strArray);
        WriteLogChar(lf);
        String.Dispose(lfdString);
        Stringliste.StepNext(HKestringliste);
     END; (* WHILE *)

  END; (* IF HKestringliste nicht leer *)

  Stringliste.Dispose(HKestringliste);

END OpShowAlleAlsStringliste;


(* --------------------------------------------------------------------------*)

PROCEDURE OpSaveHKe();

(* Alle Hilfskraefte auf Datei auslagern *)
(* HID-Liste auf Datei auslagern *)

  VAR Dateiname  : TString;
      strArray   : ARRAY [0..255] OF CHAR;
      outMessage : TString;
      FileHandle : THandleInVDat;


BEGIN

   Dateiname := String.Create();
   String.Schreibe(Dateiname, DateinameHKe);

   (* HilfskraefteOrdner auf Datei auslagern *)
   IF OpenInVDat(FileHandle, Dateiname, SetWriteInVDat()) THEN
      IF NOT SaveinHKe(FileHandle, outMessage) THEN
         WriteLogString("Fehler bei SaveinHKe, ");
         String.Lies(outMessage, strArray);
         WriteLogString(strArray); WriteLogChar(lf);
         String.Dispose(outMessage);
      END; (* IF Save NOT OK *)
   ELSE
      WriteLogString("Fehler bei OpenInVDat fuer Save HKe"); WriteLogChar(lf);
   END; (* IF OpenInVDat NOT OK *)

   IF NOT CloseInVDat(FileHandle) THEN
      WriteLogString("Fehler bei CloseInVDat fuer Save HKe"); WriteLogChar(lf);
   END; (* IF *)

   (* HIDOrdner auf Datei auslagern *)
   String.Schreibe(Dateiname, DateinameHID);
   IF OpenInVDat(FileHandle, Dateiname, SetWriteInVDat()) THEN
      IF NOT SaveInHID(FileHandle, outMessage) THEN
         WriteLogString("Fehler bei SaveInHID, ");
         String.Lies(outMessage, strArray);
         WriteLogString(strArray); WriteLogChar(lf);
         String.Dispose(outMessage);
      END; (* IF Save NOT OK *)
   ELSE
      WriteLogString("Fehler bei OpenInVDat fuer Save HID"); WriteLogChar(lf);
   END; (* IF OpenInVDat NOT OK *)

   IF NOT CloseInVDat(FileHandle) THEN
      WriteLogString("Fehler bei CloseInVDat fuer Save HID"); WriteLogChar(lf);
   END; (* IF *)

   String.Dispose(Dateiname);

END OpSaveHKe;



(* --------------------------------------------------------------------------*)

PROCEDURE OpLoadHKe();

(* Alle Hilfskraefte von Datei einspielen *)

  VAR Dateiname  : TString;
      strArray   : ARRAY [0..255] OF CHAR;
      outMessage : TString;
      FileHandle : THandleInVDat;


BEGIN

   Dateiname := String.Create();
   String.Schreibe(Dateiname, DateinameHKe);

   (* HilfskraefteOrdner von Datei einspielen *)
   IF OpenInVDat(FileHandle, Dateiname, SetReadInVDat()) THEN
      IF NOT LoadinHKe(FileHandle, outMessage) THEN
         WriteLogString("Fehler bei LoadinHKe, ");
         String.Lies(outMessage, strArray);
         WriteLogString(strArray); WriteLogChar(lf);
         String.Dispose(outMessage);
      END; (* IF Load OK *)
   ELSE
      WriteLogString("Fehler bei OpenInVDat fuer Load"); WriteLogChar(lf);
   END; (* IF OpenInVDat NOT OK *)

   IF NOT CloseInVDat(FileHandle) THEN
      WriteLogString("Fehler bei CloseInVDat fuer Load"); WriteLogChar(lf);
   END; (* IF Close NOT OK *)

   (* HIDOrdner von Datei einspielen *)
   String.Schreibe(Dateiname, DateinameHID);
   IF OpenInVDat(FileHandle, Dateiname, SetReadInVDat()) THEN
      IF NOT LoadInHID(FileHandle, outMessage) THEN
         WriteLogString("Fehler bei LoadInHID, ");
         String.Lies(outMessage, strArray);
         WriteLogString(strArray); WriteLogChar(lf);
         String.Dispose(outMessage);
      END; (* IF Load OK *)
   ELSE
      WriteLogString("Fehler bei OpenInVDat fuer Load HID"); WriteLogChar(lf);
   END; (* IF OpenInVDat NOT OK *)

   IF NOT CloseInVDat(FileHandle) THEN
      WriteLogString("Fehler bei CloseInVDat fuer Load HID"); WriteLogChar(lf);
   END; (* IF Close NOT OK *)

   String.Dispose(Dateiname);

END OpLoadHKe;



(* --------------------------------------------------------------------------*)

BEGIN (* Modul-Initialisierung *)

(* Prozedur-Pointer der Setze-Operationen in Arrays - getrennt nach Typ -
   speichern
*)
   SetzePROCsString[0]  := SetzeNameinHK;
   SetzePROCsString[1]  := SetzeVornameinHK;
   SetzePROCsString[2]  := SetzeGeburtsortinHK;
   SetzePROCsString[3]  := SetzeStaatsanginHK;
   SetzePROCsString[4]  := SetzeStrasseinHK;
   SetzePROCsString[5]  := SetzePostleitzahlinHK;
   SetzePROCsString[6]  := SetzeStadtinHK;
   SetzePROCsString[7]  := SetzeLandinHK;
   SetzePROCsString[8]  := SetzeTelefoninHK;
   SetzePROCsString[9]  := SetzeFaxinHK;
   SetzePROCsString[10] := SetzeeMailinHK;
   SetzePROCsString[11] := SetzeDienstNameinHK;
   SetzePROCsString[12] := SetzeDienstGebaeudeinHK;
   SetzePROCsString[13] := SetzeDienstZimmerinHK;
   SetzePROCsString[14] := SetzeDienstStrinHK;
   SetzePROCsString[15] := SetzeDienstPlzinHK;
   SetzePROCsString[16] := SetzeDienstStadtinHK;
   SetzePROCsString[17] := SetzeDienstLandinHK;
   SetzePROCsString[18] := SetzeDienstTelinHK;
   SetzePROCsString[19] := SetzeDienstFaxinHK;
   SetzePROCsString[20] := SetzeDiensteMailinHK;
   SetzePROCsString[21] := SetzeBankinstOrtinHK;
   SetzePROCsString[22] := SetzeKontonrinHK;
   SetzePROCsString[23] := SetzeAbschlussinHK;
   SetzePROCsString[24] := SetzeInteresseninHK;
   SetzePROCsString[25] := SetzeBemerkungeninHK;

   SetzePROCsDatum[0] := SetzeGeburtsdatuminHK;
   SetzePROCsDatum[1] := SetzeVerfuegAbinHK;

   SetzePROCsChar[0]  := SetzeGeschlechtinHK;
   SetzePROCsChar[1]  := SetzeFamstandinHK;

   SetzePROCsCardinal[0] := SetzePersonalnrinHK;
   SetzePROCsCardinal[1] := SetzeKinderinHK;
   SetzePROCsCardinal[2] := SetzeBlzinHK;
   SetzePROCsCardinal[3] := SetzeVorStdzahlinHK;

END TestOpHKe.
