(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: HIDVerwSchablone.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:48:29 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE (* -- Behaelter-ADT *) HIDVerwSchablone;

  IMPORT IDVerwaltung;
  IMPORT HID; 

  TYPE TIDVerwaltung  = IDVerwaltung.TIDVerwaltung;
       TObjektreferenz  = IDVerwaltung.TObjektreferenz;

  PROCEDURE Create () : TIDVerwaltung;
  VAR aktDIV : TIDVerwaltung;
  BEGIN
    aktDIV := IDVerwaltung.Create (CreateID,
				   CreateFolgeNr,
                                   DisposeID,
                                   DuplicateID,
                                   EqualizeID,
                                   IsEqualID,
                                   SmallerID,
                                   SameID);
    RETURN aktDIV
  END Create;

  PROCEDURE Dispose (VAR inoutDIV : TIDVerwaltung);
  BEGIN
    IDVerwaltung.Dispose (inoutDIV);
  END Dispose;


  PROCEDURE HoleNeueID (inoutDIV : TIDVerwaltung) : TID;
  (* -- Liefert eine z.Z. unbenutzte ID *)
  BEGIN
    RETURN HID.ID (IDVerwaltung.HoleNeueID (inoutDIV))
  END HoleNeueID;

  PROCEDURE UeberlasseAlteID (inoutDIV : TIDVerwaltung;
                              inID        : TID);
  BEGIN
    IDVerwaltung.UeberlasseAlteID (inoutDIV, 
                                   HID.OBJEKTREFERENZ (inID))
  END UeberlasseAlteID;

  PROCEDURE GibHoechsteID(inIV : TIDVerwaltung): TID;
  (* -- Vorbed.: es existiert wenigsten eine ID
     -- gibt die hoechste zuletzt vergebene ID zurueck
     -- Nachbed.: ist ID der Rueckgabewert dann gilt fr alle ID's i:
                  i > ID: i ist noch nicht vergeben *)
  BEGIN
    RETURN HID.ID(IDVerwaltung.GibHoechsteID(inIV));
  END GibHoechsteID;

  PROCEDURE SetzeHoechsteID(inoutIV : TIDVerwaltung; inID : TID);
  (* -- Vorbed.: fuer alle ID's i:
                 i > inID: i ist noch nicht vergeben *)
  BEGIN 
	IDVerwaltung.SetzeHoechsteID(inoutIV,HID.OBJEKTREFERENZ (inID));
  END SetzeHoechsteID;

  PROCEDURE FreieIDsVorhanden(inIV : TIDVerwaltung): BOOLEAN;
  (* -- Vorbed.: keine
     -- Nachbed.: FreieIDsVorhanden(inIV) <=> FreieIDListe ist nicht leer *)
  BEGIN
	RETURN IDVerwaltung.FreieIDsVorhanden(inIV);
  END FreieIDsVorhanden;

  PROCEDURE ErsteFreieID(inIV : TIDVerwaltung): TID;
  (* -- Vorbed.: FreieIDsVorhanden(inIV)
     -- gibt die erste Id der freien ID-Liste zurueck.
     -- Nachbed.: keine
  *)
  BEGIN
	RETURN HID.ID(IDVerwaltung.ErsteFreieID(inIV));
  END ErsteFreieID;
  

  PROCEDURE NaechsteFreieID(inIV : TIDVerwaltung;
                             VAR inoutID: TID): BOOLEAN;
  (* -- Vorbed.: inoutID existiert in der freien ID-Liste
     -- gibt die auf inID folgende freie ID der freien ID-Liste zurueck.
     -- Nachbed.: NaechsteFreieID(inIV, inoutID) <=> inID ist die letzte ID
                                                     der Liste
  *) 
  VAR aktObjekt : TObjektreferenz;
  BEGIN
        aktObjekt:=HID.OBJEKTREFERENZ(inoutID);
	(* Die etwas seltsam anmutende Logik kommt vom Bibliotheksmodul  *)
	(* IDVerwaltung.NaechsteFreieID *)
	IF NOT IDVerwaltung.NaechsteFreieID(inIV,aktObjekt) THEN
		inoutID:= HID.ID(aktObjekt);
		RETURN TRUE;
	ELSE
		RETURN FALSE;
        END (* IF *);
  END NaechsteFreieID;

(* ----------------------------------------------------------- 
   -- Typkonvertierungen von TID und TObjektreferenz  
   ----------------------------------------------------------- *)

  PROCEDURE CreateID (inSeqNr : CARDINAL) : TObjektreferenz;
  VAR aktID     : TID;
      outObjekt : TObjektreferenz;
  BEGIN
    aktID := HID.Create (inSeqNr);
    outObjekt := HID.OBJEKTREFERENZ (aktID);
    RETURN outObjekt;
  END CreateID;

  PROCEDURE CreateFolgeNr (inID : TObjektreferenz) : CARDINAL;
  VAR aktFolgeNr: CARDINAL;
  BEGIN
    aktFolgeNr := HID.CreateFolgeNr (HID.ID(inID));
    RETURN aktFolgeNr;
  END CreateFolgeNr;
  
  PROCEDURE DisposeID (VAR inoutReferenz : TObjektreferenz);
  VAR akTID : TID;
  BEGIN
    akTID := HID.ID (inoutReferenz);
    HID.Dispose (akTID);
    inoutReferenz := HID.OBJEKTREFERENZ (akTID);
  END DisposeID;

  PROCEDURE DuplicateID (inReferenz : TObjektreferenz) : TObjektreferenz;
  (* -- Ungetypte Version der Operation ID.Duplicate *)
  VAR Source,
      outDuplikat : TID;
  BEGIN
    Source := HID.ID (inReferenz);
    outDuplikat := HID.Duplicate (Source);
    RETURN HID.OBJEKTREFERENZ (outDuplikat)
  END DuplicateID;

  PROCEDURE EqualizeID (inoutZiel, inQuelle : TObjektreferenz);
  VAR ZielID,
      QuellID : TID;
  BEGIN
    (* -- Vorbed.: siehe ID.Equalize *)
    ZielID := HID.ID (inoutZiel);
    QuellID := HID.ID (inQuelle);
    HID.Equalize (ZielID, QuellID)
  END EqualizeID;

  PROCEDURE IsEqualID (inReferenz1, 
                         inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Objekt1,
      Objekt2 : TID;
  BEGIN
    Objekt1 := HID.ID (inReferenz1);
    Objekt2 := HID.ID (inReferenz2);
    RETURN HID.IsEqual (Objekt1, Objekt2)
  END IsEqualID;

  PROCEDURE SmallerID (inReferenz1, 
                         inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Objekt1,
      Objekt2 : TID;
  BEGIN
    Objekt1 := HID.ID (inReferenz1);
    Objekt2 := HID.ID (inReferenz2);
    RETURN HID.SmallerSeqNr (Objekt1, Objekt2)
  END SmallerID;

  PROCEDURE SameID (inReferenz1, 
                         inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Objekt1,
      Objekt2 : TID;
  BEGIN
    Objekt1 := HID.ID (inReferenz1);
    Objekt2 := HID.ID (inReferenz2);
    RETURN HID.SameSeqNr (Objekt1, Objekt2)
  END SameID;

END HIDVerwSchablone.
