(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: HIDVerwaltung.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:48:30 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE (* -- Teilsystem *) HIDVerwaltung;

  IMPORT HID;
  IMPORT HIDVerwADO;
  IMPORT HIDVerwIMPORTE;
  IMPORT HIDSpeichern;
  FROM String IMPORT TString;

  (* TYPE 	TID = HID.TID; *)

  PROCEDURE CreateIDVerwaltung();
  (* -- Vorbed.: keine
     -- Generiert ein neues abstraktes Datenobjekt  IDVerwaltung
     -- Nachbed.: IDVerwaltung existiert  *)
  BEGIN
	HIDVerwADO.CreateIDVerwaltung();
  END CreateIDVerwaltung;

  PROCEDURE DisposeIDVerwaltung();
  (* -- Vorbed.: keine
     -- Entfernt das ADO IDVerwaltung dem System
     -- Nachbed.: ADO IDVerwaltung existiert nicht mehr *)
  BEGIN
	HIDVerwADO.DisposeIDVerwaltung();
  END DisposeIDVerwaltung;

  PROCEDURE HoleNeueID () : TID;
  (* -- Vorbed.: keine
     -- Liefert eine z.Z. unbenutzte ID. Sei ouTID der
     -- Rueckgabeparameter, dann gilt:
     -- Nachbed.: (ouTID existiert) AND (ouTID NOT IN IDVerwaltung) *)
  BEGIN
	RETURN HIDVerwADO.GetNewID();
  END HoleNeueID;

  PROCEDURE UeberlasseAlteID (inID  : TID);
  (* -- Vorbed.: inID NOT IN IDVerwaltung)
     -- Uebergibt eine z.Z. unbenutzte ID an die ID-Verwaltung
     -- Nachbed.: inID IN IDVerwaltung *)
  BEGIN
	 HIDVerwADO.LeaveOldID(inID);  
  END UeberlasseAlteID;

(* die folgenden Prozeduren sollen das kontrollierte Aufsetzen der ID-Verwalt-*)
(* nach dem Speichern/Laden der Daten auf Platte untersttzen                 *)

  PROCEDURE LoadInHID( inH: THandle; outM: TString ): BOOLEAN;
  (* -- Vorbed.: inH verweist auf eine zum Lesen geoeffnete Datei
     -- restauriert das TS HIDVerwaltung. Ldt den HIDOrdner und die SequenzNr
        neu. Falls das Laden fehlschlgt, wird false zurckgegeben und outM 
	enthaelt eine Fehlernachricht.
     -- Nachbed.: IDVerwaltung ist geladen  *)
  BEGIN
	RETURN HIDSpeichern.Load(inH,outM);
  END LoadInHID;

  PROCEDURE SaveInHID( inH: THandle; outM: TString ): BOOLEAN;
  (* -- Vorbed.: inH verweist auf eine zum Schreiben geoeffnete Datei
     -- restauriert das TS HIDVerwaltung. Speichert den HIDOrdner und die 
	SequenzNr auf den Sekundaerspeicher. Falls das Speichern fehlschlaegt,
	wird false zurueckgegeben und in outM eine Fehlernachricht ausgegeben.
     -- Nachbed.: IDVerwaltung ist gespeichert  *)
  BEGIN
	RETURN HIDSpeichern.Save(inH,outM);
  END SaveInHID;

(* Exporte des ADT HID zur Bearbeitung von IDs (wozu auch immer das gut     *)
(* sein soll)								    *)

  PROCEDURE CreateHID (inSeqNumber : CARDINAL) : TID;
  (* -- Vorbed.: keine
     -- Berechnet aus SeqNr die entsprechende ID der internen ID-Folge.
     -- Nachbed.: Rueckgabe-ADO ID existiert *)
  BEGIN
	RETURN HID.Create(inSeqNumber);
  END CreateHID;

  PROCEDURE DisposeHID (VAR inoutID : TID);
  (* -- Vorbed.: keine
     -- Entfernt das ADO inoutID aus dem System
     -- Nachbed.: ADO inoutID existiert nicht mehr *)
  BEGIN
        HID.Dispose(inoutID);
  END DisposeHID;    

  PROCEDURE DuplicateHID (inQuellID : TID) : TID;
  (* -- Vorbed.: keine
     -- Erzeugt ein neues ID-Objekt und gleicht dieses an den Zustand bzw.
     -- die Attribute von inQuellID an. Bezeichnet Duplikat das Rueckgabe-ADO
     -- der Operation, so gilt:
     -- Nachbed.: IsEqual (Duplikat, inQuellID) AND NOT (Duplikat = inQuellID) *)
  BEGIN
        RETURN HID.Duplicate(inQuellID) ;
  END DuplicateHID;  	

  PROCEDURE IsEqualHID (inID1, inID2 : TID) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Ueberprueft die beiden IDs auf Zustandsgleichheit
     -- Nachbed.: keine; *)
  BEGIN
        RETURN HID.IsEqual(inID1,inID2);
  END IsEqualHID;

  PROCEDURE EqualizeHID (inoutZielID, inQuellID : TID);
  (* -- Vorbed.: keine
     -- Gleicht den Zustand /die Attribute der ZielID an die QuellID an
     -- Nachbed.: IsEqual (inQuellID, inoutZielID) *)
  BEGIN
        HID.Equalize(inoutZielID, inQuellID);
  END EqualizeHID;

  PROCEDURE LiesSeqNrHID (inID : TID) : CARDINAL;
  (* -- Vorbed.: keine
     -- Liefert die Nummer des Folgeglieds
     -- Nachbed.: keine *)
  BEGIN
        RETURN HID.LiesSeqNr(inID) ;
  END LiesSeqNrHID; 

  PROCEDURE SameSeqNrHID (inID1, inID2 : TID) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob inID1 und inID2 aus dem gleichen Folgeglied generiert wurden.
     -- Bezeichne same den boole'schen Rueckgabeparameter, dann gilt:
     -- Nachbed.: same <=> (inID1.SeqNr = inID2.SeqNr) *)
  BEGIN
        RETURN HID.SameSeqNr(inID1, inID2);
  END SameSeqNrHID;

  PROCEDURE SmallerSeqNrHID (inID1, inID2 : TID) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob inID1 aus einem kleineren Folgeglied generiert wurde als inID2.
     -- Bezeichne smaller den boole'schen Rueckgabeparameter, dann gilt:
     -- Nachbed.: smaller <=> (inID1.SeqNr < inID2.SeqNr) *)
  BEGIN
        RETURN HID.SmallerSeqNr(inID1, inID2);
  END SmallerSeqNrHID;

  PROCEDURE CreateFolgeNrHID (inID : TID) : CARDINAL;
  (* -- Vorbed.: keine
     -- Berechnet aus ID die entsprechende FolgeNr der internen ID-Folge.
     -- Nachbed.: Rueckgabe-ADO FolgeNr existiert *)   
  BEGIN
        RETURN HID.CreateFolgeNr(inID);
  END CreateFolgeNrHID; 

(* Hier noch zwei Operationen fr den Typ HID: Konversion in un von TString *)
(* dabei wird natuerlich nur die Schluessel-Nr konvertiert.                 *)

  PROCEDURE HID2Str( inID : TID ; IDLaenge: CARDINAL): TString ;
  (* -- Vorbed.: inID existiert
     -- gibt die SchluesselNr des ID als String zurueck. IDLaenge steuert die
        Breite des Strings. Es werden fuehrende Leerzeichen ausgegeben, bis
        die Gesamtlaenge=Laenge ist. Falls IDLaenge=0 angegeben wird, wird der
        String 'getrimmt' returniert.
     -- Nachbed.: keine *)
  BEGIN
	RETURN HID.HID2Str( inID , IDLaenge, 1);
  END HID2Str;

  PROCEDURE HID2Str4SL( inID : TID ; IDLaenge: CARDINAL): TString ;
  (* -- Vorbed.: inID existiert
     -- gibt die SchluesselNr des ID als String zurueck. IDLaenge steuert die
        Breite des Strings. Es werden fuehrende Leerzeichen ausgegeben, bis
        die Gesamtlaenge=Laenge ist. Falls IDLaenge=0 angegeben wird, wird der
        String 'getrimmt' returniert. Diese Variante gibt als Fuellzeichen 
	Nullen aus.
     -- Nachbed.: keine *)
  BEGIN
	RETURN HID.HID2Str( inID , IDLaenge, 0);
  END HID2Str4SL;

  PROCEDURE Str2HID( inStr: TString): TID ;
  (* -- Vorbed.: inString existiert
     -- gibt einen ID zurueck dessen SequenzNr aus der in inStr enthaltenen SchluesselNr
        berechnet worden ist.
     -- Nachbed.: keine *)      
  BEGIN
	RETURN HID.Str2HID(inStr);
  END Str2HID;    
                                                     
(* Plausibprfung Herbert Schoenhammer *)

  PROCEDURE PruefeHID (inString : TString;
                        VAR outMessage : TString) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob der inString in eine gueltige ID umgewandelt werden kann
     -- Nachbed.: keine *)    
  BEGIN
	RETURN HID.PruefeHID(inString, outMessage);
  END PruefeHID ;

END HIDVerwaltung.
