IMPLEMENTATION MODULE (* -- GADT *) IDVerwaltung;

  (* angepasste Version fuer Sopra 1585 1996  durch Hartwig Toedter 

     Aufgenommen wurden die Funktionen:
        GibHoechsteID,
        SetzeHoechsteID,
        ErsteFreieID,
        NaechsteFreieID
     fuer die Moeglichkeit, die IDVerwaltung aus einer Datei wiederherstellen
     zu koennen.

     Fuer die erweiterte Funktionalitaet muss eine weitere generische
     Operation CreateFolgeNr vom Basistyp angeboten werden. Diese Opeartion
     berechnet zu einer ID die Folgenummer *)

  FROM Storage IMPORT ALLOCATE, DEALLOCATE;
  FROM SYSTEM  IMPORT TSIZE;
  IMPORT GENERIC, SortierteObjektliste;

  TYPE TIDVerwaltungDS = RECORD
                           IDFolge     : CARDINAL;
                           FreieIDs    : SortierteObjektliste.TSortierteObjektliste;
                           CreateID    : TCreateID;
                           CreateFolgeNr : TCreateFolgeNr;
                         END;
       (* -- In IDFolge wird die Nummer der als naechstes
             zu berechnenden ID gespeichert *)
        TIDVerwaltung   = POINTER TO TIDVerwaltungDS;

  PROCEDURE Create (inCreateID        : TCreateID;
                    inCreateFolgeNr   : TCreateFolgeNr;
                    inDisposeObjekt   : GENERIC.TDisposeObjekt;
                    inDuplicateObjekt : GENERIC.TDuplicateObjekt;
                    inEqualizeObjekt  : GENERIC.TEqualizeObjekt;
                    inIsEqualObjekt   : GENERIC.TIsEqualObjekt;
                    inSmallerSeqNr    : GENERIC.TSmallerValue;
                    inSameSeqNr       : GENERIC.TSameValue
                    ) : TIDVerwaltung;
  (* -- Generiert und initialisert ein neues abstraktes Datenobjekt outIV *)
  VAR outIV : TIDVerwaltung;
  BEGIN
    (* -- Vorbed.: keine *)
    ALLOCATE (outIV, TSIZE (TIDVerwaltungDS));
    WITH outIV^ DO
       FreieIDs := SortierteObjektliste.Create (inDisposeObjekt,
                                                inDuplicateObjekt,
                                                inEqualizeObjekt,
                                                inIsEqualObjekt,
                                                inSmallerSeqNr,
                                                inSameSeqNr);
       IDFolge := 1;
       CreateID := inCreateID;
       CreateFolgeNr := inCreateFolgeNr
     END; (* WITH *)
     RETURN outIV
    (* -- Nachbed.: Rueckgabe-ADO outIV existiert *)
  END Create;

  PROCEDURE Dispose (VAR inoutIV : TIDVerwaltung);
  (* -- Entfernt das ADO inoutIV aus dem System *)
  BEGIN
    (* -- Vorbed.: keine *)
    SortierteObjektliste.Dispose (inoutIV^.FreieIDs);
    DEALLOCATE (inoutIV, TSIZE (TIDVerwaltungDS));
    (* -- Nachbed.: Das ADO inoutIV existiert nicht mehr *)
  END Dispose;

  PROCEDURE HoleNeueID (inoutIV : TIDVerwaltung) : TObjektreferenz;
  (* -- Liefert eine z.Z. unbenutzte ID *)
  VAR outID : TObjektreferenz;
  BEGIN
    (* -- Vorbed.: keine *)
    WITH inoutIV^ DO
      IF SortierteObjektliste.IsEmpty (FreieIDs) THEN
        outID := CreateID (IDFolge);
        INC (IDFolge)
      ELSE 
        SortierteObjektliste.GoSmallest (FreieIDs);
        outID := SortierteObjektliste.Current (FreieIDs);
        SortierteObjektliste.Delete (FreieIDs)
      END (* IF *)
    END; (* WITH *)
    RETURN outID
    (* -- Nachbed.: (outID existiert) AND (outID NOT IN inoutIV) *)
  END HoleNeueID;

  PROCEDURE UeberlasseAlteID (inoutIV : TIDVerwaltung; 
                              inID    : TObjektreferenz);
  (* -- Uebergibt eine z.Z. unbenutzte ID an die ID-Verwaltung *)
  BEGIN
    (* -- Vorbed.: inID NOT IN inoutIV *)
    SortierteObjektliste.Insert (inoutIV^.FreieIDs, inID);
    (* -- Nachbed.: inID IN inoutIV *)
  END UeberlasseAlteID;

  PROCEDURE GibNeueID (inIV : TIDVerwaltung): TObjektreferenz;
  (* -- Gibt die naechste neue ID zurueck *)
  VAR outID : TObjektreferenz;
  BEGIN
     (* -- Vorbed.: es existiert wenigsten eine ID *)
     outID := inIV^.CreateID (inIV^.IDFolge);
     RETURN outID
     (* Nachbed.: fr alle ID's i gilt:
                  i > outID: i ist noch nicht vergeben *) 
  END GibNeueID;

  PROCEDURE SetzeNeueID (inoutIV : TIDVerwaltung;
                         inID : TObjektreferenz);
  (* -- Berechnet zu einer ID die zugehrige Folgenummer und berschreibt
     -- damit die Instanzvariable IDFolge *)
  BEGIN
     (* -- Vorbed.: fuer alle ID's i:
                 i > inID: i ist noch nicht vergeben *)
     inoutIV^.IDFolge := inoutIV^.CreateFolgeNr (inID);
     (* Nachbed.: IDFolge = Folgenummer von inID *) 
  END SetzeNeueID;

  PROCEDURE FreieIDsVorhanden(inIV : TIDVerwaltung): BOOLEAN;
  VAR IDsVorhanden : BOOLEAN;
  BEGIN
  (* -- Vorbed.: keine *)
     IF SortierteObjektliste.IsEmpty (inIV^.FreieIDs) THEN
        IDsVorhanden := FALSE;
     ELSE 
        IDsVorhanden := TRUE;
     END; (* IF *)
     RETURN IDsVorhanden 
  (* -- Nachbed.: FreieIDsVorhanden(inIV) <=> FreieIDListe ist nicht leer *)
  END FreieIDsVorhanden;

  PROCEDURE ErsteFreieID(inIV : TIDVerwaltung): TObjektreferenz;
  (* -- gibt die erste Id der freien ID-Liste zurueck *)
  VAR outID : TObjektreferenz;
  BEGIN
     (* -- Vorbed.: FreieIDsVorhanden(inIV) *)
     WITH inIV^ DO
        IF SortierteObjektliste.IsEmpty (FreieIDs) THEN
           outID := CreateID (IDFolge);
        ELSE 
           SortierteObjektliste.GoSmallest (FreieIDs);
           outID := SortierteObjektliste.Current (FreieIDs);
        END (* IF *)
     END; (* WITH *)
     RETURN outID
     (* -- Nachbed.: keine *)
  END ErsteFreieID;

   PROCEDURE NaechsteFreieID(inIV : TIDVerwaltung; 
                             inoutID: TObjektreferenz): BOOLEAN;
  (* -- gibt die auf inID folgende freie ID der freien ID-Liste zurueck *)
  VAR outID : TObjektreferenz;
  BEGIN
  (* -- Vorbed.: inoutID existiert in der freien ID-Liste *)
     WITH inIV^ DO
        IF SortierteObjektliste.Search (FreieIDs, inoutID) THEN
           SortierteObjektliste.StepNext (FreieIDs);
           IF NOT SortierteObjektliste.OffEnd (FreieIDs) THEN
              inoutID := SortierteObjektliste.Current (FreieIDs);
              RETURN FALSE;
           ELSE
              RETURN TRUE
           END (* IF *)
        END (* IF *)
     END; (* WITH *)
  (* -- Nachbed.: NaechsteFreieID(inIV, inoutID) <=> inID ist die letzte ID 
                                                     der Liste *)
  END NaechsteFreieID;

END IDVerwaltung.
