(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: Kst.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:48:05 $
# $Revision: 1.1.1.1 $
Bemerkungen:

IMPLEMENTATION MODULE Kst   ADT-Modul Kostenstelle

Erstentwurf: Sigrid Preuss
Implementation : Herbert Schoenhammer, Herbert.Schoenhammer@Fernuni-Hagen.de
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE Kst;             (* ENTHALTEN IN Kostenstellen TS_Ksten *)


 FROM Storage IMPORT ALLOCATE, DEALLOCATE;
 FROM SYSTEM IMPORT TSIZE;
 FROM Datum IMPORT TDatum, TJahr, CreateDatum, SetzeDatumEternity, DisposeDatum,
                   EqualizeDatum, IsEqualDatum, SmallerDatum, LiesJahrInDatum,
                   IstDatumEternity, DuplicateDatum, CreateJahr, DisposeJahr,
                   IsEqualJahr, EqualizeJahr, DuplicateJahr, DatumVonString,
                   EqualizeJahr, SetzeDatumEternity ;
 FROM Stringliste  IMPORT TStringliste;
 FROM GENERIC IMPORT TObjektreferenz;

 IMPORT String;

 FROM KstID IMPORT TKstID, CreateKstID, DisposeKstID, IsEqualKstID,
                   EqualizeKstID, DuplicateKstID, SmallerKstID, SameKstID,
                   IstGesetztKID;

 TYPE TString = String.TString;

 TYPE TKstDS = RECORD
                KostenstellenID : TKstID;
                EingerichtetAm : TDatum;
                GesperrtAm : TDatum;
                LiquidiertAm : TDatum;
                Name : TString;
                Jahr : TJahr;
                Kontingente : CARDINAL;
                Aequivalent : CARDINAL;
               END;

 TYPE TKst = POINTER TO TKstDS;

 (* Operationen zur Erzeugung von Ausprgungen des Entittstyps Kostenstelle *)
 (*==========================================================================*)

 PROCEDURE CreateKst () : TKst;
  (* -- Vorbed.: keine
     -- Generiert ein neues abstraktes Datenobjekt Kostenstelle Kst vom Typ TKst
     -- Nachbed.: Rckgabe ADO Kst existiert *)
  VAR aktKst : TKst;
      tempDatum : TDatum;
      tempJahr : TJahr;
      egal : BOOLEAN;
  BEGIN
    (* -- Vorbed.: keine *)

    (* ADO vom Typ TJahr erzeugen *)
    tempDatum := CreateDatum();
    tempJahr := LiesJahrInDatum (tempDatum);

    ALLOCATE (aktKst, TSIZE (TKstDS)) ;
    WITH aktKst^ DO

       KostenstellenID := CreateKstID();

       EingerichtetAm := CreateDatum ();
       GesperrtAm := CreateDatum ();
       LiquidiertAm := CreateDatum ();

       Name := String.Create ();

       Jahr := CreateJahr ();
       EqualizeJahr (Jahr, tempJahr);

       Kontingente := 0;
       Aequivalent := 0;

    END; (* WITH *)

    DisposeDatum (tempDatum);
    DisposeJahr (tempJahr);

    RETURN aktKst
  END CreateKst;
        

 (* TYP-Konvertierung fuer OrdnerSchablone und Ordner                        *)
 (*==========================================================================*)

  PROCEDURE OBJEKTREFERENZ (inObjekt : TKst) : TObjektreferenz;
  (* -- Typtransfer von TKst nach TObjektreferenz *)
  VAR outRef : TObjektreferenz;
  BEGIN
    (* -- Vorbed.: keine *)
    outRef := inObjekt;
    RETURN outRef
     (* -- Nachbed.: inObjekt = BELEG (outRef)  *)
  END OBJEKTREFERENZ;

  PROCEDURE KOSTENSTELLE (inObjektreferenz : TObjektreferenz) : TKst;
  (* -- Typtransfer von TObjektreferenz nach TKst *)
  VAR outObjekt : TKst;
  BEGIN
    (* -- Vorbed.: inObjektreferenz referenziert ein Beleg-Objekt *)
    outObjekt := inObjektreferenz;
    RETURN outObjekt
    (* -- Nachbed.: Referenz von outObjekt = inObjektreferenz *)
  END KOSTENSTELLE;


 (* Standard-Operationen auf dem Entittstyp Kostenstellen                   *)
 (*==========================================================================*)

 PROCEDURE DisposeKst (VAR inoutKst : TKst);
  (* -- Vorbed.: keine
     -- Entfernt das abstraktes Datenobjekt inoutKst aus dem System
     -- Nachbed.: ADO inoutKst existiert *)
  BEGIN
    WITH inoutKst^ DO
       DisposeKstID (KostenstellenID);
       DisposeDatum (EingerichtetAm);
       DisposeDatum (GesperrtAm);
       DisposeDatum (LiquidiertAm);
       String.Dispose (Name);
       DisposeJahr (Jahr);
    END; (* WITH *)
    DEALLOCATE (inoutKst, TSIZE (TKstDS));
    inoutKst := NIL
  END DisposeKst;


 PROCEDURE DuplicateKst (inQuellKst : TKst) : TKst;
  (* -- Vorbed.: keine
     -- Erzeugt ein neues Kst-Objekt und gleicht dieses an den Zustand bzw. 
     -- die Attribute von inQuellKst an. Bezeichnet Duplikat das Rueckgabe-ADO
     -- der Operation, so gilt:
     -- Nachbed.: IsEqual (Duplikat, inQuellKst) AND NOT (Duplikat = inQuellKst)
  *)
  VAR aktDuplikat : TKst;
  BEGIN
      aktDuplikat := CreateKst();
      EqualizeKst (aktDuplikat, inQuellKst);
      RETURN aktDuplikat;
  END DuplicateKst;

 PROCEDURE IsEqualKst (inKst1, inKst2 : TKst) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Ueberprueft die beiden Kostenstellen auf Zustandsgleichheit 
     -- Nachbed.: keine; *)
  VAR gleich : BOOLEAN;
  BEGIN
    gleich :=
            IsEqualKstID (inKst1^.KostenstellenID, inKst2^.KostenstellenID)
        AND IsEqualDatum (inKst1^.EingerichtetAm, inKst2^.EingerichtetAm)
        AND IsEqualDatum (inKst1^.GesperrtAm, inKst2^.GesperrtAm) 
        AND IsEqualDatum (inKst1^.LiquidiertAm, inKst2^.LiquidiertAm)
        AND String.IsEqual (inKst1^.Name, inKst2^.Name)
        AND IsEqualJahr (inKst1^.Jahr, inKst2^.Jahr)
        AND (inKst1^.Kontingente = inKst2^.Kontingente)
        AND  (inKst1^.Aequivalent = inKst2^.Aequivalent);
    RETURN gleich
  END IsEqualKst;


 PROCEDURE EqualizeKst (inoutZielKst, inQuellKst : TKst);
  (* -- Vorbed.: keine
     -- Gleicht den Zustand /die Attribute der ZielKst an die QuellKst an
     -- Nachbed.: IsEqual (inQuellKst, inoutZielKst) *)
 BEGIN
   WITH inoutZielKst^ DO
      EqualizeKstID (KostenstellenID, inQuellKst^.KostenstellenID);
      EqualizeDatum (EingerichtetAm, inQuellKst^.EingerichtetAm);
      EqualizeDatum (GesperrtAm, inQuellKst^.GesperrtAm);
      EqualizeDatum (LiquidiertAm, inQuellKst^.LiquidiertAm);
      String.Equalize (Name, inQuellKst^.Name);
      EqualizeJahr (Jahr, inQuellKst^.Jahr);
      Kontingente := inQuellKst^.Kontingente;
      Aequivalent := inQuellKst^.Aequivalent;
   END; (* WITH *)
 END EqualizeKst;
      

 PROCEDURE SmallerKst (inSmallerKst, inGreaterKst : TKst) : BOOLEAN;
 BEGIN
   RETURN SmallerKstID (inSmallerKst^.KostenstellenID,
                        inGreaterKst^.KostenstellenID)
 END SmallerKst;


 PROCEDURE SameKst (inKst1, inKst2 : TKst) : BOOLEAN;
 BEGIN
   RETURN SameKstID (inKst1^.KostenstellenID,
                     inKst2^.KostenstellenID)
 END SameKst;


 (* Operationen bezogen auf die extern zu vergebende KstID*)
 (*=======================================================*)

 PROCEDURE SetzeKIDinKst (inK  : TKstID; inoutKst: TKst);
  (* -- Vorbed.: keine
     -- Setzt im ADO die extern vergebene KostenstellenID. 
     -- Die Prfung auf bereits vorhandene KostenstellenID mu
     -- vorher mittels ExistKst geprft sein.
     -- Nachbed.: inoutKst.ID := inK *)
 BEGIN
   EqualizeKstID (inoutKst^.KostenstellenID, inK);
 END SetzeKIDinKst;   

     
 PROCEDURE LiesKIDinKst ( inKst: TKst):TKstID;
 VAR aktKstID : TKstID;
 BEGIN
   aktKstID := DuplicateKstID (inKst^.KostenstellenID);
   RETURN aktKstID
 END LiesKIDinKst;


 PROCEDURE IstGesetztKstID (inKst : TKst): BOOLEAN;
 BEGIN
   RETURN  IstGesetztKID (inKst^.KostenstellenID)
 END IstGesetztKstID;


 (* Standard- und Prf-Operationen auf den Attributen des Entittstyps         *)
 (* Kostenstelle                                                               *)
 (*============================================================================*)


 PROCEDURE SetzeEingAminKst (inoutKst: TKst; inD  : TDatum);
 BEGIN
   EqualizeDatum (inoutKst^.EingerichtetAm, inD);
 END SetzeEingAminKst;

 PROCEDURE LiesEingAminKst ( inKst: TKst):TDatum;
 VAR aktDatum : TDatum;
 BEGIN
   aktDatum := DuplicateDatum (inKst^.EingerichtetAm);
   RETURN aktDatum
 END LiesEingAminKst;

 PROCEDURE SetzeGesperrtAminKst (inoutKst: TKst; inD  : TDatum);
 BEGIN
   EqualizeDatum (inoutKst^.GesperrtAm, inD);
 END SetzeGesperrtAminKst;

 PROCEDURE LiesGesperrtAminKst ( inKst: TKst):TDatum;
 VAR aktDatum : TDatum;
 BEGIN
   aktDatum := DuplicateDatum (inKst^.GesperrtAm);
   RETURN aktDatum
 END LiesGesperrtAminKst;

 PROCEDURE IstGesperrtAminKst ( inKst: TKst):BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob Datum GesperrtAm ungleich dem default Datum 31.12.9999 ist
     -- Nachbed.: (TRUE <=> (inKSt.GespertAm <> 31.12.9999) AND
                  (FALSE <=> (inKST.GesperrtAm = 31.12.9999 ) *)
 VAR not_istGesperrt : BOOLEAN;
 BEGIN
   not_istGesperrt := IstDatumEternity (inKst^.GesperrtAm);
   RETURN (NOT not_istGesperrt)
 END IstGesperrtAminKst;

 PROCEDURE SetzeLiquidAminKst (inoutKst: TKst; inD  : TDatum);
 BEGIN
   EqualizeDatum (inoutKst^.LiquidiertAm, inD);
 END SetzeLiquidAminKst;

 PROCEDURE LiesLiquidAminKst ( inKst: TKst):TDatum;
 VAR aktDatum : TDatum;
 BEGIN
   aktDatum := DuplicateDatum (inKst^.LiquidiertAm);
   RETURN aktDatum
 END LiesLiquidAminKst;

 PROCEDURE IstLiquidiertAminKst ( inKst: TKst):BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob Datum LiquidiertAm ungleich dem default Datum 31.12.9999 ist
     -- Nachbed.: (TRUE <=> (inKSt.LiquidiertAm <> 31.12.9999) AND
                  (FALSE <=> (inKST.LiquidiertAm = 31.12.9999 ) *)
 VAR not_istLiquid : BOOLEAN;
 BEGIN
   not_istLiquid := IstDatumEternity (inKst^.LiquidiertAm);
   RETURN NOT not_istLiquid
 END IstLiquidiertAminKst;

 PROCEDURE SetzeNameinKst ( inoutKst: TKst; inN: TString);
 BEGIN
   String.Equalize (inoutKst^.Name, inN)
 END SetzeNameinKst;

 PROCEDURE LiesNameinKst ( inKst: TKst):TString;
 VAR aktText : TString;
 BEGIN
   aktText := String.Duplicate (inKst^.Name);
   RETURN aktText
 END LiesNameinKst;

 PROCEDURE SetzeJahrinKst ( inoutKst: TKst; inJahr: TJahr);
 BEGIN
   EqualizeJahr (inoutKst^.Jahr, inJahr);
 END SetzeJahrinKst;

 PROCEDURE LiesJahrinKst ( inKst: TKst):TJahr;
 BEGIN
   RETURN DuplicateJahr (inKst^.Jahr);
 END LiesJahrinKst;

 PROCEDURE SetzeKontinginKst ( inoutKst: TKst; inKonti: CARDINAL);
 BEGIN
   inoutKst^.Kontingente := inKonti;
 END SetzeKontinginKst;

 PROCEDURE LiesKontinginKst ( inKst: TKst):CARDINAL;
 VAR konting : CARDINAL;
 BEGIN
   konting := inKst^.Kontingente;
   RETURN konting
 END LiesKontinginKst;

 PROCEDURE SetzeAequivinKst ( inoutKst: TKst; inAequi: CARDINAL);
 BEGIN
   inoutKst^.Aequivalent := inAequi;
 END SetzeAequivinKst;

 PROCEDURE LiesAequivinKst ( inKst: TKst):CARDINAL;
 VAR aequiv : CARDINAL;
 BEGIN
   aequiv := inKst^.Aequivalent;
   RETURN aequiv
 END LiesAequivinKst;


END Kst.







