(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: KstOSchablone.mod,v $
# $Author: Sopra $
# $Date: 2001/03/13 08:30:51 $
# $Revision: 1.6 $
Bemerkungen:

IMPLEMENTATION MODULE KstenOrdnerSchablone   ADT-Modul KostenstelleOrdnerSchablone

Implementation : Herbert Schoenhammer, Herbert.Schoenhammer@Fernuni-Hagen.de

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE KstOSchablone;   (* ENTHALTEN IN Kostenstellen TS_Ksten *)

  IMPORT Kst;
  IMPORT KstID;

  FROM KstenIMPORTE IMPORT LiesSystemDatumInSysDat,
                           ReadLineInVDat,
                           WriteLineInVDat,
                           EofInVDat,
                           THandle;

  IMPORT String;
  IMPORT Stringliste;
  FROM Datum IMPORT TDatum, TJahr,
                    CreateDatum, DisposeDatum, SetzeDatumEternity,
                    SetzeJahrInDatum, DatumVonString, DatumAlsString,
                    DisposeJahr, LiesJahrInDatum, JahrVonString, JahrAlsString;
  FROM WholeStr IMPORT ConvResults, CardToStr, StrToCard;
  FROM InOut IMPORT WriteString, WriteLn;

  IMPORT SortierteObjektliste;

  TYPE TKstenOrdner     = SortierteObjektliste.TSortierteObjektliste;
       TObjektreferenz  = SortierteObjektliste.TObjektreferenz;
       TString = String.TString;

  TYPE TStringliste = Stringliste.TStringliste;


(* ------------------------------------------------------------------
   -- StandardOperationen fuer SortierteObjektliste
   ------------------------------------------------------------------ *)


  PROCEDURE CreateKstenOrdner () : TKstenOrdner;
  (*    -- Generiert ein neues abstraktes Datenobjekt outListe *)
    VAR outListe : TKstenOrdner;
  BEGIN
    (* -- Generiert ein neues abstraktes Datenobjekt outListe *)
    outListe := SortierteObjektliste.Create (DisposeKst,
                                             DuplicateKst,
                                             EqualizeKst,
                                             IsEqualKst,
                                             SmallerKstID,
                                             SameKstID    );
    RETURN outListe
    (* -- Nachbed.: ADO outListe existiert AND IsEmpty (outListe) *)
  END CreateKstenOrdner;


  (* -- GEMEINSAME VORBEDINGUNG FUER ALLE UEBRIGEN OPERATIONEN:  
     --     ADOs inListe bzw. inoutListe existieren *)


  (* PROCEDURE DisposeKstenOrdner (VAR inoutListe : TKstenOrdner);
  BEGIN
    SortierteObjektliste.Dispose (inoutListe);
  END DisposeKstenOrdner; *)


  PROCEDURE CurrentInKstenOrdner(inListe : TKstenOrdner) : TKst;
  (* -- Liefert die Kostenstelle an der aktuellen Position *)
  BEGIN
    (* -- Vorbed.:NOT (OffStart (inListe) OR OffEnd (inListe)) *)
    RETURN Kst.KOSTENSTELLE (SortierteObjektliste.Current (inListe))
    (* -- Nachbed.: NOT (OffStart (inListe) OR OffEnd (inListe)) *)
  END CurrentInKstenOrdner;


  PROCEDURE UpdateInKstenOrdner (inoutListe : TKstenOrdner;
                    inObjekt   : TKst);
  (* -- Ueberschreibt die Attributwerte der Kostenstelle an der aktuellen 
     -- Position mit denen von inObjekt, 
     -- d.h. anschliessend sind die Kst an der aktuellen
     -- Position und inObjekt zustandsgleich *)
  BEGIN
    (* -- Vorbed.: NOT (OffStart (inoutListe) OR OffEnd (inoutListe))  *)
    SortierteObjektliste.Update (inoutListe, Kst.OBJEKTREFERENZ (inObjekt))
    (* -- Nachbed.: NOT (OffStart (inoutListe) OR OffEnd (inoutListe)  *);
  END UpdateInKstenOrdner;


  PROCEDURE InsertInKstenOrdner (inoutListe : TKstenOrdner;
                    inObjekt   : TKst);
  (* -- Einfugen eines Objekts bzgl. der Sortierreihenfolge
     -- Anschliessend befindet sich inObjekt an der aktuellen Position *)
   
  BEGIN
    (* -- Vorbed.: IsEmpty (inoutListe) OR NOT (IsFull (inoutListe) 
       -- OR OffStart(inoutListe))  *)
    SortierteObjektliste.Insert (inoutListe, Kst.OBJEKTREFERENZ (inObjekt))
    (* -- Nachbed.: NOT (OffStart (inoutListe) OR OffEnd (inoutListe) *)
  END InsertInKstenOrdner;


  PROCEDURE DeleteInKstenOrdner (inoutListe : TKstenOrdner); 
  (* -- Loeschen der Kst an der aktuellen Position; das Listenelement rechts
     -- neben dem zu loeschenden Element befindet sich anschliessend an der 
     -- (unveranderten) aktuellen Position. 
     -- Wird die letzte Kst geloescht, steht die aktuelle Position auf OffEnd *)
  BEGIN 
    (* -- Vorbed.: NOT (OffStart (inoutListe) OR OffEnd (inoutListe)) *)
    SortierteObjektliste.Delete (inoutListe); 
    (* -- Nachbed.: (inoutListe.len_neu = inoutListe.len_alt - 1 ) 
                  AND NOT (IsFull (inoutListe) OR OffStart(inoutListe)) *)
  END DeleteInKstenOrdner; 


  (* PROCEDURE SearchInKstenOrdner (inoutListe : TKstenOrdner;
                    inObjekt   : TKst) : BOOLEAN; *)
  (* -- Suche nach inObjekt wird ein zustandsgleiches Objekt gefunden, 
     -- ist dort die neue aktuelle Position; sonst OffEnd *)
  (* BEGIN *)
    (* -- Vorbed.: keine *)
    (* RETURN SortierteObjektliste.Search (inoutListe, Kst.OBJEKTREFERENZ (inObjekt)) *)
    (* -- Nachbed.: (outfound => NOT (OffStart (inoutListe) OR 
       -- OffEnd (inoutListe))) AND (NOT outfound => OffEnd (inoutListe)) *)
  (* END SearchInKstenOrdner; *)


  PROCEDURE GoSmallestInKstenOrdner (inoutListe : TKstenOrdner);
  (* -- Die Position des Objekts mit der kleinsten ID wird zur aktuellen 
     -- Position *)
  BEGIN
    (* -- Vorbed.: keine *)
    SortierteObjektliste.GoSmallest (inoutListe);
    (* -- Nachbed.: inoutListe.pos_neu = 1 *)
  END GoSmallestInKstenOrdner;

 
  PROCEDURE StopAtInKstenOrdner (inoutListe: TKstenOrdner;
                                 inObjekt : TObjektreferenz) : BOOLEAN;
   (* -- Vorbed.: keine
      -- Suche nach einem bzgl. SameValue Objekt gleichen Objekt. Wird ein
      -- solches Objekt gefunden, ist dort die neue aktuelle Position *)
  BEGIN
    RETURN SortierteObjektliste.StopAt(inoutListe, inObjekt)
  END StopAtInKstenOrdner;


  (* PROCEDURE GoLargestInKstenOrdner (inoutListe : TKstenOrdner); *)
  (* -- Die Position des Objekts mit der groessten ID wird zur aktuellen 
     -- Position *)
  (* BEGIN *)
    (* -- Vorbed.: keine *)
    (* SortierteObjektliste.GoLargest (inoutListe); *)
    (* -- Nachbed.: inoutListe.pos_neu = inoutListe.len_alt *)
  (* END GoLargestInKstenOrdner; *)


  PROCEDURE StepNextInKstenOrdner (inoutListe : TKstenOrdner);
  (* -- Die Position hinter der aktuellen Position wird zur aktuellen 
     -- Position *)
  BEGIN
    (* -- Vorbed.: NOT OffEnd (inoutListe) *)
    SortierteObjektliste.StepNext (inoutListe);
    (* -- Nachbed.: (inoutListe.pos_neu = inoutListe.pos_alt + 1) 
       --           OR (IsEmpty (inoutListe) => OffEnd (inoutListe) *)
  END StepNextInKstenOrdner;


  (* PROCEDURE StepBackInKstenOrdner (inoutListe : TKstenOrdner); *)
  (* Die vorherige Position wird zur aktuellen Position. *)
  (* BEGIN *)
    (* -- Vorbed.: OffStart (inoutListe) *)
    (* SortierteObjektliste.StepBack (inoutListe); *)
    (* -- Nachbed.: (inoutListe.pos_neu = inoutListe.pos_alt - 1) 
       --           OR (IsEmpty (inoutListe) => OffStart (inoutListe) *)
  (* END StepBackInKstenOrdner; *)


  PROCEDURE IsEmptyInKstenOrdner (inListe : TKstenOrdner) : BOOLEAN;
  (* -- Test, ob die Liste leer ist: IsEmpty (inListe) <=> (inListe.len = 0) *)
  BEGIN
    (* -- Vorbed.: keine *)
    RETURN SortierteObjektliste.IsEmpty (inListe);
    (* -- Nachbed.:IsEmpty (inListe) => OffStart (inListe) OR OffEnd (inListe)*)
  END IsEmptyInKstenOrdner;


  PROCEDURE IsFullInKstenOrdner (inListe : TKstenOrdner) : BOOLEAN;
  (* -- Test, ob die Liste voll ist: IsFull (inListe) <=> OL.len = inListe.maxlen *)
  BEGIN
    (* -- Vorbed.: keine *)
    RETURN SortierteObjektliste.IsFull (inListe);
    (* -- Nachbed.: keine *)
  END IsFullInKstenOrdner;


  (* PROCEDURE OffStartInKstenOrdner (inListe : TKstenOrdner) : BOOLEAN; *)
  (* -- Test, ob die Position vor dem ersten Objekt aktuell ist: 
     -- OffStart (inListe) <=> inListe.pos = 0 *)
  (* BEGIN *)
    (* -- Vorbed.: keine *)
    (* RETURN SortierteObjektliste.OffStart (inListe); *)
    (* -- Nachbed.: keine *)
  (* END OffStartInKstenOrdner;*)


  PROCEDURE OffEndInKstenOrdner (inListe : TKstenOrdner) : BOOLEAN;
  (* Test, ob die Position nach dem letzten Listenelement aktuell ist. *)
  BEGIN
    (* -- Vorbed.: keine *)
    RETURN SortierteObjektliste.OffEnd (inListe);
    (* -- Nachbed.: keine *)
  END OffEndInKstenOrdner;


(* ------------------------------------------------------------------
   -- Typkonvertierungen von TKst und TObjektreferenz
   ------------------------------------------------------------------ *)


  PROCEDURE DisposeKst (VAR inoutReferenz : TObjektreferenz);
  (* -- ... *)
  VAR aktKst : TKst;
  BEGIN
    (* -- Vorbed.: ... *)
    aktKst := Kst.KOSTENSTELLE (inoutReferenz);
    Kst.DisposeKst (aktKst);
    inoutReferenz := Kst.OBJEKTREFERENZ (aktKst);
    (* -- Nachbed.: ... *)
  END DisposeKst;


  PROCEDURE DuplicateKst (inReferenz : TObjektreferenz) : TObjektreferenz;
  (* -- ... *)
  VAR Source,
      outDuplikat : TKst;
  BEGIN
    (* -- Vorbed.: ... *)
    Source := Kst.KOSTENSTELLE (inReferenz);
    outDuplikat := Kst.DuplicateKst (Source);
    RETURN Kst.OBJEKTREFERENZ (outDuplikat)
    (* -- Nachbed.: ... *)
  END DuplicateKst;


  PROCEDURE EqualizeKst (inoutZiel, inQuelle : TObjektreferenz);
  (* -- ... *)
  VAR ZielKst,
      QuellKst : TKst;
  BEGIN
    (* -- Vorbed.: ... *)
    ZielKst := Kst.KOSTENSTELLE (inoutZiel);
    QuellKst := Kst.KOSTENSTELLE (inQuelle);
    Kst.EqualizeKst (ZielKst, QuellKst)
    (* -- Nachbed.: ... *)
  END EqualizeKst;


  PROCEDURE IsEqualKst (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  (* -- ... *)
  VAR Kst1,
      Kst2 : TKst;
  BEGIN
    (* -- Vorbed.: ... *)
    Kst1 := Kst.KOSTENSTELLE (inReferenz1);
    Kst2 := Kst.KOSTENSTELLE (inReferenz2);
    RETURN Kst.IsEqualKst (Kst1, Kst2)
    (* -- Nachbed.: ... *)
  END IsEqualKst;


  PROCEDURE SameKstID (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  (* -- ... *)
  VAR Kst1, Kst2 : TKst;
  BEGIN
    (* -- Vorbed.: ... *)
    Kst1 := Kst.KOSTENSTELLE (inReferenz1);
    Kst2 := Kst.KOSTENSTELLE (inReferenz2);
    RETURN Kst.SameKst (Kst1,
                        Kst2)
    (* -- Nachbed.: ... *)
  END SameKstID;


  PROCEDURE SmallerKstID (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  (* -- ... *)
  VAR Kst1, Kst2 : TKst;
  BEGIN
    (* -- Vorbed.: ... *)
    Kst1 := Kst.KOSTENSTELLE (inReferenz1);
    Kst2 := Kst.KOSTENSTELLE (inReferenz2);
    RETURN Kst.SmallerKst (Kst1,
                           Kst2)
    (* -- Nachbed.: ... *)
  END SmallerKstID;


(*============================================================================*)	
(* Modul-Lokale Prozeduren                                                    *)
(*============================================================================*)


 PROCEDURE LoescheAllesInKstenOrdner (inoutListe : TKstenOrdner );
 (*
    vb: keine
        Loescht alle Kostenstellen aus inoutListe. Wird bentigt, um die
        Liste vor dem Restaurieren des Ordners von Festplatte zu leeren.
    nb: Kostenstellenordner ist leer *)

 BEGIN
   
   GoSmallestInKstenOrdner (inoutListe);
   WHILE NOT IsEmptyInKstenOrdner (inoutListe) DO    
      DeleteInKstenOrdner (inoutListe);
   END; (* WHILE *)

END LoescheAllesInKstenOrdner;


(*============================================================================*)
(* HiKS-spezifische Operationen                                               *)
(*============================================================================*)



 PROCEDURE InsertKst (inK: TKst; VAR outMessage : TString;
                      inoutListe : TKstenOrdner; inVariante : BOOLEAN):BOOLEAN;
 (*
    vb: sei K die neu einzufgende Kostenstelle. Dann muss gelten: 
        ADO Konditionsordner existiert and not isfull and KID ist zugewiesen

    --  Bei inVariante = TRUE wird die Kostenstelle inK gemaess der
        Prozessspezifikationen D3.2.2 in den KostenstellenOrdner eingefuegt; 
        d.h. im Attribut EingerichtetAm wird das aktuelle HiKS-Tagesdatum 
        gesetzt.

    --  Bei inVariante = FALSE wird die Kostenstelle unveraendert in den Ordner
        eingefuegt.

    nb: K ist in Kostenstellenordner eingefgt *)

 VAR heute : TDatum;
     erfolgreich : BOOLEAN;
     aktKst : TKst;
 BEGIN

   outMessage := String.Create();

   IF ExistKst (inK, inoutListe)

   THEN  (*Fehler: KostenstellenID schon vergeben *)
     erfolgreich := FALSE;
     String.Schreibe (outMessage, "Kostenstelle bereits vorhanden")
   ELSIF IsFullInKstenOrdner (inoutListe) 
   THEN  (* Fehler: Ordner ist voll *)
     erfolgreich := FALSE;
     String.Schreibe (outMessage, "Ordner ist bereits voll")

   ELSIF NOT Kst.IstGesetztKstID (inK) (* Fehler: Noch keine KstID gesetzt *)
   THEN
     erfolgreich := FALSE;
     String.Schreibe (outMessage, "KostenstellenID ist noch nicht gesetzt");

   ELSE  (* Kostenstelle in den Ordner einfuegen lt. Prozess D3.2.2 *)

     aktKst := DuplicateKst (inK);

     IF inVariante THEN
        (* Lt. Prozespezifikation das Tagesdatum setzen *)
        heute := LiesSystemDatumInSysDat();
        Kst.SetzeEingAminKst (aktKst, heute);
        DisposeDatum (heute);
     END; (* IF *)

     InsertInKstenOrdner (inoutListe, aktKst);
     erfolgreich := TRUE;
     String.Dispose (outMessage);
     Kst.DisposeKst (aktKst);

   END; (* IF *)

   RETURN erfolgreich
 END InsertKst;



 PROCEDURE ExistKst(inKst: TKst; inoutListe : TKstenOrdner):BOOLEAN;
 (* vb: keine
        Prft, ob die KstID zur eingegebenen Kst bereits existiert; wenn ja,
	gibt sie TRUE zurck, sonst FAlSE
    nb: Die gefundene Kostenstelle ist an der aktuellen Position in der Liste *)
 BEGIN
   RETURN StopAtInKstenOrdner (inoutListe, Kst.OBJEKTREFERENZ(inKst))
 END ExistKst;


 PROCEDURE FirstKst (VAR outKst:TKst; inoutListe:TKstenOrdner):BOOLEAN;
 (* vb: Keine
	Liefere die erste Kostenstelle im Ordner zurck. Sie rc der boolesche
        Rueckgabewert, dann gilt
    nb: ADO outKst existiert AND
        rc=TRUE <=> erste Kostenstelle im Ordner existiert AND
                    IsEqualKst (outKst, erste Kst im Ordner)
        rc=FALSE <=> keine Kostenstelle im Ordner existiert               *)
 VAR existfirst : BOOLEAN;
 BEGIN
   IF IsEmptyInKstenOrdner (inoutListe)
   THEN  (* Fehler: Liste ist leer *)
     existfirst := FALSE
   ELSE  (* Kst mit kleinster ID ermitteln und zurueckgeben *)
     GoSmallestInKstenOrdner (inoutListe);
     outKst := CurrentInKstenOrdner(inoutListe);
     existfirst := TRUE
   END; (* IF *)
   RETURN existfirst
 END FirstKst;


 PROCEDURE NextKst (inK:TKst; inoutK:TKst; inoutListe:TKstenOrdner):BOOLEAN;
 (* vb: inK existiert in KostenstellenOrdner
	  Suche inK im KostenstellenOrdner und falls ein Nachfolger outK existiert 
        gib diesen zurck, false sonst.
    nb: -- *)
 VAR existnext : BOOLEAN;
     aktKst : TKst;
 BEGIN
   IF ExistKst (inK, inoutListe)
   THEN   (* inK gefunden; Nachfolger ermitteln *)
     StepNextInKstenOrdner (inoutListe); (* zum Nachfolger steppen *)
     IF NOT OffEndInKstenOrdner (inoutListe)  (* Nachfolger existiert *)
     THEN
          aktKst := CurrentInKstenOrdner(inoutListe);
          Kst.EqualizeKst (inoutK, aktKst);
          Kst.DisposeKst (aktKst);
          existnext := TRUE;
     ELSE  (* Nachfolger existiert nicht *)
          existnext := FALSE;
     END; (* IF NOT OffEndInKstenOrdner *)
   ELSE   (* inK existiert nicht => auch kein Nachfolger *)
      existnext := FALSE
   END; (* IF ExistKst *)
   RETURN existnext
 END NextKst;


 PROCEDURE FirstLfdKst (VAR outKst : TKst; inoutListe:TKstenOrdner) : BOOLEAN;
 (* Reini :
    gibt die erste Kst zurck, die weder liquidiert noch gesperrt ist *)
 VAR existlfdkst, dispose : BOOLEAN;
     aktKst : TKst;
 BEGIN
   IF IsEmptyInKstenOrdner (inoutListe)
   THEN  (* Fehler: Liste ist leer *)
     existlfdkst := FALSE;
   ELSE (* Sequentielles Durchsuchen der inoutListe *)
     GoSmallestInKstenOrdner (inoutListe);
     aktKst := CurrentInKstenOrdner(inoutListe);
     dispose := TRUE;
     WHILE (NOT OffEndInKstenOrdner (inoutListe)) (* Durchsuchen der Liste *)
       AND ( Kst.IstGesperrtAminKst (aktKst)
       OR    Kst.IstLiquidiertAminKst (aktKst) )
     DO
       StepNextInKstenOrdner (inoutListe);
       Kst.DisposeKst (aktKst);
       dispose := FALSE;

       IF NOT OffEndInKstenOrdner (inoutListe) THEN
          aktKst := CurrentInKstenOrdner(inoutListe);
          dispose := TRUE;
       END;

     END; (*WHILE *)

     IF dispose THEN
        Kst.DisposeKst (aktKst);
     END;

     IF OffEndInKstenOrdner (inoutListe)
     THEN   (* Nichts gefunden  *)
        existlfdkst := FALSE;
     ELSE   (* Rueckgabe der Kostenstelle *)
        outKst := CurrentInKstenOrdner(inoutListe);
        existlfdkst := TRUE
     END; (* IF OffEndInKstenOrdner *)
   END; (* IF IsEmptyInKstenOrdner *)
   RETURN existlfdkst
 END FirstLfdKst;


 PROCEDURE NextLfdKst (inoutKst : TKst; inoutListe:TKstenOrdner) : BOOLEAN;
 (* Reini :
    gibt die naechste Kst zurck, die weder liquidiert noch gesperrt ist *)
 VAR existlfdkst : BOOLEAN;
     aktKst : TKst;
 BEGIN

   IF IsEmptyInKstenOrdner (inoutListe)
   THEN  (* Fehler: Liste ist leer *)
     existlfdkst := FALSE;
   ELSE

     IF ExistKst (inoutKst, inoutListe)
     THEN (* inoutKst gefunden und an aktueller Position *)

       IF (NOT OffEndInKstenOrdner (inoutListe))
       THEN 
          StepNextInKstenOrdner (inoutListe);

          (*Durchsuchen der Liste*)
          existlfdkst := FALSE;
          WHILE (NOT OffEndInKstenOrdner (inoutListe)) AND (NOT existlfdkst)
          DO
            aktKst := CurrentInKstenOrdner(inoutListe);
 
            IF NOT (Kst.IstGesperrtAminKst(aktKst)
                                          OR Kst.IstLiquidiertAminKst(aktKst))
            THEN 
              Kst.EqualizeKst (inoutKst, aktKst );
              existlfdkst := TRUE
            ELSE
              StepNextInKstenOrdner (inoutListe);
            END; (* IF (Kst.IstGesperrt ...*)

            Kst.DisposeKst (aktKst);
          END; (*WHILE *)

       ELSE   (* inoutKst war schon letzte Kst der Liste *)
          existlfdkst := FALSE
       END; (* IF (NOT OffEndInKstenOrdner  *)

     ELSE  (* inoutKst nicht gefunden *)
       existlfdkst := FALSE
     END; (* IF ExistKst *)

   END; (* IF IsEmptyInKstenOrdner *)

   RETURN existlfdkst

 END NextLfdKst;


 PROCEDURE FirstAktKst (VAR outKst : TKst; inoutListe:TKstenOrdner) : BOOLEAN;
 (* Gabi :
    gibt die erste Kst zurck, die nicht liquidiert *)
 VAR existlfdkst, dispose : BOOLEAN;
     aktKst : TKst;

 BEGIN
   IF IsEmptyInKstenOrdner (inoutListe)
   THEN  (* Fehler: Liste ist leer *)
     existlfdkst := FALSE;
   ELSE (* Sequentielles Durchsuchen der inoutListe *)
     GoSmallestInKstenOrdner (inoutListe);
     aktKst := CurrentInKstenOrdner(inoutListe);
     dispose := TRUE;

     WHILE (NOT OffEndInKstenOrdner (inoutListe)) (* Durchsuchen der Liste *)
       AND  Kst.IstLiquidiertAminKst (aktKst)
     DO
       StepNextInKstenOrdner (inoutListe);
       Kst.DisposeKst (aktKst);
       dispose := FALSE;
       IF NOT OffEndInKstenOrdner (inoutListe) THEN
          aktKst := CurrentInKstenOrdner(inoutListe);
          dispose := TRUE;
       END;
     END; (*WHILE *)

     IF dispose THEN
        Kst.DisposeKst (aktKst);
     END;

     IF OffEndInKstenOrdner (inoutListe)
     THEN   (* Nichts gefunden  *)
        existlfdkst := FALSE;
     ELSE   (* Rueckgabe der Kostenstelle *)
        outKst := CurrentInKstenOrdner(inoutListe);
        existlfdkst := TRUE
     END; (* IF OffEndInKstenOrdner *)
   END; (* IF IsEmptyInKstenOrdner *)

   RETURN existlfdkst
 END FirstAktKst;


 PROCEDURE NextAktKst (inoutKst : TKst; inoutListe:TKstenOrdner) : BOOLEAN;
 (* Gabi :
    gibt die naechste Kst zurck, die nicht liquidiert ist *)
 VAR existaktkst : BOOLEAN;
     aktKst : TKst;
 BEGIN

   IF IsEmptyInKstenOrdner (inoutListe)
   THEN  (* Fehler: Liste ist leer *)
     existaktkst := FALSE;
   ELSE

     IF ExistKst (inoutKst, inoutListe)
     THEN (* inoutKst gefunden und an aktueller Position *)

       IF (NOT OffEndInKstenOrdner (inoutListe))
       THEN 
          StepNextInKstenOrdner (inoutListe);

          (*Durchsuchen der Liste*)
          existaktkst := FALSE;
          WHILE (NOT OffEndInKstenOrdner (inoutListe)) AND (NOT existaktkst)
          DO
            aktKst := CurrentInKstenOrdner(inoutListe);
 
            IF NOT Kst.IstLiquidiertAminKst(aktKst)
            THEN 
              Kst.EqualizeKst (inoutKst, aktKst );
              existaktkst := TRUE
            ELSE
              StepNextInKstenOrdner (inoutListe);
            END; (* IF (Kst.IstGesperrt ...*)

            Kst.DisposeKst (aktKst);
          END; (*WHILE *)

       ELSE   (* inoutKst war schon letzte Kst der Liste *)
          existaktkst := FALSE
       END; (* IF (NOT OffEndInKstenOrdner  *)

     ELSE  (* inoutKst nicht gefunden *)
       existaktkst := FALSE
     END; (* IF ExistKst *)

   END; (* IF IsEmptyInKstenOrdner *)

   RETURN existaktkst

 END NextAktKst;


 PROCEDURE SearchKstZuKID ( inKID: TKstID; VAR outKst : TKst;
                                           inoutListe:TKstenOrdner):BOOLEAN;
 (* vb: inKID ist gueltig
        Gib Kostenstellendatensatz der zu KID gehrt zurck
    nb: -- *)
 VAR existiert : BOOLEAN;
     aktKst : TKst;
 BEGIN
   aktKst := Kst.CreateKst();
   Kst.SetzeKIDinKst (inKID, aktKst);
   IF ExistKst (aktKst, inoutListe)
   THEN   (* Kostenstelle mit Kst.ID = inKID gefunden *)
     existiert := TRUE;
     outKst := CurrentInKstenOrdner(inoutListe);
   ELSE   (* inKID nicht in Liste gefunden *)
     existiert := FALSE;
   END; (* IF *)
   Kst.DisposeKst (aktKst);
   RETURN existiert
 END SearchKstZuKID;


PROCEDURE BerechneEtatKst (inKstID: TKstID; inoutListe : TKstenOrdner): INTEGER;
(* vb: inKst existiert im Ordner
       Berechne Etat der Kostenstelle aus Kontingente * quivalente
   nb: Liefert das Produkt der Attribute  Kontingente * quivalente  *)
VAR etat, konting, aequiv : INTEGER;
    aktKst : TKst;
    gefunden : BOOLEAN;
BEGIN

   IF SearchKstZuKID (inKstID, aktKst, inoutListe)
   THEN
      konting := Kst.LiesKontinginKst (aktKst);
      aequiv := Kst.LiesAequivinKst (aktKst);
      etat := konting * aequiv;
      Kst.DisposeKst (aktKst);
   ELSE
      etat := 0;
   END; (* IF *)

   RETURN etat
END BerechneEtatKst;


PROCEDURE AendernKst (inKst: TKst; VAR outMessage : TString; inoutListe : TKstenOrdner): BOOLEAN;
(* Gabi
   nderung vornehmen durch berschreiben der Attribute
   nicht gendert werden drfen die Attribute EingerichtetAm, LiquidiertAm,
   Name und Jahr sowie die ID*)
VAR aktKst : TKst;
    erfolgreich : BOOLEAN;
    aktGesperrtAm : TDatum;
    konting, aequiv : CARDINAL;
BEGIN

    IF ExistKst (inKst, inoutListe)
    THEN    (* Kostenstelle gefunden => Aendern*)
       aktKst := Kst.DuplicateKst (CurrentInKstenOrdner(inoutListe) );

       aktGesperrtAm := Kst.LiesGesperrtAminKst (inKst);
       Kst.SetzeGesperrtAminKst (aktKst, aktGesperrtAm);

       konting := Kst.LiesKontinginKst (inKst);
       Kst.SetzeKontinginKst (aktKst, konting);

       aequiv := Kst.LiesAequivinKst (inKst);
       Kst.SetzeAequivinKst (aktKst, aequiv);

       UpdateInKstenOrdner (inoutListe, aktKst);

       erfolgreich := TRUE;

       Kst.DisposeKst (aktKst);
       DisposeDatum (aktGesperrtAm);

    ELSE
       erfolgreich := FALSE;
       outMessage := String.Create();
       String.Schreibe (outMessage, "Kostenstelle existiert nicht in Ordner");
    END; (* IF ExistKst*)

    RETURN erfolgreich
END AendernKst;


PROCEDURE AendernUndLiquidKst (inKst: TKst; VAR outMessage : TString; inoutListe : TKstenOrdner): BOOLEAN;
(* Sucht die Kostenstelle im Ordner die Kst mit OrdnerKst.ID = inKSt.ID  und berschreibt
   die Attribute der OrdnerKst. Nicht gendert werden drfen die Attribute EingerichtetAm,
   Name und Jahr sowie die ID*)
VAR aktKst : TKst;
    erfolgreich : BOOLEAN;
    aktGesperrtAm, aktLiquidAm : TDatum;
    konting, aequiv : CARDINAL;
BEGIN

    IF ExistKst (inKst, inoutListe)
    THEN    (* Kostenstelle gefunden => Aendern*)
       aktKst := Kst.DuplicateKst (CurrentInKstenOrdner(inoutListe) );

       aktGesperrtAm := Kst.LiesGesperrtAminKst (inKst);
       Kst.SetzeGesperrtAminKst (aktKst, aktGesperrtAm);

       aktLiquidAm := Kst.LiesLiquidAminKst (inKst);
       Kst.SetzeLiquidAminKst (aktKst, aktLiquidAm);

       konting := Kst.LiesKontinginKst (inKst);
       Kst.SetzeKontinginKst (aktKst, konting);

       aequiv := Kst.LiesAequivinKst (inKst);
       Kst.SetzeAequivinKst (aktKst, aequiv);

       UpdateInKstenOrdner (inoutListe, aktKst);

       erfolgreich := TRUE;

       Kst.DisposeKst (aktKst);
       DisposeDatum (aktGesperrtAm);
       DisposeDatum (aktLiquidAm);

    ELSE
       erfolgreich := FALSE;
       outMessage := String.Create();
       String.Schreibe (outMessage, "Kostenstelle existiert nicht in Ordner");
    END; (* IF ExistKst*)

    RETURN erfolgreich
END AendernUndLiquidKst;



(* StringOperationen                                    *)
(*======================================================*)


PROCEDURE GibKstenAlsStringlisteInKsten(inoutListe : TKstenOrdner): TStringliste;
(* Gib die Kostenstellen die nicht liquidiert sind mit folgender
   Formatierung als Stringliste aus;
   Formatierung: KstenID (5Zeichen); 1 Leerzeichen; Name (20Zeichen);
   1 Leerzeichen; Jahr (4Zeichen);
   1 Leerzeichen; Sperrdatum wenn vorhanden, sonst Leerzeichen (10Zeichen);
   insgesamt also 42 Zeichen 	*)
VAR strListe : TStringliste;
    blank, null_str,
    aktString,
    aktName,
    tempname,
    tempDatumString, tempDatumStringJahr, prefix : TString;
    aktKst : TKst;
    aktKstID : TKstID;
    tempJahr : TJahr;
    tempDatum : TDatum;
    exist, dispose_marker : BOOLEAN;
    i, laenge : CARDINAL;
CONST laenge_kstid = 5;
      laenge_name = 20;

BEGIN

  blank := String.Create ();
  String.Schreibe (blank, " ");

  null_str := String.Create ();
  String.Schreibe (null_str, "0");

  strListe := Stringliste.Create ();
  exist := FirstAktKst (aktKst, inoutListe);
  dispose_marker := exist;

  WHILE  exist DO

    (* KstID *)
    aktKstID := Kst.LiesKIDinKst (aktKst);
    aktString := KstID.KstIDAlsString (aktKstID);
    KstID.DisposeKstID (aktKstID);
         (* In Ausgabe auf 5 Zeichen formatieren *)
         laenge := String.Laenge(aktString);
         IF laenge < laenge_kstid THEN
            prefix := String.Create();
               FOR i := 1 TO (laenge_kstid - laenge) DO
                 String.Anhaengen (null_str, prefix)
               END; (* FOR *)
            String.Anhaengen (aktString, prefix);
            String.Equalize (aktString, prefix);
            String.Dispose (prefix);
          END; (* IF *)
   String.Anhaengen (blank, aktString);

    (* Name *)
    aktName := Kst.LiesNameinKst (aktKst);
         laenge := String.Laenge(aktName);
         IF laenge < laenge_name THEN
            (* In Ausgabe auf laenge_name Zeichen verlaengern *)
               FOR i := 1 TO (laenge_name - laenge) DO
                 String.Anhaengen (blank, aktName)
               END; (* FOR *)
         ELSIF
            (* In Ausgabe auf laenge_name Zeichen abschneiden *)
            laenge > laenge_name THEN
            tempname := String.Create();
            String.Auslesen (aktName, 0, laenge_name, tempname);
            String.Equalize (aktName, tempname);
            String.Dispose (tempname);
         END; (* IF *)
    String.Anhaengen (aktName, aktString);
    String.Dispose (aktName);
    String.Anhaengen (blank, aktString);

    (* Jahr *)
    tempJahr := Kst.LiesJahrinKst (aktKst);
    tempDatumStringJahr := JahrAlsString (tempJahr);
    String.Anhaengen (tempDatumStringJahr, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (tempDatumStringJahr);
    DisposeJahr (tempJahr);

    (* Sperrdatum, wenn gesetzt *)
    IF Kst.IstGesperrtAminKst (aktKst)
    THEN
       tempDatum := Kst.LiesGesperrtAminKst (aktKst);
       tempDatumString := DatumAlsString (tempDatum);
       String.Anhaengen (tempDatumString, aktString);
       String.Dispose (tempDatumString);
       DisposeDatum (tempDatum);
    ELSE
       FOR i:=1 TO 10 DO
       String.Anhaengen (blank, aktString);
       END; (* FOR*)
    END; (* IF *)

    Stringliste.Append (strListe, aktString);
    String.Dispose (aktString);

    exist := NextAktKst (aktKst, inoutListe);

  END; (* WHILE *)

  IF dispose_marker
  THEN
     Kst.DisposeKst (aktKst);
  END;

  String.Dispose (blank);
  String.Dispose (null_str);

  RETURN strListe

END GibKstenAlsStringlisteInKsten;


PROCEDURE GibAllesAlsStringlisteInKsten(inoutListe : TKstenOrdner): TStringliste;
(* Gib die Kostenstellen die nicht liquidiert sind mit folgender
   Formatierung als Stringliste aus;
   Formatierung: KstenID (5Zeichen); 1 Leerzeichen; Name (20Zeichen);
   1 Leerzeichen; Jahr (4Zeichen);
   1 Leerzeichen; Sperrdatum wenn vorhanden, sonst Leerzeichen (10Zeichen);
   insgesamt also 42 Zeichen 	*)
VAR strListe : TStringliste;
    blank, null_str, komma_str,
    aktString,
    aktName, tempname,
    tempDatumString, tempDatumStringJahr, prefix, konting_str, aequiv_str : TString;
    aktKst, outKst : TKst;
    aktKstID : TKstID;
    tempJahr : TJahr;
    tempDatum : TDatum;
    exist, dispose_marker : BOOLEAN;
    i, laenge, konting, aequiv : CARDINAL;
    arrayofchar : ARRAY [1..255] OF CHAR;
CONST laenge_kstid = 5;
      laenge_name = 20;
      laenge_betrag = 10;

BEGIN

  blank := String.Create ();
  String.Schreibe (blank, " ");

  null_str := String.Create ();
  String.Schreibe (null_str, "0");

  komma_str := String.Create ();
  String.Schreibe (komma_str, ",");

  strListe := Stringliste.Create ();
  exist := FirstKst (aktKst, inoutListe);
  dispose_marker := exist;

  WHILE  exist DO

    (* KstID *)
    aktKstID := Kst.LiesKIDinKst (aktKst);
    aktString := KstID.KstIDAlsString (aktKstID);
    KstID.DisposeKstID (aktKstID);
         (* In Ausgabe auf 5 Zeichen formatieren *)
         laenge := String.Laenge(aktString);
         IF laenge < laenge_kstid THEN
            prefix := String.Create();
               FOR i := 1 TO (laenge_kstid - laenge) DO
                 String.Anhaengen (null_str, prefix)
               END; (* FOR *)
            String.Anhaengen (aktString, prefix);
            String.Equalize (aktString, prefix);
            String.Dispose (prefix);
          END; (* IF *)
   String.Anhaengen (blank, aktString);

    (* Name *)
    aktName := Kst.LiesNameinKst (aktKst);
         laenge := String.Laenge(aktName);
         IF laenge < laenge_name THEN
            (* In Ausgabe auf laenge_name Zeichen verlaengern *)
            prefix := String.Create();
               FOR i := 1 TO (laenge_name - laenge) DO
                 String.Anhaengen (blank, aktName)
               END; (* FOR *)
            String.Dispose (prefix);
         ELSIF
            (* In Ausgabe auf laenge_name Zeichen verlaengern *)
            laenge > laenge_name THEN
            tempname := String.Create();
            String.Auslesen (aktName, 0, laenge_name, tempname);
            String.Equalize (aktName, tempname);
            String.Dispose (tempname);
         END; (* IF *)
    String.Anhaengen (aktName, aktString);
    String.Dispose (aktName);
    String.Anhaengen (blank, aktString);

    (* Jahr *)
    tempJahr := Kst.LiesJahrinKst (aktKst);
    tempDatumStringJahr := JahrAlsString (tempJahr);
    String.Anhaengen (tempDatumStringJahr, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (tempDatumStringJahr);
    DisposeJahr (tempJahr);

    (* EingerichtetAm *)
    tempDatum := Kst.LiesEingAminKst (aktKst);
    tempDatumString := DatumAlsString (tempDatum);
    String.Anhaengen (tempDatumString, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (tempDatumString);
    DisposeDatum (tempDatum);


    (* SperrDatum, wenn gesetzt *)
    IF Kst.IstGesperrtAminKst (aktKst)
    THEN
       tempDatum := Kst.LiesGesperrtAminKst (aktKst);
       tempDatumString := DatumAlsString (tempDatum);
       String.Anhaengen (tempDatumString, aktString);
       String.Anhaengen (blank, aktString);
       String.Dispose (tempDatumString);
       DisposeDatum (tempDatum);
    ELSE
       FOR i:=1 TO 11 DO
       String.Anhaengen (blank, aktString);
       END; (* FOR*)
    END; (* IF *)


    (* LiquidierDatum, wenn gesetzt *)
    IF Kst.IstLiquidiertAminKst (aktKst)
    THEN
       tempDatum := Kst.LiesLiquidAminKst (aktKst);
       tempDatumString := DatumAlsString (tempDatum);
       String.Anhaengen (tempDatumString, aktString);
       String.Anhaengen (blank, aktString);
       String.Dispose (tempDatumString);
       DisposeDatum (tempDatum);
    ELSE
       FOR i:=1 TO 11 DO
       String.Anhaengen (blank, aktString);
       END; (* FOR*)
    END; (* IF *)


    (* Kontingente *)
    konting := Kst.LiesKontinginKst (aktKst);
    CardToStr (konting, arrayofchar);
    konting_str := String.Create();
    String.Schreibe (konting_str, arrayofchar);
     (* In Ausgabe auf laenge_betrag Zeichen formatieren *)
     laenge := String.Laenge(konting_str);

   (*   IF laenge = 1 THEN
         String.Einfuegen (null_str, 0, konting_str);
         laenge := laenge + 1;
      END;

      IF laenge = 2 THEN
         String.Einfuegen (null_str, 0, konting_str);
         laenge := laenge + 1;
      END; *)

      (* String hat jetzt mindestens die Lnge 3 *)
      (* String.Einfuegen (komma_str, laenge-2, konting_str); *)

      IF laenge < laenge_betrag THEN
         prefix := String.Create();
         FOR i := 1 TO (laenge_betrag - laenge) DO
             String.Anhaengen (blank, prefix)
         END; (* FOR *)
         String.Anhaengen (prefix, aktString);
         String.Anhaengen (konting_str, aktString);
         String.Anhaengen (blank, aktString);
      ELSE
         String.Schreibe (konting_str, "**********");
         String.Anhaengen (konting_str, aktString);
         String.Anhaengen (blank, aktString);
      END;
      String.Dispose (konting_str);

    (* Aequivalente *)
    aequiv := Kst.LiesAequivinKst (aktKst);
    CardToStr (aequiv, arrayofchar);
    aequiv_str := String.Create();
    String.Schreibe (aequiv_str, arrayofchar);
      (* In Ausgabe auf laenge_betrag Zeichen formatieren *)
      laenge := String.Laenge(aequiv_str);

      (* Sollte Aequivalent ein DM-Betrag sein, das folgende aktivieren *)
      IF laenge = 1 THEN
         String.Einfuegen (null_str, 0, aequiv_str);
         laenge := laenge + 1;
      END;

      IF laenge = 2 THEN
         String.Einfuegen (null_str, 0, aequiv_str);
         laenge := laenge + 1;
      END;

      String.Einfuegen (komma_str, laenge-2, aequiv_str);

      IF laenge < laenge_betrag THEN
         prefix := String.Create();
         FOR i := 1 TO (laenge_betrag - laenge - 1) DO
             String.Anhaengen (blank, prefix)
         END; (* FOR *)
         String.Anhaengen (prefix, aktString);
         String.Anhaengen (aequiv_str, aktString);
      ELSE
         String.Schreibe (aequiv_str, "**********");
         String.Anhaengen (aequiv_str, aktString);
      END;
      String.Dispose (aequiv_str);


    Stringliste.Append (strListe, aktString);
    String.Dispose (aktString);

    outKst := Kst.CreateKst();
    exist := NextKst (aktKst, outKst, inoutListe);
    IF exist THEN
       Kst.EqualizeKst (aktKst, outKst);
    END;
    Kst.DisposeKst (outKst);

  END; (* WHILE *)

  IF dispose_marker
  THEN
     Kst.DisposeKst (aktKst);
  END; (* IF *)

  String.Dispose (blank);
  String.Dispose (null_str);
  String.Dispose (komma_str);

  RETURN strListe

END GibAllesAlsStringlisteInKsten;


 (* Prozeduren zum Laden und Speichern des Teilsystems *)
 (*====================================================*)

PROCEDURE LoadinKsten(inH:THandle; VAR outMessage : TString;
                      inoutKstenOrdner : TKstenOrdner) : BOOLEAN;
   (* vb: Datei existiert und ist nicht leer und zum Lesen geoeffnet
	    Lade den Inhalt des KostenStellenOrdners mit dem Inhalt der 
          Datei auf die inH verweist.
      nb: KostenstellenOrdner ist geladen *)
VAR success : BOOLEAN;
    str, message, name, insert_message, outm_str, undef_str_kurz,
    undef_str_lang, jahr_message  : TString;
    kstID : TKstID;
    aktKst : TKst;
    eingam, gesperram, liquidam : TDatum;
    konting, aequiv : CARDINAL;
    result : ConvResults;
    CharArray : ARRAY [1..255] OF CHAR;
    jahr : TJahr;

BEGIN

 (* Leeren des vorhandenen Ordners *)
 LoescheAllesInKstenOrdner (inoutKstenOrdner);

 (* Sequentielles Einlesen der Kostenstellen *)


 str := String.Create();
 message := String.Create();
 aktKst := Kst.CreateKst();

 (* undef_str wird benoetigt, um Datum mit "undef" wieder einlesen zu koennen *)
 undef_str_kurz := String.Create();
 String.Schreibe (undef_str_kurz, "undef");
 undef_str_lang := String.Create();
 String.Schreibe (undef_str_lang, "undef     ");


 success := TRUE;

 WHILE NOT EofInVDat (inH) AND success DO

   (* KostenstellenID *)
   IF ReadLineInVDat (inH, str, message) AND success THEN
      IF KstID.PruefeKstID (str, outm_str) THEN
         kstID := KstID.StringInKstID (str);
         Kst.SetzeKIDinKst (kstID, aktKst);
         KstID.DisposeKstID (kstID);
         success := TRUE;
      ELSE
         success := FALSE;
         String.Dispose (outm_str);
      END;
   ELSE
      success := FALSE;
   END;

   (* Name *)
   name := String.Create ();
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, name, message) AND success THEN
      Kst.SetzeNameinKst (aktKst, name);
      String.Dispose (name);
      success := TRUE;
   ELSE
      success := FALSE;
   END;

   (* Datum: EingerichtetAm *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN

      (* Aus "undef     " wird "undef" gemacht, sonst Konvertierungsfehler *)
      IF String.IsEqual (str, undef_str_lang) THEN
         String.Equalize (str, undef_str_kurz)
      END;

      IF NOT DatumVonString (str, eingam, message) THEN
         success := FALSE
      ELSE
         Kst.SetzeEingAminKst (aktKst, eingam);
         DisposeDatum (eingam);
         success := TRUE;
      END;
   ELSE
      success := FALSE;
   END;

   (* Datum: GesperrtAm *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN

      (* Aus "undef     " wird "undef" gemacht, sonst Konvertierungsfehler *)
      IF String.IsEqual (str, undef_str_lang) THEN
         String.Equalize (str, undef_str_kurz)
      END;

      IF NOT DatumVonString (str, gesperram, message) THEN
         success := FALSE
      ELSE
         Kst.SetzeGesperrtAminKst (aktKst, gesperram);
         DisposeDatum (gesperram);
         success := TRUE;
      END;
   ELSE
      success := FALSE;
   END;

   (* Datum: LiquidiertAm *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN

      (* Aus "undef     " wird "undef" gemacht, sonst Konvertierungsfehler *)
      IF String.IsEqual (str, undef_str_lang) THEN
         String.Equalize (str, undef_str_kurz)
      END;

      IF NOT DatumVonString (str, liquidam, message) THEN
         success := FALSE
      ELSE
         Kst.SetzeLiquidAminKst (aktKst, liquidam);
         DisposeDatum (liquidam);
         success := TRUE;
      END;
   ELSE
      success := FALSE;
   END;

   (* Kontingente *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN
      String.Lies (str, CharArray);
      StrToCard(CharArray,konting,result);
      IF (result=strAllRight) THEN
         Kst.SetzeKontinginKst (aktKst, konting);
         success := TRUE;
      ELSE
         success := FALSE;
      END;
   ELSE
      success := FALSE;
   END;     

   (* Aequivalent *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN
      String.Lies (str, CharArray);
      StrToCard(CharArray,aequiv,result);
      IF (result=strAllRight) THEN
         Kst.SetzeAequivinKst (aktKst, aequiv);
         success := TRUE;
      ELSE
         success := FALSE;
      END;
   ELSE
      success := FALSE;
   END;     

   (* Jahr *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN
      IF JahrVonString (str, jahr, jahr_message) THEN
         Kst.SetzeJahrinKst (aktKst, jahr);
         DisposeJahr (jahr);
         success := TRUE;
      ELSE
         success := FALSE;
         String.Dispose (jahr_message);
      END;
   ELSE
      success := FALSE;
   END;


   (* Erzeugen eines ADO's vom Typ TKst, Setzen der Attribute
      und Einfgen in den KostenstellenOrdner                       *)
   IF success THEN
      IF InsertKst (aktKst, insert_message, inoutKstenOrdner, FALSE) THEN
         success := TRUE;
      ELSE
         success := FALSE;
         outMessage := String.Create();
         String.Equalize (outMessage, insert_message);
         String.Dispose (insert_message);
      END; (* IF InsertKst *)
   ELSE
      outMessage := String.Create();
      String.Schreibe (outMessage,
      "KostenstellenOrdner konnte NICHT erfolgreich restauriert werden !");
   END;

 END; (* WHILE *)

 String.Dispose (str);
 String.Dispose (message);
 String.Dispose (undef_str_kurz);
 String.Dispose (undef_str_lang);
 Kst.DisposeKst (aktKst);

 RETURN success;

END LoadinKsten;



PROCEDURE SaveinKsten(inH:THandle; VAR outMessage : TString;
                      inoutKstenOrdner : TKstenOrdner) : BOOLEAN;	
   (* vb: Datei existiert und ist zum Schreiben geoeffnet
	    Speichert den Inhalt des KostenstellenOrdner in der Datei auf die inH
	    verweist.
      nb: KostenstellenOrdner-Inhalt ist auf SekundaerSpeicher geschrieben *)
VAR id_str, eingam_str, gesperram_str, liquidam_str, name_str,
    jahr_str, outm_str, konting_str, aequiv_str : TString;
    eingam, gesperram, liquidam : TDatum;
    tempjahr : TJahr;
    aktKst, outKst : TKst;
    aktKstID : TKstID;
    next_kst, erfolgreich, dispose_marker : BOOLEAN;
    konting, aequiv : CARDINAL;
    arrayofchar : ARRAY [1..255] OF CHAR;
BEGIN



 IF NOT FirstKst (aktKst, inoutKstenOrdner) THEN

    (* KostenstellenOrdner ist leer *)
    erfolgreich := TRUE;

 ELSE

    (* KostenstellenOrdner ist nicht leer *)
    next_kst := TRUE;
    erfolgreich := TRUE;
    dispose_marker := TRUE;

    WHILE next_kst AND erfolgreich DO

       aktKstID := Kst.LiesKIDinKst (aktKst);
       id_str := KstID.KstIDAlsString (aktKstID);

       eingam := Kst.LiesEingAminKst (aktKst);
       eingam_str := DatumAlsString (eingam);

       gesperram := Kst.LiesGesperrtAminKst (aktKst);
       gesperram_str := DatumAlsString (gesperram);

       liquidam := Kst.LiesLiquidAminKst (aktKst);
       liquidam_str := DatumAlsString (liquidam);

       name_str := Kst.LiesNameinKst (aktKst);

       konting := Kst.LiesKontinginKst (aktKst);
       CardToStr (konting, arrayofchar);
       konting_str := String.Create();
       String.Schreibe (konting_str, arrayofchar);

       aequiv := Kst.LiesAequivinKst (aktKst);
       CardToStr (aequiv, arrayofchar);
       aequiv_str := String.Create();
       String.Schreibe (aequiv_str, arrayofchar);

       (* Jahr *)
       tempjahr := Kst.LiesJahrinKst (aktKst);
       jahr_str := JahrAlsString (tempjahr);

       (* Kostenstellendaten auf Datei schreiben *)
       outm_str := String.Create();
       erfolgreich := WriteLineInVDat (inH, id_str, outm_str) AND
                      WriteLineInVDat (inH, name_str, outm_str) AND
                      WriteLineInVDat (inH, eingam_str, outm_str) AND
                      WriteLineInVDat (inH, gesperram_str, outm_str) AND
                      WriteLineInVDat (inH, liquidam_str, outm_str) AND
                      WriteLineInVDat (inH, konting_str, outm_str) AND
                      WriteLineInVDat (inH, aequiv_str, outm_str) AND
                      WriteLineInVDat (inH, jahr_str, outm_str);

       DisposeJahr (tempjahr);
       DisposeDatum (eingam);
       DisposeDatum (gesperram);
       DisposeDatum (liquidam);
       KstID.DisposeKstID (aktKstID);

       String.Dispose (id_str);
       String.Dispose (eingam_str);
       String.Dispose (gesperram_str);
       String.Dispose (liquidam_str);
       String.Dispose (name_str);
       String.Dispose (konting_str);
       String.Dispose (aequiv_str);
       String.Dispose (jahr_str);

       String.Dispose (outm_str);

       outKst := Kst.CreateKst ();
       next_kst := NextKst (aktKst, outKst, inoutKstenOrdner);
       Kst.EqualizeKst (aktKst, outKst);
       Kst.DisposeKst (outKst);

    END; (* WHILE *)

    IF NOT erfolgreich
    THEN 
      erfolgreich := FALSE;
      outMessage := String.Create ();
      String.Schreibe (outMessage,
              "KostenstellenOrdner konnte nicht gesichert werden.");
    END; (* IF WriteLineInVDat *)

    IF dispose_marker
    THEN
       Kst.DisposeKst (aktKst);
    END;
            
 END; (* IF NOT FirstKst *)

 RETURN erfolgreich;
END SaveinKsten;


END KstOSchablone.
