(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: ksten.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:48:05 $
# $Revision: 1.1.1.1 $
Bemerkungen:

IMPLEMENTATION MODULE Ksten   TS-Export-Schnittstelle Kostenstellen

Implementation : Schoenhammer Herbert

Enthaltene Komponenten:
  ADT Kostenstelle                  Kst.def Kst.mod
  ADT KostenstellenID               KstID.def KstID.mod
  ADT KostenstellenOrdnerSchablone  KstenOrdnerSchablone.def /... .mod
  ADT KostenstellenOrdner           KstenOrdner.def / ... .mod

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE Ksten;     (* EXPORT-Schnittstelle des TS Kostenstellen *)

(* TS Kostenstellen *)


FROM Datum        IMPORT TDatum, TJahr;
FROM String       IMPORT TString;
FROM Stringliste  IMPORT TStringliste;
FROM KstenIMPORTE IMPORT THandle;

IMPORT Kst, KstID, KstenOrdner;


TYPE TKstID       = KstID.TKstID;
     TKst         = Kst.TKst;


(*==========================================================================*)
(* Operationen zur Erzeugung von Ausprgungen des Entittstyps Kostenstelle *)
(* Durchreichen bzw. Umbenennen der Operationen aus dem ADT-Modul Kst       *)
(*==========================================================================*)

PROCEDURE CreateKst () : TKst;
BEGIN
  RETURN Kst.CreateKst()
END CreateKst;
        
(* Standard-Operationen auf dem Entittstyp Kostenstellen *)
(*========================================================*)

PROCEDURE DisposeKst (VAR inoutKst : TKst);
BEGIN
  Kst.DisposeKst (inoutKst)
END DisposeKst;

PROCEDURE DuplicateKst (inQuellKst : TKst) : TKst;
BEGIN
  RETURN Kst.DuplicateKst (inQuellKst)
END DuplicateKst;

PROCEDURE IsEqualKst (inKst1, inKst2 : TKst) : BOOLEAN;
BEGIN
  RETURN Kst.IsEqualKst (inKst1, inKst2)
END IsEqualKst;

PROCEDURE EqualizeKst (inoutZielKst, inQuellKst : TKst);
BEGIN
  Kst.EqualizeKst (inoutZielKst, inQuellKst)
END EqualizeKst;


(* Kostenstellen ID                                       *)
(*========================================================*)


PROCEDURE CreateKstID (): TKstID;
BEGIN
  RETURN KstID.CreateKstID ()
END CreateKstID;


PROCEDURE DuplicateKstID (inKstID : TKstID): TKstID;
BEGIN
  RETURN KstID.DuplicateKstID (inKstID)
END DuplicateKstID;


PROCEDURE DisposeKstID (VAR inoutKstID : TKstID);
BEGIN
  KstID.DisposeKstID (inoutKstID)
END DisposeKstID;


PROCEDURE IsEqualKstID (inKstID1, inKstID2 : TKstID) : BOOLEAN;
BEGIN
  RETURN KstID.IsEqualKstID (inKstID1, inKstID2)
END IsEqualKstID;


PROCEDURE EqualizeKstID (inoutKstID1, inKstID2 : TKstID);
BEGIN
   KstID.EqualizeKstID (inoutKstID1, inKstID2)
END EqualizeKstID;


PROCEDURE SetzeKIDinKst (inK  : TKstID; inoutKst: TKst);
BEGIN
  Kst.SetzeKIDinKst (inK, inoutKst)
END SetzeKIDinKst;


PROCEDURE LiesKIDinKst ( inKst: TKst):TKstID;
BEGIN
  RETURN Kst.LiesKIDinKst (inKst)
END LiesKIDinKst;


PROCEDURE ExistKst(inKst: TKst):BOOLEAN;
BEGIN
  RETURN Kst.IstGesetztKstID (inKst)
END ExistKst;


PROCEDURE KstIDAlsString (inKstID : TKstID) : TString;
BEGIN
  RETURN KstID.KstIDAlsString (inKstID)
END KstIDAlsString;


PROCEDURE StringInKstID (inString : TString) : TKstID;
BEGIN
  RETURN KstID.StringInKstID (inString)
END StringInKstID;


PROCEDURE PruefeKstID  (inString : TString;
                                VAR outMessage : TString) : BOOLEAN;
BEGIN
   RETURN KstID.PruefeKstID (inString, outMessage)
END PruefeKstID;


(* Operationen auf den Attributen des Entittstyps Kondition *)
(*===========================================================*)


PROCEDURE SetzeEingAminKst (inoutKst: TKst; inD  : TDatum);
BEGIN
  Kst.SetzeEingAminKst (inoutKst, inD)
END SetzeEingAminKst;

PROCEDURE LiesEingAminKst ( inKst: TKst):TDatum;
BEGIN
  RETURN Kst.LiesEingAminKst (inKst)
END LiesEingAminKst;

PROCEDURE SetzeGesperrtAminKst (inoutKst: TKst; inD  : TDatum);
BEGIN
  Kst.SetzeGesperrtAminKst (inoutKst, inD)
END SetzeGesperrtAminKst;

PROCEDURE LiesGesperrtAminKst ( inKst: TKst):TDatum;
BEGIN
  RETURN Kst.LiesGesperrtAminKst (inKst)
END LiesGesperrtAminKst;

PROCEDURE IstGesperrtAminKst ( inKst: TKst):BOOLEAN;
(* Prueft, ob Datum GesperrtAm ungleich dem default Datum 31.12.9999 *)
BEGIN
  RETURN Kst.IstGesperrtAminKst (inKst)
END IstGesperrtAminKst;

PROCEDURE SetzeLiquidAminKst (inoutKst: TKst; inD  : TDatum);
BEGIN
  Kst.SetzeLiquidAminKst (inoutKst, inD)
END SetzeLiquidAminKst;

PROCEDURE LiesLiquidAminKst ( inKst: TKst):TDatum;
BEGIN
  RETURN Kst.LiesLiquidAminKst ( inKst)
END LiesLiquidAminKst;

PROCEDURE IstLiquidiertAminKst ( inKst: TKst):BOOLEAN;
(* Prueft, ob Datum LiquidiertAm ungleich dem default Datum 31.12.9999 *)
BEGIN
  RETURN Kst.IstLiquidiertAminKst (inKst)
END IstLiquidiertAminKst;

PROCEDURE SetzeNameinKst ( inoutKst: TKst; inN: TString);
BEGIN
  Kst.SetzeNameinKst ( inoutKst, inN)
END SetzeNameinKst;

PROCEDURE LiesNameinKst ( inKst: TKst):TString;
BEGIN
  RETURN Kst.LiesNameinKst (inKst)
END LiesNameinKst;

PROCEDURE SetzeJahrinKst ( inoutKst: TKst; inJahr: TJahr);
BEGIN
  Kst.SetzeJahrinKst ( inoutKst, inJahr)
END SetzeJahrinKst;

PROCEDURE LiesJahrinKst ( inKst: TKst):TJahr;
BEGIN
  RETURN  Kst.LiesJahrinKst ( inKst)
END LiesJahrinKst;

PROCEDURE SetzeKontinginKst ( inoutKst: TKst; inKonti: CARDINAL);
BEGIN
  Kst.SetzeKontinginKst ( inoutKst, inKonti)
END SetzeKontinginKst;

PROCEDURE LiesKontinginKst ( inKst: TKst):CARDINAL;
BEGIN
  RETURN Kst.LiesKontinginKst ( inKst)
END LiesKontinginKst;

PROCEDURE SetzeAequivinKst ( inoutKst: TKst; inAequi: CARDINAL);
BEGIN
  Kst.SetzeAequivinKst ( inoutKst, inAequi)
END SetzeAequivinKst;

PROCEDURE LiesAequivinKst ( inKst: TKst):CARDINAL;
BEGIN
  RETURN  Kst.LiesAequivinKst ( inKst)
END LiesAequivinKst;


(* String-Operationen                        *)
(*===========================================*)

PROCEDURE GibKstenAlsStringlisteInKsten(): TStringliste;
BEGIN
   RETURN KstenOrdner.GibKstenAlsStringlisteInKsten ()
END GibKstenAlsStringlisteInKsten;


PROCEDURE GibAllesAlsStringlisteInKsten(): TStringliste;
BEGIN
   RETURN KstenOrdner.GibAllesAlsStringlisteInKsten ()
END GibAllesAlsStringlisteInKsten;


(* Operationen auf den Kostenstellen Ordnern *)
(*===========================================*)


PROCEDURE InsertKst ( inK: TKst; VAR outMessage : TString ) : BOOLEAN;
(* vb: sei K die neu einzufgende Kostenstelle. Dann muss gelten: 
       ADO Konditionsordner existiert and not isfull and KID ist zugewiesen
	 Fuegt eine Kostenstelle gemaess der Prozessspezifikationen D3.2.2 in den
	 KostenstellenOrdner ein. 
   nb: K ist in Kostenstellenordner eingefgt *) 
BEGIN
  RETURN KstenOrdner.InsertKst (inK, outMessage)
END InsertKst;


PROCEDURE FirstKst (VAR outKst:TKst):BOOLEAN;
(* vb: Keine
	 Liefere die erste Kostenstelle im Ordner zurck
   nb: -- *)
BEGIN
   RETURN KstenOrdner.FirstKst (outKst)
END FirstKst;


PROCEDURE NextKst (inK:TKst; inoutK:TKst):BOOLEAN;
(* vb: inK existiert in KostenstellenOrdner 
	 Suche inK im KostenstellenOrdner und falls ein Nachfolger outK existiert gib
	 diesen zurck, false sonst. 
   nb: --	*)
BEGIN
   RETURN KstenOrdner.NextKst (inK, inoutK)
END NextKst;


PROCEDURE FirstLfdKst (VAR outKst : TKst) : BOOLEAN;
(* Reini :
   gibt die erste Kst zurck, die weder liquidiert noch gesperrt ist *)
BEGIN
   RETURN KstenOrdner.FirstLfdKst (outKst)
END FirstLfdKst;


PROCEDURE NextLfdKst (inoutKst : TKst) : BOOLEAN;
(* Reini :
   gibt die naechste Kst zurck, die weder liquidiert noch gesperrt ist *)
BEGIN
   RETURN KstenOrdner.NextLfdKst (inoutKst)
END NextLfdKst;


PROCEDURE FirstAktKst (VAR outKst : TKst) : BOOLEAN;
(* Reini :
   gibt die erste Kst zurck, die nicht liquidiert ist *)
BEGIN
   RETURN KstenOrdner.FirstAktKst (outKst)
END FirstAktKst;


PROCEDURE NextAktKst (inoutKst : TKst) : BOOLEAN;
(* Reini :
   gibt die naechste Kst zurck, die nicht liquidiert ist *)
BEGIN
   RETURN KstenOrdner.NextAktKst (inoutKst)
END NextAktKst;


PROCEDURE SearchKstZuKID( inKID: TKstID; VAR outKst : TKst):BOOLEAN;
(* vb: inKID ist gueltig
	 Gib Kostenstellendatensatz der zu KID gehrt zurck
   nb: --	*)
BEGIN
   RETURN KstenOrdner.SearchKstZuKID (inKID, outKst)
END SearchKstZuKID;


PROCEDURE AendernKst (inKst: TKst; VAR outMessage : TString): BOOLEAN;
(* Gabi
   nderung vornehmen durch berschreiben der Attribute
   nicht gendert werden drfen die Attribute EingerichtetAm, LiquidiertAm,
   Name und Jahr sowie die ID *)                                                    
BEGIN
   RETURN KstenOrdner.AendernKst (inKst, outMessage)
END AendernKst;


PROCEDURE AendernUndLiquidKst (inKst: TKst; VAR outMessage : TString): BOOLEAN;
(* nderung vornehmen durch berschreiben der Attribute
   nicht gendert werden drfen die Attribute EingerichtetAm,
   Name und Jahr sowie die ID *)
BEGIN
   RETURN KstenOrdner.AendernUndLiquidKst (inKst, outMessage)
END AendernUndLiquidKst;


PROCEDURE BerechneEtatKst (inKstID: TKstID): INTEGER;
(* vb: inKst existiert im Ordner
	 Berechne Etat der Kostenstelle aus Kontingente * quivalente
   nb: Liefert das Produkt der Attribute  Kontingente * quivalente *)
BEGIN
   RETURN KstenOrdner.BerechneEtatKst (inKstID)
END BerechneEtatKst;


(* Prozeduren zum Laden und Speichern des Teilsystems *)
(*====================================================*)


PROCEDURE LoadinKsten(inH : THandle; VAR outMessage : TString) : BOOLEAN;
(* vb: Datei existiert und ist nicht leer und zum Lesen geoeffnet
	 Lade den Inhalt des KostenStellenOrdners mit dem Inhalt der Datei auf die inH
	 verweist.
   nb: KostenstellenOrdner ist geladen *)
BEGIN
  RETURN KstenOrdner.LoadinKsten ( inH, outMessage)
END LoadinKsten;
	

PROCEDURE SaveinKsten(inH:THandle; VAR outMessage : TString) : BOOLEAN;	
(* vb: Datei existiert und ist zum Schreiben geoeffnet
	 Speichert den Inhalt des KostenstellenOrdner in der Datei auf die inH
	 verweist.
   nb: KostenstellenOrdner-Inhalt ist auf SekundaerSpeicher geschrieben	*)
BEGIN
   RETURN KstenOrdner.SaveinKsten ( inH, outMessage)
END SaveinKsten;



END Ksten.






