(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: SysDatADO.mod,v $
# $Author: Sopra $
# $Date: 2001/03/12 14:37:03 $
# $Revision: 1.2 $
# Bemerkungen: 
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE SysDatADO; 

IMPORT SysDatIMPORTE, Datum, WholeStr, String;
FROM Storage IMPORT ALLOCATE, DEALLOCATE;
FROM SYSTEM  IMPORT TSIZE;                

TYPE 	THandle=SysDatIMPORTE.THandle;
	TDatum=Datum.TDatum;
	TString=String.TString;

TYPE  	TVersionDS=[0..MaxBackup];
	TVersion=POINTER TO TVersionDS;
	TBackup=RECORD
			Version:	TVersion;
			BDatum:		TDatum;
		END;
	TBackupTable=ARRAY[1..MaxBackup] OF TBackup;


(* ADO Backuptable ist statisch und wird deshalb einfach hier erzeugt *)
VAR 	BackupTable:	TBackupTable;
	AktVersion: 	TVersion;
	SystemDatum:	TDatum;
	AutoDatum:	TDatum;


PROCEDURE CreateBackupTable();
(* Da die Variable AktVersion schon existiert, muss nur noch der Zhler init *)
(* ialisiert werden. Die Datum ADOs werden erzeugt und leer belassen.  *)
(* Damit tritt auch bei teilweise gefllter Tabelle kein Fehler auf und das *)
(* Datum wird als 'undef' angezeigt *)
VAR i: TVersionDS;
BEGIN
	FOR i:=1 TO MaxBackup DO
		WITH BackupTable[i] DO
			Version:=CreateVersion();
			Version^:=i;
			BDatum:=Datum.CreateDatum();
		END (* WITH *);
	END (* FOR *);
END CreateBackupTable;

PROCEDURE CreateVersion() : TVersion;
(*
vb: --
Gibt ein leeres ADO vom Typ TVersion zurck
nb: ADO existiert
*) 
VAR Version: TVersion;
BEGIN
        ALLOCATE (Version, TSIZE (TVersionDS)) ;
        RETURN Version; 	
END CreateVersion;

PROCEDURE DisposeVersion(VAR inoutV: TVersion);
(*
vb: ADO inoutV existiert
Lscht ein ADO vom Typ TVersion aus dem Speicher
nb: inoutV existiert nicht mehr
*) 
BEGIN
	DEALLOCATE (inoutV, TSIZE (TVersionDS)) ;
	inoutV:=NIL;
END DisposeVersion;

PROCEDURE VersionAlsString(inV:TVersion): TString;
(*
vb: --
Erzeugt ein neues ADO vom Typ TString und  beschreibt es mit dem String
aequivalent von inV
nb: --
*)
VAR str:TString;
    CharArray: ARRAY[1..255] OF CHAR;
BEGIN
	str:=String.Create();
	WholeStr.CardToStr(inV^,CharArray);
	String.Schreibe(str, CharArray);
	RETURN str;
END VersionAlsString;
	
PROCEDURE VersionVonString (inString : TString; VAR outV:TVersion;
                          VAR outMessage:TString) : BOOLEAN;
(*
vb: --
Ldt das ADO outV mit der Version die im Stringaequivalent inString enthalten
war. Gelingt die Konversion nicht wird FALSE und eine Fehlermeldung zurck
gegeben
nb: --
*)
VAR card: 	CARDINAL;
    CharArray: 	ARRAY[1..255] OF CHAR;
    Result: 	WholeStr.ConvResults;
    success: 	BOOLEAN;
BEGIN
        String.Lies(inString, CharArray);
        WholeStr.StrToCard(CharArray,card,Result);
	IF  (card >= 0) AND 
	    (card <= MaxBackup) AND 
	    (Result = WholeStr.strAllRight) THEN
	  outV:=CreateVersion();
	  outV^:=card;
	  success:=TRUE;
	ELSE
	  outMessage:=String.Create();
	  String.Schreibe(outMessage,
		"Fehler bei der Konvertierung nach TVersion!");
          success:= FALSE;
	END (* IF *);
        RETURN success;
END VersionVonString;    

PROCEDURE IsZeroVersion(inV:TVersion):BOOLEAN;
BEGIN
	RETURN (inV^ = 0);
END IsZeroVersion;

PROCEDURE LiesBackupVersion():TVersion;
(*
vb: --
Gibt ein ADO mit dem aktuellen Wert des Backup-Versionszhlers zurck
nb: --
*)
VAR V: TVersion;
BEGIN
	V:=CreateVersion();
	V^:=AktVersion^;
	RETURN V;
END LiesBackupVersion;

PROCEDURE SetzeBackupVersion(inV:TVersion);
(*
vb: --
Setzt den aktuellen Inhalt des BackupVersionszhlers auf inV. Dient dazu, das   naechste Backup gezielt unter einer bestimmten Version anzulegen
nb: --
*)
BEGIN
  	AktVersion^ := inV^;
END SetzeBackupVersion; 

PROCEDURE LiesBackupDatum(V:TVersion):TDatum;
(*
vb: --
Gib das Datum zur BackupVersion zurck
nb: --
*)
VAR D: TDatum;
BEGIN
	D:=Datum.CreateDatum();
	Datum.EqualizeDatum(D,BackupTable[V^].BDatum);
	RETURN D;
END LiesBackupDatum;

PROCEDURE NeuesBackup();
(*
vb: Neues Backup ist angelegt worden
Inkrementiere den Versionszhler und schreibe das aktuelle SystemDatum in die
BackupTabelle
nb: Sei V der Versionszhler und sei Maxbackup die maximale Anzahl vor-
gehaltener Backups. Dann gilt V=(V mod Maxbackups) + 1
*)
VAR dat: TDatum;
BEGIN
	(* Hole SystemDatum *)
	dat:=LiesSystemDatum();
	WITH BackupTable[AktVersion^] DO 
        	Version^:=AktVersion^;
	        Datum.EqualizeDatum(BDatum,dat );
	END (* WITH *);
        Datum.DisposeDatum(dat);
	(* erhoehe Versionszaehler zyklisch um 1 *)
	AktVersion^:= (AktVersion^ MOD MaxBackup) + 1;
        
END NeuesBackup;

PROCEDURE Load(inH: THandle; VAR outM:TString):BOOLEAN;
(* Wird beim Systemstart zur Restaurierung der BackupTabelle aufgerufen
vb: ADO BackupTable existiert, ADO AktVersion existiert
Lade den Inhalt der Datei HiKS.ini in die Backuptabelle und setze den Versions
zhler, lade das Datum des letzten Auto-Betriebes. Falls erfolgreich wird true
zurckgegeben. Im Fehlerfall enthlt inoutM eine Fehlernachricht.
nb: Backuptabelle ist mit Versionen und Datum der Backups geladen
*)

VAR     CharArray: ARRAY [1..255] OF CHAR;
        success: BOOLEAN;
        str: String.TString;
	dat: Datum.TDatum;
        i: CARDINAL;
	Result: WholeStr.ConvResults;
	card: CARDINAL;
BEGIN
        success:=TRUE;
        str:=String.Create();
	dat:=Datum.CreateDatum();
        i:=0;
	(* Zuerst AktVersion einlesen *)
        success:=SysDatIMPORTE.ReadLine(inH,str,outM);
	IF success THEN
	  String.Lies(str,CharArray);
	  WholeStr.StrToCard(CharArray,card,Result);
	  IF (Result=WholeStr.strAllRight) THEN
	    AktVersion^:=card;
	  ELSE
	    outM:=String.Create(); 
	    String.Schreibe(outM, "Fehler beim Lesen des Versionszhlers!");
	    success:=FALSE;
	  END (* IF *);
	END (* IF *);
	(* jetzt den Inhalt der BackupTabelle laden *)
        REPEAT
	  INC(i);
          success:=SysDatIMPORTE.ReadLine(inH,str,outM);
	  IF success THEN
	    String.Lies(str,CharArray);
            WholeStr.StrToCard(CharArray,card,Result);
	    BackupTable[i].Version^:=card;
	    IF Result=WholeStr.strAllRight THEN
	      success:=SysDatIMPORTE.ReadLine(inH,str,outM);
	      IF success THEN
	        success:=Datum.DatumVonString(str,dat,outM);
	        IF success THEN
	          Datum.EqualizeDatum(BackupTable[i].BDatum,dat);
	        END (* IF *);
	      END (* IF *);
	    END (* IF *);
	  END (* IF *);
        UNTIL (i=MaxBackup) OR NOT success OR SysDatIMPORTE.Eof(inH);
	(* letztes AutoDatum laden *)
        IF success THEN
          success:=SysDatIMPORTE.ReadLine(inH,str,outM);
          IF success THEN
            success:=Datum.DatumVonString(str,dat,outM);
            IF success THEN
              Datum.EqualizeDatum(AutoDatum,dat);
            END (* IF *);
          END (* IF *);
        END (* IF *);     
	String.Dispose(str);
	Datum.DisposeDatum(dat);
        RETURN success    	
END Load;

PROCEDURE Save(inH: THandle;VAR outM:TString):BOOLEAN;     
(*
vb: ADO BackupTable existiert
Schreibe den Inhalt der BackupTabelle in die Datei HiKS.ini und speichere den
Stand des Versionszhlers und den Inhalt des letzten Automatischen Betriebs.
Falls erfolgreich wird true zurckgegeben. Im Fehlerfall enthlt inoutM eine Fehlernachricht.
nb: Inhalt von SysDat ist in HiKS.ini
*)
VAR 	CharArray: ARRAY [1..255] OF CHAR;
	success: BOOLEAN; 
	str: String.TString;
	i: CARDINAL;
BEGIN
	success:=TRUE;
	str:=String.Create();
	i:=1;
        (* Zuerst AktVersion wegschreiben *)
        WholeStr.CardToStr(AktVersion^,CharArray);
        String.Schreibe(str,CharArray);
        success:=SysDatIMPORTE.WriteLine(inH,str,outM);
	IF NOT success THEN
	  outM:=String.Create();
          String.Schreibe(outM, "Fehler beim Schreiben des Versionszhlers!");
          success:=FALSE;
        END (* IF *);
        (* jetzt den Inhalt der Backuptabelle speichern *)
       	WHILE (i<= MaxBackup) AND success DO
	  WholeStr.CardToStr(BackupTable[i].Version^,CharArray);
	  String.Schreibe(str,CharArray);
	  success:=SysDatIMPORTE.WriteLine(inH,str,outM);
	  IF success THEN
	    str:=Datum.DatumAlsString(BackupTable[i].BDatum);
	    success:=SysDatIMPORTE.WriteLine(inH,str,outM) 
	  END (* IF *);
	  INC(i);
	END (* WHILE *);
	(* Letztes AutoDatum speichern *)
	IF success THEN
	  str:=Datum.DatumAlsString(AutoDatum);
	  success:=SysDatIMPORTE.WriteLine(inH,str,outM) ;
	END (* IF *);
	String.Dispose(str);
	RETURN success	
END  Save;

PROCEDURE FirstBackup(VAR outV:TVersion; VAR outD:TDatum):BOOLEAN;
(*
vb: Backups existieren e.g. D<>UNDEF
liefert das erste Paar BackupVersion/BackupDatum zurck, also 1/Datum. Falls
D fr Backup 1 UNDEF ist, wird false zurckgegeben
nb: --
*)
BEGIN
	outD:=Datum.DuplicateDatum(BackupTable[1].BDatum);
	outV:=CreateVersion();
	outV^:=1;
	IF Datum.IstDatumEternity(BackupTable[1].BDatum) THEN
		RETURN FALSE;
	ELSE
		RETURN TRUE;
	END (* IF *);

END  FirstBackup;

PROCEDURE NextBackup(VAR inoutV:TVersion; VAR inoutD:TDatum):BOOLEAN;
(*
vb: inoutV<MaxBackup and inoutD <> UNDEF
liefert das nchste Paar BackupVersion/BackupDatum zurck, falls FirstBackup
erfolgreich war. Falls es keine BackupVersion mehr gibt wird false zurckgegeben
nb: --
*)
VAR i	: TVersionDS;
    ok	: BOOLEAN;
BEGIN
        ok:=TRUE;
	IF inoutV^ < MaxBackup THEN
		i:=inoutV^ + 1;
		IF Datum.IstDatumEternity(BackupTable[i].BDatum) THEN
			ok:=FALSE;
		ELSE
			inoutV^:=BackupTable[i].Version^;
			Datum.EqualizeDatum(inoutD, BackupTable[i].BDatum);
		END (* IF *);
        ELSE
		ok:=FALSE;
	END (* IF *);
        RETURN ok;
	
END NextBackup;

PROCEDURE LiesLetztesAutoDatum():TDatum;
(*
vb: --
Gibt ein DatumADO zurck, dass das Datum des letzten Automatischen Betriebes enthlt.
nb: --
*)
VAR AD: TDatum;
BEGIN
        AD:=Datum.CreateDatum();
        Datum.EqualizeDatum(AD,AutoDatum);
        RETURN AD    
END LiesLetztesAutoDatum;

PROCEDURE SetzeLetztesAutoDatum( inD: TDatum);
(*
vb: --
Setze das ADO LetztesAutoDatum auf den Wert inD            
nb: --
*)
BEGIN
	Datum.EqualizeDatum(AutoDatum,inD);   
END  SetzeLetztesAutoDatum;

PROCEDURE LiesSystemDatum():TDatum;
(*
vb: --
Gibt ein DatumADO zurck, dass das Systemdatum enthlt.
nb: --
*)
VAR SD: TDatum;
BEGIN
	SD:=Datum.CreateDatum();
	Datum.EqualizeDatum(SD,SystemDatum);
	RETURN SD
END LiesSystemDatum;

PROCEDURE SetzeSystemDatum( inD: TDatum);
(*
vb: --
Setze das Systemdatum auf den Wert inD
nb: --
*)
BEGIN
	Datum.EqualizeDatum(SystemDatum,inD);
END SetzeSystemDatum;   

(*********************************Initialisierungen****************************)

BEGIN
	SystemDatum:= Datum.CreateDatum();
	AutoDatum:=Datum.CreateDatum();
	AktVersion:=CreateVersion();
	AktVersion^:=1;
	CreateBackupTable();
END SysDatADO.	



