(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: VtrListe.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 16:51:11 $
# $Revision: 1.2 $
# Bemerkungen: 

IMPLEMENTATION MODULE VtrListe   ADT SortierteVertraegeListe

Entwurf dur TS_HKe: Sigrid Preuss

Implementation und Anpassung an TS_Vtre: Schoenhammer Herbert

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE VtrListe;

(* ADT SortierteVertragsListe *)

(* IMPORTE *)


  IMPORT Vtr;
  IMPORT SortierteObjektliste;

  TYPE TVtrListe =  SortierteObjektliste.TSortierteObjektliste;
       TObjektreferenz = SortierteObjektliste.TObjektreferenz;


(* SEMANTIK:

Die Semantik ist dem Modul "SortierteObjektliste.def" entnommen

Die Semantik wird auf Basis zweier lokaler Integervariablen len und pos,
die den Zustand der sortierten Objektliste beschreiben, und einer
lokalen Kapazitaetskonstanten maxlen spezifiziert: 
Sei len die Laenge der sortierten Objektliste, maxlen > 0 die Maximal-
kapazitaet der Liste, d.h. 0 <= len <= maxlen, und pos die Position 
des aktuellen Listenelements mit 0 <= pos <= len + 1.
Die Indizes alt und neu bezeichnen den Wert einer Zustandsvariablen vor
bzw. nach Ausfuehren einer Operation. Werden die neuen Zustaende der
Variablen nicht explizit angegeben, so sind sie invariant.
*)

PROCEDURE  Create () : TVtrListe;
(*
Vb:	keine
Generiert und initialisert ein ADO TVtrListe.
Nb:	Rueckgabe-ADO TVtrListe existiert AND IsEmpty(TVtrListe)
*)

  VAR outVtrListe : TVtrListe;

BEGIN
  outVtrListe := SortierteObjektliste.Create (DisposeVtr,
                                             DuplicateVtr,
                                             EqualizeVtr,
                                             IsEqualVtr,
                                             SmallerVertrag,
                                             SameVertrag);
  RETURN outVtrListe;

END Create;


PROCEDURE Dispose (VAR inoutVtrListe : TVtrListe);
(*
Vb:	IsEmpty (inoutVtrListe : TVtrListe)
Entfernt das ADO inoutVtrListe aus dem System.
Nb:	ADO inoutVtrListe existiert nicht mehr
*)
BEGIN
  SortierteObjektliste.Dispose (inoutVtrListe);
END Dispose;


PROCEDURE Current(inVtrListe : TVtrListe) : TVtr;
(*
Vb:	NOT (OffStart (inVtrListe) OR OffEnd (inVtrListe))
Liefert das Objekt an der aktuellen Position.
Nb:	NOT (OffStart(inVtrListe) OR OffEnd(inVtrListe))
*)
BEGIN
  RETURN Vtr.VERTRAG (SortierteObjektliste.Current (inVtrListe));
END Current;


PROCEDURE Update (inoutVtrListe : TVtrListe;
                  inObjekt     : TVtr);
(*
Vb:	NOT (OffStart (inoutVtrListe) OR OffEnd (inoutVtrListe))
Ueberschreibt unter Beachtung der Ordnung die Attributwerte des
Objekts an der aktuellen Position mit denen von inObjekt.
Anschliessend sind das Objekt an der ggf. neuen aktuellen
Position und inObjekt zustandsgleich
Nb:	NOT (OffStart (inoutVtrListe) OR OffEnd (inoutVtrListe)
*)
BEGIN
  SortierteObjektliste.Update (inoutVtrListe, Vtr.OBJEKTREFERENZ (inObjekt));
END Update;


PROCEDURE Insert (inoutVtrListe : TVtrListe;
                  inObjekt     : TVtr);
(*
Vb:	keine
Einfuegen eines Objekts bzgl. der Sortierreihenfolge
Anschliessend befindet sich inObjekt an der aktuellen Position
Nb:	(1 <= inoutVtrListe.pos_neu <= inoutVtrListe.len) AND
              (inoutVtrListe.len_neu = inoutVtrListe.len_alt + 1) AND NOT
              (IsEmpty (inoutVtrListe) OR OffStart (inoutVtrListe)
               OR OffEnd (inoutVtrListe) )
*)

BEGIN
  SortierteObjektliste.Insert (inoutVtrListe, Vtr.OBJEKTREFERENZ (inObjekt));
END Insert;


PROCEDURE Delete (inoutVtrListe : TVtrListe);
(*
Vb:	NOT (OffStart (inoutVtrListe) OR OffEnd (inoutVtrListe))
Loeschen des Objekts an der aktuellen Position; das Listenelement rechts
neben dem zu loeschenden Element befindet sich anschliessend an der
(unveraenderten) aktuellen Position.
Wird das letzte Objekt geloescht, steht die aktuelle Position auf OffEnd
Nb:     (inoutVtrListe.len_neu = inoutVtrListe.len_alt - 1) OR
         OffStart (inoutVtrListe))
*)
BEGIN
  SortierteObjektliste.Delete (inoutVtrListe);
END Delete;


PROCEDURE Search (inoutVtrListe : TVtrListe;
                  inObjekt     : TVtr) : BOOLEAN;
(*
Vb:	keine
Suche nach inObjekt; wird ein zustandsgleiches Objekt gefunden,
ist dort die neue aktuelle Position; sonst OffEnd
Sei found der boole'sche Rueckgabeparameter, dann gilt:
Nb: (found => NOT (OffStart (inoutVtrListe) OR OffEnd (inoutVtrListe))) AND
    (NOT found => OffEnd (inoutVtrListe))
*)

BEGIN
  RETURN SortierteObjektliste.Search (inoutVtrListe,
                                      Vtr.OBJEKTREFERENZ (inObjekt));
END Search;


PROCEDURE GoSmallest (inoutVtrListe : TVtrListe);
(*
Vb:	keine
Die Position des kleinsten Objekts der Liste wird zur aktuellen Position
Nb:	inoutVtrListe.pos_neu = 1
*)
BEGIN
  SortierteObjektliste.GoSmallest (inoutVtrListe);
END GoSmallest;


PROCEDURE StopAt (inoutVtrListe : TVtrListe;
                  inObjekt     : TVtr) : BOOLEAN;
(*
Vb:	keine
Suche nach einem bzgl. SameValue Objekt gleichen Objekt. Wird ein
solches Objekt gefunden, ist dort die neue aktuelle Position;
Seien found der boole'sche Rueckgabeparameter und aktObjekt bzw.
MaxObjekt die Objekte an pos_neu bzw. pos = len (falls diese
existieren), dann gilt:
Nb:    (found => NOT (OffStart (inoutVtrListe) OR OffEnd (inoutVtrListe)) AND
                 Objekt.SameValue (inObjekt, aktObjekt)) AND
       (NOT found => (IsEmpty (inoutVtrListe) AND OffEnd (inoutVtrListe)) OR
                     (NOT IsEmpty (inoutVtrListe) AND OffEnd (inoutVtrListe) AND
                      Objekt.SmallerValue (MaxObjekt, inObjekt)) OR
                      (NOT (OffStart (inoutVtrListe) OR
                            OffEnd (inoutVtrListe)) AND
                       Objekt.SmallerValue (aktObjekt, inObjekt))
*)

BEGIN
  RETURN SortierteObjektliste.StopAt (inoutVtrListe, Vtr.OBJEKTREFERENZ (inObjekt));
END StopAt;



PROCEDURE GoLargest (inoutVtrListe : TVtrListe);
(*
Vb:	keine
Die Position des groessten Objekts der Liste wird zur aktuellen Position
Nb:	inoutVtrListe.pos_neu = inoutVtrListe.len
*)

BEGIN
  SortierteObjektliste.GoLargest (inoutVtrListe);
END GoLargest;


PROCEDURE StepNext (inoutVtrListe : TVtrListe);
(*
Vb:	NOT OffEnd (inoutVtrListe)
Die Position hinter der aktuellen Position wird zur aktuellen Position
Nb:	(inoutVtrListe.pos_neu = inoutVtrListe.pos_alt + 1
         OR (IsEmpty (inoutVtrListe) => OffEnd (inoutVtrListe)
*)

BEGIN
  SortierteObjektliste.StepNext (inoutVtrListe);
END StepNext;



PROCEDURE StepBack (inoutVtrListe : TVtrListe);
(*
Vb:	NOT OffStart (inoutVtrListe)
Die Position vor der aktuellen Position wird zur aktuellen Position
Nb:	inoutVtrListe.pos_neu = inoutVtrListe.pos_alt - 1) OR
        (IsEmpty (inoutVtrListe) => OffStart (inoutVtrListe)
*)

BEGIN
  SortierteObjektliste.StepBack (inoutVtrListe);
END StepBack;


PROCEDURE IsEmpty (inVtrListe : TVtrListe) : BOOLEAN;
(*
Vb:	keine
Test, ob die Liste leer ist:
IsEmpty (inVtrListe) <=> (inVtrListe.len = 0)
Nb:	IsEmpty (inVtrListe) => OffStart (inVtrListe) OR OffEnd (inVtrListe)
*)

BEGIN
  RETURN SortierteObjektliste.IsEmpty (inVtrListe);
END IsEmpty;


PROCEDURE OffStart (inVtrListe : TVtrListe) : BOOLEAN;
(*
Vb:	keine
Test, ob die Position vor dem ersten Objekt aktuell ist:
OffStart (inVtrListe) <=> inVtrListe.pos = 0
Nb:	keine
*)

BEGIN
  RETURN SortierteObjektliste.OffStart (inVtrListe);
END OffStart;


PROCEDURE OffEnd (inVtrListe : TVtrListe) : BOOLEAN;
(*
Vb:	keine
Test, ob die Position hinter dem letzten Objekt aktuell ist:
OffEnd (inVtrListe) <=> inVtrListe.pos = inVtrListe.len + 1
Nb:	Rueckgabe-ADO TVtrListe existiert AND IsEmpty(TVtrListe)
*)

BEGIN
  RETURN SortierteObjektliste.OffEnd (inVtrListe);
END OffEnd;


(* ------------------------------------------------------------------
   -- Typkonvertierungen von TVtr und TObjektreferenz
   ------------------------------------------------------------------
*)

PROCEDURE DisposeVtr (VAR inoutReferenz : TObjektreferenz);
 VAR aktVtr : TVtr;
BEGIN
  aktVtr := Vtr.VERTRAG (inoutReferenz);
  Vtr.DisposeVtr (aktVtr);
  inoutReferenz := Vtr.OBJEKTREFERENZ (aktVtr);
END DisposeVtr;

PROCEDURE DuplicateVtr (inReferenz : TObjektreferenz) : TObjektreferenz;
  VAR Source      : TVtr;
      outDuplikat : TVtr;
BEGIN
  Source := Vtr.VERTRAG (inReferenz);
  outDuplikat := Vtr.DuplicateVtr (Source);
  RETURN Vtr.OBJEKTREFERENZ (outDuplikat)
END DuplicateVtr;

PROCEDURE EqualizeVtr (inoutZiel, inQuelle : TObjektreferenz);
  VAR ZielVtr  : TVtr;
      QuellVtr : TVtr;
BEGIN
 ZielVtr := Vtr.VERTRAG (inoutZiel);
 QuellVtr := Vtr.VERTRAG (inQuelle);
 Vtr.EqualizeVtr (ZielVtr, QuellVtr)
END EqualizeVtr;

PROCEDURE IsEqualVtr (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Vtr1 : TVtr;
      Vtr2 : TVtr;
BEGIN
  Vtr1 := Vtr.VERTRAG (inReferenz1);
  Vtr2 := Vtr.VERTRAG (inReferenz2);
  RETURN Vtr.IsEqualVtr (Vtr1, Vtr2)
END IsEqualVtr;

PROCEDURE SmallerVertrag (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Vtr1 : TVtr;
      Vtr2 : TVtr;
BEGIN
  Vtr1 := Vtr.VERTRAG (inReferenz1);
  Vtr2 := Vtr.VERTRAG (inReferenz2);
  RETURN Vtr.SmallerVertrag (Vtr1, Vtr2)
END SmallerVertrag;

PROCEDURE SameVertrag (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Vtr1 : TVtr;
      Vtr2 : TVtr;
BEGIN
  Vtr1 := Vtr.VERTRAG (inReferenz1);
  Vtr2 := Vtr.VERTRAG (inReferenz2);
  RETURN Vtr.SameVertrag (Vtr1, Vtr2)
END SameVertrag;

END VtrListe.
