(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: VtrOSchablone.mod,v $
# $Author: Sopra $
# $Date: 2001/03/14 08:51:03 $
# $Revision: 1.14 $
Bemerkungen:

IMPLEMENTATION MODULE VtrOrdnerSchablone   ADT-Modul VertraegeOrdnerSchablone

Implementation : Herbert Schoenhammer, Herbert.Schoenhammer@Fernuni-Hagen.de
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE VtrOSchablone;        (* ENTHALTEN IN Vertraege TS_Vtre *)


  FROM VtreIMPORTE IMPORT TVtrID,
                          HoleNeueHID,
                          UeberlasseHID,
                          DuplicateHID,
                          DisposeHID,
                          IsEqualHID,

                          TPersID,
                          IsEqualPersID,
                          DisposePersID,
                          PersIDAlsString,
                          StringInPersID,
                          PruefePersID,

                          TKstID,
                          IsEqualKstID,
                          DisposeKstID,
                          KstIDAlsString,
                          StringInKstID,
                          PruefeKstID,

                          THandle,
                          WriteLineInVDat,
                          ReadLineInVDat,
                          EofInVDat;
                          
  IMPORT Vtr;
  IMPORT VtrListe;

  IMPORT String;
  IMPORT Stringliste;
  FROM Storage IMPORT ALLOCATE, DEALLOCATE;
  FROM SYSTEM IMPORT TSIZE;
  FROM WholeStr IMPORT ConvResults, CardToStr, StrToCard;
  FROM Datum IMPORT TDatum, CreateDatum, DisposeDatum, DuplicateDatum, EqualizeDatum,
                    IsEqualDatum, SmallerDatum, DatumInZeitraum, DatumAlsString,
                    DatumVonString, Wochentag, MinDatum, MaxDatum, EqualizeDatum;



  TYPE TString = String.TString;
       TStringliste = Stringliste.TStringliste;

  TVtrOrdnerDS = RECORD
                   Ordner             : VtrListe.TVtrListe;
                 END;
  TVtrOrdner   = POINTER TO TVtrOrdnerDS;



(*==============================================================================*)
(* Lokale Funktion zur Berechnung von Zeitraum-Ueberschneidungen                *)
(*==============================================================================*)


 PROCEDURE ZeitraumUeberschneidung (inVtr : TVtr; 
                                    ZtrAnfang, ZtrEnde : TDatum ) : BOOLEAN;
 (* Berechnet, ob sich der Vertragszeitraum [inVtr.BueltigVon, inVtr.GueltigBis] 
    mit dem EingabeIntervall [ZtrAnfang, ZtrEnde] ueberdeckt. 
    Sei ueberdeckt der boolesche
    Rueckgabeparameter, dann gilt:
    ueberdeckt <=> ein Tag aus [ZtrAnfang, ZtrEnde] liegt in
    [inVtr.BueltigVon, inVtr.GueltigBis]                                   *)

 VAR ueberschneidet : BOOLEAN;
     vtrVon, vtrBis : TDatum;
 BEGIN
   vtrVon := Vtr.LiesGueltigVoninVtr (inVtr);
   vtrBis := Vtr.LiesGueltigBisinVtr (inVtr);

   ueberschneidet := (SmallerDatum (ZtrAnfang, vtrVon) AND SmallerDatum (vtrBis, ZtrEnde))
                  OR (SmallerDatum (vtrVon, ZtrAnfang) AND SmallerDatum (ZtrEnde, vtrBis))
                  OR DatumInZeitraum (vtrVon, ZtrAnfang, ZtrEnde)
                  OR DatumInZeitraum (vtrBis, ZtrAnfang, ZtrEnde) ;
   DisposeDatum (vtrVon);
   DisposeDatum (vtrBis);

   RETURN ueberschneidet
 END ZeitraumUeberschneidung;  

 PROCEDURE PruefeDatum (gueltigVon, gueltigBis : TDatum; VAR outMessage : TString) : BOOLEAN;
 (* vb: keine
        Es wird geprueft, ob ob das angegebene Datum TagVon ein Montag ist. Wenn nicht,
        wird der eingegebene Wochentag ausgegeben. Es wird geprueft, ob das eingegebene
        Datum TagBis ein Sonntag oder undef ist.Wenn nicht, wird der eingegebene
        Wochentag ausgegeben.
        Sei rc der boolesche Rueckgabeparameter, dann gilt:
    nb: rc=TRUE <=> (wenn TagVon =Montag AND TagBis = (Sonntag OR undef))
        rc = FALSE sonst *)

 VAR success : BOOLEAN;
     TagVon, TagBis : CARDINAL;

 BEGIN

  success := TRUE;
  TagVon:= Wochentag(gueltigVon);
  TagBis:= Wochentag(gueltigBis);

  IF SmallerDatum (gueltigBis, gueltigVon) THEN
     success := FALSE;
     outMessage := String.Create();
     String.Schreibe (outMessage, "Das VertragsEnde liegt vor VertragsBeginn");
  ELSE
     IF (TagVon # 1) THEN
        outMessage := String.Create();
        success:= FALSE;
        CASE TagVon OF
	    2 : String.Schreibe(outMessage,
	   	"Das GueltigVon Datum fllt auf einen Dienstag!");
           |3 :  String.Schreibe(outMessage,
                "Das GueltigVon Datum fllt auf einen Mittwoch!");
           |4 :  String.Schreibe(outMessage,
                 "Das GueltigVon Datum fllt auf einen Donnerstag!");
           |5 :  String.Schreibe(outMessage,
                 "Das GueltigVon Datum fllt auf einen Freitag!");
           |6 :  String.Schreibe(outMessage,
                  "Das GueltigVon Datum fllt auf einen Samstag!");
           |7 :  String.Schreibe(outMessage,
                  "Das GueltigVon Datum fllt auf einen Sonntag!");
           |0 :  String.Schreibe(outMessage,
                  "Das GueltigVon Datum ist undefiniert!");
        END (* CASE *);
     ELSIF  (TagBis # 7) AND (TagBis # 0) THEN
        outMessage := String.Create();
        success:= FALSE;
        CASE TagBis OF
            1 :  String.Schreibe(outMessage,
                 "Das GueltigBis Datum fllt auf einen Montag!");
           |2 : String.Schreibe(outMessage,
                 "Das GueltigBis Datum fllt auf einen Dienstag!");
           |3 :  String.Schreibe(outMessage,
                 "Das GueltigBis Datum fllt auf einen Mittwoch!");
           |4 :  String.Schreibe(outMessage,
                 "Das GueltigBis Datum fllt auf einen Donnerstag!");
           |5 :  String.Schreibe(outMessage,
                 "Das GueltigBis Datum fllt auf einen Freitag!");
           |6 :  String.Schreibe(outMessage,
                 "Das GueltigBis Datum fllt auf einen Samstag!");
        END (* CASE *);
      END; (* IF (TagVon # 1) *)
    END; (* IF SmallerDatum *)

   RETURN success;
 END PruefeDatum;

 PROCEDURE LoescheAllesInVtreOrdner (inoutVtreOrdner : TVtrOrdner );
 (* vb: keine
        Loescht alle Vertraege aus inoutVtreOrdner. Wird bentigt, um die
        Liste vor dem Restaurieren des Ordners von Festplatte zu leeren.
    nb: inoutVtreOrdner ist leer *)

 BEGIN

   VtrListe.GoSmallest (inoutVtreOrdner^.Ordner);
   WHILE NOT VtrListe.IsEmpty (inoutVtreOrdner^.Ordner) DO
      VtrListe.Delete (inoutVtreOrdner^.Ordner);
   END; (* WHILE *)

END LoescheAllesInVtreOrdner;


(*=============================================================================*)
(* StandardOperationen fuer SortierteObjektliste                               *)
(*=============================================================================*)



  PROCEDURE CreateVtrOrdner () : TVtrOrdner;
  (*    -- Generiert ein neues abstraktes Datenobjekt outOrdner *)
    VAR VtrOrdner : TVtrOrdner;
  BEGIN
    (* -- Generiert ein neues abstraktes Datenobjekt outListe *)
    ALLOCATE (VtrOrdner, TSIZE(TVtrOrdnerDS));
    VtrOrdner^.Ordner := VtrListe.Create();
    RETURN VtrOrdner
    (* -- Nachbed.: ADO outListe existiert AND IsEmpty (outListe) *)
  END CreateVtrOrdner;


  (* -- GEMEINSAME VORBEDINGUNG FUER ALLE UEBRIGEN OPERATIONEN:  
     --     ADOs inListe bzw. inoutListe existieren *)


  (* PROCEDURE DisposeVtrOrdner (VAR inoutVtrOrdner : TVtrOrdner);
  BEGIN
    VtrListe.Dispose(inoutVtrOrdner^.Ordner);
    DEALLOCATE (inoutVtrOrdner, TSIZE(TVtrOrdner));
    inoutVtrOrdner := NIL;
  END DisposeVtrOrdner; *)

  PROCEDURE IsEmptyVtrOrdner (inVtrOrdner : TVtrOrdner) : BOOLEAN;
  (* VB: keine
         Test, ob der Ordner leer ist
     Nb: IsEmpty (inHKOrdner) <=> kap = 0 *)
  VAR leer : BOOLEAN;
  BEGIN
    RETURN VtrListe.IsEmpty (inVtrOrdner^.Ordner);
  END IsEmptyVtrOrdner;


  PROCEDURE InsertVtr ( inoutVtr: TVtr; VAR outMessage : TString;
                        inoutVtrOrdner : TVtrOrdner;
                        inVariante : BOOLEAN ) : BOOLEAN;
   
  (*
     Vb: NOT Vertrag.ExistiertVtrID(inoutVtr)
         Fuegt den Vertrag in den Ordner inoutVtrOrdner ein und setzt
         den Identifikator des Vertrages

         inVariante=TRUE : Es wird eine neue VertragsID aus der
                           HIDVerwaltung angefordert und in den Vertrag
                           eingefuegt.

         inVariante=FALSE: Es wird keine VertragsID generiert. Der Vertrag
                           wird mit seiner bestehenden VertragsID in die Liste
                           eingefuegt. Wird benoetigt zum Restaurieren des
                           Ordners von Datei.

     Nb: (kap_neu = kap_alt+1) and Vertrag.ExistiertVtrID(inoutVtr)
         Vtr.GueltigVon <= Vtr.GueltigBis AND
         PruefeDatum (Vtr.GueltigVon, Vtr.Gueltig.Bis *)


  VAR aktVtrID : TVtrID;
      gueltigVon, gueltigBis : TDatum;
      erfolgreich : BOOLEAN;
  BEGIN

    IF inVariante THEN

     (* Pruefen, ob gueltigVon <= guelitgBis *)
     gueltigVon := Vtr.LiesGueltigVoninVtr (inoutVtr);
     gueltigBis := Vtr.LiesGueltigBisinVtr (inoutVtr);


      (* VertragsAnfang = Montag ?? und VertragsEnde = Freitag ?? *)
      IF NOT PruefeDatum (gueltigVon, gueltigBis, outMessage) THEN
         erfolgreich := FALSE;
      ELSE


       (* Einfuegen mit neu generierter VertragsID *)
       IF NOT Vtr.ExistiertVtrID(inoutVtr) THEN
          aktVtrID := HoleNeueHID();
          Vtr.SetzeVtrID(inoutVtr, aktVtrID);
          VtrListe.Insert(inoutVtrOrdner^.Ordner,inoutVtr);
          DisposeHID(aktVtrID);
          erfolgreich := TRUE;
       ELSE
          outMessage := String.Create();
          String.Schreibe(outMessage, "VtrID ist bereits im Ordner vorhanden.");
          erfolgreich := FALSE;
       END; (* IF NOT Vtr.ExistiertVtrID *)

      END; (* IF (Wochentag ... *)

     DisposeDatum (gueltigVon);
     DisposeDatum (gueltigBis);

    ELSE

       (* Einfuegen mit vorhandener VertragsID *)
       IF NOT Vtr.ExistiertVtrID(inoutVtr) THEN
          outMessage := String.Create();
          String.Schreibe(outMessage, "Keine VtrID im VertragsObjekt gesetzt.");
          erfolgreich := FALSE;
       ELSE
          VtrListe.Insert(inoutVtrOrdner^.Ordner,inoutVtr);
          erfolgreich := TRUE;
       END; (* IF NOT Vtr.ExistiertVtrID *)

    END; (* IF inVariante *)

    RETURN erfolgreich
  END InsertVtr;


 PROCEDURE SearchVtr( inVtrID: TVtrID; VAR outVtr : TVtr; 
                      inoutVtrOrdner:TVtrOrdner):BOOLEAN;
 (* Vb: keine
        Sucht den Vertrag mit der uebergebenen inVtr.ID; wird der Vertrag gefunden,
        so wird sie TRUE zurueckgeliefert
    Nb: (existiert=>(inVtr existiert AND Vtr.ID=inVtr.ID AND Rueckgabe = TRUE)
         AND (nicht rc => (inVtr.ID  existiert nicht AND Rueckgabe = FALSE))  *)
  VAR aktVtrID  : TVtrID;
      existiert : BOOLEAN;
      aktVtr    : TVtr;
  BEGIN
    existiert := FALSE;
    IF NOT IsEmptyVtrOrdner (inoutVtrOrdner) THEN
      WITH inoutVtrOrdner^ DO
         VtrListe.GoSmallest (Ordner);
         WHILE NOT (existiert OR VtrListe.OffEnd (Ordner)) DO
           aktVtr := VtrListe.Current (Ordner);
           aktVtrID := Vtr.LiesVtrIDinVtr (aktVtr);
           IF IsEqualHID (aktVtrID, inVtrID) THEN
             existiert := TRUE;
             outVtr := Vtr.DuplicateVtr (aktVtr);
           ELSE
             VtrListe.StepNext (Ordner)
           END; (* IF *)
           DisposeHID (aktVtrID);
           Vtr.DisposeVtr (aktVtr)
         END; (* WHILE nicht gefunden*)
      END (* WITH inoutVtrOrdner^*)
    END; (* IF Ordner nicht leer*)
    RETURN existiert
  END SearchVtr;


 PROCEDURE AendernVtr (inVtr : TVtr; VAR outMessage : TString; 
                       inoutVtrOrdner:TVtrOrdner):BOOLEAN;
 (* Vb.: keine
         Sucht im Ordner den Vertrag mit der uebergebenen inVtr.ID; 
         wird der Vertrag Vtr gefunden, so werden die Attribute GueltigBis, StornoAm,
         Stundenzahl, Taetigkeit und Begruendung des Ordnervertrages denen von 
         inVtr angeglichen.
    Nb.: Vtr.GueltigVon <= Vtr.GueltigBis AND
         Vtr.Ende = Sonntag | undef AND
         inVtr.Ende <= Vtr.Ende *)

 VAR erfolgreich, egal : BOOLEAN;
     gueltigvon, gueltigbis, gueltig_alt_bis, stornoam : TDatum;
     stundenzahl : CARDINAL;
     taetigkeit, begruendung : TString;
     current_vtr : TVtr;
 BEGIN


   IF VtrListe.StopAt (inoutVtrOrdner^.Ordner, inVtr) THEN

      gueltigbis := Vtr.LiesGueltigBisinVtr (inVtr);
      stornoam := Vtr.LiesStornoAminVtr (inVtr);
      stundenzahl := Vtr.LiesStdzahlinVtr (inVtr);
      taetigkeit := Vtr.LiesTaetigktinVtr (inVtr);
      begruendung := Vtr.LiesBegruendinVtr (inVtr);

      current_vtr := VtrListe.Current (inoutVtrOrdner^.Ordner);
      gueltigvon := Vtr.LiesGueltigVoninVtr (current_vtr);
      gueltig_alt_bis := Vtr.LiesGueltigBisinVtr (current_vtr);

     IF SmallerDatum (gueltig_alt_bis, gueltigbis) THEN
        erfolgreich := FALSE;
        outMessage := String.Create();
        String.Schreibe (outMessage, "Das Vertragsende kann nicht verlngert werden");
     ELSE

        IF NOT PruefeDatum (gueltigvon , gueltigbis, outMessage) THEN
         erfolgreich := FALSE;
        ELSE
         Vtr.SetzeGueltigBisinVtr (current_vtr, gueltigbis);
         egal := Vtr.SetzeStornoAminVtr (current_vtr, stornoam);
         egal := Vtr.SetzeStdzahlinVtr (current_vtr, stundenzahl);
         egal := Vtr.SetzeBegruendinVtr (current_vtr, begruendung);
         VtrListe.Update (inoutVtrOrdner^.Ordner, current_vtr);
         erfolgreich := TRUE;
        END; (* IF NOT ((Wochentag ... *)

      END; (* SmallerDatum *)

      Vtr.DisposeVtr (current_vtr);
      DisposeDatum (gueltigvon);
      DisposeDatum (gueltigbis);
      DisposeDatum (stornoam);
      String.Dispose (taetigkeit);
      String.Dispose (begruendung);

   ELSE
      outMessage := String.Create();
      String.Schreibe (outMessage, "Zu ndernder Vertrag nicht im Ordner gefunden");
      erfolgreich := FALSE;
   END; (* IF *)

   RETURN erfolgreich;
 END AendernVtr;


 PROCEDURE DeleteVtr( inVtr: TVtr; inoutVtrOrdner:TVtrOrdner):BOOLEAN;
 (* Vb: keine
        Sucht den Vertrag mit der uebergebenen inVtr.ID im Ordner;
        wird der Vertrag gefunden, so wird er aus dem Ordner entfernt und die
        frei werdende VtrID an die HIDVerwaltung zurckgegeben.
        Sei rc der boolesche Rueckgabeparameter, so gilt:
    Nb: (rc=>(inVtr aus Ordner entfernt, inVtrID an HIDVerwaltung zurckgegeben)
         AND (nicht rc => (inVtr.ID  existiert nicht im Ordner))  *)
  VAR rc : BOOLEAN;
      freie_id : TVtrID;
  BEGIN

    IF VtrListe.StopAt (inoutVtrOrdner^.Ordner, inVtr) THEN

       (* VertragsID zureuckgeben *)
       freie_id := Vtr.LiesVtrIDinVtr (inVtr);
       UeberlasseHID (freie_id);
       DisposeHID (freie_id);

       (* Vertrag im Ordner loeschen *)
       VtrListe.Delete (inoutVtrOrdner^.Ordner);
       rc := TRUE;

    ELSE

       (* Vertrag nicht im Ordner gefunden *)
       rc := FALSE;
    END;

    RETURN rc;

 END DeleteVtr;


 (* Iteratoren ueber nicht stornierte Vertraege *)
 (*=============================================*)


 PROCEDURE FirstVtr (VAR outVtr:TVtr; inoutVtrOrdner:TVtrOrdner):BOOLEAN;
 (* Vb: NOT IsEmpty(inoutVtrOrdner)
        Der erste nicht stornierte Vertrag wird in outVtr zurueckgegeben
        und inoutHKOrdner.pos auf diesen Vertrag positioniert.
    Nb: (outVtr enthaelt den Vertrag *)

  VAR existiert, next_vtr : BOOLEAN;
      aktVtr    : TVtr;
  BEGIN
    existiert := FALSE;
    IF FirstVtrUndStorno (aktVtr, inoutVtrOrdner)
    THEN
      next_vtr := TRUE;
      WHILE (NOT existiert) AND next_vtr DO
        IF NOT Vtr.IstStornoInVtr (aktVtr) THEN
         existiert := TRUE;
         outVtr := Vtr.DuplicateVtr (aktVtr);
        ELSE
         next_vtr := NextVtrUndStorno(aktVtr, inoutVtrOrdner)
        END;
      END; (* WHILE *)
      Vtr.DisposeVtr(aktVtr);
    END; (* IF FirstVtrUndStorno *)
    RETURN existiert
 END FirstVtr;


 PROCEDURE NextVtr (inoutVtr:TVtr; inoutVtrOrdner:TVtrOrdner):BOOLEAN;
 (* vb: inoutVtr existiert in VertraegeOrdner
	Suche inVtr im VertraegeOrdner und falls ein nicht stornierter Nachfolger 
      existiert gib diesen in inoutVtr zurck, false sonst.
    nb: -- *)
 VAR existiert : BOOLEAN;
     aktVtr : TVtr;
 BEGIN
   aktVtr := Vtr.CreateVtr();
   Vtr.EqualizeVtr (aktVtr, inoutVtr);

   existiert := FALSE;
   WHILE NextVtrUndStorno (aktVtr, inoutVtrOrdner) AND NOT existiert DO
      IF NOT Vtr.IstStornoInVtr (aktVtr) THEN
         existiert := TRUE;
         Vtr.EqualizeVtr (inoutVtr, aktVtr);
      END;  (* IF *)
   END; (* WHILE *)

   Vtr.DisposeVtr (aktVtr);

   RETURN existiert
 END NextVtr;


 PROCEDURE FirstVldVtrAbDatum (inDatum : TDatum; VAR outVtr:TVtr;
                               inoutVtrOrdner:TVtrOrdner):BOOLEAN;
 (* Vb: NOT IsEmpty(inoutVtrOrdner)
        Der erste nicht stornierte Vertrag, dessen GueltigBis >= inDatum
        wird in outVtr zurueckgegeben
        und inoutHKOrdner.pos auf diesen Vertrag positioniert.
    Nb: (outVtr enthaelt den Vertrag *)

  VAR existiert, next_vtr : BOOLEAN;
      aktVtr    : TVtr;
      gueltigbis : TDatum;
  BEGIN
    existiert := FALSE;
    IF FirstVtrUndStorno (aktVtr, inoutVtrOrdner)
    THEN
      next_vtr := TRUE;
      WHILE (NOT existiert) AND next_vtr DO
        gueltigbis := Vtr.LiesGueltigBisinVtr (aktVtr);
        IF (NOT Vtr.IstStornoInVtr (aktVtr)) AND
           (NOT SmallerDatum (gueltigbis, inDatum)) THEN
         existiert := TRUE;
         outVtr := Vtr.DuplicateVtr (aktVtr);
        ELSE
         next_vtr := NextVtrUndStorno(aktVtr, inoutVtrOrdner)
        END;
        DisposeDatum (gueltigbis);
      END; (* WHILE *)
      Vtr.DisposeVtr(aktVtr);
    END; (* IF FirstVtrUndStorno *)
    RETURN existiert
 END FirstVldVtrAbDatum;


 PROCEDURE NextVldVtrAbDatum (inDatum : TDatum; inoutVtr:TVtr; 
                              inoutVtrOrdner:TVtrOrdner):BOOLEAN;
 (* Vb: NOT IsEmpty(inoutVtrOrdner)
        Der nchste nicht stornierte Vertrag, dessen GueltigBis >= inDatum
        wird in outVtr zurueckgegeben
        und inoutHKOrdner.pos auf diesen Vertrag positioniert.
    Nb: (outVtr enthaelt den Vertrag *)

  VAR existiert, next_vtr : BOOLEAN;
      aktVtr : TVtr;
     gueltigbis : TDatum;
  BEGIN
    aktVtr := Vtr.CreateVtr ();
    Vtr.EqualizeVtr (aktVtr, inoutVtr);

    existiert := FALSE;
    IF NextVtrUndStorno (aktVtr, inoutVtrOrdner)
    THEN
      next_vtr := TRUE;
      WHILE (NOT existiert) AND next_vtr DO
        gueltigbis := Vtr.LiesGueltigBisinVtr (aktVtr);
        IF (NOT Vtr.IstStornoInVtr (aktVtr)) AND
           (NOT SmallerDatum (gueltigbis, inDatum)) THEN
         existiert := TRUE;
         Vtr.EqualizeVtr (inoutVtr, aktVtr);
        ELSE
         next_vtr := NextVtrUndStorno(aktVtr, inoutVtrOrdner)
        END;
        DisposeDatum (gueltigbis);
      END; (* WHILE *)
      Vtr.DisposeVtr(aktVtr);
    END; (* IF FirstVtrUndStorno *)
    RETURN existiert
 END NextVldVtrAbDatum;


 PROCEDURE FirstVtrImZeitraum( inGueltigVon, inGueltigBis:TDatum;
                               VAR outVtr : TVtr; 
                               inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 (*
    vb: VertraegeOrdner existiert
        Hole den ersten nicht stornierten Vertrag, dessen GueltigVon-Datum
        im Zeitraum inGueltigVon-inGueltigBis liegt
        Sei OK der boolesche Rueckgabecode, dann gilt:
    nb: OK <=> nicht stornierter Vertrag im Zeitraum existiert
 *)
 VAR existiert, next_vtr : BOOLEAN;
     aktVtr : TVtr;

 BEGIN

    existiert := FALSE;
    IF FirstVtr (aktVtr, inoutVtrOrdner)
    THEN
      next_vtr := TRUE;
      WHILE (NOT existiert) AND next_vtr DO
        IF ZeitraumUeberschneidung (aktVtr, inGueltigVon, inGueltigBis) THEN
         existiert := TRUE;
         outVtr := Vtr.DuplicateVtr (aktVtr);
        ELSE
         next_vtr := NextVtr(aktVtr, inoutVtrOrdner)
        END;
      END; (* WHILE *)
      Vtr.DisposeVtr(aktVtr);
    END; (* IF FirstVtrUndStorno *)

   RETURN existiert
 END FirstVtrImZeitraum;


 PROCEDURE NextVtrImZeitraum( inGueltigVon, inGueltigBis:TDatum;
                              inoutVtr : TVtr; inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 (*
    vb: VertraegeOrdner existiert
        Falls mit FirstVtrImZeitraumHolen ein nicht stornierter Vertrag,
        dessen GueltigVon-Datum im Zeitraum
        inGueltigVon-inGueltigBis gefunden wurde, dann holt diese Prozedur den
        naechsten Vertrag.
        Sei OK der boolesche Rueckgabecode, dann gilt:
    nb: OK <=> Weiterer nicht stornierter Vertrag im Zeitraum existiert
 *)
 VAR existiert : BOOLEAN;
     aktVtr : TVtr;
 BEGIN
   aktVtr := Vtr.CreateVtr();
   Vtr.EqualizeVtr (aktVtr, inoutVtr);

   existiert := FALSE;
   WHILE NextVtr (aktVtr, inoutVtrOrdner) AND NOT existiert DO
      IF ZeitraumUeberschneidung (aktVtr, inGueltigVon, inGueltigBis) THEN
         existiert := TRUE;
         Vtr.EqualizeVtr (inoutVtr, aktVtr);
      END;  (* IF *)
   END; (* WHILE *)

   Vtr.DisposeVtr (aktVtr);
   RETURN existiert
 END NextVtrImZeitraum;


 PROCEDURE FirstValidVtrZuHK (inPersID : TPersID; VAR outVtr : TVtr;
                              inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 (* Ersten gueltigter Vertrag (nicht storniert) einer Hilfskraft ausgeben *)
 VAR aktVtr : TVtr;
     exist, vtr_zu_hk: BOOLEAN;
 BEGIN

   vtr_zu_hk :=  FirstVtrZuHK (inPersID, aktVtr, inoutVtrOrdner);

   IF NOT vtr_zu_hk
   THEN
     exist := FALSE;
   ELSE
     exist := FALSE;

     WHILE (NOT  exist) AND vtr_zu_hk DO
        IF NOT Vtr.IstStornoInVtr (aktVtr) THEN
           exist := TRUE;
           outVtr := Vtr.DuplicateVtr (aktVtr);
        ELSE
           vtr_zu_hk := NextVtrZuHK (inPersID, aktVtr, inoutVtrOrdner)
        END; (* IF n_storno_vtr *)
     END; (* WHILE *)

     Vtr.DisposeVtr (aktVtr);

   END; (* IF NOT vtr_zu_hk *)

   RETURN exist
 END FirstValidVtrZuHK;


 PROCEDURE NextValidVtrZuHK  (inPersID : TPersID; inoutVtr : TVtr;
                              inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 (* Naechsten gueltiger Vertrag (nicht storniert) einer Hilfskraft ausgeben *)

 VAR aktVtr : TVtr;
     exist, vtr_zu_hk : BOOLEAN;
 BEGIN

   aktVtr := Vtr.CreateVtr();
   Vtr.EqualizeVtr (aktVtr, inoutVtr);

   vtr_zu_hk :=  NextVtrZuHK (inPersID, aktVtr, inoutVtrOrdner);

   IF NOT vtr_zu_hk
   THEN
     exist := FALSE;
   ELSE
     exist := FALSE;

     WHILE  (NOT  exist) AND vtr_zu_hk  DO
        IF NOT Vtr.IstStornoInVtr (aktVtr) THEN
           exist := TRUE;
           Vtr.EqualizeVtr (inoutVtr, aktVtr);
        ELSE
           vtr_zu_hk := NextVtrZuHK (inPersID, aktVtr, inoutVtrOrdner)
        END; (* IF n_storno_vtr *)
     END; (* WHILE *)

   END; (* IF NOT vtr_zu_hk *)

   Vtr.DisposeVtr (aktVtr);
   RETURN exist
 END NextValidVtrZuHK;


 PROCEDURE FirstVldVtrZuHKinZtraum (inPersID : TPersID; inAnfang,inEnde :TDatum;
                                    VAR outVtr : TVtr; inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
  (* Ersten gueltigten Vertrag (nicht storniert) einer Hilfskraft ausgeben
    im Zeitraum *)

 VAR aktVtr : TVtr;
     exist, vldvtr_zu_hk : BOOLEAN;
 BEGIN

   vldvtr_zu_hk :=  FirstValidVtrZuHK (inPersID, aktVtr, inoutVtrOrdner);
   IF NOT vldvtr_zu_hk
   THEN
     exist := FALSE;
   ELSE
     exist := FALSE;

     WHILE (NOT exist) AND vldvtr_zu_hk DO
         IF ZeitraumUeberschneidung (aktVtr, inAnfang, inEnde) THEN
           exist := TRUE;
           outVtr := Vtr.DuplicateVtr (aktVtr);
        ELSE
           vldvtr_zu_hk := NextValidVtrZuHK (inPersID, aktVtr, inoutVtrOrdner)
        END; (* IF ZeitraumUeberschneidung *)
     END; (* WHILE *)

   Vtr.DisposeVtr (aktVtr);
   END; (* IF NOT vtr_zu_hk *)

   RETURN exist
 END FirstVldVtrZuHKinZtraum;


 PROCEDURE NextVldVtrZuHKinZtraum  (inPersID : TPersID; inAnfang,inEnde : TDatum;
                                    inoutVtr : TVtr; inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
  (* Naechsten gueltigten Vertrag (nicht storniert) einer Hilfskraft ausgeben
    im Zeitraum *)

 VAR aktVtr : TVtr;
     existiert, next_vtr : BOOLEAN;
 BEGIN

   aktVtr := Vtr.CreateVtr();
   Vtr.EqualizeVtr (aktVtr, inoutVtr);

   next_vtr := NextValidVtrZuHK (inPersID, aktVtr, inoutVtrOrdner);

   IF NOT next_vtr
   THEN
     existiert := FALSE;
   ELSE

     existiert := FALSE;
     WHILE  next_vtr AND (NOT existiert) DO
         IF ZeitraumUeberschneidung (aktVtr, inAnfang, inEnde) THEN
           existiert := TRUE;
           Vtr.EqualizeVtr (inoutVtr, aktVtr);
        ELSE
           next_vtr := NextValidVtrZuHK (inPersID, aktVtr, inoutVtrOrdner)
        END; (* IF ZeitraumUeberschneidung *)
     END; (* WHILE *)

   Vtr.DisposeVtr (aktVtr);
   END; (* IF NOT next_vtr_in_ztr *)

   RETURN existiert
 END NextVldVtrZuHKinZtraum;


 PROCEDURE FirstVldVtrZuHKinZtpunkt (inPersID : TPersID; inDatum : TDatum;
                                     VAR outVtr : TVtr; 
                                     inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
  (* Ersten gueltigten Vertrag (nicht storniert) einer Hilfskraft ausgeben
    im Zeitpunkt *)
 VAR existiert, next_vtr : BOOLEAN;
     aktVtr : TVtr;
     vtrVon, vtrBis : TDatum;
 BEGIN
    
    next_vtr := FirstValidVtrZuHK(inPersID, aktVtr, inoutVtrOrdner);

    IF NOT next_vtr 
    THEN  (* Es existiert kein Vertrag im Zeitpunkt zur HK *)
       existiert := FALSE
    ELSE

       existiert := FALSE;
       WHILE (NOT existiert) AND next_vtr DO
        vtrVon := Vtr.LiesGueltigVoninVtr (aktVtr);
        vtrBis := Vtr.LiesGueltigBisinVtr (aktVtr);
          IF DatumInZeitraum (inDatum, vtrVon, vtrBis) THEN
             existiert := TRUE;
             outVtr := Vtr.DuplicateVtr (aktVtr);
          ELSE
             next_vtr := NextValidVtrZuHK(inPersID, aktVtr, inoutVtrOrdner);
          END; (* IF *)
       END; (* WHILE *)
          
       Vtr.DisposeVtr (aktVtr);
       DisposeDatum (vtrVon);
       DisposeDatum (vtrBis);
    END; (* IF NOT next_vtr *)
   
    RETURN existiert
 END FirstVldVtrZuHKinZtpunkt;

 PROCEDURE NextVldVtrZuHKinZtpunkt  (inPersID : TPersID; inDatum : TDatum;
                                     inoutVtr : TVtr; 
                                     inoutVtrOrdner : TVtrOrdner): BOOLEAN;
  (* Naechsten gueltigten Vertrag (nicht storniert) einer Hilfskraft ausgeben
    im Zeitpunkt *)
 VAR existiert, next_vtr : BOOLEAN;
     aktVtr : TVtr;
     vtrVon, vtrBis : TDatum;
 BEGIN

   aktVtr := Vtr.CreateVtr();
   Vtr.EqualizeVtr (aktVtr, inoutVtr);

    next_vtr := NextValidVtrZuHK(inPersID, aktVtr, inoutVtrOrdner);

    IF NOT next_vtr
    THEN  (* Es existiert kein gueltiger Vertrag zur HK *)
       existiert := FALSE
    ELSE

       existiert := FALSE;
       WHILE (NOT existiert) AND next_vtr DO
        vtrVon := Vtr.LiesGueltigVoninVtr (aktVtr);
        vtrBis := Vtr.LiesGueltigBisinVtr (aktVtr);
          IF DatumInZeitraum (inDatum, vtrVon, vtrBis) THEN
             existiert := TRUE;
             Vtr.EqualizeVtr (inoutVtr, aktVtr);
          ELSE
             next_vtr := NextValidVtrZuHK(inPersID, aktVtr, inoutVtrOrdner);
          END; (* IF *)
       END; (* WHILE *)

       Vtr.DisposeVtr (aktVtr);
       DisposeDatum (vtrVon);
       DisposeDatum (vtrBis);
    END; (* IF NOT next_vtr *)

    RETURN existiert
 END NextVldVtrZuHKinZtpunkt;


 PROCEDURE BerMaxStdZahlDerHK(inVtr: TVtr; 
                              inoutVtrOrdner : TVtrOrdner): INTEGER;
	(* Gabi
	S. 45: ab (*HK nach VKond prefn *)
	bis .. Stunden := Max()... (=Rckgabe)
	    END (*FOR*)
	AnzahlMonate wird nicht benoetigt !!!!
	*)
 VAR stunden, tempMaxStunden : CARDINAL;
     aktV : TVtr;
     vGueltigVon, vGueltigBis : TDatum;
     persID : TPersID;
     next_aktV, dispose_aktV : BOOLEAN;
 BEGIN

   aktV := Vtr.DuplicateVtr (inVtr);
   persID := Vtr.LiesPersIDinVtr (aktV);
   vGueltigVon := Vtr.LiesGueltigVoninVtr (aktV);
   vGueltigBis := Vtr.LiesGueltigBisinVtr (aktV);

   next_aktV := FirstVldVtrZuHKinZtraum (persID, vGueltigVon, vGueltigBis,
                                         aktV, inoutVtrOrdner);
   dispose_aktV := next_aktV;

   tempMaxStunden := 0;

   WHILE next_aktV DO
      stunden := Vtr.LiesStdzahlinVtr (aktV);
      tempMaxStunden := tempMaxStunden + stunden;
      next_aktV := NextVldVtrZuHKinZtraum (persID, vGueltigVon, vGueltigBis,
                                            aktV, inoutVtrOrdner);
   END; (* WHILE *)

   IF dispose_aktV THEN
      Vtr.DisposeVtr (aktV);
   END;
   DisposePersID (persID);
   DisposeDatum (vGueltigVon);
   DisposeDatum (vGueltigBis);

   RETURN tempMaxStunden;
 END BerMaxStdZahlDerHK;



PROCEDURE BerMaxBeschZeitDerHK (inHKID : TPersID; VAR outVonDatum, outBisDatum : TDatum;
                                inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 (* -- vb: VertrageOrdner existiert
    --     Durchluft alle gueltigen Vertrage zu einer Hilfskraft und gibt das
    --     kleinste gueltigVon in outVonDatum sowie das gre gueltigBis in
    --     outBisDatum zurueck. Sei rc der boolesche Rueckgabewert, dann gilt:
    -- nb: rc <=> Vertrge existieren ANd
           (not rc) <=> sonst *)
 VAR aktVtr : TVtr;
     rc : BOOLEAN;
     vonDatum, bisDatum, tempDatum : TDatum;

 BEGIN

     IF FirstValidVtrZuHK (inHKID, aktVtr, inoutVtrOrdner) THEN

        rc := TRUE;
        outVonDatum := Vtr.LiesGueltigVoninVtr (aktVtr);
        outBisDatum := Vtr.LiesGueltigBisinVtr (aktVtr);

        WHILE NextValidVtrZuHK (inHKID, aktVtr, inoutVtrOrdner) DO

            vonDatum :=  Vtr.LiesGueltigVoninVtr (aktVtr);
            bisDatum :=  Vtr.LiesGueltigBisinVtr (aktVtr);

            IF SmallerDatum (vonDatum, outVonDatum) THEN
               EqualizeDatum (outVonDatum, vonDatum);
            END;

            IF SmallerDatum (outBisDatum, bisDatum) THEN
               EqualizeDatum (outBisDatum, bisDatum);
            END;

            DisposeDatum (vonDatum);
            DisposeDatum (bisDatum);

       END; (* WHILE *)

       Vtr.DisposeVtr (aktVtr);

     ELSE
        (* Kein Vertrag zur inHKID im Ordner *)
        rc := FALSE;
     END; (* IF FirstValidVtrZuHK *)

     RETURN rc;

 END BerMaxBeschZeitDerHK;


 (* Iterator ueber alle Vertraege (einschl. der stornierten) *)
 (*==========================================================*)


 PROCEDURE FirstVtrUndStorno(VAR outVtr : TVtr; 
                             inoutVtrOrdner : TVtrOrdner):BOOLEAN;
 (*
    vb: VertraegeOrdner existiert
        Holt den ersten Vertrag
        Sei OK der boolesche Rueckgabecode, dann gilt:
   nb: OK <=> Mindestens ein Vertrag existiert
 *)
 VAR  existiert : BOOLEAN;
 BEGIN
  IF NOT IsEmptyVtrOrdner (inoutVtrOrdner) THEN
    VtrListe.GoSmallest(inoutVtrOrdner^.Ordner);
    outVtr := VtrListe.Current(inoutVtrOrdner^.Ordner);
    existiert := TRUE;
  ELSE
    existiert := FALSE;
  END; (* IF *)
  RETURN existiert;
 END FirstVtrUndStorno;


 PROCEDURE NextVtrUndStorno (inoutVtr : TVtr; 
                             inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 (* Vb:  Vertrag.ExistiertVtrID(inoutVtr) AND
         Search(inoutVtrOrdner,inoutVtr.VtrID,x)
         Rueckgabe des Vertrages an der naechsten Position; (pos_alt+1, falls existent)
    Nb:  (inoutVtrOrdner.pos_neu = inoutVtrOrdner.pos_alt + 1) AND
         (inoutVtr enthaelt die Hilfskraft auf inoutVtrOrdner.pos_alt AND
         (Rueckgabewert = TRUE) <=>(inoutVtrOrdner.pos_neu<=kap)) *)
  VAR aktVtrID : TVtrID;
      existiert        : BOOLEAN;
      lfdVtr     : TVtr;
  BEGIN
    existiert := FALSE;
    WITH inoutVtrOrdner^ DO
      IF VtrListe.StopAt (Ordner, inoutVtr) THEN
        VtrListe.StepNext (Ordner);
        IF NOT VtrListe.OffEnd (Ordner) THEN
          lfdVtr := VtrListe.Current (Ordner);
          Vtr.EqualizeVtr (inoutVtr, lfdVtr);
          Vtr.DisposeVtr (lfdVtr);
          existiert := TRUE
        END (* IF NOT VtrListe.OffEnd *)
      END (* IF VtrListe.StopAt *)
    END; (* WITH inoutVtrOrdner^*)
    RETURN existiert;
 END NextVtrUndStorno;


 PROCEDURE  FirstVtrZuHK (inPersID : TPersID; VAR outVtr : TVtr;
                          inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 (* Ersten Vertrag einer Hilfskraft ausgeben *)
  VAR aktPersID  : TPersID;
      existiert : BOOLEAN;
      aktVtr    : TVtr;
  BEGIN
    existiert := FALSE;
    IF NOT IsEmptyVtrOrdner (inoutVtrOrdner) THEN
      WITH inoutVtrOrdner^ DO
         VtrListe.GoSmallest (Ordner);
         WHILE NOT (existiert OR VtrListe.OffEnd (Ordner)) DO
           aktVtr := VtrListe.Current (Ordner);
           aktPersID := Vtr.LiesPersIDinVtr (aktVtr);
           IF IsEqualPersID (aktPersID, inPersID) THEN
             existiert := TRUE;
             outVtr := Vtr.DuplicateVtr (aktVtr)
           ELSE
             VtrListe.StepNext (Ordner)
           END; (* IF *)
           DisposePersID (aktPersID);
           Vtr.DisposeVtr (aktVtr)
         END; (* WHILE nicht gefunden*)
      END (* WITH inoutVtrOrdner^*)
    END; (* IF Ordner nicht leer*)
    RETURN existiert
 END FirstVtrZuHK;


 PROCEDURE  NextVtrZuHK  (inPersID : TPersID; inoutVtr : TVtr;
                          inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 (* Naechsten Vertrag einer Hilfskraft ausgeben *)
  VAR aktPersID  : TPersID;
      existiert, gefunden : BOOLEAN;
      aktVtr    : TVtr;
  BEGIN
    IF NOT IsEmptyVtrOrdner (inoutVtrOrdner) THEN
      WITH inoutVtrOrdner^ DO
         gefunden := VtrListe.Search (Ordner, inoutVtr);
         IF NOT gefunden THEN
           existiert := FALSE
         ELSE
           (* inoutVtr in Ordner gefunden, Nachfolger vergleichen *)
           VtrListe.StepNext (Ordner);
           existiert := FALSE;
         WHILE NOT (existiert OR VtrListe.OffEnd (Ordner)) DO
           aktVtr := VtrListe.Current (Ordner);
           aktPersID := Vtr.LiesPersIDinVtr (aktVtr);
           IF IsEqualPersID (aktPersID, inPersID) THEN
             existiert := TRUE;
             Vtr.EqualizeVtr (inoutVtr, aktVtr);
           ELSE
             VtrListe.StepNext (Ordner)
           END; (* IF HKe.IsEqualPersID *)
           DisposePersID (aktPersID);
           Vtr.DisposeVtr (aktVtr)
         END; (* WHILE NOT (existiert OR ...) *)
         END; (* IF nichtgefunden *)
      END (* WITH inoutVtrOrdner^*)
    END; (* IF Ordner nicht leer*)
    RETURN existiert
 END NextVtrZuHK;
 

 PROCEDURE  ExistVtrZuHK (inHKID:TPersID; inoutVtrOrdner : TVtrOrdner):BOOLEAN;
 (* Prueft, ob zu der HK inHKID jemals ein Vertrag existiert hat.
	 Dabei ist es voellig egal, ob der Vertrag storniert ist usw. 
       Dies Voraussetzung fr Loesche HK *)
 VAR aktVtr : TVtr;
     exist : BOOLEAN;
 BEGIN
   exist := FirstVtrZuHK (inHKID, aktVtr, inoutVtrOrdner);
   IF exist THEN
     Vtr.DisposeVtr (aktVtr)
   END; (* IF *)
   RETURN exist
 END ExistVtrZuHK;


 PROCEDURE  FirstVtrZuKst (inKstID : TKstID; VAR outVtr : TVtr;
                           inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 (* Ersten Vertrag einer Kostenstelle ausgeben *)
  VAR aktKstID  : TKstID;
      existiert : BOOLEAN;
      aktVtr    : TVtr;
  BEGIN
    existiert := FALSE;
    IF NOT IsEmptyVtrOrdner (inoutVtrOrdner) THEN
      WITH inoutVtrOrdner^ DO
         VtrListe.GoSmallest (Ordner);
         WHILE NOT (existiert OR VtrListe.OffEnd (Ordner)) DO
           aktVtr := VtrListe.Current (Ordner);
           aktKstID := Vtr.LiesKstIDinVtr (aktVtr);
           IF IsEqualKstID (aktKstID, inKstID) THEN
             existiert := TRUE;
             outVtr := Vtr.DuplicateVtr (aktVtr)
           ELSE
             VtrListe.StepNext (Ordner)
           END; (* IF *)
           DisposeKstID (aktKstID);
           Vtr.DisposeVtr (aktVtr)
         END; (* WHILE nicht gefunden*)
      END (* WITH inoutVtrOrdner^*)
    END; (* IF Ordner nicht leer*)
    RETURN existiert
 END FirstVtrZuKst;


 PROCEDURE  NextVtrZuKst  (inKstID : TKstID; inoutVtr : TVtr;
                           inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 (* Naechsten Vertrag einer Kostenstelle ausgeben *)
  VAR aktKstID  : TKstID;
      existiert, gefunden : BOOLEAN;
      aktVtr    : TVtr;
  BEGIN
    IF NOT IsEmptyVtrOrdner (inoutVtrOrdner) THEN
      WITH inoutVtrOrdner^ DO
         gefunden := VtrListe.Search (Ordner, inoutVtr);
         IF NOT gefunden THEN
           existiert := FALSE
         ELSE
           (* inoutVtr in Ordner gefunden, Nachfolger vergleichen *)
           VtrListe.StepNext (Ordner);
           existiert := FALSE;
         WHILE NOT (existiert OR VtrListe.OffEnd (Ordner)) DO
           aktVtr := VtrListe.Current (Ordner);
           aktKstID := Vtr.LiesKstIDinVtr (aktVtr);
           IF IsEqualKstID (aktKstID, inKstID) THEN
             existiert := TRUE;
             Vtr.EqualizeVtr (inoutVtr, aktVtr);
           ELSE
             VtrListe.StepNext (Ordner)
           END; (* IF HKe.IsEqualPersID *)
           DisposeKstID (aktKstID);
           Vtr.DisposeVtr (aktVtr)
         END; (* WHILE NOT (existiert OR ...) *)
         END; (* IF nichtgefunden *)
      END (* WITH inoutVtrOrdner^*)
    END; (* IF Ordner nicht leer*)
    RETURN existiert
 END NextVtrZuKst;


 PROCEDURE FirstVtrZuHKinZeitraum (inPersID : TPersID; inAnfang,inEnde :TDatum;
                                   VAR outVtr : TVtr; inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 VAR aktVtr : TVtr;
     persID : TPersID;
     exist, vtr_zu_hk, gleiche_persID : BOOLEAN;
 BEGIN

   vtr_zu_hk :=  FirstVtrZuHK (inPersID, aktVtr, inoutVtrOrdner);
   IF NOT vtr_zu_hk
   THEN
     exist := FALSE;
   ELSE
     exist := FALSE;

     WHILE (NOT exist) AND vtr_zu_hk DO
        
        IF ZeitraumUeberschneidung (aktVtr, inAnfang, inEnde) THEN
           exist := TRUE;
           outVtr := Vtr.DuplicateVtr (aktVtr);
        ELSE
           vtr_zu_hk := NextVtrZuHK (inPersID, aktVtr, inoutVtrOrdner)
        END; (* IF ZeitraumUeberschneidung *)

     END; (* WHILE *)

     Vtr.DisposeVtr (aktVtr);
   END; (* IF NOT vtr_zu_hk *)

   RETURN exist;
 END FirstVtrZuHKinZeitraum;


 PROCEDURE NextVtrZuHKinZeitraum  (inPersID : TPersID; 
                                   inAnfang,inEnde : TDatum;
                                   inoutVtr : TVtr; inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 VAR existiert : BOOLEAN;
     aktVtr : TVtr;
 BEGIN
   aktVtr := Vtr.CreateVtr();
   Vtr.EqualizeVtr (aktVtr, inoutVtr);

   existiert := FALSE;
   WHILE NextVtrZuHK (inPersID, aktVtr, inoutVtrOrdner) AND NOT existiert DO
      IF ZeitraumUeberschneidung (aktVtr, inAnfang, inEnde) THEN
         existiert := TRUE;
         Vtr.EqualizeVtr (inoutVtr, aktVtr);
      END;  (* IF *)
   END; (* WHILE *)

  Vtr.DisposeVtr (aktVtr);
  RETURN existiert
 END NextVtrZuHKinZeitraum;


 PROCEDURE FirstVtrZuKstinZeitraum (inKstID : TKstID; 
                                    inAnfang,inEnde : TDatum;
                                    VAR outVtr : TVtr; 
                                    inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 VAR aktVtr : TVtr;
     kstID : TKstID;
     exist, vtr_zu_kst : BOOLEAN;
 BEGIN

   vtr_zu_kst :=  FirstVtrZuKst (inKstID, aktVtr, inoutVtrOrdner);

   IF NOT vtr_zu_kst
   THEN
     exist := FALSE;
   ELSE
     exist := FALSE;

     WHILE (NOT exist) AND vtr_zu_kst DO
        
        IF ZeitraumUeberschneidung (aktVtr, inAnfang, inEnde) THEN
           exist := TRUE;
           outVtr := Vtr.DuplicateVtr (aktVtr);
        ELSE
           vtr_zu_kst := NextVtrZuKst (inKstID, aktVtr, inoutVtrOrdner)
        END; (* IF ZeitraumUeberschneidung *)

     END; (* WHILE *)

     Vtr.DisposeVtr (aktVtr);
   END; (* IF NOT vtr_zu_kst *)
   RETURN exist
 END FirstVtrZuKstinZeitraum;


 PROCEDURE NextVtrZuKstinZeitraum  (inKstID : TKstID; 
                                    inAnfang,inEnde : TDatum;
                                    inoutVtr : TVtr; 
                                    inoutVtrOrdner : TVtrOrdner) : BOOLEAN;
 VAR existiert : BOOLEAN;
     aktVtr : TVtr;
 BEGIN
   aktVtr := Vtr.CreateVtr();
   Vtr.EqualizeVtr (aktVtr, inoutVtr);

   existiert := FALSE;
   WHILE NextVtrZuKst (inKstID, aktVtr, inoutVtrOrdner) AND NOT existiert DO
      IF ZeitraumUeberschneidung (aktVtr, inAnfang, inEnde) THEN
         existiert := TRUE;
         Vtr.EqualizeVtr (inoutVtr, aktVtr);
      END;  (* IF *)
   END; (* WHILE *)

  Vtr.DisposeVtr (aktVtr);
  RETURN existiert
 END NextVtrZuKstinZeitraum;


 PROCEDURE FirstVtrZuHKinZeitpunkt (inPersID : TPersID;
                                    inDatum : TDatum;
                                    VAR outVtr : TVtr; 
                                    inoutVtrOrdner : TVtrOrdner ) : BOOLEAN;
 VAR aktVtr : TVtr;
     vtrVon, vtrBis : TDatum;
     exist, vtr_zu_hk : BOOLEAN;
 BEGIN

   vtr_zu_hk :=  FirstVtrZuHK (inPersID, aktVtr, inoutVtrOrdner);
   IF NOT vtr_zu_hk
   THEN
     exist := FALSE;
   ELSE
     exist := FALSE;

     WHILE (NOT exist) AND vtr_zu_hk DO

        vtrVon := Vtr.LiesGueltigVoninVtr (aktVtr);
        vtrBis := Vtr.LiesGueltigBisinVtr (aktVtr);
        
        IF DatumInZeitraum (inDatum, vtrVon, vtrBis) THEN
           exist := TRUE;
           outVtr := Vtr.DuplicateVtr (aktVtr);
        ELSE
           vtr_zu_hk := NextVtrZuHK (inPersID, aktVtr, inoutVtrOrdner)
        END; (* IF ZeitraumUeberschneidung *)

     END; (* WHILE *)

     Vtr.DisposeVtr (aktVtr);
     DisposeDatum (vtrVon);
     DisposeDatum (vtrBis);

   END; (* IF NOT vtr_zu_hk *)

   RETURN exist
 END FirstVtrZuHKinZeitpunkt;


 PROCEDURE NextVtrZuHKinZeitpunkt (inPersID : TPersID; 
                                   inDatum : TDatum;
                                   inoutVtr : TVtr; 
                                   inoutVtrOrdner : TVtrOrdner ) : BOOLEAN;
 VAR vtrVon, vtrBis : TDatum;
     existiert, vtr_zu_hk : BOOLEAN;
     aktVtr : TVtr;
 BEGIN
   aktVtr := Vtr.CreateVtr();
   Vtr.EqualizeVtr (aktVtr, inoutVtr);

   existiert := FALSE;
   vtr_zu_hk := NextVtrZuHK (inPersID, aktVtr, inoutVtrOrdner);

   WHILE vtr_zu_hk AND NOT existiert DO
      vtrVon := Vtr.LiesGueltigVoninVtr (aktVtr);
      vtrBis := Vtr.LiesGueltigBisinVtr (aktVtr);
       IF DatumInZeitraum (inDatum, vtrVon, vtrBis) THEN
         existiert := TRUE;
         Vtr.EqualizeVtr (inoutVtr, aktVtr);
       ELSE
         vtr_zu_hk := NextVtrZuHK (inPersID, aktVtr, inoutVtrOrdner);
       END;    (* IF *)
      DisposeDatum (vtrVon);
      DisposeDatum (vtrBis);
   END; (* WHILE *)

  Vtr.DisposeVtr (aktVtr);
  RETURN existiert
END NextVtrZuHKinZeitpunkt;



(* StringOperationen                                    *)
(*======================================================*)

PROCEDURE GibVtrAlsStringlisteInVtr(inoutListe : TVtrOrdner): TStringliste;
VAR strListe : TStringliste;
    blank, null_str,
    aktString,
    tempDatumString, outMessage, prefix, persString, kstString : TString;
    aktVtr : TVtr;
    aktVtrID : TVtrID;
    aktPersID : TPersID;
    aktKstID : TKstID;
    erstellDatum : TDatum;
    exist, dispose_marker : BOOLEAN;
    i, laenge : CARDINAL;

CONST laenge_id = 5;

BEGIN

  blank := String.Create ();
  String.Schreibe (blank, " ");

  null_str := String.Create ();
  String.Schreibe (null_str, "0");

  strListe := Stringliste.Create ();
  exist := FirstVtrUndStorno (aktVtr, inoutListe);
  dispose_marker := exist;

  WHILE  exist DO

    (* VtrID *)
    aktVtrID := Vtr.LiesVtrIDinVtr (aktVtr);
    aktString := Vtr.VtrIDAlsString (aktVtrID);
         (* In Ausgabe auf 5 Zeichen formatieren *)
         laenge := String.Laenge(aktString);
         IF laenge < laenge_id THEN
            prefix := String.Create();
               FOR i := 1 TO (laenge_id - laenge) DO
                 String.Anhaengen (null_str, prefix)
               END; (* FOR *)
            String.Anhaengen (aktString, prefix);
            String.Equalize (aktString, prefix);
            String.Dispose (prefix);
         END; (* IF *)
    String.Anhaengen (blank, aktString);

    (* PersID *)
    IF Vtr.ExistiertPersIDinVtr (aktVtr) THEN
       (* PersID ist gesetzt *)
       aktPersID := Vtr.LiesPersIDinVtr (aktVtr);
       persString := PersIDAlsString (aktPersID);
         (* In Ausgabe auf 5 Zeichen formatieren *)
         laenge := String.Laenge(persString);
         IF laenge < laenge_id THEN
            prefix := String.Create();
               FOR i := 1 TO (laenge_id - laenge) DO
                 String.Anhaengen (null_str, prefix)
               END; (* FOR *)
            String.Anhaengen (prefix, aktString);
            String.Dispose (prefix);
         END; (* IF *)
       String.Anhaengen (persString, aktString);
       String.Anhaengen (blank, aktString);
       String.Dispose (persString);
    ELSE
       (* PersID ist nicht gesetzt *)
       FOR i:=1 TO laenge_id DO
         String.Anhaengen (blank, aktString);
       END; (* FOR *)
    END; (* IF Vtr.ExistiertPersID *)

    (* KstID *)
    aktKstID := Vtr.LiesKstIDinVtr (aktVtr);
    kstString := KstIDAlsString (aktKstID);
         (* In Ausgabe auf 5 Zeichen formatieren *)
         laenge := String.Laenge(kstString);
         IF laenge < laenge_id THEN
            prefix := String.Create();
               FOR i := 1 TO (laenge_id - laenge) DO
                 String.Anhaengen (null_str, prefix)
               END; (* FOR *)
            String.Anhaengen (prefix, aktString);
            String.Dispose (prefix);
         END; (* IF *)
    String.Anhaengen (kstString, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (kstString);

    (* ErstellungsDatum *)
    erstellDatum := Vtr.LiesErstellDatinVtr (aktVtr);
    tempDatumString := DatumAlsString (erstellDatum);
    String.Anhaengen (tempDatumString, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (tempDatumString);
    DisposeDatum (erstellDatum);

    Stringliste.Append (strListe, aktString);

    String.Dispose (aktString);

    exist := NextVtrUndStorno (aktVtr, inoutListe);
  END; (* WHILE *)

  IF dispose_marker THEN
    DisposeHID (aktVtrID);
    String.Dispose (aktString);
    Vtr.DisposeVtr (aktVtr);
  END; (* IF *)

  String.Dispose (blank);
  String.Dispose (null_str);

  RETURN strListe;
 END GibVtrAlsStringlisteInVtr;



PROCEDURE GibAllesAlsStringlisteInVtr(inoutListe : TVtrOrdner): TStringliste;
VAR strListe : TStringliste;
    blank, null_str,
    aktString,
    tempDatumString, outMessage, prefix, persString, kstString, 
    stdzahl_str, taetigkt_str, begruend_str, temptgkt, tempbgrd : TString;
    aktVtr : TVtr;
    aktVtrID : TVtrID;
    aktPersID : TPersID;
    aktKstID : TKstID;
    erstellDatum, vonDatum, bisDatum, stornoDatum : TDatum;
    exist, dispose_marker : BOOLEAN;
    i, laenge, stdzahl : CARDINAL;
    arrayofchar : ARRAY [1..255] OF CHAR;
CONST laenge_id = 5;
      laenge_tb = 10;
BEGIN

  blank := String.Create ();
  String.Schreibe (blank, " ");

  null_str := String.Create ();
  String.Schreibe (null_str, "0");

  strListe := Stringliste.Create ();
  exist := FirstVtrUndStorno (aktVtr, inoutListe);
  dispose_marker := exist;

  WHILE  exist DO

    (* VtrID *)
    aktVtrID := Vtr.LiesVtrIDinVtr (aktVtr);
    aktString := Vtr.VtrIDAlsString (aktVtrID);
         (* In Ausgabe auf 5 Zeichen formatieren *)
         laenge := String.Laenge(aktString);
         IF laenge < laenge_id THEN
            prefix := String.Create();
               FOR i := 1 TO (laenge_id - laenge) DO
                 String.Anhaengen (null_str, prefix)
               END; (* FOR *)
            String.Anhaengen (aktString, prefix);
            String.Equalize (aktString, prefix);
            String.Dispose (prefix);
         END; (* IF *)
    String.Anhaengen (blank, aktString);

    (* PersID *)
    IF Vtr.ExistiertPersIDinVtr (aktVtr) THEN
       (* PersID ist gesetzt *)
       aktPersID := Vtr.LiesPersIDinVtr (aktVtr);
       persString := PersIDAlsString (aktPersID);
         (* In Ausgabe auf 5 Zeichen formatieren *)
         laenge := String.Laenge(persString);
         IF laenge < laenge_id THEN
            prefix := String.Create();
               FOR i := 1 TO (laenge_id - laenge) DO
                 String.Anhaengen (null_str, prefix)
               END; (* FOR *)
            String.Anhaengen (prefix, aktString);
            String.Dispose (prefix);
         END; (* IF *)
       String.Anhaengen (persString, aktString);
       String.Anhaengen (blank, aktString);
       String.Dispose (persString);
    ELSE
       (* PersID ist nicht gesetzt *)
       FOR i:=1 TO laenge_id DO
         String.Anhaengen (blank, aktString);
       END; (* FOR *)
    END; (* IF Vtr.ExistiertPersID *)

    (* KstID *)
    aktKstID := Vtr.LiesKstIDinVtr (aktVtr);
    kstString := KstIDAlsString (aktKstID);
         (* In Ausgabe auf 5 Zeichen formatieren *)
         laenge := String.Laenge(kstString);
         IF laenge < laenge_id THEN
            prefix := String.Create();
               FOR i := 1 TO (laenge_id - laenge) DO
                 String.Anhaengen (null_str, prefix)
               END; (* FOR *)
            String.Anhaengen (prefix, aktString);
            String.Dispose (prefix);
         END; (* IF *)
    String.Anhaengen (kstString, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (kstString);

    (* ErstellungsDatum *)
    erstellDatum := Vtr.LiesErstellDatinVtr (aktVtr);
    tempDatumString := DatumAlsString (erstellDatum);
    String.Anhaengen (tempDatumString, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (tempDatumString);
    DisposeDatum (erstellDatum);

     (* Ab hier keine Stringliste mehr; nur zum Testen eingefuegt *)

    (* GueltigVon-Datum *)
    vonDatum := Vtr.LiesGueltigVoninVtr (aktVtr);
    tempDatumString := DatumAlsString (vonDatum);
    String.Anhaengen (tempDatumString, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (tempDatumString);
    DisposeDatum (vonDatum);

    (* GueltigBis-Datum *)
    bisDatum := Vtr.LiesGueltigBisinVtr (aktVtr);
    tempDatumString := DatumAlsString (bisDatum);
    String.Anhaengen (tempDatumString, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (tempDatumString);
    DisposeDatum (bisDatum);

    (* Storno-Datum *)
    stornoDatum := Vtr.LiesStornoAminVtr (aktVtr);
    tempDatumString := DatumAlsString (stornoDatum);
    String.Anhaengen (tempDatumString, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (tempDatumString);
    DisposeDatum (stornoDatum);

    (* Stundenzahl *)
    stdzahl := Vtr.LiesStdzahlinVtr (aktVtr);
    CardToStr (stdzahl, arrayofchar);
    stdzahl_str := String.Create();
    String.Schreibe (stdzahl_str, arrayofchar);
      (* In Ausgabe auf 5 Zeichen formatieren *)
      laenge := String.Laenge(stdzahl_str);
     IF laenge < laenge_id THEN
      FOR i := 1 TO (laenge_id - laenge) DO
          String.Anhaengen (blank, stdzahl_str)
      END; (* FOR *)
     ELSE
       String.Schreibe (stdzahl_str, "*****");
     END;
    String.Anhaengen (stdzahl_str, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (stdzahl_str);

    (* Taetigkeit *)
    taetigkt_str := Vtr.LiesTaetigktinVtr (aktVtr);
    laenge := String.Laenge(taetigkt_str);
    IF laenge < laenge_tb THEN
       FOR i := 1 TO (laenge_tb - laenge) DO
           String.Anhaengen (blank, taetigkt_str)
       END; (* FOR *)
    ELSIF
       laenge > laenge_tb THEN
       temptgkt := String.Create();
       String.Auslesen (taetigkt_str, 0, laenge_tb, temptgkt);
       String.Equalize (taetigkt_str, temptgkt);
       String.Dispose (temptgkt);
    END; (* IF *)
    String.Anhaengen (taetigkt_str, aktString);
    String.Anhaengen (blank, aktString);
    String.Dispose (taetigkt_str);

    (* Begruendung *)
    begruend_str := Vtr.LiesBegruendinVtr (aktVtr);
    laenge := String.Laenge(begruend_str);
    IF laenge < laenge_tb THEN
       FOR i := 1 TO (laenge_tb - laenge) DO
           String.Anhaengen (blank, begruend_str)
       END; (* FOR *)
    ELSIF
       laenge > laenge_tb THEN
       tempbgrd := String.Create();
       String.Auslesen (begruend_str, 0, laenge_tb, tempbgrd);
       String.Equalize (begruend_str, tempbgrd);
       String.Dispose (tempbgrd);
    END; (* IF *)
    String.Anhaengen (begruend_str, aktString);
    String.Dispose (begruend_str);

    Stringliste.Append (strListe, aktString);

    String.Dispose (aktString);

    exist := NextVtrUndStorno (aktVtr, inoutListe);
  END; (* WHILE *)

  IF dispose_marker THEN
    DisposeHID (aktVtrID);
    String.Dispose (aktString);
    Vtr.DisposeVtr (aktVtr);
  END; (* IF *)

  String.Dispose (blank);
  String.Dispose (null_str);

  RETURN strListe;
 END GibAllesAlsStringlisteInVtr;



 (* Prozeduren zum Laden und Speichern des Teilsystems *)
 (*====================================================*)

 PROCEDURE SaveinVtre (inH : THandle;
                       VAR outMessage : TString; inoutVtreOrdner : TVtrOrdner) : BOOLEAN;
 (*
    vb: VertraegeOrdner existiert
        Sichert den VertraegeOrdner auf Datei
        Sei OK der boolesche RFirstValidVtrZuHK (persID, aktV, inoutVtrOrdner^.Ordner);ueckgabecode, dann gilt:
    nb: OK <=>  Sicherung korrekt
 *)
 VAR  erfolgreich, next_vtr : BOOLEAN;
      outm_str, id_str, persid_str, kstid_str, gueltigvon_str,
      gueltigbis_str, erstell_str, storno_str, taetigkt_str, begruend_str,
      stdzahl_str : TString;
      arrayofchar : ARRAY [1..255] OF CHAR;
      aktVtr : TVtr;
      aktVtrID : TVtrID;
      pers_id : TPersID;
      kst_id : TKstID;
      gueltigvon, gueltigbis, erstell, storno : TDatum;
      stdzahl : CARDINAL;
 BEGIN


 IF NOT FirstVtrUndStorno (aktVtr, inoutVtreOrdner) THEN

    (* VertrgeOrdner ist leer *)
      erfolgreich := TRUE;

 ELSE

    (* KostenstellenOrdner ist nicht leer *)
    next_vtr := TRUE;
    erfolgreich := TRUE;

    WHILE next_vtr AND erfolgreich DO

       aktVtrID := Vtr.LiesVtrIDinVtr (aktVtr);
       id_str := Vtr.VtrIDAlsString (aktVtrID);

       pers_id := Vtr.LiesPersIDinVtr (aktVtr);
       persid_str := PersIDAlsString (pers_id);

       kst_id := Vtr.LiesKstIDinVtr (aktVtr);
       kstid_str := KstIDAlsString (kst_id);

       gueltigvon := Vtr.LiesGueltigVoninVtr (aktVtr);
       gueltigvon_str := DatumAlsString (gueltigvon);

       gueltigbis := Vtr.LiesGueltigBisinVtr (aktVtr);
       gueltigbis_str := DatumAlsString (gueltigbis);

       erstell := Vtr.LiesErstellDatinVtr (aktVtr);
       erstell_str := DatumAlsString (erstell);

       storno := Vtr.LiesStornoAminVtr (aktVtr);
       storno_str := DatumAlsString (storno);

       stdzahl := Vtr.LiesStdzahlinVtr (aktVtr);
       CardToStr (stdzahl, arrayofchar);
       stdzahl_str := String.Create();
       String.Schreibe (stdzahl_str, arrayofchar);

       taetigkt_str := Vtr.LiesTaetigktinVtr (aktVtr);

       begruend_str := Vtr.LiesBegruendinVtr (aktVtr);

       outm_str := String.Create();
       erfolgreich := WriteLineInVDat (inH, id_str, outm_str) AND
                      WriteLineInVDat (inH, persid_str, outm_str) AND
                      WriteLineInVDat (inH, kstid_str, outm_str) AND
                      WriteLineInVDat (inH, gueltigvon_str, outm_str) AND
                      WriteLineInVDat (inH, gueltigbis_str, outm_str) AND
                      WriteLineInVDat (inH, erstell_str, outm_str) AND
                      WriteLineInVDat (inH, storno_str, outm_str) AND
                      WriteLineInVDat (inH, stdzahl_str, outm_str) AND
                      WriteLineInVDat (inH, taetigkt_str, outm_str) AND
                      WriteLineInVDat (inH, begruend_str, outm_str);


       DisposeHID (aktVtrID);
       String.Dispose (id_str);

       DisposePersID (pers_id);
       String.Dispose (persid_str);

       DisposeKstID (kst_id);
       String.Dispose (kstid_str);

       DisposeDatum (gueltigvon);
       String.Dispose (gueltigvon_str);

       DisposeDatum (gueltigbis);
       String.Dispose (gueltigbis_str);

       DisposeDatum (erstell);
       String.Dispose (erstell_str);

       DisposeDatum (storno);
       String.Dispose (storno_str);

       String.Dispose (taetigkt_str);
       String.Dispose (begruend_str);

       (* String.Dispose (outm_str); *)

       next_vtr := NextVtrUndStorno (aktVtr, inoutVtreOrdner);

    END; (* WHILE *)

    IF erfolgreich
    THEN
      erfolgreich := TRUE;
    ELSE
      erfolgreich := FALSE;
      outMessage := String.Create ();
      String.Schreibe (outMessage,
              "KostenstellenOrdner konnte nicht gesichert werden.");
    END; (* IF WriteLineInVDat *)

    Vtr.DisposeVtr (aktVtr);

 END; (* IF NOT FirstKst *)

 RETURN erfolgreich;
 END SaveinVtre;

 PROCEDURE LoadinVtre (inH : THandle;
                       VAR outMessage : TString; inoutVtreOrdner : TVtrOrdner) : BOOLEAN;
 (*
    vb: VertraegeOrdner ist mit Save gespeichert worden
        Loescht alle Elemente in einem vorhandenen VertraegeOrdner
        Erstellt den VertraegeOrdner aus der mit Save erstellten Datei
        Sei OK der boolesche Rueckgabecode, dann gilt:
   nb: OK <=>  Der VertraegeOrdner ist korrekt erstellt
 *)

 VAR success, egal : BOOLEAN;
     str, message, outm_str, insert_message, taetigkt, begruend,
     undef_str_kurz, undef_str_lang : TString;
     aktVtr : TVtr;
     aktVtrID : TVtrID;
     persid : TPersID;
     kstID : TKstID;
     gueltigvon, gueltigbis, erstell, storno : TDatum;
     stdzahl : CARDINAL;
     CharArray : ARRAY [1..255] OF CHAR;
     result : ConvResults;
 BEGIN

 (* Leeren des vorhandenen Ordners *)
 LoescheAllesInVtreOrdner (inoutVtreOrdner);

 (* Sequentielles Einlesen der Kostenstellen *)


 str := String.Create();
 message := String.Create();
 aktVtr := Vtr.CreateVtr();

 (* undef_str wird benoetigt, um Datum mit "undef" wieder einlesen zu koennen *)
 undef_str_kurz := String.Create();
 String.Schreibe (undef_str_kurz, "undef");
 undef_str_lang := String.Create();
 String.Schreibe (undef_str_lang, "undef     ");

 success := TRUE;

 WHILE NOT EofInVDat (inH) AND success DO

   (* VertragsID *)
   IF ReadLineInVDat (inH, str, message) AND success THEN
      IF Vtr.PruefeVtrID (str, outm_str) THEN
         aktVtrID := Vtr.StringInVtrID (str);
         Vtr.SetzeVtrID (aktVtr, aktVtrID);
         DisposeHID (aktVtrID);
         String.Dispose (outm_str);
         success := TRUE;
      ELSE
         success := FALSE;
         String.Dispose (outm_str);
      END;
   ELSE
      success := FALSE;
   END;

   (* PersonenID *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN
   IF PruefePersID (str, outm_str) THEN
         persid := StringInPersID (str);
         Vtr.SetzePersIDinVtr (aktVtr, persid );
         DisposePersID (persid);
         String.Dispose (outm_str);
         success := TRUE;
      ELSE
         success := FALSE;
         String.Dispose (outm_str);
      END;
   ELSE
      success := FALSE;
   END;


   (* KostenstellenID *)
   IF (NOT EofInVDat (inH)) AND
       ReadLineInVDat (inH, str, message) AND success THEN
      IF PruefeKstID (str, outm_str) THEN
         kstID := StringInKstID (str);
         Vtr.SetzeKstIDinVtr (aktVtr, kstID);
         DisposeKstID (kstID);
         String.Dispose (outm_str);
         success := TRUE;
      ELSE
         success := FALSE;
         String.Dispose (outm_str);
      END;
   ELSE
      success := FALSE;
   END;


   (* Datum: GueltigVon *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN

      (* Aus "undef     " wird "undef" gemacht, sonst Konvertierungsfehler *)
      IF String.IsEqual (str, undef_str_lang) THEN
         String.Equalize (str, undef_str_kurz)
      END;

      IF NOT DatumVonString (str, gueltigvon, message) THEN
         success := FALSE
      ELSE
         Vtr.SetzeGueltigVoninVtr (aktVtr, gueltigvon);
         DisposeDatum (gueltigvon);
         success := TRUE;
      END;
   ELSE
      success := FALSE;
   END;


   (* Datum: GueltigBis *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN

      (* Aus "undef     " wird "undef" gemacht, sonst Konvertierungsfehler *)
      IF String.IsEqual (str, undef_str_lang) THEN
         String.Equalize (str, undef_str_kurz)
      END;

      IF NOT DatumVonString (str, gueltigbis, message) THEN
         success := FALSE
      ELSE
         Vtr.SetzeGueltigBisinVtr (aktVtr, gueltigbis);
         DisposeDatum (gueltigbis);
         success := TRUE;
      END;
   ELSE
      success := FALSE;
   END;


   (* Datum: ErstelltAm *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN

      (* Aus "undef     " wird "undef" gemacht, sonst Konvertierungsfehler *)
      IF String.IsEqual (str, undef_str_lang) THEN
         String.Equalize (str, undef_str_kurz)
      END;

      IF NOT DatumVonString (str, erstell, message) THEN
         success := FALSE
      ELSE
         egal := Vtr.SetzeErstellDatinVtr (aktVtr, erstell);
         DisposeDatum (erstell);
         success := TRUE;
      END;
   ELSE
      success := FALSE;
   END;


   (* Datum: StorniertAm *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN

      (* Aus "undef     " wird "undef" gemacht, sonst Konvertierungsfehler *)
      IF String.IsEqual (str, undef_str_lang) THEN
         String.Equalize (str, undef_str_kurz)
      END;

      IF NOT DatumVonString (str, storno, message) THEN
         success := FALSE
      ELSE
         egal := Vtr.SetzeStornoAminVtr (aktVtr, storno);
         DisposeDatum (storno);
         success := TRUE;
      END;
   ELSE
      success := FALSE;
   END;


   (* Stundenzahl *)
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, str, message) AND success THEN
      String.Lies (str, CharArray);
      StrToCard(CharArray,stdzahl,result);
      IF (result=strAllRight) THEN
         egal := Vtr.SetzeStdzahlinVtr (aktVtr, stdzahl);
         success := TRUE;
      ELSE
         success := FALSE;
      END;
   ELSE
      success := FALSE;
   END;


   (* Taetigkeit *)
   taetigkt := String.Create ();
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, taetigkt, message) AND success THEN
      egal := Vtr.SetzeTaetigktinVtr (aktVtr, taetigkt);
      String.Dispose (taetigkt);
      success := TRUE;
   ELSE
      success := FALSE;
   END;


   (* Begruendung *)
   begruend := String.Create ();
   IF (NOT EofInVDat (inH)) AND
      ReadLineInVDat (inH, begruend, message) AND success THEN
      egal := Vtr.SetzeBegruendinVtr (aktVtr, begruend);
      String.Dispose (begruend);
      success := TRUE;
   ELSE
      success := FALSE;
   END;


   (* Erzeugen eines ADO's vom Typ TKst, Setzen der Attribute
      und Einfgen in den KostenstellenOrdner                       *)
   IF success THEN
      IF InsertVtr (aktVtr, insert_message, inoutVtreOrdner, FALSE) THEN
         success := TRUE;
      ELSE
         success := FALSE;
         outMessage := String.Create();
         String.Equalize (outMessage, insert_message);
         String.Dispose (insert_message);
      END; (* IF InsertKst *)
   ELSE
      outMessage := String.Create();
      String.Schreibe (outMessage,
      "VertrgeOrdner konnte NICHT erfolgreich restauriert werden !");
   END;

 END; (* WHILE *)

 String.Dispose (str);
 String.Dispose (message);
 String.Dispose (undef_str_kurz);
 String.Dispose (undef_str_lang);
 Vtr.DisposeVtr (aktVtr);

 RETURN success;
 END LoadinVtre;


 
END VtrOSchablone.
