(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: VtreOrdner.mod,v $
# $Author: Sopra $
# $Date: 2001/03/13 19:17:49 $
# $Revision: 1.5 $
Bemerkungen:

IMPLEMENTATION MODULE VtreOrdner   ADT-Modul VertraegeOrdner
                                   generiert das ADO VertraegeOrdner

Implementation : Herbert Schoenhammer, Herbert.Schoenhammer@Fernuni-Hagen.de
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE VtreOrdner;           (* ENTHALTEN IN Vertraege TS_Vtre *)


  IMPORT VtrOSchablone;
  FROM VtreIMPORTE IMPORT TVtrID, TPersID, TKstID, THandle;

  FROM Datum IMPORT TDatum;

  FROM String IMPORT TString;
  FROM Stringliste IMPORT TStringliste;



(* Lokale Datenstruktur als Modulgedaechtnis                         *)
(*===================================================================*)

  VAR VtrOrdner : TVtrOrdner;


(* Standard-Ordner Operationen                                                *)
(*============================================================================*)


PROCEDURE CreateVtrOrdner;
  (*    -- Generiert ein neues abstraktes Datenobjekt outOrdner *)
  BEGIN
    VtrOrdner := VtrOSchablone.CreateVtrOrdner ();
  END CreateVtrOrdner;


  (* -- GEMEINSAME VORBEDINGUNG FUER ALLE UEBRIGEN OPERATIONEN:  
     --     ADOs inListe bzw. inoutListe existieren *)



PROCEDURE InsertVtr ( inoutVtr: TVtr; VAR outMessage : TString) : BOOLEAN;
  BEGIN
   RETURN VtrOSchablone.InsertVtr (inoutVtr, outMessage, VtrOrdner, TRUE)
  END InsertVtr;


PROCEDURE SearchVtr( inVtrID: TVtrID; VAR outVtr : TVtr):BOOLEAN;
 (* vb: inVtr ist gueltig
	  Gib Vertrag der zu VtrID gehrt zurck
    nb: -- *)
 BEGIN
    RETURN  VtrOSchablone.SearchVtr (inVtrID, outVtr, VtrOrdner)
 END SearchVtr;


PROCEDURE AendernVtr ( inVtr: TVtr; VAR outMessage : TString) : BOOLEAN;
  BEGIN
   RETURN VtrOSchablone.AendernVtr (inVtr, outMessage, VtrOrdner)
  END AendernVtr;


PROCEDURE DeleteVtr ( inVtr: TVtr) : BOOLEAN;
  BEGIN
   RETURN VtrOSchablone.DeleteVtr (inVtr, VtrOrdner)
  END DeleteVtr;


(* Iteratoren ueber nicht stornierte Vertraege *)
(*=============================================*)


PROCEDURE FirstVtr (VAR outVtr:TVtr):BOOLEAN;
 (* vb: Keine
        Liefere den ersten nicht stornierten Vertrag im Ordner zurck
    nb: -- *)
 BEGIN
   RETURN VtrOSchablone.FirstVtr (outVtr, VtrOrdner)
 END FirstVtr;


PROCEDURE NextVtr (inoutVtr: TVtr):BOOLEAN;
 (* vb: inK existiert in VertrageOrdner
        Suche inoutVtr im VertraegeOrdner und falls ein nicht stronierter 
        Nachfolger existiert gib diesen zurck, false sonst.
    nb: -- *)
 BEGIN
   RETURN VtrOSchablone.NextVtr (inoutVtr, VtrOrdner)
 END NextVtr;


PROCEDURE FirstVldVtrAbDatum (inDatum: TDatum; VAR outVtr:TVtr):BOOLEAN;
 (* Vb: NOT IsEmpty(inoutVtrOrdner)
        Der erste nicht stornierte Vertrag, dessen GueltigBis >= inDatum
        wird in outVtr zurueckgegeben
        und inoutHKOrdner.pos auf diesen Vertrag positioniert.
    Nb: (outVtr enthaelt den Vertrag *)

 BEGIN
   RETURN VtrOSchablone.FirstVldVtrAbDatum (inDatum, outVtr, VtrOrdner)
 END FirstVldVtrAbDatum;


PROCEDURE NextVldVtrAbDatum (inDatum: TDatum; inoutVtr: TVtr):BOOLEAN;
 (* Vb: NOT IsEmpty(inoutVtrOrdner)
        Der nchste nicht stornierte Vertrag, dessen GueltigBis >= inDatum
        wird in outVtr zurueckgegeben
        und inoutHKOrdner.pos auf diesen Vertrag positioniert.
    Nb: (outVtr enthaelt den Vertrag *)
 BEGIN
   RETURN VtrOSchablone.NextVldVtrAbDatum (inDatum, inoutVtr, VtrOrdner)
 END NextVldVtrAbDatum;


PROCEDURE FirstVtrImZeitraum( inGueltigVon, inGueltigBis:TDatum;
                               VAR outVtr : TVtr) : BOOLEAN;
 (*
    vb: VertraegeOrdner existiert
        Hole den ersten nicht stornierten Vertrag, dessen GueltigVon-Datum
        im Zeitraum inGueltigVon-inGueltigBis liegt
        Sei OK der boolesche Rueckgabecode, dann gilt:
    nb: OK <=> nicht stornierter Vertrag im Zeitraum existiert
 *)
 BEGIN
   RETURN VtrOSchablone.FirstVtrImZeitraum (inGueltigVon, inGueltigBis, outVtr, VtrOrdner)
 END FirstVtrImZeitraum;


PROCEDURE NextVtrImZeitraum( inGueltigVon, inGueltigBis:TDatum;
                                      inoutVtr : TVtr) : BOOLEAN;
 (*
    vb: VertraegeOrdner existiert
        Falls mit FirstVtrImZeitraumHolen ein nicht stornierter Vertrag,
        dessen GueltigVon-Datum im Zeitraum
        inGueltigVon-inGueltigBis gefunden wurde, dann holt diese Prozedur den
        naechsten Vertrag.
        Sei OK der boolesche Rueckgabecode, dann gilt:
    nb: OK <=> Weiterer nicht stornierter Vertrag im Zeitraum existiert
 *)
 BEGIN
   RETURN VtrOSchablone.NextVtrImZeitraum (inGueltigVon, inGueltigBis,
                                            inoutVtr, VtrOrdner)
 END NextVtrImZeitraum;


PROCEDURE FirstValidVtrZuHK (inPersID : TPersID; VAR outVtr : TVtr) : BOOLEAN;
 (* Ersten gueltigter Vertrag (nicht storniert) einer Hilfskraft ausgeben *)
 BEGIN
   RETURN VtrOSchablone.FirstValidVtrZuHK (inPersID, outVtr, VtrOrdner)
 END FirstValidVtrZuHK;


PROCEDURE NextValidVtrZuHK  (inPersID : TPersID; inoutVtr : TVtr) : BOOLEAN;
 (* Naechsten gueltiger Vertrag (nicht storniert) einer Hilfskraft ausgeben *)
 BEGIN
   RETURN VtrOSchablone.NextValidVtrZuHK (inPersID, inoutVtr, VtrOrdner)
 END NextValidVtrZuHK;


PROCEDURE FirstVldVtrZuHKinZtraum (inPersID : TPersID; inAnfang,inEnde :TDatum;
                                    VAR outVtr : TVtr) : BOOLEAN;
 (* Ersten gueltigten Vertrag (nicht storniert) einer Hilfskraft ausgeben
    im Zeitraum *)
 BEGIN
   RETURN VtrOSchablone.FirstVldVtrZuHKinZtraum (inPersID, inAnfang, inEnde,
                                                 outVtr, VtrOrdner)
 END FirstVldVtrZuHKinZtraum;


PROCEDURE NextVldVtrZuHKinZtraum  (inPersID : TPersID; inAnfang,inEnde : TDatum;
                                    inoutVtr : TVtr) : BOOLEAN;
 (* Naechsten gueltigten Vertrag (nicht storniert) einer Hilfskraft ausgeben
    im Zeitraum *)
 BEGIN
   RETURN VtrOSchablone.NextVldVtrZuHKinZtraum (inPersID, inAnfang, inEnde,
                                                 inoutVtr, VtrOrdner)
 END NextVldVtrZuHKinZtraum;


PROCEDURE FirstVldVtrZuHKinZtpunkt (inPersID : TPersID; inDatum : TDatum;
                                     VAR outVtr : TVtr) : BOOLEAN;
 (* Ersten gueltigten Vertrag (nicht storniert) einer Hilfskraft ausgeben
    im Zeitpunkt *)
 BEGIN
   RETURN VtrOSchablone.FirstVldVtrZuHKinZtpunkt (inPersID, inDatum, outVtr,
                                                  VtrOrdner)
 END FirstVldVtrZuHKinZtpunkt;


PROCEDURE NextVldVtrZuHKinZtpunkt  (inPersID : TPersID; inDatum : TDatum;
                                     inoutVtr : TVtr): BOOLEAN;
 (* Naechsten gueltigten Vertrag (nicht storniert) einer Hilfskraft ausgeben
    im Zeitpunkt *)
 BEGIN
   RETURN VtrOSchablone.NextVldVtrZuHKinZtpunkt (inPersID, inDatum,
                                                  inoutVtr, VtrOrdner)
 END NextVldVtrZuHKinZtpunkt;


PROCEDURE BerMaxStdZahlDerHK(inVtr: TVtr): INTEGER;
	(* Gabi
	S. 45: ab (*HK nach VKond prefn *)
	bis .. Stunden := Max()... (=Rckgabe)
	    END (*FOR*)
	AnzahlMonate wird nicht benoetigt !!!!
	*)
 BEGIN
   RETURN VtrOSchablone.BerMaxStdZahlDerHK (inVtr, VtrOrdner)
 END BerMaxStdZahlDerHK;


PROCEDURE BerMaxBeschZeitDerHK(inHKID : TPersID;
                               VAR outVonDatum, outBisDatum : TDatum): BOOLEAN;
 BEGIN
   RETURN VtrOSchablone.BerMaxBeschZeitDerHK (inHKID, outVonDatum,
                                                      outBisDatum, VtrOrdner)
 END BerMaxBeschZeitDerHK;



(* Iterator ueber alle Vertraege (einschl. der stornierten) *)
(*==========================================================*)


PROCEDURE FirstVtrUndStorno(VAR outVtr : TVtr):BOOLEAN;
 (*
    vb: VertraegeOrdner existiert
        Holt den ersten Vertrag
        Sei OK der boolesche Rueckgabecode, dann gilt:
   nb: OK <=> Mindestens ein Vertrag existiert
 *)
 BEGIN
   RETURN VtrOSchablone.FirstVtrUndStorno (outVtr, VtrOrdner)
 END FirstVtrUndStorno;


PROCEDURE NextVtrUndStorno (inoutVtr : TVtr) : BOOLEAN;
 (*
    vb: VertraegeOrdner existiert
        Falls mit FirstVtrHolen ein Vertrag gefunden wurde, dann holt diese 
        Prozedur den naechsten Vertrag.
        Sei OK der boolesche Rueckgabecode, dann gilt:
   nb: OK <=> Weiterer Vertrag im Zeitraum existiert
 *)
 BEGIN
    RETURN VtrOSchablone.NextVtrUndStorno (inoutVtr, VtrOrdner)
 END NextVtrUndStorno;


PROCEDURE  FirstVtrZuHK (inPersID : TPersID; VAR outVtr : TVtr) : BOOLEAN;
 (* Ersten Vertrag einer Hilfskraft ausgeben *)
 BEGIN
   RETURN VtrOSchablone.FirstVtrZuHK (inPersID, outVtr, VtrOrdner)
 END FirstVtrZuHK;


PROCEDURE  NextVtrZuHK  (inPersID : TPersID; inoutVtr : TVtr) : BOOLEAN;
 (* Naechsten Vertrag einer Hilfskraft ausgeben *)
 BEGIN
  RETURN VtrOSchablone.NextVtrZuHK (inPersID, inoutVtr, VtrOrdner)
 END NextVtrZuHK;


PROCEDURE  ExistVtrZuHK (inHKID:TPersID):BOOLEAN;
 (* Prueft, ob zu der HK inHKID jemals ein Vertrag existiert hat.
	 Dabei ist es voellig egal, ob der Vertrag storniert ist usw. 
       Dies Voraussetzung fr Loesche HK *)
 BEGIN
    RETURN VtrOSchablone.ExistVtrZuHK (inHKID, VtrOrdner)
 END ExistVtrZuHK;


PROCEDURE  FirstVtrZuKst (inKstID : TKstID; VAR outVtr : TVtr) : BOOLEAN;
 (* Ersten Vertrag einer Kostenstelle ausgeben *)
 BEGIN
   RETURN VtrOSchablone.FirstVtrZuKst (inKstID, outVtr, VtrOrdner)
 END FirstVtrZuKst;


PROCEDURE  NextVtrZuKst  (inKstID : TKstID; inoutVtr : TVtr) : BOOLEAN;
 (* Naechsten Vertrag einer Kostenstelle ausgeben *)
 BEGIN
   RETURN VtrOSchablone.NextVtrZuKst (inKstID, inoutVtr, VtrOrdner)
 END NextVtrZuKst;


PROCEDURE FirstVtrZuHKinZeitraum (inPersID : TPersID; inAnfang,inEnde :TDatum;
                                   VAR outVtr : TVtr) : BOOLEAN;
  (* Ersten Vertrag zu einer Hilfskraft im Zeitraum ausgeben *)
 BEGIN
    RETURN VtrOSchablone.FirstVtrZuHKinZeitraum (inPersID, inAnfang, inEnde,
                                                 outVtr, VtrOrdner)
 END FirstVtrZuHKinZeitraum;


PROCEDURE NextVtrZuHKinZeitraum  (inPersID : TPersID; inAnfang,inEnde : TDatum;
                                   inoutVtr : TVtr) : BOOLEAN;
  (* Naechsten Vertrag zu einer Hilfskraft im Zeitraum ausgeben *)
 BEGIN
    RETURN VtrOSchablone.NextVtrZuHKinZeitraum (inPersID, inAnfang, inEnde,
                                                 inoutVtr, VtrOrdner)
 END NextVtrZuHKinZeitraum;


PROCEDURE FirstVtrZuKstinZeitraum (inKstID : TKstID; inAnfang,inEnde : TDatum;
                                    VAR outVtr : TVtr) : BOOLEAN;
  (* Ersten Vertrag zu einer Kostenstelle im Zeitraum ausgeben *)
 BEGIN
    RETURN VtrOSchablone.FirstVtrZuKstinZeitraum (inKstID, inAnfang, inEnde,
                                                  outVtr, VtrOrdner)
 END FirstVtrZuKstinZeitraum;


PROCEDURE NextVtrZuKstinZeitraum  (inKstID : TKstID; inAnfang,inEnde : TDatum;
                                    inoutVtr : TVtr) : BOOLEAN;
  (* Naechtsen Vertrag zu einer Kostenstelle im Zeitraum ausgeben *)
 BEGIN
    RETURN VtrOSchablone.NextVtrZuKstinZeitraum (inKstID, inAnfang, inEnde,
                                                  inoutVtr, VtrOrdner)
 END NextVtrZuKstinZeitraum;


PROCEDURE FirstVtrZuHKinZeitpunkt (inPersID : TPersID; inDatum : TDatum;
                                    VAR outVtr : TVtr) : BOOLEAN;
   (* Ersten Vertag zu einer Hilfskraft ausgeben, wobei gelten mu:
      outVtr.GueltigVon <= inDatum <= outVtr.GueltigBis *)
 BEGIN
   RETURN VtrOSchablone.FirstVtrZuHKinZeitpunkt (inPersID, inDatum,
                                                 outVtr, VtrOrdner)
 END FirstVtrZuHKinZeitpunkt;


PROCEDURE NextVtrZuHKinZeitpunkt (inPersID : TPersID; inDatum : TDatum;
                                   inoutVtr : TVtr) : BOOLEAN;
   (* Naechsten Vertag zu einer Hilfskraft ausgeben, wobei gelten mu:
      outVtr.GueltigVon <= inDatum <= outVtr.GueltigBis *)
 BEGIN
   RETURN VtrOSchablone.NextVtrZuHKinZeitpunkt (inPersID, inDatum,
                                                inoutVtr, VtrOrdner)
 END NextVtrZuHKinZeitpunkt;



(* StringOperationen                                    *)
(*======================================================*)


PROCEDURE GibVtrAlsStringlisteInVtr(): TStringliste;
 BEGIN
   RETURN VtrOSchablone.GibVtrAlsStringlisteInVtr(VtrOrdner)
 END GibVtrAlsStringlisteInVtr;


PROCEDURE GibAllesAlsStringlisteInVtr(): TStringliste;
 BEGIN
   RETURN VtrOSchablone.GibAllesAlsStringlisteInVtr(VtrOrdner)
 END GibAllesAlsStringlisteInVtr;


(* Prozeduren zum Laden und Speichern des Teilsystems *)
(*====================================================*)


PROCEDURE SaveinVtre (inH : THandle;
                       VAR outMessage : TString) : BOOLEAN;
 (* vb: VertraegeOrdner existiert
        Sichert den VertraegeOrdner auf Datei
        Sei OK der boolesche Rueckgabecode, dann gilt:
    nb: OK <=>  Sicherung korrekt *)
 BEGIN
   RETURN VtrOSchablone.SaveinVtre (inH, outMessage, VtrOrdner)
 END SaveinVtre;


 PROCEDURE LoadinVtre (inH : THandle;
                       VAR outMessage : TString) : BOOLEAN;
 (* vb: VertraegeOrdner ist mit Save gespeichert worden
        Loescht alle Elemente in einem vorhandenen VertraegeOrdner
        Erstellt den VertraegeOrdner aus der mit Save erstellten Datei
        Sei OK der boolesche Rueckgabecode, dann gilt:
    nb: OK <=>  Der VertraegeOrdner ist korrekt erstellt *)
 BEGIN
   RETURN VtrOSchablone.LoadinVtre (inH, outMessage, VtrOrdner)
 END LoadinVtre;


 (* Architektur-ADO VertraegeOrdner erzeugen                            *)
 (***********************************************************************)

BEGIN

   CreateVtrOrdner

END VtreOrdner.
